/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithm;

import javax.vecmath.Vector3f;

/**
 * This class is used in order to represent a bonded neighbor of an atom. The
 * instances of this class can be accessed through the Atom-Objects
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class Bond {

	public Atom neigbor;
	public float originalLength;
	private Atom source;

	/**
	 * Constructs a new Object !NOT TO BE USED BY AUTHORS!
	 */
	public Bond(Atom source, Atom neigbor, float originalLength) {
		this.source = source;
		this.neigbor = neigbor;

		this.originalLength = originalLength;
	}

	/**
	 * Please use getOriginalLength() instead.
	 * 
	 * @return the ideal bond length in angstrom
	 */
	@Deprecated
	public float getOriginalLenght() {
		return this.originalLength;
	}
	
	/**
	 * Returns the ideal length of this bond, as determined by the pdb-file or
	 * the profile applied to the molecule
	 * 
	 * @return the ideal bond length in angstrom
	 */
	public float getOriginalLength() {
		return this.originalLength;
	}

	/**
	 * Please use getCurrentLength() instead.
	 * 
	 * @return the current length in angstrom
	 */
	@Deprecated
	public float getCurrentLenght() {
		return source.position.distance(neigbor.position);
	}
	
	/**
	 * Returns the current length of this bond
	 * 
	 * @return the current length in angstrom
	 */
	public float getCurrentLength() {
		return source.position.distance(neigbor.position);
	}

	/**
	 * Returns the direction of the neighbor in respect to the atom this bond is
	 * associated with.
	 * 
	 * @return an unnormalized vector that points into the direction of the
	 *         neigboring atom
	 */
	public Vector3f getDirection() {
		Vector3f v = new Vector3f(neigbor.position);
		v.sub(source.position);
		return v;
	}

	/**
	 * Returns the instance of the bonded neighbor of the atom that contains
	 * this bond
	 * 
	 * @return the neigboring atom
	 */
	public Atom getBondedNeighbor() {
		return this.neigbor;
	}
}
