/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.coarsegrainsupport;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.vecmath.Matrix3f;

import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * This is the controlpanel used in the Conformation-Change-mode of the Molecule Editor 
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
public class ConformationPanel extends ControlPanel implements ActionListener {
	TitledPanel tp1 = new TitledPanel("Fixed Camera");
	JPanel tp2 = new JPanel();
	TitledPanel tp3 = new TitledPanel("Current Conformation");
	JButton undoBtn = new JButton("<<Undo");
	JButton resetInitialBtn = new JButton("Reset to Initial");
	JButton defaultBtn = new JButton("Reset to Default");

	JButton cam1Btn = new JButton("Front View");
	JButton cam2Btn = new JButton("Side View");
	JButton cam3Btn = new JButton("Top View");

	// JRadioButton rb1 = new JRadioButton("Select / Rotate Camera");
	// JRadioButton rb2 = new
	// JRadioButton("Rotate Selected around chosen Center");
	// JRadioButton rb3 = new JRadioButton("Translate Selected");

	private static final long serialVersionUID = 1L;

	public ConformationPanel(MoleculeEditor editor) {
		super(editor);
		// ButtonGroup bg = new ButtonGroup();
		// bg.add(rb1);
		// bg.add(rb2);
		// bg.add(rb3);
		// rb1.setSelected(true);
		this.headerTitle.setText("  Edit Conformation  ");
		this.headerText
				.setText("<html>Specify the default conformation of a molecule that uses this profile. For this, select multiple Atoms/Particles and move them synchronously by either rotating them around a specified atom (use this to bend strctures) or translating them freely<br> </html>");

		this.componentPanel.add(tp3);
		this.componentPanel.add(tp1);
		this.componentPanel.add(tp2);
		// rb1.addActionListener(this);
		// rb2.addActionListener(this);
		// rb3.addActionListener(this);

		cam1Btn.addActionListener(this);
		cam2Btn.addActionListener(this);
		cam3Btn.addActionListener(this);

		undoBtn.addActionListener(this);
		resetInitialBtn.addActionListener(this);
		defaultBtn.addActionListener(this);

		tp1.setLayout(new BoxLayout(tp1, BoxLayout.Y_AXIS));
		JPanel btnPnl = new JPanel();
		JPanel p1 = new JPanel();

		btnPnl.setLayout(new GridLayout(2, 2));
		JLabel l1 = new JLabel("   Switch to a fixed camera position");
		p1.add(l1);
		p1.add(Box.createGlue());
		p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));

		tp1.add(p1);

		JPanel p2 = new JPanel();
		p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
		p2.add(Box.createHorizontalGlue());
		p2.add(btnPnl);
		p2.add(Box.createHorizontalGlue());
		tp1.add(p2);

		btnPnl.add(cam3Btn);
		btnPnl.add(cam2Btn);
		btnPnl.add(cam1Btn);
		btnPnl.add(new JPanel());

		// tp2.setLayout(new BoxLayout(tp2, BoxLayout.Y_AXIS));
		// tp2.add(rb1);
		// tp2.add(Box.createGlue());
		// tp2.add(rb2);
		// tp2.add(Box.createGlue());
		// tp2.add(rb3);

		tp3.setLayout(new BoxLayout(tp3, BoxLayout.Y_AXIS));
		JPanel btnPnl2 = new JPanel();

		btnPnl2.setLayout(new BoxLayout(btnPnl2, BoxLayout.X_AXIS));

		tp3.add(Box.createGlue());
		tp3.add(btnPnl2);
		tp3.add(Box.createGlue());

		btnPnl2.add(Box.createHorizontalGlue());
		btnPnl2.add(undoBtn);
		btnPnl2.add(Box.createHorizontalStrut(5));
		btnPnl2.add(resetInitialBtn);
		btnPnl2.add(Box.createHorizontalStrut(5));
		btnPnl2.add(defaultBtn);
		btnPnl2.add(Box.createHorizontalGlue());

		componentPanel.add(logPanel);

	}

	public void actionPerformed(ActionEvent arg0) {

		if (arg0.getSource() == cam1Btn) {

			Matrix3f m = Cm2_Toolkit
					.copyMatrix(this.editor.display.initialRotationMatrix);
			this.editor.display.setUpCamera();
			this.editor.currentMolecule.setRotationMatrix(m);
			this.editor.currentMolecule.move();

		} else if (arg0.getSource() == cam2Btn) {
			Matrix3f m = Cm2_Toolkit
					.copyMatrix(this.editor.display.initialRotationMatrix);
			this.editor.display.setUpCamera();
			Cm2_Toolkit.rotate_Y_Axis_Of_CoordinateSystem(m, 270f);
			this.editor.currentMolecule.setRotationMatrix(m);
			this.editor.currentMolecule.move();

		} else if (arg0.getSource() == cam3Btn) {
			Matrix3f m = Cm2_Toolkit
					.copyMatrix(this.editor.display.initialRotationMatrix);
			this.editor.display.setUpCamera();
			Cm2_Toolkit.rotate_X_Axis_Of_CoordinateSystem(m, 90f);
			this.editor.currentMolecule.setRotationMatrix(m);
			this.editor.currentMolecule.move();
		} else if (arg0.getSource() == undoBtn) {
			this.editor.undoConformation(true);
		} else if (arg0.getSource() == resetInitialBtn) {
			this.editor.undoConformation(false);
		} else if (arg0.getSource() == defaultBtn) {
			this.editor.setDefaultConformation();
		}

	}

	@Override
	void updateComponents() {

		this.undoBtn.setEnabled(!this.editor.history.isEmpty());

	}

}
