/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.coarsegrainsupport;


import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.plaf.basic.BasicComboBoxEditor;

import org.cellmicrocosmos.cm2.gui.dialogs.LimitedDocument;



/**
 * This class describes the panel to display or modify the textual properties of the selected particle(s)
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
public class ParticlePropertiesPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	JComponent comp1;
	JComponent comp2;
	JComponent comp3;

	JPanel p1 = new JPanel();
	JPanel p2 = new JPanel();

	JLabel l1 = new JLabel("<html><b>Particle ID  </b></html>", JLabel.TRAILING);
	JLabel l2 = new JLabel("<html><b>Atom Name  </b></html>", JLabel.TRAILING);
	JLabel l3 = new JLabel("<html><b>Element  </b></html>", JLabel.TRAILING);

	private final DefaultComboBoxModel model1 = new DefaultComboBoxModel();
	private final DefaultComboBoxModel model2 = new DefaultComboBoxModel();
	private final DefaultComboBoxModel model3 = new DefaultComboBoxModel();

	public JButton cBtn = new JButton("C");

	public ParticlePropertiesPanel(MoleculeEditor editor, boolean editable,
			String iD, String name, String element, Color color) {

		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		if (editable) {
			this.comp1 = new JComboBox();
			this.comp2 = new JComboBox();
			this.comp3 = new JComboBox();
			JComboBox cb1 = (JComboBox) comp1;
			JComboBox cb2 = (JComboBox) comp2;
			JComboBox cb3 = (JComboBox) comp3;

			((JComboBox) this.comp1).addActionListener(this);
			((JComboBox) this.comp2).addActionListener(this);
			((JComboBox) this.comp3).addActionListener(this);

			cb1.setModel(model1);
			cb2.setModel(model2);
			cb3.setModel(model3);
			cb1.setEditable(true);
			cb2.setEditable(true);
			cb3.setEditable(true);

			final JTextField tf1 = new JTextField();
			final JTextField tf2 = new JTextField();
			final JTextField tf3 = new JTextField();

			BasicComboBoxEditor bcbe = new BasicComboBoxEditor() {

				public Component getEditorComponent() {
					return tf1;
				}
			};
			BasicComboBoxEditor bcbe2 = new BasicComboBoxEditor() {

				public Component getEditorComponent() {
					return tf2;
				}
			};
			BasicComboBoxEditor bcbe3 = new BasicComboBoxEditor() {

				public Component getEditorComponent() {
					return tf3;
				}
			};

			cb1.setEditor(bcbe);
			cb2.setEditor(bcbe2);
			cb3.setEditor(bcbe3);
			tf1.setDocument(new LimitedDocument(tf1, 20));
			tf2.setDocument(new LimitedDocument(tf2, 3));
			tf3.setDocument(new LimitedDocument(tf3, 2));

			tf1.setText(iD);
			tf2.setText(name);
			tf3.setText(element);

		} else {

			this.comp1 = new JTextField();

			this.comp2 = new JTextField();

			this.comp3 = new JTextField();

			JTextField tf1 = (JTextField) this.comp1;

			JTextField tf2 = (JTextField) this.comp2;

			JTextField tf3 = (JTextField) this.comp3;

			tf1.setEnabled(false);

			tf2.setEnabled(false);

			tf3.setEnabled(false);

			tf1.setBorder(BorderFactory.createEmptyBorder());

			tf2.setBorder(BorderFactory.createEmptyBorder());

			tf3.setBorder(BorderFactory.createEmptyBorder());

			tf1.setDisabledTextColor(this.getForeground());

			tf2.setDisabledTextColor(this.getForeground());

			tf3.setDisabledTextColor(this.getForeground());

		}

		comp1.setFont(comp1.getFont().deriveFont(
				comp1.getFont().getSize2D() + 2));

		comp2.setFont(comp2.getFont().deriveFont(Font.BOLD,
				comp1.getFont().getSize2D() + 2));

		comp3.setFont(comp3.getFont().deriveFont(Font.BOLD,
				comp1.getFont().getSize2D() + 2));

		p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));

		p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));

		p1.add(Box.createGlue());
		p2.add(Box.createGlue());

		p1.add(l1);

		p2.add(l2);

		p1.add(comp1);

		p2.add(comp2);

		p2.add(l3);
		p2.add(comp3);

		p2.add(Box.createHorizontalStrut(5));
		p2.add(cBtn);
		cBtn.setFont(cBtn.getFont().deriveFont(Font.BOLD));
		if (color != null)
			cBtn.setForeground(color);
		else
			// cBtn.setForeground(Color.cyan);

			cBtn.setEnabled(editable);
		cBtn.setPreferredSize(new Dimension(25, 25));
		cBtn.setMinimumSize(new Dimension(25, 25));
		cBtn.setMaximumSize(new Dimension(25, 25));
		cBtn.setOpaque(true);
		cBtn.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				final Point pointAsBoolean = new Point(0, 0);
				final JColorChooser colorChooser = new JColorChooser();
				final JDialog dialog = JColorChooser.createDialog(null,
						"Pick a Color", true, colorChooser,
						new ActionListener() {

							public void actionPerformed(final ActionEvent arg0) {

								pointAsBoolean.x = 1;

							}

						}, null);

				Color newColor;

				colorChooser.setColor(cBtn.getBackground());

				dialog.toFront();
				dialog.setVisible(true);

				newColor = colorChooser.getColor();
				if (pointAsBoolean.x == 0
						|| newColor.equals(cBtn.getBackground()))
					return;

				cBtn.setForeground(newColor);
			}

		});

		int add = editable ? 26 : 0;

		comp1.setMinimumSize(new Dimension(230 + add, 25));
		comp1.setMaximumSize(new Dimension(230 + add, 25));
		comp1.setPreferredSize(new Dimension(230 + add, 25));

		comp2.setMinimumSize(new Dimension(50 + add, 25));
		comp2.setMaximumSize(new Dimension(50 + add, 25));
		comp2.setPreferredSize(new Dimension(50 + add, 25));

		comp3.setMinimumSize(new Dimension(30 + add, 25));
		comp3.setMaximumSize(new Dimension(30 + add, 25));
		comp3.setPreferredSize(new Dimension(30 + add, 25));

		// p2.add(Box.createHorizontalStrut(30), 3);

		this.add(Box.createGlue());
		this.add(p1);

		this.add(Box.createGlue());
		this.add(Box.createVerticalStrut(5));
		this.add(Box.createGlue());
		this.add(p2);
		this.add(Box.createGlue());

	}

	public void actionPerformed(ActionEvent arg0) {

		JComboBox cb = (JComboBox) arg0.getSource();

		JTextField tf = (JTextField) (cb.getEditor().getEditorComponent());

		tf.setText((String) cb.getSelectedItem());

	}

}
