/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.dataobjects;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.LineArray;
import javax.media.j3d.LineAttributes;
import javax.media.j3d.Shape3D;
import javax.vecmath.Color3f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;

/**
 * This class represents the constrains needed for the molecules.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 *
 */
public class Constraint_3D {

	// public float originalLength;
	public float currentLength;
	public Atom_3D atomA;
	public Atom_3D atomB;
	BranchGroup shapeBG;

	final LineArray pla = new LineArray(4, GeometryArray.COORDINATES
			| GeometryArray.COLOR_3);;

	final Appearance app = new Appearance();

	final LineAttributes la = new LineAttributes();
	public Point3f centerPoint;

	public Constraint_3D(Atom_3D a, Atom_3D b) {
		this.atomA = a;
		this.atomB = b;

		// if(!this.atomA.currentNeighbors.contains(this.atomB))this.atomA.currentNeighbors.addElement(this.atomB);
		// if(!this.atomB.currentNeighbors.contains(this.atomA))this.atomB.currentNeighbors.addElement(this.atomA);

		// originalLength=Cm2_Toolkit.calculateLengthBetween(a.position,b.position);

		this.update();

		la.setPatternMask(LineAttributes.PATTERN_DASH);
		la.setLineWidth(400 * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
		app.setLineAttributes(la);

		Cm2_Toolkit.setRenderingAttributes(app);
		
		final Shape3D shape = new Shape3D();
		shape.setAppearance(app);
		shape.setPickable(false);

		shape.addGeometry(pla);
		pla.setCapability(Geometry.ALLOW_INTERSECT);
		pla.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		pla.setCapability(LineArray.ALLOW_COLOR_WRITE);

		shapeBG = new BranchGroup();
		shapeBG.setCapability(BranchGroup.ALLOW_DETACH);
		shapeBG.addChild(shape);

	}

	public void update() {

		Point3f[] coords;

		Color3f[] colors;

		currentLength = Cm2_Toolkit.calculateLengthBetween(atomA.position,
				atomB.position);

		coords = new Point3f[4];
		coords[0] = new Point3f(atomA.position);
		coords[3] = new Point3f(atomB.position);

		coords[0].scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
		coords[3].scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);

		coords[1] = new Point3f(coords[0]);
		coords[1].sub(coords[3]);
		if(atomA instanceof Particle) {
			Vector3f v = new Vector3f(coords[1]);
			v.normalize();
			v.scale(atomB.radiusCov*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
			coords[1]=new Point3f(v);
		}else if(atomB instanceof Particle) {
			Vector3f v = new Vector3f(coords[1]);
			v.normalize();
			v.scale((currentLength*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE)-(atomB.radiusCov*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE));
			coords[1]=new Point3f(v);
		}else {
			
			Vector3f v = new Vector3f(coords[1]);
			v.normalize();
			v.scale(atomB.radiusCov*CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
			coords[1]=new Point3f(v);
		}
		
		
		coords[1].add(coords[3]);
		coords[2] = coords[1];
		this.centerPoint = coords[2];

		colors = new Color3f[] { new Color3f(atomA.color),
				new Color3f(atomA.color), new Color3f(atomB.color),
				new Color3f(atomB.color) };

		pla.setCoordinates(0, coords);

		pla.setColors(0, colors);

	}

	public BranchGroup getBranchGroup() {

		return shapeBG;
	}

}
