/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.dataobjects;

import java.util.Vector;

/**
 * The particles are extended atom models needed to represent
 * coarse grains.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 *
 */
public class Particle extends Atom_3D {

	public Vector<Atom_3D> includedAtoms = new Vector<Atom_3D>();
	public String particleIdentifier;

	Float mass=null;
	

	public Particle(Atom_3D[] includedAtoms) {
		super();
		this.set(includedAtoms);

//		color = Color.cyan;
//		radiusCov = 0f;
//		radiusVDW = 0f;
	}

	public boolean includes(Atom_3D a) {
		return includedAtoms.contains(a);
	}

	public void set(Atom_3D[] includedAtoms) {
		this.includedAtoms.clear();
		
		
		float mVDW=0f;
		float mCOV=0f;
		for (int i = 0; i < includedAtoms.length; i++) {
			this.includedAtoms.addElement(includedAtoms[i]);
			mVDW=Math.max(includedAtoms[i].radiusVDW,mVDW);
			mCOV=Math.max(includedAtoms[i].radiusCov,mCOV);
		}
		MyBounds bb = new MyBounds(includedAtoms);
		this.position = bb.center;

		
		this.radiusCov=Math.max(bb.widthX,this.radiusCov);
		this.radiusCov=Math.max(bb.widthY,this.radiusCov);
		this.radiusCov=Math.max(bb.widthZ,this.radiusCov);
		
		this.radiusCov/=2f;
		this.radiusVDW=this.radiusCov;
		
		this.radiusCov+=mCOV;
		this.radiusVDW+=mVDW;
		
		
		
		
	}

	public float getMassValue() {
		if(this.mass==null) {
			float m=0;
			for(int i=0;i<this.includedAtoms.size();i++) {
				m+=this.includedAtoms.get(i).massValue;
			}
			this.mass=new Float(m);
			return mass;
		}else return mass;
		
	}

	
	

}
