/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.functionality3d;

import java.awt.Font;
import java.util.Enumeration;

import javax.media.j3d.Appearance;
import javax.media.j3d.Billboard;
import javax.media.j3d.BoundingBox;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.Material;
import javax.media.j3d.Node;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.QuadArray;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Color3f;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Layer;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


import com.sun.j3d.utils.geometry.Cone;
import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.GeometryInfo;
import com.sun.j3d.utils.geometry.NormalGenerator;
import com.sun.j3d.utils.geometry.Primitive;
import com.sun.j3d.utils.geometry.Text2D;

/**
 * The Coordinate System. Contains a shape for the bars and some numbers next to
 * it. Constructs dependent on the size of the MembraneModel that is currently
 * active.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class CoordinateBar {

	Color3f color;
	BranchGroup branchGroup = new BranchGroup();
	BranchGroup branchGroup2 = new BranchGroup();
	TransformGroup transformGroup = new TransformGroup();
	Shape3D shape = new Shape3D();

	Primitive[] Cxyz = new Primitive[4];
	private Appearance appear;

	public CoordinateBar() {
		Cm2_Toolkit.setAllCapabilitysForBranchGroup(this.branchGroup);
		Cm2_Toolkit.setAllCapabilitysForBranchGroup(this.branchGroup2);
		Cm2_Toolkit.setAllCapabilitysForShape3D(this.shape);
		Cm2_Toolkit.setAllCapabilitysForTransformGroup(this.transformGroup);
		this.shape.setPickable(false);

		if (DataPool.J3DSettings.bright_appearance)
			this.color = new Color3f(
					CM2_CONSTANTS.J3D.COLOR_COORDINATEBARS_BRIGHT);
		else
			this.color = new Color3f(
					CM2_CONSTANTS.J3D.COLOR_COORDINATEBARS_DARK);

		this.branchGroup.addChild(this.transformGroup);

	}

	public synchronized void setBarsVisible(final boolean b) {

		final BranchGroup parent = (BranchGroup) this.branchGroup.getParent();
		parent.removeChild(this.branchGroup);

		final Enumeration<?> en = this.branchGroup2.getAllChildren();

		while (en.hasMoreElements()) {
			final Object o = en.nextElement();
			if (o instanceof Node)
				this.branchGroup2.removeChild((Node) o);
		}

		if (b) {

			// Start of measuring
			final MembraneModel model = DataPool.getMembraneModel();

			// The (angstroem) length of the axis
			// (actually 4 axis, Y is divided into positive and negative)
			float X = model.getSize().width;
			float Z = model.getSize().height;
			float up = 0;
			float down = 0;

			// determining the optimal length of all 4 Axis

			final Layer currentLayer = DataPool.getCurrentLayer();
			if (currentLayer != null) {
				currentLayer.getLayerId();

			}

			int count = model.getProteinCount();
			for (int i = 0; i < count; i++) {
				final Molecule_3D mol = model.getProteinAt(i);

				final Point3f lower = mol.getLower(false);

				final Point3f upper = mol.getUpper(false);

				up = Math.max(upper.y, up);
				down = Math.min(lower.y, down);
			}
			count = model.getLipidCountExtra();

			for (int i = 0; i < count; i++) {

				final Molecule_3D mol = model.getLipidAtExtra(i);

				final Point3f lower = mol.getLower(false);

				final Point3f upper = mol.getUpper(false);

				up = Math.max(upper.y, up);
				down = Math.min(lower.y, down);
			}
			count = model.getLipidCountIntra();
			for (int i = 0; i < count; i++) {
				final Molecule_3D mol = model.getLipidAtIntra(i);

				final Point3f lower = mol.getLower(false);

				final Point3f upper = mol.getUpper(false);

				up = Math.max(upper.y, up);
				down = Math.min(lower.y, down);
			}

			final float maxY = up;
			final float minY = down;

			// the width of the bars ()
			final float widthOfBars = 0.015f;

			// the distance from the bars for all numbers
			final float textDist = -0.15f;

			// creating count of numbers shown on each bar by dividing without
			// any
			// rest
			final int cX = (int) X / 100;
			final int cmaxY = (int) (maxY / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE) / 100;
			final int cminY = (int) (minY / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE) / 100;
			final int cZ = (int) Z / 100;

			// scaling the axis for j3d
			X *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
			Z *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;

			X += widthOfBars * 4;
			Z += widthOfBars * 4;
			// Geometry for the shape (this time in quads)
			final Point3f[] points = new Point3f[48];

			int index = 0;

			// X axis
			points[index++] = new Point3f(0, 0, 0);
			points[index++] = new Point3f(X, 0, 0);
			points[index++] = new Point3f(X, 0, widthOfBars);
			points[index++] = new Point3f(0, 0, widthOfBars);

			points[index++] = new Point3f(0, 0, widthOfBars);
			points[index++] = new Point3f(X, 0, widthOfBars);
			points[index++] = new Point3f(X, widthOfBars, widthOfBars);
			points[index++] = new Point3f(0, widthOfBars, widthOfBars);

			points[index++] = new Point3f(0, widthOfBars, widthOfBars);
			points[index++] = new Point3f(X, widthOfBars, widthOfBars);
			points[index++] = new Point3f(X, widthOfBars, 0);
			points[index++] = new Point3f(0, widthOfBars, 0);

			points[index++] = new Point3f(0, 0, 0);
			points[index++] = new Point3f(0, widthOfBars, 0);
			points[index++] = new Point3f(X, widthOfBars, 0);
			points[index++] = new Point3f(X, 0, 0);

			// Z axis
			points[index++] = new Point3f(0, 0, 0);
			points[index++] = new Point3f(0, 0, Z);
			points[index++] = new Point3f(0, widthOfBars, Z);
			points[index++] = new Point3f(0, widthOfBars, 0);

			points[index++] = new Point3f(0, 0, 0);
			points[index++] = new Point3f(widthOfBars, 0, 0);
			points[index++] = new Point3f(widthOfBars, 0, Z);
			points[index++] = new Point3f(0, 0, Z);

			points[index++] = new Point3f(widthOfBars, 0, 0);
			points[index++] = new Point3f(widthOfBars, widthOfBars, 0);
			points[index++] = new Point3f(widthOfBars, widthOfBars, Z);
			points[index++] = new Point3f(widthOfBars, 0, Z);

			points[index++] = new Point3f(0, widthOfBars, 0);
			points[index++] = new Point3f(0, widthOfBars, Z);
			points[index++] = new Point3f(widthOfBars, widthOfBars, Z);
			points[index++] = new Point3f(widthOfBars, widthOfBars, 0);

			// Y axis
			points[index++] = new Point3f(0, maxY, 0);
			points[index++] = new Point3f(0, minY, 0);
			points[index++] = new Point3f(widthOfBars, minY, 0);
			points[index++] = new Point3f(widthOfBars, maxY, 0);

			points[index++] = new Point3f(0, maxY, 0);
			points[index++] = new Point3f(0, maxY, widthOfBars);
			points[index++] = new Point3f(0, minY, widthOfBars);
			points[index++] = new Point3f(0, minY, 0);

			points[index++] = new Point3f(0, maxY, widthOfBars);
			points[index++] = new Point3f(widthOfBars, maxY, widthOfBars);
			points[index++] = new Point3f(widthOfBars, minY, widthOfBars);
			points[index++] = new Point3f(0, minY, widthOfBars);

			points[index++] = new Point3f(widthOfBars, maxY, widthOfBars);
			points[index++] = new Point3f(widthOfBars, maxY, 0);
			points[index++] = new Point3f(widthOfBars, minY, 0);
			points[index++] = new Point3f(widthOfBars, minY, widthOfBars);

			// feeding the shape with the geometry
			final QuadArray geo = new QuadArray(points.length,
					GeometryArray.NORMALS | GeometryArray.COORDINATES
							| GeometryArray.COLOR_4);

			final GeometryInfo gi = new GeometryInfo(GeometryInfo.QUAD_ARRAY);

			gi.setCoordinates(points);

			geo.setCoordinates(0, points);

			new NormalGenerator().generateNormals(gi);

			geo.setCapability(Geometry.ALLOW_INTERSECT);

			this.shape.setGeometry(geo);

			this.appear = new Appearance();
			final PolygonAttributes pattrib = new PolygonAttributes();
			pattrib.setPolygonMode(PolygonAttributes.POLYGON_FILL);
			pattrib.setCullFace(PolygonAttributes.CULL_NONE);
			this.appear.setPolygonAttributes(pattrib);

			final Material mat = new Material();
			mat.setLightingEnable(true);

			mat.setShininess(50);

			// if (DataPool.J3DSettings.bright_appearance)
			// color = new
			// Color3f(CM2_CONSTANTS.J3D.COLOR_COORDINATEBARS_BRIGHT);
			// else
			// color = new Color3f(CM2_CONSTANTS.J3D.COLOR_COORDINATEBARS_DARK);

			mat.setAmbientColor(this.color);

			this.appear.setMaterial(mat);

			Cm2_Toolkit.setRenderingAttributes(this.appear);
			
			this.appear.setCapability(Appearance.ALLOW_MATERIAL_READ);
			this.appear.setCapability(Appearance.ALLOW_MATERIAL_WRITE);

			this.appear
					.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_READ);
			this.appear
					.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);

			// mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
			// mat.setCapability(Material.ALLOW_COMPONENT_READ);

			this.shape.setAppearance(this.appear);

			for (int i = 0; i < this.Cxyz.length; i++) {

				final TransformGroup tr = new TransformGroup();
				final Transform3D trans = new Transform3D();
				final Matrix3f m = Cm2_Toolkit.createAUnitMatrix();

				this.Cxyz[i] = i != 3 ? new Cone(
						2.5f * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,
						5 * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,
						Primitive.GENERATE_NORMALS, this.appear)
						: new Cylinder(
								2.5f * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,
								1 * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE,
								Primitive.GENERATE_NORMALS, this.appear);

				if (i == 0)
					trans.setTranslation(new Vector3f(X
							+ ((Cone) this.Cxyz[i]).getHeight() / 2f
							- widthOfBars, 0, 0 - widthOfBars / 2f));
				else if (i == 1)
					trans.setTranslation(new Vector3f(0 - widthOfBars / 2f,
							maxY + ((Cone) this.Cxyz[i]).getHeight() / 2f
									- widthOfBars / 2f, 0 - widthOfBars / 2f));
				else if (i == 2)
					trans.setTranslation(new Vector3f(0 - widthOfBars / 2f, 0,
							Z + ((Cone) this.Cxyz[i]).getHeight() / 2f
									- widthOfBars));
				else if (i == 3)
					trans.setTranslation(new Vector3f(0 - widthOfBars / 2f,
							minY - ((Cylinder) this.Cxyz[i]).getHeight() / 2f
									- widthOfBars / 2f, 0 - widthOfBars / 2f));

				if (i == 0)
					Cm2_Toolkit.rotate_Z_Axis_Of_CoordinateSystem(m, -90);
				else if (i == 2)
					Cm2_Toolkit.rotate_X_Axis_Of_CoordinateSystem(m, 90);
				else if (i == 3)
					Cm2_Toolkit.rotate_X_Axis_Of_CoordinateSystem(m, 180);

				this.Cxyz[i].setPickable(false);

				tr.addChild(this.Cxyz[i]);
				this.branchGroup2.addChild(tr);

				trans.setRotation(m);

				tr.setTransform(trans);
			}

			// Putting the axis letters in
			this.addMyTextNode("+X", new Point3f(X + 0.3f, 0, 0), this.color,
					new Font("Arial", Font.BOLD, 35));

			this.addMyTextNode("+Y", new Point3f(0, maxY + 0.3f, 0),
					this.color, new Font("Arial", Font.BOLD, 35));

			this.addMyTextNode("-Y", new Point3f(0, minY - 0.3f, 0),
					this.color, new Font("Arial", Font.BOLD, 35));

			this.addMyTextNode("+Z", new Point3f(0, 0, Z + 0.3f), this.color,
					new Font("Arial", Font.BOLD, 35));

			/*
			 * Calculation and positioning of the numbers shown next to the
			 * bars.
			 */
			final float step = 100 * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
			int pos = 0;
			for (int i = 0; i < cX; i++) {
				pos += 100;

				this
						.addMyTextNode("" + pos, new Point3f((i + 1) * step, 0,
								textDist), this.color, new Font("Arial",
								Font.BOLD, 25));

			}
			pos = 0;
			for (int i = 0; i < cZ; i++) {
				pos += 100;

				this.addMyTextNode("" + pos, new Point3f(textDist, 0, (i + 1)
						* step), this.color, new Font("Arial", Font.BOLD, 25));

			}

			pos = 0;
			for (int i = 0; i < cmaxY; i++) {
				pos += 100;

				this.addMyTextNode("" + pos, new Point3f(textDist, (i + 1)
						* step, textDist), this.color, new Font("Arial",
						Font.BOLD, 25));

			}
			pos = 0;
			for (int i = 0; i < cminY * (-1); i++) {
				pos += 100;

				this.addMyTextNode("" + pos * (-1), new Point3f(textDist,
						(i + 1) * step * (-1), textDist), this.color, new Font(
						"Arial", Font.BOLD, 25));

			}

			if (this.shape.getParent() == null) {
				this.transformGroup.addChild(this.shape);
				this.branchGroup.addChild(this.branchGroup2);
			}
			final Transform3D owntrans = new Transform3D();
			owntrans.setTranslation(new Vector3f(0 - widthOfBars,
					(0 - widthOfBars / 2), 0 - widthOfBars));
			this.transformGroup.setTransform(owntrans);

		} else {

			if (this.shape.getParent() == this.transformGroup) {

				this.transformGroup.removeChild(this.shape);
				this.branchGroup.removeChild(this.branchGroup2);
			}
		}

		parent.addChild(this.branchGroup);

	}

	/**
	 * Adds a specified text to this Graph that is set to be Billboarding
	 * (always facing the user)
	 * 
	 * @param text
	 * @param position
	 * @param color
	 * @param font
	 */
	private void addMyTextNode(final String text, final Point3f position,
			final Color3f color, final Font font) {
		// this.branchGroup.removeChild(this.branchGroup2);
		// creating the text
		final Text2D textX = new Text2D(text, color, font.getFontName(), font
				.getSize(), font.getStyle());
		// those are necessary for the billboard to work
		final TransformGroup objScale = new TransformGroup();
		final TransformGroup objTrans = new TransformGroup();
		final TransformGroup billTrans = new TransformGroup();
		billTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		billTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);

		this.branchGroup2.addChild(objTrans);

		final Transform3D t3d = new Transform3D();

		objScale.addChild(billTrans);
		billTrans.addChild(textX);

		// determining the center of the text used as the rotation point
		final BoundingBox textB = (BoundingBox) textX.getBounds();
		final Point3d upper = new Point3d();
		textB.getUpper(upper);
		final Point3f rotationPt = new Point3f((float) (0 + upper.x / 2),
				(float) (0 + upper.y / 2), (float) (0 + upper.z / 2));

		final BoundingSphere bounds = new BoundingSphere(
				new Point3d(rotationPt), 100000.0);

		// Setting up the billboard node
		final Billboard bill = new Billboard(billTrans);
		bill.setSchedulingBounds(bounds);
		bill.setAlignmentMode(Billboard.ROTATE_ABOUT_POINT);
		objTrans.addChild(bill);
		bill.setRotationPoint(rotationPt);

		// positioning of the entire Scene
		t3d.setTranslation(new Vector3f((float) (position.x - upper.x / 2),
				(float) (position.y - upper.y / 2),
				(float) (position.z - upper.z / 2)));

		objScale.setTransform(t3d);

		this.branchGroup2.addChild(objScale);

	}

	public BranchGroup getBranchGroup() {
		return this.branchGroup;
	}

	public Appearance getAppearance() {
		return this.appear;
	}

	public void setColor(final Color3f color) {
		this.color = color;

	}

}
