/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.functionality3d;

import java.awt.Color;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.media.j3d.BoundingBox;
import javax.swing.BorderFactory;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.gui.elements.MembraneView;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;

/**
 * 
 * This class is used to display intersecting lipids.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class IntersectionsDisplayer {

	private final MembraneView mView;

	private static Hashtable<Molecule_3D, HistoricalMolecule> hash = new Hashtable<Molecule_3D, HistoricalMolecule>();

	public IntersectionsDisplayer(final MembraneView mview) {
		this.mView = mview;
	}

	public void updateIntersectionDispay() {

		// if there is no collision detection at all
		if(!DataPool.J3DSettings.atomCollisionCheck&&!DataPool.J3DSettings.shapeCollisionCheck) {
			this.mView.intersectPanel.setBorder(null);
			this.mView.intersectPanel.nonelabel.setVisible(true);
			this.mView.intersectPanel.nonelabel.setText("Disabled");
			this.mView.intersectPanel.lipidsBtn.setVisible(false);
			this.mView.intersectPanel.showBtn.setVisible(false);
			
			this.mView.intersectPanel.label
					.setToolTipText("Intersection check has been disabled!");
			this.mView.intersectPanel.label.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("CV_Collisions"));
		
			final MembraneModel model = DataPool.getMembraneModel();
			final Molecule_3D[] mols = model.getAllMoleculesInMembrane();
			for (int i = 0; i < mols.length; i++) {
				mols[i].setPositionIsInAValidSpace(true);
			}
			return;
		}
		
		// start the collision check
		final MembraneModel model = DataPool.getMembraneModel();
		final Molecule_3D[] mols = model.getAllMoleculesInMembrane();
		this.mView.intersectPanel.showBtn.setVisible(true);
		if (!model.anyCurrentIntersections(true)) {
			this.mView.intersectPanel.label
					.setToolTipText("No current intersections!");
			this.mView.intersectPanel.label.setIcon(Cm2_IconDepot
					.getSpecifiedIcon("CV_No_Collisions"));
			// this.mView.intersectPanel.showBtn.setVisible(false);
			this.mView.intersectPanel.lipidsBtn.setVisible(false);
			this.mView.intersectPanel.nonelabel.setText("None");
			this.mView.intersectPanel.nonelabel.setVisible(true);
			
			this.mView.intersectPanel.setBorder(null);
			for (int i = 0; i < mols.length; i++) {
				mols[i].setPositionIsInAValidSpace(true);
			}
			return;

		}
		this.mView.intersectPanel.nonelabel.setVisible(false);
		this.mView.intersectPanel.label
				.setToolTipText("Some objects intersect currently!");
		this.mView.intersectPanel.label.setIcon(Cm2_IconDepot
				.getSpecifiedIcon("CV_Collisions"));
		;
		// this.mView.intersectPanel.showBtn.setVisible(true);
		this.mView.intersectPanel.setBorder(BorderFactory.createLineBorder(
				Color.red, 2));

		final Vector<Molecule_3D> prots = new Vector<Molecule_3D>();
		final Vector<Molecule_3D> lips = new Vector<Molecule_3D>();
		final IntersectManager inter = new IntersectManager();

		final boolean b = this.mView.intersectPanel.showBtn.isSelected();
		// this if clause was original from Tim and caused the repainting of borders
		// not to function if e.g. in the settings menu the values of the spinner
		// has been changed from non-intersecting to intersecting state more than
		// two times
//		if (!b)
			IntersectionsDisplayer.hash.clear();
		for (int i = 0; i < mols.length; i++) {

			if (!b)
				mols[i].setPositionIsInAValidSpace(true);

			if (mols[i] instanceof Protein_3D)
				prots.addElement(mols[i]);
			else
				lips.addElement(mols[i]);

		}

		if (b) {

			final Molecule_3D[] molsToCheck = this.getMoleculesToCheck(mols);

			for (int i = 0; i < molsToCheck.length; i++) {

				if (inter.intercectsBorderOrAnyMolecule(molsToCheck[i], true)) {

					molsToCheck[i].setPositionIsInAValidSpace(false);

				} else {
					molsToCheck[i].setPositionIsInAValidSpace(true);
				}

			}
		}

		for (int i = 0; i < prots.size(); i++) {
			if (inter.intersectsMoleculeMoleculesVector(prots.get(i), lips,
					true)) {
				this.mView.intersectPanel.lipidsBtn.setVisible(true);
				return;

			}

		}
		this.mView.intersectPanel.lipidsBtn.setVisible(false);

	}

	private Molecule_3D[] getMoleculesToCheck(final Molecule_3D[] mols) {

		final Vector<Molecule_3D> ret = new Vector<Molecule_3D>() {
			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			@Override
			public synchronized void addElement(final Molecule_3D mol) {
				if (!this.contains(mol))
					super.addElement(mol);
			}
		};

		System.currentTimeMillis();
		for (int i = 0; i < mols.length; i++) {
			final HistoricalMolecule histmol = IntersectionsDisplayer.hash
					.get(mols[i]);
			// if(histmol==null) {
			// ret.addElement(mols[i]);
			//				
			// for(int j=0;j<mols.length;j++) {
			// if(mols[j]!=mols[i]&&mols[j].getAlignedGeometricBounds().intersect(mols[i].getAlignedGeometricBounds()))
			// {
			// ret.addElement(mols[j]);
			// }
			// }
			// }else {
			//				
			// hash.remove(mols[i]);
			//				
			// if(!histmol.isEqual(mols[i])) {
			// ret.addElement(mols[i]);
			// for(int j=0;j<mols.length;j++) {
			// if(mols[j]!=mols[i]&&mols[j].getAlignedGeometricBounds().intersect(histmol.getBounds()))
			// {
			// ret.addElement(mols[j]);
			// }
			// }
			// }
			// }

			// for(int j=0;j<mols.length;j++) {
			// if(mols[j]!=mols[i]&&mols[j].getAlignedGeometricBounds().intersect(mols[i].getAlignedGeometricBounds()))
			// {
			// ret.addElement(mols[j]);
			// }
			// }

			IntersectionsDisplayer.hash.remove(mols[i]);

			if (histmol == null || !histmol.isEqual(mols[i])) {
				ret.addElement(mols[i]);
				for (int j = 0; j < mols.length; j++) {
					if (mols[j] != mols[i]
							&& (mols[j].getAlignedGeometricBounds().intersect(
									mols[i].getAlignedGeometricBounds()) || (histmol != null && mols[j]
									.getAlignedGeometricBounds().intersect(
											histmol.getBounds())))) {
						ret.addElement(mols[j]);
					}
				}
			}

		}

		final Enumeration<Molecule_3D> en = IntersectionsDisplayer.hash.keys();

		while (en.hasMoreElements()) {

			final BoundingBox bb = (IntersectionsDisplayer.hash.get(en
					.nextElement())).getBounds();

			for (int j = 0; j < mols.length; j++) {
				if (mols[j].getAlignedGeometricBounds().intersect(bb)) {
					ret.addElement(mols[j]);
				}
			}

		}

		// System.out.println("---------------");
		// System.out.println("Time: "+(System.currentTimeMillis()-t));
		// System.out.println("Ret: "+ret.size());
		// System.out.println("Hash: "+hash.size());
		// System.out.println("---------------");
		// System.out.println();

		IntersectionsDisplayer.hash.clear();

		for (int i = 0; i < mols.length; i++) {
			IntersectionsDisplayer.hash.put(mols[i], new HistoricalMolecule(
					mols[i]));
		}

		return ret.toArray(new Molecule_3D[0]);

	}

	class HistoricalMolecule {

		private final Point3f position;
		private final Matrix3f matrix;
		private final BoundingBox bounds;
		private final Atom_3D[] atoms;

		public HistoricalMolecule(final Molecule_3D mol) {

			this.position = new Point3f(mol.getPosition());
			this.matrix = Cm2_Toolkit.copyMatrix(mol.getRotationMatrix());
			this.bounds = new BoundingBox(mol.getAlignedGeometricBounds());
			this.atoms = mol.getAtoms(false, true);

		}

		public boolean isEqual(final Molecule_3D mol) {

			if (!this.position.equals(mol.getPosition()))
				return false;
			if (!this.matrix.equals(this.matrix))
				return false;
			if (!this.getBounds().equals(mol.getAlignedGeometricBounds()))
				return false;

			if (this.atoms.length != mol.atoms.length)
				return false;

			for (int i = 0; i < this.atoms.length; i++) {
				if (!this.atoms[i].position.equals(mol.atoms[i].position))
					return false;
			}

			return true;
		}

		public BoundingBox getBounds() {
			return this.bounds;
		}

	}

	public static void clearHashtable() {
		IntersectionsDisplayer.hash.clear();
	}

}
