/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.functionality3d;

import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.util.Enumeration;

import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.swing.JButton;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;

/**
 * Manages the microdomains of a membrane.
 *          
 * @author Tim Dingersen dackelnose@gmx.net
 */
public class MicroDomainManager implements MouseListener, MouseMotionListener {

	private final BranchGroup branchGroup;

	private final Canvas3D canvas;

	private final PickNDragManager PNDmanager;

	private JButton acceptBtn;

	private Shape tempgeometry;

	private final MicroDomain_3D defaultArea;

	public MicroDomainManager(final Canvas3D canvas, final BranchGroup microBG,
			final PickNDragManager PNDmanager, final MicroDomain_3D defaultArea) {
		this.canvas = canvas;
		this.branchGroup = microBG;
		this.PNDmanager = PNDmanager;
		this.defaultArea = defaultArea;
	}

	public void startDomainMode(final MicroDomain_3D domainToChange) {

		if (domainToChange == null) {
			DataPool.drawingDomain = new MicroDomain_3D();

			DataPool.drawingDomain.id = -1;
			DataPool.drawingDomain.layerId = DataPool.getCurrentLayerID();

		} else {
			DataPool.drawingDomain = domainToChange;
			if (DataPool.drawingDomain.geometry == MicroDomain_3D.RECTANGULAR)
				this.tempgeometry = new Rectangle(
						(Rectangle) DataPool.drawingDomain.geoObject);
			else if (DataPool.drawingDomain.geometry == MicroDomain_3D.ELLIPSE)
				this.tempgeometry = (Ellipse2D.Double) ((Ellipse2D.Double) DataPool.drawingDomain.geoObject)
						.clone();
			else
				this.tempgeometry = new Polygon(
						((Polygon) DataPool.drawingDomain.geoObject).xpoints,
						((Polygon) DataPool.drawingDomain.geoObject).ypoints,
						((Polygon) DataPool.drawingDomain.geoObject).npoints);

		}

		DataPool.drawingDomain.setValid(true);

		this.canvas.addMouseListener(this);
		this.canvas.addMouseMotionListener(this);

		

	}

	public void finishDomainMode(final boolean apply) {
		this.canvas.removeMouseListener(this);
		this.canvas.removeMouseMotionListener(this);

		this.branchGroup.removeChild(DataPool.drawingDomain.branch);

		if (apply) {

			if (DataPool.drawingDomain.id == -1) {

				DataPool.getMembraneModel().addMicroDomain(
						DataPool.drawingDomain);
				DataPool.drawingDomain.id = DataPool.getMembraneModel()
						.countMicroDomains();

			}

		}
		if (apply == false && DataPool.drawingDomain.id != -1) {

			if (this.tempgeometry instanceof Rectangle)
				DataPool.drawingDomain
						.setGeometryType(MicroDomain_3D.RECTANGULAR);
			else if (this.tempgeometry instanceof Ellipse2D.Double)
				DataPool.drawingDomain.setGeometryType(MicroDomain_3D.ELLIPSE);
			else
				DataPool.drawingDomain.setGeometryType(MicroDomain_3D.POLYGON);

			DataPool.drawingDomain.geoObject = this.tempgeometry;
			DataPool.drawingDomain.updateShape3D();
			DataPool.drawingDomain.setValid(true);
		}

		DataPool.getMembraneModel().organizeDomains();

	}

	public void setDomainPaintMode(final int mode) {

		DataPool.drawingDomain.setGeometryType(mode);

		DataPool.drawingDomain.updateShape3D();
		this.validateDrawingMicroDomain();
		this.updateDisplayOfMicroDomains();

	}

	public void mouseClicked(final MouseEvent arg0) {

	}

	public void mouseEntered(final MouseEvent arg0) {

	}

	public void mouseExited(final MouseEvent arg0) {

	}

	public void mousePressed(final MouseEvent arg0) {
		if (arg0.getButton() != MouseEvent.BUTTON1)
			return;

		if (!DataPool.drawingDomain.branch.isLive())
			this.branchGroup.addChild(DataPool.drawingDomain.branch);

		final Point3f p = this.PNDmanager.calculateLocation(arg0.getX(), arg0
				.getY(), 0);
		DataPool.drawingDomain.startPainting(new Point(
				(int) (p.x / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE),
				(int) (p.z / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE)));
	}

	public void mouseReleased(final MouseEvent arg0) {

	}

	public void mouseDragged(final MouseEvent arg0) {

		if (arg0.getModifiers() != InputEvent.BUTTON1_MASK)
			return;

		final Point3f p = this.PNDmanager.calculateLocation(arg0.getX(), arg0
				.getY(), 0);
		DataPool.drawingDomain.continuePainting(new Point(
				(int) (p.x / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE),
				(int) (p.z / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE)));

		this.validateDrawingMicroDomain();

	}

	public void mouseMoved(final MouseEvent arg0) {

	}

	public void setAcceptButton(final JButton ok) {
		this.acceptBtn = ok;

	}

	private void validateDrawingMicroDomain() {

		final MembraneModel model = DataPool.getMembraneModel();
		boolean valid = false;

		if (DataPool.drawingDomain.geoObject instanceof Polygon) {
			final Polygon poly = (Polygon) DataPool.drawingDomain.geoObject;
			int maxx = 0;
			int maxy = 0;
			int minx = DataPool.getMembraneModel().getSize().width;
			int miny = DataPool.getMembraneModel().getSize().height;
			for (int i = 0; i < poly.npoints; i++) {
				maxx = Math.max(maxx, poly.xpoints[i]);
				maxy = Math.max(maxy, poly.ypoints[i]);

				minx = Math.min(minx, poly.xpoints[i]);
				miny = Math.min(miny, poly.ypoints[i]);
			}

			if (Math.abs(minx - maxx) > 1 || Math.abs(miny - maxy) > 1) {
				if (poly.npoints > 2)
					valid = true;
			}
		} else if (!DataPool.drawingDomain.geoObject.getBounds().isEmpty()) {
			valid = true;
		}

		if (valid) {
			for (int i = 0; i < model.countMicroDomains(); i++) {

				if (DataPool.drawingDomain == model.getMicroDomain(i)
						|| !model.getMicroDomain(i).isEnabled())
					continue;

				final Shape shape = model.getMicroDomain(i).geoObject;

				final Area in = new Area(DataPool.drawingDomain.geoObject);

				in.intersect(new Area(shape));

				if (!in.isEmpty()) {
					valid = false;
					break;
				}

			}
		}

		if (valid) {
			this.acceptBtn.setEnabled(true);

			DataPool.drawingDomain.setValid(valid);
		} else {
			this.acceptBtn.setEnabled(false);

			DataPool.drawingDomain.setValid(valid);
		}

	}

	public void updateDisplayOfMicroDomains() {

		final Enumeration<?> en = this.branchGroup.getAllChildren();

		while (en.hasMoreElements()) {
			final Object o = en.nextElement();
			if (o instanceof BranchGroup
					&& ((BranchGroup) o).getUserData() instanceof MicroDomain_3D)
				if (((BranchGroup) o).getUserData() != this.defaultArea)
					this.branchGroup.removeChild((BranchGroup) o);

		}

		for (int i = 0; i < DataPool.getMembraneModel().countMicroDomains(); i++) {
			if (DataPool.getMembraneModel().getMicroDomain(i).isEnabled()
					&& DataPool.getMembraneModel().getMicroDomain(i).layerId == DataPool
							.getCurrentLayerID())
				this.branchGroup.addChild(DataPool.getMembraneModel()
						.getMicroDomain(i).branch);

		}

	}

	public void higlightMicroDomainIn3D(final int domainID) {
		final MembraneModel model = DataPool.getMembraneModel();

		// int layerId = DataPool.getCurrentLayerID();
		// Vector<?>[] lists =
		// {model.getAllLipidsExtra(),model.getAllLipidsIntra()};
		// for(int a=0;a<lists.length;a++) {
		// Vector<?>list=lists[a];
		// int length=list.size();
		// for(int i=0;i<length;i++) {
		// Lipid_3D mol = (Lipid_3D) list.get(i);

		// if(mol.getLayerID()==layerId&&(domainID<0||mol.getDomainID()==domainID))
		// mol.setTransparent(false);
		// else mol.setTransparent(true);
		// }
		// }

		// if(domainID<1)
		// this.defaultArea.setHighlighted(true);
		// else this.defaultArea.setHighlighted(false);

		for (int i = 0; i < model.countMicroDomains(); i++) {
			final MicroDomain_3D micro = model.getMicroDomain(i);
			// if(micro.layerId!=layerId)continue;
			if (micro.id == domainID)
				micro.setHighlighted(true);
			else
				micro.setHighlighted(false);

		}
		if (domainID == 0)
			this.defaultArea.setHighlighted(true);
		else
			this.defaultArea.setHighlighted(false);

	}

	public int getHighlightedID() {
		final MembraneModel model = DataPool.getMembraneModel();
		if (this.defaultArea.isHigh)
			return 0;
		for (int i = 0; i < model.countMicroDomains(); i++) {
			final MicroDomain_3D micro = model.getMicroDomain(i);
			// if(micro.layerId!=layerId)continue;
			if (micro.isHigh)
				return micro.id;

		}
		return -1;
	}
}
