/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui;

import java.awt.Component;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.io.Compatibility;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;


/**
 * 
 * The compatibility panel is used to show the compatibility 
 * of the actual PDB settings to external PDB viewers.
 * The compatibilities are defined in CM2_CONSTANTS.
 * 
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public abstract class CompatibilityPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	Compatibility[] comps = CM2_CONSTANTS.IO.INCLUDED_COMPATIBILITIES;

	ImageIcon green = Cm2_IconDepot.getSpecifiedIcon("Generate_Green");
	ImageIcon warning = Cm2_IconDepot.getSpecifiedIcon("Warning");

	public CompatibilityPanel() {

		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		for (int i = 0; i < this.comps.length; i++) {

			final JRadioButton b = new JRadioButton(this.comps[i]
					.getApplicationName());
			b.setFont(b.getFont().deriveFont(b.getFont().getSize2D() + 3));
			b.setFocusPainted(false);
			b.setFocusable(false);
			b.setFont(b.getFont().deriveFont(9f));
			b.setIconTextGap(5);
			b.setIcon(this.green);
			this.add(b);
		}

		this.add(Box.createHorizontalGlue());

	}

	public void update() {

		for (int i = 0; i < this.getComponentCount(); i++) {
			final Component c = this.getComponent(i);
			if (c instanceof JRadioButton) {
				final JRadioButton b = (JRadioButton) c;

				final String tooltext = this.checkCompatibility(b.getText());

				if (tooltext == null) {
					b.setIcon(this.green);
					b
							.setToolTipText("<html>"
									+ b.getText()
									+ " should work fine<br>with the current configuration</html>");
				} else {
					b.setIcon(this.warning);
					b.setToolTipText(tooltext);
				}
			}
		}

	}

	public abstract String checkCompatibility(String text);

	public String getToolText(final String name, final int requiredAtomserials,
			final int requiredChains, final boolean caseSensitiveChainIds,
			final boolean assignResSeqNumbers,
			final boolean atomserials_All99999,
			final boolean atomserials_RestartNumberingWith0,
			final boolean adoptTitleSections,  
			final boolean swapYZ, final int shiftXZ, final int shiftY, 
			final boolean sortByType, final boolean writeReverseParsingInfo) {

		Compatibility c = null;
		for (int i = 0; i < CM2_CONSTANTS.IO.INCLUDED_COMPATIBILITIES.length; i++) {
			if (CM2_CONSTANTS.IO.INCLUDED_COMPATIBILITIES[i]
					.getApplicationName().equals(name)) {
				c = CM2_CONSTANTS.IO.INCLUDED_COMPATIBILITIES[i];
				break;
			}
		}
		if (c == null)
			return null;

		final Vector<String> out = new Vector<String>();

		if (c.requiredAtomserials() != null
				&& requiredAtomserials != c.requiredAtomserials().intValue()) {
			if (c.requiredAtomserials() == Compatibility.ATOMSERIALS_BOTH)
				out.add("Requires complete atom serials");
			else if (c.requiredAtomserials() == Compatibility.ATOMSERIALS_PROTSONLY)
				out.add("Requires protein-only atom serials");
			else
				out.add("Atom serials not allowed");
		}

		if (c.requiredChains() != null
				&& requiredChains != c.requiredChains().intValue()) {
			if (c.requiredChains() == Compatibility.ATOMSERIALS_BOTH)
				out.add("Requires complete chain-ID's");
			else if (c.requiredChains() == Compatibility.ATOMSERIALS_PROTSONLY)
				out.add("Requires protein-only chain-ID's");
			else
				out.add("Chain-ID's not allowed");
		}

		if (c.caseSensitiveChainIds() != null
				&& caseSensitiveChainIds != c.caseSensitiveChainIds()
						.booleanValue()) {
			if (c.caseSensitiveChainIds())
				out.add("Requires support of case-sensitive chain-ID's");
			else
				out.add("Case-sensitive chain-ID's not allowed");
		}

		if (c.assignResSeqNumbers() != null
				&& assignResSeqNumbers != c.assignResSeqNumbers()
						.booleanValue()) {

			if (c.assignResSeqNumbers())
				out.add("Requires sequential resSeq's for each molecule");
			else
				out.add("Overwriting of resSeq's forbidden");

		}

		if (c.atomserials_All99999() != null
				&& atomserials_All99999 != c.atomserials_All99999()
						.booleanValue()) {
			if (requiredAtomserials == Compatibility.ATOMSERIALS_NONE) {
				if (c.atomserials_All99999())
					out.add("Requires all remaining atomserials to be 99999");
				else {
					if (c.atomserials_RestartNumberingWith0() != null && c.atomserials_RestartNumberingWith0())
						out.add("Requires counting of atomserials greater than 99999 to start over at 0");
					else
						out.add("Requires counting of atomserials greater than 99999 to start over at 1");
				}
			}
		}
		
		if (c.atomserials_RestartNumberingWith0() != null
				&& atomserials_RestartNumberingWith0 != c.atomserials_RestartNumberingWith0()
						.booleanValue()) {
			if (requiredAtomserials == Compatibility.ATOMSERIALS_NONE) {
				if (c.atomserials_RestartNumberingWith0())
					out.add("Requires counting of atomserials greater than 99999 to start over at 0");
				else
					out.add("Requires counting of atomserials greater than 99999 to start over at 1");
			}
		}

		if (c.adoptTitleSections() != null
				&& adoptTitleSections != c.adoptTitleSections().booleanValue()) {
			if (c.adoptTitleSections())
				out.add("Adoption of title sections required");
			else
				out.add("Adoption of title sections may cause problems");
		}

		if (c.sortByType() != null && sortByType != c.sortByType().booleanValue()) {
			if (c.sortByType())
				out.add("Requires to sort the molecules by their type");
			else
				out.add("Requires to sort the molecules by the membrane side");
		}
		
		if (c.writeReverseParsingInfo() != null &&  writeReverseParsingInfo != c.writeReverseParsingInfo().booleanValue()) {
			if (c.writeReverseParsingInfo())
				out.add("Requires to enable reverse parsing");
			else
				out.add("Requires to disable reverse parsing");
		}
		
		if (c.swapYZ() != null && swapYZ != c.swapYZ().booleanValue()) {
			if (c.swapYZ())
				out.add("Requires to swap the Y and Z Axis");
			else
				out.add("Requires the original Y and Z Axis");
		}

		if (c.shiftXZ() != null && shiftXZ != c.shiftXZ().intValue()) {
			if (c.shiftXZ() == Compatibility.TRANSFORM_NONE)
				out.add("No further XZ-transformation allowed");
			else if (c.shiftXZ() == Compatibility.TRANSFORM_PHYSICAL)
				out.add("Requires XZ-transformation to physical center");
			else
				out.add("Requires XZ-transformation to the bottom");
		}

		if (c.shiftY() != null && shiftY != c.shiftY().intValue()) {
			if (c.shiftY() == Compatibility.TRANSFORM_NONE)
				out.add("No further Y-transformation allowed");
			else if (c.shiftY() == Compatibility.TRANSFORM_PHYSICAL)
				out.add("Requires Y-transformation to physical center");
			else
				out.add("Requires Y-transformation to the bottom");
		}

		if (out.isEmpty())
			return null;
		
		if (c.remark() != null) {
			out.add(c.remark());
		}
		
		String s = "";
		for (int i = 0; i < out.size(); i++) {
			s += "-" + out.get(i) + "<br>";
		}

		return "<html>Possible incompatibilities:<br><br>" + s + "</html>";
	}

}
