/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;


/**
 * The component that appears in the filechooser before generating a Membrane.
 * Provides Options for the user and displays information of the sections that
 * will be written
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
@SuppressWarnings("serial")
public class MylFCAccessoryForGenerating extends JPanel {

	JPanel main = new JPanel();

	WritePanel wp = new WritePanel();

	MyCompatibilityPanel cpanel = new MyCompatibilityPanel();

	public MylFCAccessoryForGenerating() {

		// this.setPreferredSize(new Dimension(180, 410));

		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		this.add(this.main);

		this.main.setLayout(new BoxLayout(this.main, BoxLayout.Y_AXIS));

		final JPanel p = new JPanel();

		final Color clr = new JProgressBar().getForeground();
		
	

		p.setBorder(BorderFactory.createLineBorder(clr, 1));

		final JPanel info = new JPanel();

		info.setBorder(BorderFactory.createTitledBorder(null,
				"Compatibilities", TitledBorder.DEFAULT_JUSTIFICATION,
				TitledBorder.DEFAULT_POSITION, this.getFont().deriveFont(
						Font.BOLD, this.getFont().getSize2D() + 2f)));
		this.main.add(info);
		info.setLayout(new BoxLayout(info, BoxLayout.X_AXIS));

		info.add(this.cpanel);
		info.add(Box.createHorizontalGlue());

		this.wp.setBorder(BorderFactory.createTitledBorder(null,
				"Assured Contents", TitledBorder.DEFAULT_JUSTIFICATION,
				TitledBorder.DEFAULT_POSITION, this.getFont().deriveFont(
						Font.BOLD, this.getFont().getSize2D() + 2f)));
		
		
	
		this.main.add(Box.createVerticalStrut(4));
		this.main.add(this.wp);
		
		this.wp.setUp();
		this.cpanel.update();

	}

	/**
	 * The panel that displays information about the sections that will be
	 * written
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class WritePanel extends JPanel {

		private static final long serialVersionUID = 1L;

		JRadioButton a1 = new JRadioButton("PROTEIN  Atom Serials");

		JRadioButton a2 = new JRadioButton("PROTEIN  CONECT's");

		JRadioButton a3 = new JRadioButton("PROTEIN  Secondary Structure");

		JRadioButton a4 = new JRadioButton("PROTEIN  Other Sections");

		JRadioButton b1 = new JRadioButton("LIPID        Atom Serials");

		JRadioButton b2 = new JRadioButton("LIPID        CONECT's");

		JRadioButton b3 = new JRadioButton("LIPID        Secondary Structure");

		JRadioButton b4 = new JRadioButton("LIPID        Other Sections");

		String t1 = "<html>Included Sections:<br><br>-Atom Serials</html>";

		String t2 = "<html>Included Sections:<br><br>-CONECT</html>";

		String t3 = "<html>Included Sections:<br><br>-HELIX<br>-SHEET<br>-TURN</html>";

		String t4 = "<html>Included Sections:<br><br>Primary Strucures<br>"
				+ "-DBREF<br>-SEQADV<br>-SEQRES<br>-MODRES<br><br>"
				+ "Heterogens<br>" + "-HET<br><br>"
				+ "Miscellaneous Features<br>" + "-SITE<br><br>"
				+ "Connectivities<br>" + "-SSBOND<br>-LINK<br>-CISPEP</html>";

		public WritePanel() {
			this.setBorder(BorderFactory.createLineBorder(Color.GRAY));

			this.setLayout(new GridLayout(8, 0));

			this.add(this.a1);
			this.add(this.a2);
			this.add(this.a3);
			this.add(this.a4);

			this.add(this.b1);
			this.add(this.b2);
			this.add(this.b3);
			this.add(this.b4);

			this.a1.setToolTipText(this.t1);
			this.b1.setToolTipText(this.t1);
			this.a2.setToolTipText(this.t2);
			this.b2.setToolTipText(this.t2);
			this.a3.setToolTipText(this.t3);
			this.b3.setToolTipText(this.t3);
			this.a4.setToolTipText(this.t4);
			this.b4.setToolTipText(this.t4);

			final ImageIcon red = Cm2_IconDepot
					.getSpecifiedIcon("Generate_Red");

			for (int i = 0; i < this.getComponentCount(); i++) {
				final Component c = this.getComponent(i);
				if (c instanceof JRadioButton) {
					final JRadioButton b = (JRadioButton) c;
					b.setFocusPainted(false);
					b.setFocusable(false);
					b.setFont(b.getFont().deriveFont(9f));
					b.setIconTextGap(2);
					b.setIcon(red);

				}
			}

		}

		public void setUp() {

			final ImageIcon green = Cm2_IconDepot
					.getSpecifiedIcon("Generate_Green");
			final int serials = DataPool.LimitationSettings.atomserials;
			final int chains = DataPool.LimitationSettings.chains;
			if (serials == 0) {
				this.a1.setIcon(green);
				this.b1.setIcon(green);
				this.a2.setIcon(green);
				this.b2.setIcon(green);
			}
			if (serials == 1) {
				this.a1.setIcon(green);
				this.a2.setIcon(green);

			}
			if (chains == 0) {
				this.a3.setIcon(green);
				this.b3.setIcon(green);
				this.a4.setIcon(green);
				this.b4.setIcon(green);
			}
			if (chains == 1) {
				this.a3.setIcon(green);

				this.a4.setIcon(green);

			}

		}
	}

	class MyCompatibilityPanel extends CompatibilityPanel {

		public String checkCompatibility(final String name) {

			return this.getToolText(name,
					DataPool.LimitationSettings.atomserials,
					DataPool.LimitationSettings.chains,
					DataPool.LimitationSettings.caseSensitiveChainIds,
					DataPool.LimitationSettings.assignResSeqNumbers,
					DataPool.LimitationSettings.all99999,
					DataPool.LimitationSettings.restartNumberingWith0,
					DataPool.LimitationSettings.adoptTitleSections,
					DataPool.CoordinateTransformationSettings.swapYZ,
					DataPool.CoordinateTransformationSettings.shiftXZ,
					DataPool.CoordinateTransformationSettings.shiftY,
					DataPool.CoordinateTransformationSettings.sortByType,
					DataPool.LimitationSettings.reverseParsing);

		}

	}
}
