/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * The Frame that displays textual contents of a pdb file. Displays the text in
 * a textArea and constructs link buttons to each section that appears in the
 * text
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class PDBFileBrowser extends JFrame implements ActionListener {

	private static final long serialVersionUID = 1L;

	final private JPanel right = new JPanel();

	final private JPanel top = new JPanel();

	final private JPanel middle = new JPanel();

	final private JPanel bottom = new JPanel();

	final private JPanel space = new JPanel();

	final private JLabel jump = new JLabel("Jump to Section");

	private JScrollPane scroll;

	final private JTextArea tp = new JTextArea();

	final private JButton close = new JButton("Close");

	/*
	 * Possible tags. Used for comparison with found section tags and for
	 * putting the them into the righ category
	 */
	final private String[] tags = new String[] { " Title", "HEADER", "OBSLTE",
			"TITLE", "CAVEAT", "COMPND", "SOURCE", "KEYWDS", "EXPDTA",
			"AUTHOR", "REVDAT", "SPRSDE", "JRNL", "REMARK",
			" Primary Structure", "DBREF", "SEQADV", "SEQRES", "MODRES",
			" Heterogen", "HET", "HETNAM", "HETSYN", "FORMUL",
			" Secondary Structure", "HELIX", "SHEET", "TURN",
			" Connectivity Annotation", "SSBOND", "LINK", "CISPEP",
			" Miscellaneous Features", "SITE", " Crystallographic", "CRYST1",
			"ORIGXn", "SCALEn", "MTRIXn", "TVECT", " Coordinate", "ATOM",
			"HETATM", " Connectivity", "CONECT" };

	private int textlength;

	public PDBFileBrowser(final File file) {
		try {
			final String[] s = new PdbReader_3D().readFile(file);
			new PDBFileBrowser(s, Cm2_Toolkit.getSimpleName(file));
		} catch (final IOException e) {
			return;
		}

	}

	/**
	 * Constructs a new Frame.
	 * 
	 * @param file
	 */
	public PDBFileBrowser(final String[] lines, final String name) {

		final Hashtable<String, String> hash = new Hashtable<String, String>();
		final Vector<MyButton> vec = new Vector<MyButton>();

		int counter = 0;

		this.tp.setLineWrap(false);
		this.tp.setEditable(false);
		try {
			String finalString = "";

			for (int i = 0; i < lines.length; i++) {
				try {
					final String s = lines[i];
					finalString = finalString.concat(s + "\n");
					counter++;
					String tag;
					try {
						tag = s.substring(0, 6).trim();
					} catch (final Exception e) {
						continue;
					}
					if (tag.startsWith("ORIGX"))
						tag = "ORIGXn";
					else if (tag.startsWith("SCALE"))
						tag = "SCALEn";
					else if (tag.startsWith("MTRIX"))
						tag = "MTRIXn";

					boolean in = false;
					for (int j = 0; j < this.tags.length; j++) {
						if (this.tags[j].equals(tag)) {
							in = true;
							break;
						}
					}

					if (!in)
						continue;

					if (!hash.containsKey(tag)) {

						vec.addElement(new MyButton(tag, counter, this));

						hash.put(tag, "");
					}

				} catch (final Exception e) {

					this.dispose();
					return;
				}
			}

			this.tp.setText(finalString);
		} catch (final Exception e) {

			this.dispose();
			return;
		}

		this.tp
				.setFont(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.FONT_PDB_TEXTFIELDS);
		this.tp.setForeground(Color.black);
		this.tp.setBackground(Color.white);
		this.right.setLayout(new BoxLayout(this.right, BoxLayout.Y_AXIS));
		this.middle.setLayout(new BoxLayout(this.middle, BoxLayout.Y_AXIS));

		this.top.add(this.jump);
		this.right.add(this.top);
		this.right.add(this.middle);
		this.right.add(this.space);
		this.right.add(this.bottom);
		this.bottom.add(this.close);
		this.textlength = counter;

		final Vector<JComponent> vec2 = new Vector<JComponent>();
		int pos = -1;
		for (int i = 0; i < vec.size(); i++) {
			final String text = vec.get(i).getText();

			for (int j = 0; j < this.tags.length; j++) {
				if (this.tags[j].equals(text)) {
					if (pos < j)
						pos = j;
					break;
				}
			}

			if (pos <= 13) {
				vec2.addElement(new JLabel(this.tags[0]));
				pos = 14;
			} else if (pos > 14 && pos < 19) {
				vec2.addElement(new JLabel(this.tags[14]));
				pos = 19;
			} else if (pos > 19 && pos < 24) {
				vec2.addElement(new JLabel(this.tags[19]));
				pos = 24;
			} else if (pos > 24 && pos < 28) {
				vec2.addElement(new JLabel(this.tags[24]));
				pos = 28;
			} else if (pos > 28 && pos < 32) {
				vec2.addElement(new JLabel(this.tags[28]));
				pos = 32;
			} else if (pos > 32 && pos < 34) {
				vec2.addElement(new JLabel(this.tags[32]));
				pos = 34;
			} else if (pos > 34 && pos < 40) {
				vec2.addElement(new JLabel(this.tags[34]));
				pos = 40;
			} else if (pos > 40 && pos < 43) {
				vec2.addElement(new JLabel(this.tags[40]));
				pos = 43;
			} else if (pos > 43 && pos < 45) {
				vec2.addElement(new JLabel(this.tags[43]));
				pos = 45;
			}

			vec2.addElement(vec.get(i));
		}

		int maxw = 0;
		for (int i = 0; i < vec2.size(); i++) {

			final JComponent c = vec2.get(i);
			if (c instanceof JLabel) {
				final JLabel label = (JLabel) c;

				final JPanel p = new JPanel();
				p.setBorder(BorderFactory.createLineBorder(Color.white));
				p.add(label);

				p.setBackground(Color.GRAY);
				label.setForeground(Color.white);
				this.middle.add(p);
				maxw = Math.max(maxw, label.getPreferredSize().width);
			} else if (c instanceof MyButton) {
				final MyButton button = (MyButton) c;

				if (this.middle.getComponentCount() > 0
						&& this.middle.getComponent(this.middle
								.getComponentCount() - 1) instanceof JPanel)
					((JPanel) this.middle.getComponent(this.middle
							.getComponentCount() - 1)).add(button);

				maxw = Math.max(maxw, button.getPreferredSize().width);
			}

		}
		maxw = Math.max(maxw, this.jump.getPreferredSize().width);
		maxw = Math.max(maxw, this.close.getPreferredSize().width);
		maxw += 5;
		int totalcount = 0;
		for (int i = 0; i < this.middle.getComponentCount(); i++) {
			if (this.middle.getComponent(i) instanceof JPanel) {

				final JPanel p = (JPanel) this.middle.getComponent(i);
				p.setLayout(new GridLayout(p.getComponentCount(), 1));
				p
						.setMaximumSize(new Dimension(maxw, 24 * p
								.getComponentCount()));
				p
						.setMinimumSize(new Dimension(maxw, 24 * p
								.getComponentCount()));
				for (int j = 0; j < p.getComponentCount(); j++) {
					p.getComponent(j).setMaximumSize(new Dimension(maxw, 24));
					p.getComponent(j).setMinimumSize(new Dimension(maxw, 24));
					totalcount++;
				}
			}
		}

		this.close.setFont(this.close.getFont().deriveFont(Font.BOLD,
				this.close.getFont().getSize2D() + 5));

		this.middle.setPreferredSize(new Dimension(maxw, totalcount * 24));

		this.scroll = new JScrollPane(this.tp);
		this.scroll.setPreferredSize(new Dimension(700, this.middle
				.getPreferredSize().height));

		this.bottom.setPreferredSize(new Dimension(this.middle
				.getPreferredSize().width,
				this.close.getPreferredSize().height + 8));

		this.top.setPreferredSize(new Dimension(
				this.middle.getPreferredSize().width, 20));

		this.getContentPane().add(this.scroll, BorderLayout.WEST);
		this.getContentPane().add(this.right, BorderLayout.EAST);

		this.close.addActionListener(this);

		int prefh = this.getToolkit().getScreenSize().height;
		prefh -= prefh / 4;
		if (this.middle.getPreferredSize().height < prefh)
			this.space.setPreferredSize(new Dimension(maxw, prefh
					- this.middle.getPreferredSize().height));

		this.pack();

		this.setTitle(name + " - Pdb Browser");
		this.setResizable(false);
		this.setIconImage(Cm2_IconDepot.getSpecifiedIcon("DB_Browse")
				.getImage());
		this.setLocation(this.getToolkit().getScreenSize().width / 2
				- this.getWidth() / 2, this.getToolkit().getScreenSize().height
				/ 2 - this.getHeight() / 2);
		this.setVisible(true);

	}

	public void actionPerformed(final ActionEvent arg0) {
		if (arg0.getSource() == this.close) {
			this.dispose();
			return;
		}

		/*
		 * Scrolls to the specified location
		 */
		final int loc = ((MyButton) arg0.getSource()).location;
		final float step = ((float) this.tp.getHeight())
				/ (float) this.textlength;
		final int newpos = (int) (step * (loc - 1));
		this.tp.scrollRectToVisible(new Rectangle(0, newpos, 0, this.scroll
				.getHeight()));
	}

	@Override
	public void dispose() {

		super.dispose();
		System.gc();
	}

	/**
	 * This type of button has the ability to save the location where the
	 * textpane should scroll to when pressed
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class MyButton extends JButton {

		private static final long serialVersionUID = 1L;

		int location;

		public MyButton(final String string, final int loc,
				final ActionListener ls) {
			super(string);
			this.location = loc;
			this.addActionListener(ls);
		}

	}
}
