/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.gui.HelpBtn;
import org.cellmicrocosmos.cm2.io.FileIOHandler;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * A small frame that shows the "About" information of this application
 * retrieved from the constant class
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class AboutPane extends JDialog {

	private static final long serialVersionUID = 1L;

	public AboutPane() {

		final JPanel left = new JPanel();
		final JPanel right = new JPanel();

		final JPanel picpanel = new JPanel();
		final JPanel btnpanel = new JPanel();

		final ImageIcon image = new ImageIcon(this.getToolkit().createImage(
				FileIOHandler.home + "cm2images" + File.separator
						+ "Cm2_Logo_Medium.JPG"));
		final JLabel piclabel = new JLabel(image);
		piclabel.setPreferredSize(new Dimension(image.getIconWidth(), image
				.getIconHeight()));

		final JTextArea ta1 = new JTextArea();
		final JTextArea ta2 = new JTextArea();
		final JTextArea ta3 = new JTextArea();
		final JTextArea ta4 = new JTextArea();

		final JButton b1 = new JButton(" Cm2 Homepage ");
		
		this.getContentPane().add(left, BorderLayout.WEST);
		this.getContentPane().add(right, BorderLayout.EAST);

		btnpanel.add(b1);
		picpanel.add(piclabel);

		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
		right.setLayout(new BoxLayout(right, BoxLayout.Y_AXIS));

		left.add(picpanel);
		left.add(new JPanel());
		left.add(new JPanel());
		left.add(btnpanel);

		right.add(new JPanel());
		right.add(ta1);
		right.add(new JPanel());
		right.add(ta2);
		right.add(new JPanel());
		right.add(ta3);
		right.add(new JPanel());
		right.add(new HelpBtn(this));
		right.add(new JPanel());
		right.add(ta4);
		right.add(new JPanel());

		ta1.setEditable(false);
		ta2.setEditable(false);
		ta3.setEditable(false);
		ta4.setEditable(false);

		ta1.setOpaque(false);
		ta2.setOpaque(false);
		ta3.setOpaque(false);
		ta4.setOpaque(false);

		ta1
				.setText(CM2_CONSTANTS.GENERAL.NAME_OF_THIS_PROGRAM
						+ "  \n"
						+ CM2_CONSTANTS.GENERAL.VERSION
						+ " "
						+ (CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.DEVELOPER_EDITION ? CM2_CONSTANTS.GENERAL.DEVELOPER_EDITION_NAME
								: CM2_CONSTANTS.GENERAL.STANDARD_EDITION_NAME)
						+ " "
						+ (Cm2_Toolkit.amIRunningOnWebstart() ? CM2_CONSTANTS.GENERAL.WEBSTART_NAME
								: CM2_CONSTANTS.GENERAL.STANDALONE_NAME));
		ta2.setText(CM2_CONSTANTS.GENERAL.COPYRIGHT);
		ta3.setText(CM2_CONSTANTS.GENERAL.OTHERNOTES);
		ta4.setText(CM2_CONSTANTS.GENERAL.CM2_HOMEPAGE);

		ta1.setFont(right.getFont().deriveFont(Font.BOLD, 15));
		ta2.setFont(right.getFont().deriveFont(Font.PLAIN, 13));
		ta3.setFont(right.getFont().deriveFont(Font.PLAIN, 10));
		ta4.setFont(right.getFont().deriveFont(Font.ITALIC, 13));

		// ta2.setForeground(Color.DARK_GRAY);
		// ta3.setForeground(Color.DARK_GRAY);
		// ta4.setForeground(Color.DARK_GRAY);

		b1.setBorder(BorderFactory.createLineBorder(Color.white, 0));

		this.setTitle("About");
		this.pack();

		this.setResizable(false);
		this.setAlwaysOnTop(true);

		final Dimension dim = this.getToolkit().getScreenSize();
		this.setLocation(dim.width / 2 - this.getWidth() / 2, dim.height / 2
				- this.getHeight() / 2);

		b1.addActionListener(new ActionListener() {
			public void actionPerformed(final ActionEvent arg0) {
				Cm2_Toolkit
						.openWebPageOnSystemBrowser(CM2_CONSTANTS.GENERAL.CM2_HOMEPAGE);
				AboutPane.this.dispose();
			}
		});

		this.setVisible(true);

	}

}
