/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.interfaces.MembraneViewListener;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.ConfigParser;
import org.cellmicrocosmos.cm2.pluginmanager.Plugin;
import org.cellmicrocosmos.cm2.pluginmanager.PluginManager;


/**
 * The Calculation Wizzard helps the user to start the calculation.
 * 
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class AlgorithmWizzard extends JDialog implements ActionListener,
		ListSelectionListener {

	private static final long serialVersionUID = 1L;

	public static final int RUN_MODE = 0;
	public static final int MANAGING_MODE = 1;

	int mode = AlgorithmWizzard.MANAGING_MODE;

	MembraneViewListener listener;

	JButton runBtn = new JButton("Start");
	JButton closeBtn = new JButton("Close");
	JButton addBtn = new JButton("Add...");
	JButton restoreBtn = new JButton("Restore...");
	JButton removeBtn = new JButton("Remove");
	JButton refreshBtn = new JButton("Refresh Table");

	MyTable table = new MyTable();

	DefaultTableModel model = new DefaultTableModel() {

		private static final long serialVersionUID = 1L;

		@Override
		public Class<?> getColumnClass(final int index) {

			return super.getColumnClass(index);

		}

		@Override
		public boolean isCellEditable(final int row, final int column) {

			return false;

		};
	};

	JPanel mainpanel = new JPanel();

	JPanel left = new JPanel();
	JPanel right = new JPanel();

	JPanel infopanel = new JPanel();
	JPanel namepanel = new JPanel();
	JPanel typepanel = new JPanel();
	JPanel pathpanel = new JPanel();
	JPanel textfieldpanel = new JPanel();
	JLabel namelabel = new JLabel("", SwingConstants.LEADING);
	JLabel typelabel = new JLabel("", SwingConstants.LEADING);
	JTextField pathlabel = new JTextField();

	JPanel leftBtnbar = new JPanel();
	JPanel rightBtnbarUpper = new JPanel();
	JPanel rightBtnbarBottom = new JPanel();
	JPanel leftinnerBtnbar = new JPanel();
	JLabel ta = new JLabel("", SwingConstants.LEADING);
	JScrollPane scroll;
	JScrollPane scroll2;
	
	JCheckBox chkStartInBackground = new JCheckBox();
	JCheckBox chkShowProtocol = new JCheckBox();
	JCheckBox chkUniqueRandom = new JCheckBox();
	JSpinner jspUniqueRandomSeed = new JSpinner(new SpinnerNumberModel(new Long(DataPool.AlgorithmSettings.uniqueRandomSeed).intValue(), Integer.MIN_VALUE, Integer.MAX_VALUE, 1));
	
	ImageIcon yesIcon = Cm2_IconDepot.getSpecifiedIcon("Generate_Green");
	ImageIcon noIcon = Cm2_IconDepot.getSpecifiedIcon("Generate_Red");

	private final PluginManager pluginmanager;

	public AlgorithmWizzard(final PluginManager pluginmanager,
			final MembraneViewListener listener) {
		this.listener = listener;
		this.pluginmanager = pluginmanager;
		if (this.listener != null)
			this.mode = AlgorithmWizzard.RUN_MODE;

		this.setModal(true);
		this.setAlwaysOnTop(true);
		this.runBtn.addActionListener(this);
		this.closeBtn.addActionListener(this);
		this.addBtn.addActionListener(this);
		this.restoreBtn.addActionListener(this);
		this.removeBtn.addActionListener(this);
		this.refreshBtn.addActionListener(this);

		this.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(final WindowEvent e) {
				pluginmanager.writeToDisc();
			}
		});

		this.getContentPane().add(this.mainpanel);
		this.mainpanel.add(this.left, BorderLayout.WEST);
		this.mainpanel.add(this.right, BorderLayout.EAST);

		this.left.setLayout(new BoxLayout(this.left, BoxLayout.Y_AXIS));
		this.right.setLayout(new BoxLayout(this.right, BoxLayout.Y_AXIS));
		this.infopanel
				.setLayout(new BoxLayout(this.infopanel, BoxLayout.Y_AXIS));

		this.leftBtnbar.setLayout(new BoxLayout(this.leftBtnbar,
				BoxLayout.X_AXIS));
		this.leftinnerBtnbar.setLayout(new BoxLayout(this.leftinnerBtnbar,
				BoxLayout.X_AXIS));
		
		this.rightBtnbarUpper.setLayout(new BoxLayout(this.rightBtnbarUpper,
				BoxLayout.X_AXIS));
//		this.rightBtnbarUpper.setSize(new Dimension(300,50));
//		this.rightBtnbarUpper.setMinimumSize(new Dimension(300,50));
//		this.rightBtnbarUpper.setMaximumSize(new Dimension(300,50));
		
		this.rightBtnbarBottom.setLayout(new BoxLayout(this.rightBtnbarBottom,
				BoxLayout.X_AXIS));
//		this.rightBtnbarBottom.setSize(new Dimension(300,50));
//		this.rightBtnbarBottom.setMinimumSize(new Dimension(300,50));
//		this.rightBtnbarBottom.setMaximumSize(new Dimension(300,50));

		this.scroll = new JScrollPane(this.table);
		this.scroll2 = new JScrollPane(this.ta);

		this.left.add(this.scroll);
		this.left.add(this.leftBtnbar);
		this.leftBtnbar.add(this.leftinnerBtnbar);

		this.right.add(this.infopanel);
		this.right.add(Box.createVerticalGlue());
		this.right.add(this.rightBtnbarUpper);
		this.right.add(Box.createVerticalGlue());
		this.right.add(this.rightBtnbarBottom);

		this.infopanel.add(Box.createVerticalGlue());
		this.infopanel.add(this.namepanel);
		this.infopanel.add(Box.createVerticalGlue());
		this.infopanel.add(this.typepanel);
		this.infopanel.add(Box.createVerticalGlue());
		this.infopanel.add(this.pathpanel);
		this.infopanel.add(Box.createVerticalGlue());
		this.infopanel.add(this.textfieldpanel);
		this.infopanel.add(Box.createVerticalGlue());
		
		this.namepanel.add(this.namelabel);
		this.typepanel.add(this.typelabel);
		this.pathpanel.add(this.pathlabel);
		this.textfieldpanel.add(this.scroll2);

		this.textfieldpanel.setBorder(BorderFactory
				.createTitledBorder("Information"));

		this.leftinnerBtnbar.add(this.addBtn);
		this.leftinnerBtnbar.add(Box.createHorizontalGlue());
		this.leftinnerBtnbar.add(this.restoreBtn);
		this.leftinnerBtnbar.add(this.removeBtn);

		this.leftBtnbar.add(Box.createHorizontalStrut(20));
		this.leftBtnbar.add(this.refreshBtn);

		// rightBtnbar.add(Box.createHorizontalGlue());

		this.chkStartInBackground.setSelected(DataPool.AlgorithmSettings.startInBackground);
		this.chkStartInBackground.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if (AlgorithmWizzard.this.chkStartInBackground.isSelected())
					DataPool.AlgorithmSettings.startInBackground = true;
				else
					DataPool.AlgorithmSettings.startInBackground = false;
			}
		});
		
		this.chkShowProtocol.setSelected(DataPool.AlgorithmSettings.showProtocol);
		this.chkShowProtocol.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if (AlgorithmWizzard.this.chkShowProtocol.isSelected())
					DataPool.AlgorithmSettings.showProtocol = true;
				else
					DataPool.AlgorithmSettings.showProtocol = false;
			}
		});
		
		this.chkUniqueRandom.setSelected(DataPool.AlgorithmSettings.uniqueRandom);
		this.chkUniqueRandom.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if (AlgorithmWizzard.this.chkUniqueRandom.isSelected()) {
					AlgorithmWizzard.this.jspUniqueRandomSeed.setEnabled(true);
					DataPool.AlgorithmSettings.uniqueRandom = true;
				} else {
					AlgorithmWizzard.this.jspUniqueRandomSeed.setEnabled(false);
					DataPool.AlgorithmSettings.uniqueRandom = false;
				}
			}
		});
		jspUniqueRandomSeed.setMaximumSize(new Dimension(60, 25));
		jspUniqueRandomSeed.setMinimumSize(new Dimension(60, 25));
		if (DataPool.AlgorithmSettings.uniqueRandom)
			jspUniqueRandomSeed.setEnabled(true);	
		else
			jspUniqueRandomSeed.setEnabled(false);
		jspUniqueRandomSeed.setValue(new Long(DataPool.AlgorithmSettings.uniqueRandomSeed).intValue());
		jspUniqueRandomSeed.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				DataPool.AlgorithmSettings.uniqueRandomSeed = Long.parseLong(AlgorithmWizzard.this.jspUniqueRandomSeed.getValue().toString());
			}
		});
		chkStartInBackground.setSelected(DataPool.AlgorithmSettings.startInBackground);
		chkShowProtocol.setSelected(DataPool.AlgorithmSettings.showProtocol);
		chkUniqueRandom.setSelected(DataPool.AlgorithmSettings.uniqueRandom);
		
		chkStartInBackground.setVisible(true);
		chkStartInBackground.setText("Background-Start");
		chkStartInBackground.setHorizontalTextPosition(SwingConstants.LEFT);
		chkStartInBackground.setMargin(new Insets(0,0,0,0));
		chkShowProtocol.setVisible(true);
		chkShowProtocol.setText("Show Protocol");
		chkUniqueRandom.setVisible(true);
//		chkUniqueRandom.setText("UniqueRandom");
		chkUniqueRandom.setText("Random Seed");
		jspUniqueRandomSeed.setVisible(true);
		
		this.rightBtnbarBottom.add(this.chkShowProtocol);
		this.rightBtnbarBottom.add(Box.createHorizontalStrut(10));
		this.rightBtnbarBottom.add(this.chkUniqueRandom);
		this.rightBtnbarBottom.add(Box.createHorizontalStrut(2));
		this.rightBtnbarBottom.add(this.jspUniqueRandomSeed);

		this.rightBtnbarUpper.add(Box.createHorizontalStrut(5));
		this.rightBtnbarUpper.add(this.chkStartInBackground);
		this.rightBtnbarUpper.add(Box.createHorizontalStrut(6));
		this.rightBtnbarUpper.add(this.runBtn);
		this.rightBtnbarUpper.add(Box.createHorizontalGlue());
		this.rightBtnbarUpper.add(this.closeBtn);

		this.scroll2.setBorder(BorderFactory.createEmptyBorder());
		this.scroll2.setBorder(BorderFactory.createEmptyBorder());

		this.pathlabel.setEditable(false);
		this.pathlabel.setBackground(this.right.getBackground());
		this.pathpanel.setBorder(BorderFactory.createTitledBorder("Filepath"));

		this.namelabel.setFont(this.namelabel.getFont().deriveFont(Font.BOLD,
				this.namelabel.getFont().getSize2D() + 5));
		this.typelabel.setFont(this.typelabel.getFont().deriveFont(Font.PLAIN,
				this.typelabel.getFont().getSize2D() + 5));

		this.runBtn.setFont(this.runBtn.getFont().deriveFont(Font.BOLD,
				this.runBtn.getFont().getSize2D() + 10));
		this.scroll.setPreferredSize(new Dimension(500, 520));
		this.scroll2.setPreferredSize(new Dimension(310, 250));
		this.right.setPreferredSize(new Dimension(330, 520));
		this.pathlabel.setPreferredSize(new Dimension(310, 20));

		this.table.setModel(this.model);

		this.model.setColumnIdentifiers(new String[] { "<html>Name</html>",
				"<html>Micro<br>Domains</html>",
				"<html>Absolute<br>Values</html>",
				"<html>Overrides<br>Lipids</html>", "<html>Type</html>",
				"<html>State</html>" });

		// table.setRowHeight(table.getRowHeight() + 2);
		this.table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		// table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		this.table.getTableHeader().setReorderingAllowed(false);

		this.table.getColumnModel().getColumn(0).setPreferredWidth(100);

		this.table.getColumnModel().getColumn(1).setMaxWidth(60);
		this.table.getColumnModel().getColumn(1).setMinWidth(60);

		this.table.getColumnModel().getColumn(2).setMaxWidth(60);
		this.table.getColumnModel().getColumn(2).setMinWidth(60);

		this.table.getColumnModel().getColumn(3).setMaxWidth(60);
		this.table.getColumnModel().getColumn(3).setMinWidth(60);
		
		this.table.getColumnModel().getColumn(4).setPreferredWidth(30);

		// table.getColumnModel().getColumn(4).setPreferredWidth(120);

		this.table.getTableHeader().setPreferredSize(new Dimension(0, 30));

		this.table.setColumnSelectionAllowed(false);

		this.table.getSelectionModel().addListSelectionListener(this);

		for (int i = 0; i < this.table.getColumnCount(); i++) {
			this.table.getColumnModel().getColumn(i).setCellRenderer(
					new DefaultTableCellRenderer());
		}

		final MyTableCellRenderer renderer = new MyTableCellRenderer();

		this.table.getColumnModel().getColumn(1).setCellRenderer(renderer);
		this.table.getColumnModel().getColumn(2).setCellRenderer(renderer);
		this.table.getColumnModel().getColumn(3).setCellRenderer(renderer);
		
		pluginmanager.performUpdate();
		this.updateTable();

		if (!CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.DEVELOPER_EDITION)
			this.refreshBtn.setVisible(false);

		if (this.mode == AlgorithmWizzard.MANAGING_MODE) {

			this.setTitle("Manage Membrane Algorithms");
			this.runBtn.setVisible(false);
		} else {
			this.setTitle("Run a Membrane Algorithm");
			this.leftBtnbar.remove(this.refreshBtn);
			this.leftBtnbar.add(Box.createHorizontalGlue());
			this.leftBtnbar.add(this.refreshBtn);
			this.leftinnerBtnbar.setVisible(false);

		}

		this.pack();
		this.setResizable(false);
		this.setLocationByPlatform(true);

		if (this.mode == AlgorithmWizzard.RUN_MODE) {
//			final int shrutsize = this.rightBtnbarUpper.getWidth() / 2
//					- this.runBtn.getWidth() / 2;
//			this.rightBtnbarUpper.add(Box.createHorizontalStrut(shrutsize), 0);
			try {
				final String previous = ConfigParser
						.getValue("last_started_algorithm")[0];
				for (int i = 0; i < pluginmanager.plugins.size(); i++) {
					if (previous.equals(pluginmanager.plugins.get(i).getName())) {

						this.table.setRowSelectionInterval(i, i);
						this.updateComponents();
						break;
					}
				}
			} catch (final Exception e1) {

			}
		}

		this.setVisible(true);

	}

	private void updateTable() {
		String selectedname = null;
		if (this.table.getSelectedRow() != -1)
			selectedname = (String) this.table.getModel().getValueAt(
					this.table.getSelectedRow(), 0);

		this.table.getSelectionModel().removeListSelectionListener(this);
		while (this.table.getRowCount() > 0) {
			this.model.removeRow(0);
		}

		for (int i = 0; i < this.pluginmanager.plugins.size(); i++) {
			final Plugin plugin = this.pluginmanager.plugins.get(i);

			final Vector<Object> rowdata = new Vector<Object>();

			rowdata.add(plugin.getName());

			if (plugin.supportsMDs() == null)
				rowdata.add("?");
			else if (plugin.supportsMDs().booleanValue() == true)
				rowdata.add(this.yesIcon);
			else
				rowdata.add(this.noIcon);

			if (plugin.supportsAbsoluteValues() == null)
				rowdata.add("?");
			else if (plugin.supportsAbsoluteValues().booleanValue() == true)
				rowdata.add(this.yesIcon);
			else
				rowdata.add(this.noIcon);
			
			if (plugin.overridesLipids() == null)
				rowdata.add("?");
			else if (plugin.overridesLipids().booleanValue() == true)
				rowdata.add(this.yesIcon);
			else
				rowdata.add(this.noIcon);

			rowdata.add(plugin.getType());

			rowdata.add(plugin.getState());

			this.model.addRow(rowdata);

		}

		if (selectedname != null) {
			for (int i = 0; i < this.table.getRowCount(); i++) {

				final Object value = this.model.getValueAt(i, 0);
				if (value != null && value.equals(selectedname)) {
					this.table.setRowSelectionInterval(i, i);
					break;
				}
			}
		}

		this.table.getSelectionModel().addListSelectionListener(this);

		this.updateComponents();
	}

	public void actionPerformed(final ActionEvent arg0) {
		if (arg0.getSource() == this.closeBtn) {
			this.pluginmanager.writeToDisc();
			this.dispose();
		} else if (arg0.getSource() == this.runBtn) {

			this.useSelectedAlgorithm();

		} else if (arg0.getSource() == this.addBtn) {
			final MyFileChooser chooser = new MyFileChooser();
			final int ret = chooser.showOpenDialog(this);
			if (ret == JFileChooser.CANCEL_OPTION)
				return;

			ConfigParser.deleteAllAt("last_algorithm_directory");
			ConfigParser.addValue("last_algorithm_directory", new File(chooser
					.getSelectedFile().getParent()).getAbsolutePath());
			final File file = chooser.getSelectedFile();
			this.setModal(false);
			this.setAlwaysOnTop(false);
			final boolean b = this.tryToAdd(file);
			this.setModal(true);
			this.setAlwaysOnTop(true);

			if (!b)
				return;
			this.updateTable();
			this.table.setRowSelectionInterval(this.table.getRowCount() - 1,
					this.table.getRowCount() - 1);
			this.updateComponents();
		} else if (arg0.getSource() == this.restoreBtn) {

			final int row = this.table.getSelectedRow();
			final Plugin plugin = this.pluginmanager.plugins.get(row);
			final MyFileChooser chooser = new MyFileChooser();
			final int ret = chooser.showOpenDialog(this);
			if (ret == JFileChooser.CANCEL_OPTION)
				return;

			ConfigParser.deleteAllAt("last_algorithm_directory");
			ConfigParser.addValue("last_algorithm_directory", new File(chooser
					.getSelectedFile().getParent()).getAbsolutePath());
			final File file = chooser.getSelectedFile();

			this.setModal(false);
			this.setAlwaysOnTop(false);
			final boolean b = this.tryToRestore(plugin, file, row);
			this.setModal(true);
			this.setAlwaysOnTop(true);

			if (!b)
				return;
			this.updateTable();
			this.updateComponents();
		} else if (arg0.getSource() == this.removeBtn) {
			final int row = this.table.getSelectedRow();
			this.pluginmanager.plugins.remove(row);
			this.pluginmanager.plugins.trimToSize();
			this.updateTable();
			this.updateComponents();
		} else if (arg0.getSource() == this.refreshBtn) {

			this.pluginmanager.performUpdate();
			this.updateTable();
		}

	}

	private boolean tryToAdd(final File file) {
		try {
			final Plugin newplugin = this.pluginmanager.readToPlugin(file);

			if (newplugin.getState() != Plugin.STATE_READY) {

				JOptionPane.showMessageDialog(null,
						"The following error occured:\n\n"
								+ newplugin.getInformation(), "ERROR",
						JOptionPane.ERROR_MESSAGE);
				return false;
			}
			if (newplugin.getName() == null) {
				
				JOptionPane.showMessageDialog(null,
						"The selected algorithm must return an name!",
						"No Name", JOptionPane.ERROR_MESSAGE);
				return false;
			}
			if (this.pluginmanager.isAlreadyIn(newplugin.getName(), file)) {
				
				JOptionPane.showMessageDialog(null,
						"The selected algorithm is already in the list!",
						"No Name", JOptionPane.ERROR_MESSAGE);
				return false;
			}

			this.pluginmanager.plugins.add(newplugin);
			return true;
		} catch (final Exception e) {

			JOptionPane.showMessageDialog(null,
					"The following error occured:\n\n" + e.getMessage(),
					"ERROR", JOptionPane.ERROR_MESSAGE);
			return false;
		}

	}

	private boolean tryToRestore(final Plugin plugin, final File file,
			final int index) {
		try {
			final Plugin newplugin = this.pluginmanager.readToPlugin(file);

			if (newplugin.getState() != Plugin.STATE_READY) {

				JOptionPane.showMessageDialog(null,
						"The following error occured:\n\n"
								+ newplugin.getInformation(), "ERROR",
						JOptionPane.ERROR_MESSAGE);
				return false;
			}
			if (newplugin.getName() == null) {
				
				JOptionPane.showMessageDialog(null,
						"The selected algorithm must return an name!",
						"No Name", JOptionPane.ERROR_MESSAGE);
				return false;
			}
			if (this.pluginmanager.isAlreadyIn(newplugin.getName(), file,
					plugin)) {
				
				JOptionPane.showMessageDialog(null,
						"The selected algorithm is already in the list!",
						"No Name", JOptionPane.ERROR_MESSAGE);
				return false;
			}

			this.pluginmanager.plugins.set(index, newplugin);
			return true;
		} catch (final Exception e) {

			JOptionPane.showMessageDialog(null,
					"The following error occured:\n\n" + e.getMessage(),
					"ERROR", JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}

	private class MyTable extends JTable {

		private static final long serialVersionUID = 1L;

		@Override
		public Component prepareRenderer(final TableCellRenderer renderer,
				final int rowIndex, final int vColIndex) {
			final Component c = super.prepareRenderer(renderer, rowIndex,
					vColIndex);

			final Object value = this.getValueAt(rowIndex, 5);

			if (value.equals(Plugin.STATE_READY))
				c.setForeground(null);
			else
				c.setForeground(Color.red.brighter());

			return c;
		}

	}

	private class MyTableCellRenderer extends DefaultTableCellRenderer {

		private static final long serialVersionUID = 1L;

		@Override
		public void setValue(final Object value) {
			if (value instanceof ImageIcon) {
				this.setIcon((ImageIcon) value);
				this.setText("");
			} else {
				this.setIcon(null);
				super.setValue(value);
			}
		}

	}

	class MyFileChooser extends JFileChooser {

		private static final long serialVersionUID = 1L;

		public MyFileChooser() {
			super();
			this.setDialogTitle("Choose your algorithm");
			this.setMultiSelectionEnabled(false);
			this.setAcceptAllFileFilterUsed(false);
			try {
				final File lastused = new File(ConfigParser
						.getValue("last_algorithm_directory")[0]);
				this.setCurrentDirectory(lastused);
			} catch (final Exception e) {

			}
			this.setFileFilter(new FileFilter() {

				@Override
				public boolean accept(final File arg0) {
					if (arg0.isDirectory())
						return true;
					if ((!DataPool.iAmWebstart && arg0.getName().toLowerCase()
							.endsWith(".java"))
							|| arg0.getName().toLowerCase().endsWith(".jar"))
						return true;
					else
						return false;
				}

				@Override
				public String getDescription() {
					if (DataPool.iAmWebstart)
						return ".jar";
					return ".java , .jar";
				}

			});
		}

	}

	public void valueChanged(final ListSelectionEvent arg0) {

		if (!arg0.getValueIsAdjusting())
			return;

		this.updateComponents();

	}

	private void updateComponents() {

		if (this.table.getSelectedRow() < 0) {
			this.restoreBtn.setEnabled(false);
			this.removeBtn.setEnabled(false);
			this.runBtn.setEnabled(false);

			this.ta.setText("");
			this.typelabel.setText("");
			this.pathlabel.setText("");
			this.namelabel.setText("");

		} else {

			final Plugin pl = this.pluginmanager.plugins.get(this.table
					.getSelectedRow());

			this.namelabel.setText(pl.getName());

			String typeonlabel = "";
			final String type = pl.getType();
			if (Plugin.INCLUDED.equals(type))
				typeonlabel = "Included in this application";
			if (Plugin.JAVA_FILE.equals(type))
				typeonlabel = "Uncompiled JAVA file";
			if (Plugin.JAR_FILE.equals(type))
				typeonlabel = "JAR file";

			// Cm2_Toolkit.extendRight(typeonlabel,50);

			this.typelabel.setText(typeonlabel);

			if (Plugin.INCLUDED.equals(type))
				this.pathlabel.setText(pl.getJavapath());
			else if (pl.getOriginalFile() == null
					|| pl.getOriginalFile().getAbsolutePath().length() < 1)
				this.pathlabel.setText("-");
			else
				this.pathlabel.setText(pl.getOriginalFile().getAbsolutePath());

			if (pl.getInformation() == null || pl.getInformation().length() < 1)
				this.ta.setText("<html>(none)</html>");
			else {

				if (pl.getInformation().toLowerCase().contains("<html>") == false) {
					this.ta
							.setText(("<html>" + pl.getInformation() + "</html>")
									.replaceAll("\n", "<br>"));
				} else
					this.ta.setText(pl.getInformation());

			}

			if (Plugin.STATE_READY.equals(pl.getState())) {
				this.runBtn.setEnabled(true);

			} else {
				this.runBtn.setEnabled(false);
			}

			if (Plugin.INCLUDED.equals(type)) {
				this.restoreBtn.setEnabled(false);
				this.removeBtn.setEnabled(false);
			} else {
				this.restoreBtn.setEnabled(true);
				this.removeBtn.setEnabled(true);
			}

		}

	}

	private void useSelectedAlgorithm() {

		this.pluginmanager.performUpdate();
		this.updateTable();

		final Plugin selected = this.pluginmanager.plugins.get(this.table
				.getSelectedRow());
		
		// check if the algorithm supports absolute values if it was selected
		if (!selected.supportsAbsoluteValues() && DataPool.getMembraneModel().getComputeAbsoluteValues()) {
			final int val = JOptionPane.showOptionDialog(
					this,
					"This algorithm does not explicitly support the lipid distribution based on absolute values.\n" +
					"Please use Percentages instead. Stop now and change the settings in the Component View Properties\n" +
					"prior starting this algorithm.\n", "WARNING",
					JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE,
					null, new String[] { "Ignore Wartning and Continue", "Stop" },
					null);
			
			if (val == 1)
				return;
		}
		
		
		

		ConfigParser.deleteAllAt("last_started_algorithm");
		ConfigParser.addValue("last_started_algorithm", selected.getName());

		if (Plugin.INCLUDED.equals(selected.getType())) {
			try {
				final MembranePackingAlgorithm algo = (MembranePackingAlgorithm) (Class
						.forName(selected.getJavapath()).newInstance());

				this.pluginmanager.writeAlgorithmSettings(selected,algo);
				this.pluginmanager.writeToDisc();
				this.dispose();
				this.listener.calculate(algo);
				return;
			} catch (final Exception e) {
				
				this.setModal(false);
				this.setAlwaysOnTop(false);
				final int val = JOptionPane.showOptionDialog(
						this,
						"An Error occured while performing algorithm!\n\n"
								+ "Error Message:\n" + e.toString(), "ERROR",
						JOptionPane.YES_NO_OPTION, JOptionPane.ERROR_MESSAGE,
						null, new String[] { "Print Stack Trace", "Close" },
						null);

				if (val == 0)
					e.printStackTrace();
				
				this.setModal(true);
				this.setAlwaysOnTop(true);
				return;
			}
		}

		if (Plugin.STATE_READY.equals(selected.getState()) == false)
			return;

		try {
			final Plugin newone = this.pluginmanager.readToPlugin(selected
					.getOriginalFile());

			final MembranePackingAlgorithm algo = this.pluginmanager
					.createMembranePackingAlgorithm(newone, newone.getOriginalFile());
			
			if(!newone.isExtendedPlugin()) {
				this.pluginmanager.writeAlgorithmSettings(selected,algo);
				this.pluginmanager.writeToDisc();
			} else {
				this.pluginmanager.writeToDisc();
			}

			this.dispose();
			this.listener.calculate(algo);

		} catch (final Exception e) {

			this.setModal(false);
			this.setAlwaysOnTop(false);
			final int val = JOptionPane.showOptionDialog(
					this,
					"An Error occured while performing algorithm!\n\n"
							+ "Error Message:\n" + e.toString(), "ERROR",
					JOptionPane.YES_NO_OPTION, JOptionPane.ERROR_MESSAGE,
					null, new String[] { "Print Stack Trace", "Close" },
					null);

			if (val == JOptionPane.YES_NO_OPTION)
				e.printStackTrace();
			this.setModal(true);
			this.setAlwaysOnTop(true);
			return;
		}

	}

}
