/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.awt.geom.Ellipse2D;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.Engine;
import org.cellmicrocosmos.cm2.functionality3d.CameraMode;


/**
 * The dialog that allows the user to change the size of the membrane
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
@SuppressWarnings("serial")
public class ChangeSizeDialog extends JDialog implements ActionListener, ChangeListener {

	// possible return values when showing up
	public static final int APPROVE = 1;

	public final int CANCEL = 2;

	private int out = this.CANCEL;

	private JButton cancelBtn, okBtn;

	private JPanel buttonbar;

	private JPanel header1,header2,header3;

	private JLabel lx, lz,lTransX,lTransZ;

	private JSpinner tfx, tfz,tfTransX,tfTransZ;

	private JPanel groundxy;
	private JPanel groundTransxy;

	private JLabel h1;

	private final Point location;



	private Engine engine;

	private Point3f[] oldPositions;

	private Shape[] oldMDGeometries;

	private Dimension oldSize;

	public ChangeSizeDialog(final Point locationOnScreen,final Engine engine, Point3f[] oldPositions,
			Shape[] oldMDGeometries, Dimension oldSize) {
		this.location = locationOnScreen;
		this.engine=engine;
		this.oldPositions=oldPositions;
		this.oldMDGeometries=oldMDGeometries;
		this.oldSize=oldSize;
	}

	

	/**
	 * Constructs the Gui referring to the mode and the model and shows the
	 * Dialog. Blocks until user cancels or accepts
	 * 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public int popUp() {

		this.setTitle("Redefine Membrane Bounds");

		this.tfx = new JSpinner();

		this.tfz = new JSpinner();
		
		this.tfTransX = new JSpinner();

		this.tfTransZ = new JSpinner();

		this.lx = new JLabel("X");
		this.lz = new JLabel("  Z");
		
		this.lTransX = new JLabel("X");
		this.lTransZ = new JLabel("  Z");
		

		this.groundxy = new JPanel();
		this.groundTransxy = new JPanel();

		this.buttonbar = new JPanel();

		this.header1 = new JPanel();
		this.header2 = new JPanel();
		this.header3 = new JPanel();

		this.tfx.setModel(new SpinnerNumberModel(
				CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE,
				CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE,
				CM2_CONSTANTS.MEMBRANE_SIZES.MAX_SIZE_OF_MEMBRANE, 1));
		this.tfz.setModel(new SpinnerNumberModel(
				CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE,
				CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE,
				CM2_CONSTANTS.MEMBRANE_SIZES.MAX_SIZE_OF_MEMBRANE, 1));
		
		this.tfTransX.setModel(new SpinnerNumberModel(
				0,
				-1000,
				1000, 1));
		this.tfTransZ.setModel(new SpinnerNumberModel(
				0,
				-1000,
				1000, 1));

		this.h1 = new JLabel("Redefine the Bounds of this Membrane", SwingConstants.LEADING);

		final JLabel label = new JLabel(
				+ CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE
				+ " <= X,Z <= "
				+ CM2_CONSTANTS.MEMBRANE_SIZES.MAX_SIZE_OF_MEMBRANE 
				+ " Angstrom",
				SwingConstants.LEADING);
		
		final JLabel label2 = new JLabel("Translation",
				SwingConstants.LEADING);
		
		this.header1.add(this.h1);
//		this.h1.setPreferredSize(new Dimension(250, 35));
		this.header2.add(label);
		label.setPreferredSize(new Dimension(250, 30));
		this.header3.add(label2);
		label2.setPreferredSize(new Dimension(250, 30));
//		// header1.setBackground(Color.white);

//		this.groundxy.setPreferredSize(new Dimension(250, 50));
//		this.groundTransxy.setPreferredSize(new Dimension(250, 50));
		this.cancelBtn = new JButton("Cancel");

		this.okBtn = new JButton("Accept");

		// block the rest of the application until this window is closed
		this.setModal(true);

		this.getContentPane().setLayout(
				new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));

		this.getContentPane().add(this.header1);
		this.getContentPane().add(this.header2);
		this.getContentPane().add(this.groundxy);
		this.getContentPane().add(this.header3);
		this.getContentPane().add(this.groundTransxy);
		this.getContentPane().add(this.buttonbar);

		this.buttonbar.add(this.okBtn);
		this.buttonbar.add(this.cancelBtn);

		this.cancelBtn.addActionListener(this);
		this.okBtn.addActionListener(this);

		this.groundxy.add(this.lx);
		this.groundxy.add(this.tfx);
		this.groundxy.add(this.lz);
		this.groundxy.add(this.tfz);
		
		this.groundTransxy.add(this.lTransX);
		this.groundTransxy.add(this.tfTransX);
		this.groundTransxy.add(this.lTransZ);
		this.groundTransxy.add(this.tfTransZ);

		this.tfx.setFont(new Font("Arial", Font.PLAIN, 13));
		this.tfz.setFont(new Font("Arial", Font.PLAIN, 13));
		
		this.tfTransX.setFont(new Font("Arial", Font.PLAIN, 13));
		this.tfTransZ.setFont(new Font("Arial", Font.PLAIN, 13));

		this.h1.setFont(new Font("Serif", Font.PLAIN, 18));

		// make sure this window is always on top(inside the application)
		this.addWindowFocusListener(new WindowFocusListener() {

			public void windowGainedFocus(final WindowEvent e) {

			}

			public void windowLostFocus(final WindowEvent e) {

				ChangeSizeDialog.this.requestFocus();

			}

		});

		this.tfx.setPreferredSize(new Dimension(50, 22));
		this.tfz.setPreferredSize(new Dimension(50, 22));
		
		this.tfTransX.setPreferredSize(new Dimension(50, 22));
		this.tfTransZ.setPreferredSize(new Dimension(50, 22));

		this.setResizable(false);
		this.pack();

		if (this.location == null)
			this.setLocationByPlatform(true);
		else
			this.setLocation(this.location);

		

		


		
		
	
		this.tfx.setValue(DataPool.getMembraneModel().getSize().width);
		this.tfz.setValue(DataPool.getMembraneModel().getSize().height);
		
		this.tfTransX.setValue(0);
		this.tfTransZ.setValue(0);
		
		
		if((DataPool.getMembraneModel().getAllMoleculesInMembrane().length<1)&&DataPool.getMembraneModel().countMicroDomains()<1) {
			this.tfTransX.setEnabled(false);
			this.tfTransZ.setEnabled(false);
		}
		
		this.tfx.addChangeListener(this);
		this.tfz.addChangeListener(this);
		this.tfTransX.addChangeListener(this);
		this.tfTransZ.addChangeListener(this);

		this.setVisible(true);
		
		// goes on here when the dialog gets closed
		return this.out;

	}

	/**
	 * Returns the Value the Spinners are set to
	 * 
	 * @return
	 */
	public Dimension getSpinnerValues() {
		Dimension size;
		try {
			size = new Dimension(((Integer) this.tfx.getValue()).intValue(),
					((Integer) this.tfz.getValue()).intValue());
		} catch (final Exception e) {
			size = null;
		}
		return size;
	}

	@SuppressWarnings("unchecked")
	public void actionPerformed(final ActionEvent arg0) {

		/*
		 * Accepts the Changes
		 */
		if (arg0.getSource() == this.okBtn) {

			this.out = ChangeSizeDialog.APPROVE;
			this.dispose();

		}
		// or cancels the changes
		if (arg0.getSource() == this.cancelBtn) {
			this.dispose();
		}

	}

	public void stateChanged(ChangeEvent arg0) {
	
		int w=((Integer) this.tfx.getValue()).intValue();
		int h=((Integer) this.tfz.getValue()).intValue();
		int x=((Integer) this.tfTransX.getValue()).intValue();
		int z=((Integer) this.tfTransZ.getValue()).intValue();
		
		
		if(arg0.getSource()==this.tfx||arg0.getSource()==this.tfz) {
			
			DataPool.getMembraneModel().setSize(new Dimension(w,h));
			this.engine.membraneview.getMembrane3D().setShowAxis(this.engine.membraneview.getMembrane3D().axisVisible);
			this.engine.membraneview.getMembrane3D().setShowRaster(this.engine.membraneview.getMembrane3D().rasterVisible);
			this.engine.membraneview.getMembrane3D().getCamera().setViewByMode(CameraMode.EXTRA,null);
		}
		else  {
			
	Molecule_3D[]mols=DataPool.getMembraneModel().getAllMoleculesInMembrane();
			
			
		
			
			
			for(int i=0;i<mols.length;i++) {
				mols[i].setPosition(new Point3f(oldPositions[i].x+x,oldPositions[i].y,oldPositions[i].z+z));
				mols[i].move();
			}
			for(int i=0;i<oldMDGeometries.length;i++) {
				if(DataPool.getMembraneModel().getMicroDomain(i).geometry==MicroDomain_3D.RECTANGULAR) {
					
					final Rectangle geo = (Rectangle) oldMDGeometries[i];
					DataPool.getMembraneModel().getMicroDomain(i).geoObject=new Rectangle(geo.x+x,geo.y+z,geo.width,geo.height);
					
				}else if(DataPool.getMembraneModel().getMicroDomain(i).geometry==MicroDomain_3D.ELLIPSE) {
					final Ellipse2D.Double geo = (Ellipse2D.Double ) oldMDGeometries[i];
					DataPool.getMembraneModel().getMicroDomain(i).geoObject=new Ellipse2D.Double (geo.x+x,geo.y+z,geo.width,geo.height);
					
				}else {
					final Polygon poly = (Polygon) oldMDGeometries[i];
					
					
					Polygon newPoly=new Polygon();
					newPoly.xpoints=new int[poly.xpoints.length];
					newPoly.ypoints=new int[poly.ypoints.length];
					newPoly.npoints=poly.npoints;
					
					for(int j=0;j<poly.xpoints.length;j++) {
						newPoly.xpoints[j]=poly.xpoints[j]+x;
						
					}
					for(int j=0;j<poly.ypoints.length;j++) {
						newPoly.ypoints[j]=poly.ypoints[j]+z;
						
					}
					newPoly.invalidate();
					
					
					
					DataPool.getMembraneModel().getMicroDomain(i).geoObject=newPoly;
					
				}
				
				DataPool.getMembraneModel().getMicroDomain(i).updateShape3D();
			}
			
		}
		

		
	}
	
	
	
	public void discardChanges() {
		
	
			
			DataPool.getMembraneModel().setSize(this.oldSize);
			this.engine.membraneview.getMembrane3D().setShowAxis(this.engine.membraneview.getMembrane3D().axisVisible);
			this.engine.membraneview.getMembrane3D().setShowRaster(this.engine.membraneview.getMembrane3D().rasterVisible);
			this.engine.membraneview.getMembrane3D().getCamera().setViewByMode(CameraMode.EXTRA,null);
		
		
			
	Molecule_3D[]mols=DataPool.getMembraneModel().getAllMoleculesInMembrane();
			
			
		
			
			
			for(int i=0;i<mols.length;i++) {
				mols[i].setPosition(oldPositions[i]);
				mols[i].move();
			}
			for(int i=0;i<oldMDGeometries.length;i++) {
				DataPool.getMembraneModel().getMicroDomain(i).geoObject=oldMDGeometries[i];
					
					
				DataPool.getMembraneModel().getMicroDomain(i).updateShape3D();
			}
			
		
	}
	
	
	
	
	
	
	public boolean checkAfterwards() {
		
		final MembraneModel model = DataPool.getMembraneModel();
		
		
	
		
		
			
			final Vector<Protein_3D> prots = new Vector<Protein_3D>();
			final Vector<Lipid_3D> lipsEC = new Vector<Lipid_3D>();
			final Vector<Lipid_3D> lipsIC = new Vector<Lipid_3D>();

			for (int i = 0; i < model.getProteinCount(); i++) {
				final Protein_3D mol = model.getProteinAt(i);
				if (this.engine.membraneview.getMembrane3D().intersector.intercects_Molecule_Border(mol, model
						.getSize())) {
					prots.add(mol);

				}

			}
			for (int i = 0; i < model.getLipidCountExtra(); i++) {
				final Lipid_3D mol = model.getLipidAtExtra(i);
				if (this.engine.membraneview.getMembrane3D().intersector.intercects_Molecule_Border(mol, model
						.getSize())) {
					lipsEC.add(mol);

				}
			}
			for (int i = 0; i < model.getLipidCountIntra(); i++) {
				final Lipid_3D mol = model.getLipidAtIntra(i);
				if (this.engine.membraneview.getMembrane3D().intersector.intercects_Molecule_Border(mol, model
						.getSize())) {
					lipsIC.add(mol);

				}
			}

			if ((prots.isEmpty()&&lipsEC.isEmpty()&&lipsIC.isEmpty())==false) {
				
				int val = JOptionPane
				.showOptionDialog(
						null,
						"Some molecules will be placed outside the membrane!\n"+
						"Should molecules outside the membrane bounds be removed?",
						"Reducing the membrane size",
						JOptionPane.YES_NO_OPTION,
						JOptionPane.WARNING_MESSAGE,
						null,
						new String[] {"Remove Outsiders","Keep Outsiders", "Cancel" },
						null);

				if (val == 2)
					return false;
				else if (val == 1) {
					DataPool.setMembraneHasBeenEditedAgain(true);
					
					if (DataPool.J3DSettings.enableFastTransitions)
						this.engine.membraneview.getMembrane3D().dockScreen(true);
					
					return true;
				}
				
//				if (JOptionPane
//						.showConfirmDialog(
//								this,
//								"Objects outside the new bounds will be removed.\n\n"
//										+ "Continue?",
//								"Reducing the membrane size",
//								JOptionPane.YES_NO_OPTION,
//								JOptionPane.WARNING_MESSAGE) == JOptionPane.NO_OPTION)
//					return false;
			}
			
			if (DataPool.J3DSettings.enableFastTransitions)
				this.engine.membraneview.getMembrane3D().dockScreen(false);
			
			if (!prots.isEmpty())
				this.engine.membraneview.getMembrane3D().removeMoleculesFrom3D(prots.toArray(new Protein_3D[0]));

			if (!lipsEC.isEmpty())
				this.engine.membraneview.getMembrane3D().removeMoleculesFrom3D(lipsEC.toArray(new Lipid_3D[0]));

			if (!lipsIC.isEmpty())
				this.engine.membraneview.getMembrane3D().removeMoleculesFrom3D(lipsIC.toArray(new Lipid_3D[0]));

			for (int i = 0; i < prots.size(); i++)
				model.deleteProtein(prots.get(i));

			for (int i = 0; i < lipsEC.size(); i++)
				model.deleteLipidExtra(lipsEC.get(i));

			for (int i = 0; i < lipsIC.size(); i++)
				model.deleteLipidIntra(lipsIC.get(i));

		
		
		
		DataPool.setMembraneHasBeenEditedAgain(true);
		
		
		if (DataPool.J3DSettings.enableFastTransitions)
			this.engine.membraneview.getMembrane3D().dockScreen(true);
		
		return true;
	}
	
	
	
	
	
}
