/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.vecmath.AxisAngle4f;
import javax.vecmath.Matrix3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;


/**
 * A Frame that lets the User type in the Values of the 
 * matrix and the y tanslation manually.
 * 
 * @author Tim Dingersen dackelnose@gmx.net 
 * 
 */
@SuppressWarnings("serial")
public class EnterMatrixDialog extends JDialog implements CaretListener,
		ActionListener {

	private static final long serialVersionUID = 1L;

	JPanel ground2 = new JPanel();

	JButton ok = new JButton("Ok");
	JButton cancel = new JButton("Cancel");
	JButton pdbdefault = new JButton("Set Unit Matrix");
	JButton axisAngleToMatrix = new JButton("Update Rotation");

	JPanel panelMatrix = new JPanel();

	JPanel panelAxisAngle = new JPanel();
	
	JTextField[] tfMatrix = new JTextField[9];
	JTextField[] tfTranslation = new JTextField[1];
	JTextField[] tfAxisAngle = new JTextField[4];

	JPanel panelTranslation = new JPanel();

	Matrix3f retmatrix = null;
	float rettranslation = 0;

	JTextField tfinfo = new JTextField() {
		public void setText(final String text) {
			super.setText(text);
			if (text.length() > 0)
				this.setBorder(BorderFactory.createLineBorder(Color.red));
			else
				this.setBorder(null);
		}
	};

	private final Molecule_3D currentMolecule;

	/**
	 * Constucts a Frame and shows the passed inital Data in its textfields
	 */
	public EnterMatrixDialog(final Molecule_3D currentMolecule,
			final Point point) {
		this.currentMolecule = currentMolecule;
		final Matrix3f matrix = currentMolecule.getRotationMatrix();
		final float translation = currentMolecule.getYTranslation();
		final float[] flsMatrix = { matrix.m00, matrix.m01, matrix.m02, matrix.m10,
				matrix.m11, matrix.m12, matrix.m20, matrix.m21, matrix.m22 };
		
		AxisAngle4f axisAngle = new AxisAngle4f();
		axisAngle.set(matrix);
		final float[] flsAxisAngle ={axisAngle.x, axisAngle.y, axisAngle.z, axisAngle.angle}; 
		
		this.setTitle("Enter Values");

		this.getContentPane().setLayout(
				new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));

		this.panelMatrix.setLayout(new GridLayout(3, 3));

		for (int i = 0; i < this.tfMatrix.length; i++) {
			this.tfMatrix[i] = new JTextField();
			this.tfMatrix[i].addCaretListener(this);
			// tfMatrix[i].setPreferredSize(new Dimension(50,20));
			this.panelMatrix.add(this.tfMatrix[i]);
			String toset = "" + flsMatrix[i];
			if (toset.length() > 10)
				toset = toset.substring(0, 10);
			this.tfMatrix[i].setText(toset);
			this.tfMatrix[i].setBorder(BorderFactory.createLoweredBevelBorder());
		}
		
		this.panelAxisAngle.setLayout(new GridLayout(1, 4));
		
		for (int i = 0; i < this.tfAxisAngle.length; i++) {
			this.tfAxisAngle[i] = new JTextField();
			this.tfAxisAngle[i].addCaretListener(this);
			// tfMatrix[i].setPreferredSize(new Dimension(50,20));
			this.panelAxisAngle.add(this.tfAxisAngle[i]);
			String toset = "" + flsAxisAngle[i];
			if (toset.length() > 10)
				toset = toset.substring(0, 10);
			this.tfAxisAngle[i].setText(toset);
			this.tfAxisAngle[i].setBorder(BorderFactory.createLoweredBevelBorder());
		}
		this.panelAxisAngle.add(axisAngleToMatrix);

		this.tfTranslation[0] = new JTextField();
		this.tfTranslation[0].setText("" + translation);

		this.panelMatrix.setPreferredSize(new Dimension(290, 75));
		this.panelAxisAngle.setPreferredSize(new Dimension(290, 40));

		this.tfinfo.setPreferredSize(new Dimension(290, 15));
		this.tfTranslation[0].setPreferredSize(new Dimension(100, 20));

		this.panelMatrix.setBorder(BorderFactory.createTitledBorder("Rotation (Matrix 3x3)"));
		this.panelAxisAngle.setBorder(BorderFactory.createTitledBorder("Axis Angle (X, Y, Z, Angle in Radians) for Rotation"));
		this.panelTranslation.setBorder(BorderFactory.createTitledBorder("Translation (Y Axis)"));
		this.tfTranslation[0].setBorder(BorderFactory.createLoweredBevelBorder());

		this.tfinfo.setBackground(this.panelMatrix.getBackground());

		this.tfinfo.setEditable(false);

		this.getContentPane().add(this.panelMatrix);
		this.getContentPane().add(this.panelAxisAngle);
		this.getContentPane().add(this.ground2);
		this.getContentPane().add(this.tfinfo);

		this.panelTranslation.add(this.tfTranslation[0]);

		this.ground2.setLayout(new BoxLayout(this.ground2, BoxLayout.X_AXIS));
		this.ground2.add(this.panelTranslation);
		this.ground2.add(this.pdbdefault);
		this.ground2.add(Box.createHorizontalStrut(10));
		this.ground2.add(Box.createGlue());
		this.ground2.add(this.ok);
		this.ground2.add(Box.createHorizontalStrut(4));
		this.ground2.add(this.cancel);

		this.pdbdefault
				.setToolTipText("Use this to restore default rotation (by pdb file)");

		this.pack();
		this.setLocation(point.x, point.y - this.getHeight());

		this.tfTranslation[0].addCaretListener(this);
		this.tfAxisAngle[0].addCaretListener(this);
		this.ok.addActionListener(this);
		this.cancel.addActionListener(this);
		this.pdbdefault.addActionListener(this);
		this.axisAngleToMatrix.addActionListener(this);

		this.setAlwaysOnTop(true);
		this.setModal(true);
		this.setVisible(true);
	}

	/**
	 * Each time a change was made, values are checked for validity
	 */
	public void caretUpdate(final CaretEvent arg0) {

		float fl = 0;
		for (int i = 0; i < this.tfMatrix.length; i++) {

			if (this.tfMatrix[i] == null)
				return;
			if (this.tfMatrix[i].getText() == null) {
				this.ok.setEnabled(false);
				this.tfinfo.setText("Empty Fields!");
				return;
			}

			try {
				fl = Float.parseFloat(this.tfMatrix[i].getText());
			} catch (final Exception e) {
				this.ok.setEnabled(false);
				this.tfinfo.setText("Invalid Number Format!");
				return;
			}

			if (fl > Float.MAX_VALUE) {
				this.ok.setEnabled(false);
				this.tfinfo.setText("Too large Numbers!");
				return;
			}

			// if (i!=9&&(fl > 1 || fl < -1)) {
			// ok.setEnabled(false);
			// tfinfo.setText("Matrix values out of range!");
			// return;
			//
			// }

			if (i == this.tfMatrix.length - 1
					&& Math.abs(fl) > CM2_CONSTANTS.MEMBRANE_SIZES.MAX_Y_TRANSLATION
							- Math.abs(this.currentMolecule.getUpper(true).y
							- this.currentMolecule.getLower(true).y)
					&& Math.abs(fl) < CM2_CONSTANTS.MEMBRANE_SIZES.MIN_Y_TRANSLATION
							+ Math.abs(this.currentMolecule.getUpper(true).y
							+ this.currentMolecule.getLower(true).y)) {
				this.ok.setEnabled(false);
				this.tfinfo.setText("Y Translation too extreme!");
				return;
			}

		}

		final Matrix3f matrix = new Matrix3f();
		matrix.m00 = Float.parseFloat(this.tfMatrix[0].getText());
		matrix.m01 = Float.parseFloat(this.tfMatrix[1].getText());
		matrix.m02 = Float.parseFloat(this.tfMatrix[2].getText());
		matrix.m10 = Float.parseFloat(this.tfMatrix[3].getText());
		matrix.m11 = Float.parseFloat(this.tfMatrix[4].getText());
		matrix.m12 = Float.parseFloat(this.tfMatrix[5].getText());
		matrix.m20 = Float.parseFloat(this.tfMatrix[6].getText());
		matrix.m21 = Float.parseFloat(this.tfMatrix[7].getText());
		matrix.m22 = Float.parseFloat(this.tfMatrix[8].getText());

		final float det = matrix.determinant();

		if (Math.abs(det - 1.0000000f) > 0.0001) {
			this.tfinfo.setText("Determinat is != 1");
			this.ok.setEnabled(false);
		} else {
			this.tfinfo.setText("");
			this.ok.setEnabled(true);
		}

	}

	/**
	 * Values from the textFields are passed to the Interface
	 */
	public void actionPerformed(final ActionEvent arg0) {

		if (arg0.getSource() == this.ok) {
			this.retmatrix = new Matrix3f();
			this.retmatrix.m00 = Float.parseFloat(this.tfMatrix[0].getText());
			this.retmatrix.m01 = Float.parseFloat(this.tfMatrix[1].getText());
			this.retmatrix.m02 = Float.parseFloat(this.tfMatrix[2].getText());
			this.retmatrix.m10 = Float.parseFloat(this.tfMatrix[3].getText());
			this.retmatrix.m11 = Float.parseFloat(this.tfMatrix[4].getText());
			this.retmatrix.m12 = Float.parseFloat(this.tfMatrix[5].getText());
			this.retmatrix.m20 = Float.parseFloat(this.tfMatrix[6].getText());
			this.retmatrix.m21 = Float.parseFloat(this.tfMatrix[7].getText());
			this.retmatrix.m22 = Float.parseFloat(this.tfMatrix[8].getText());
			this.rettranslation = Float.parseFloat(this.tfTranslation[0].getText());
			this.dispose();
		} else if (arg0.getSource() == this.cancel) {
			this.dispose();
		} else if (arg0.getSource() == this.pdbdefault) {
			this.tfMatrix[0].setText("1.0");
			this.tfMatrix[1].setText("0.0");
			this.tfMatrix[2].setText("0.0");
			this.tfMatrix[3].setText("0.0");
			this.tfMatrix[4].setText("1.0");
			this.tfMatrix[5].setText("0.0");
			this.tfMatrix[6].setText("0.0");
			this.tfMatrix[7].setText("0.0");
			this.tfMatrix[8].setText("1.0");
			this.tfAxisAngle[0].setText("0.0");
			this.tfAxisAngle[1].setText("1.0");
			this.tfAxisAngle[2].setText("0.0");
			this.tfAxisAngle[3].setText("0.0");
		} else if (arg0.getSource() == this.axisAngleToMatrix){
			AxisAngle4f axisAngle = new AxisAngle4f(Float.parseFloat(this.tfAxisAngle[0].getText()), 
													Float.parseFloat(this.tfAxisAngle[1].getText()), 
													Float.parseFloat(this.tfAxisAngle[2].getText()), 
													Float.parseFloat(this.tfAxisAngle[3].getText()));
			Matrix3f matrixFromAxisAngle = new Matrix3f();
			matrixFromAxisAngle.set(axisAngle);
			this.tfMatrix[0].setText(Float.toString(matrixFromAxisAngle.m00));
			this.tfMatrix[1].setText(Float.toString(matrixFromAxisAngle.m01));
			this.tfMatrix[2].setText(Float.toString(matrixFromAxisAngle.m02));
			this.tfMatrix[3].setText(Float.toString(matrixFromAxisAngle.m10));
			this.tfMatrix[4].setText(Float.toString(matrixFromAxisAngle.m11));
			this.tfMatrix[5].setText(Float.toString(matrixFromAxisAngle.m12));
			this.tfMatrix[6].setText(Float.toString(matrixFromAxisAngle.m20));
			this.tfMatrix[7].setText(Float.toString(matrixFromAxisAngle.m21));
			this.tfMatrix[8].setText(Float.toString(matrixFromAxisAngle.m22));
		}

	}

	public Matrix3f getNewMatrix() {
		return this.retmatrix;
	}

	public float getNewYTranslation() {
		return this.rettranslation;
	}

}
