/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.io.FileIOHandler;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.cellmicrocosmos.cm2.misc.ConfigParser;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


import com.sun.org.apache.xerces.internal.parsers.DOMParser;

/**
 * Provides a Popup Help Dialog where the User can view and browse trough the
 * program's help-pages. The selected page, may be a html or a text file, is
 * shown in the frame plus a tree, where all accessible pages can be selected.
 * In addition, the tree can be browsed by forward-back buttons. No contents are
 * directly implemented in this class; The tree builds itself up based on a
 * "Table of Contents" - file that uses the syntax of xml. That helps creating
 * or editing the progrm's help easily without changing any sourcecode.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class HelpDialog {

	String home = CM2_CONSTANTS.DEBUG.READ_HELPFILES_DIRECTLY_FROM_MAIN_FOLDER ? ""
			: FileIOHandler.home;
	// the location of the TOC file
	public File toc_file = CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.DEVELOPER_EDITION ? (new File(
			this.home + "cm2help/table_of_contents_dev.xml"))
			: (new File(this.home + "cm2help/table_of_contents.xml"));

	// the initial size of the help dialog referring to the screensize. May be
	// edited

	private JFrame frame;

	private JSplitPane splitpane;

	private JTree tree;

	private JEditorPane editorpane;

	private DefaultMutableTreeNode rootNode;

	private JButton btn1;

	private JButton btn2;

	private JButton btn3;

	private JButton btn4;

	private JPanel panel;

	// the current position in the tree
	private int position;

	// stores filepaths by their referring name
	private Hashtable<String, String> references;

	private Hashtable<String, String> quickhelp;

	private final Vector<URL> history = new Vector<URL>();

	private final Vector<String> orderedlist = new Vector<String>();

	private String addLinuxSeparator = "";

	boolean init = false;

	public HelpDialog() {
		try {
			this.buildIndexfromFile();
			this.init = true;
			// and if that fails, tree is removed and no browsing will be
			// available

		} catch (final Exception e) {

			System.err.println("HelpDialog: IOException occured while reading "
					+ this.toc_file.getPath());

			e.printStackTrace();

		}
	}

	/**
	 * Builds up the frame, initializes the tree and makes them appear on
	 * screen.
	 */
	public void popupHelp() {
		if (!this.init)
			return;
		this.popupHelp(0);

	}

	/**
	 * Builds up the frame, initializes the tree and makes them appear on
	 * screen.
	 * 
	 * @param filename
	 *            - the filepath of the page the dialog should be showing
	 *            initially
	 */
	public void popupHelp(final int location) {
		if (!this.init)
			return;
		// running on a Linux system, "file:///" is needed
		// running on a Windows system, "file://" is needed
		if (File.separator.matches("/")) {
			this.addLinuxSeparator = "/";
		}

		// initializing the basic things needed to construct the frame
		this.position = 0;

		this.frame = new JFrame("Cm2 Helper");
		this.splitpane = new JSplitPane();

		this.editorpane = new JEditorPane() {

			private static final long serialVersionUID = 7151985071156827687L;

			@Override
			public int getScrollableUnitIncrement(final Rectangle visibleRect,
					final int orientation, final int direction) {
				return 9;
			}

			@Override
			public int getScrollableBlockIncrement(final Rectangle visibleRect,
					final int orientation, final int direction) {
				return 10;
			}
		};
		this.panel = new JPanel();
		this.btn1 = new JButton();
		this.btn2 = new JButton();
		this.btn3 = new JButton("Close");
		this.btn4 = new JButton();

		// tree.setBackground(Color.white);

		this.btn1.setToolTipText("Ascend one page in the tree");
		this.btn2.setToolTipText("Descent one page in the tree");

		this.btn4.setVisible(false);
		this.btn1.setIcon(Cm2_IconDepot.getSpecifiedIcon("up"));
		this.btn2.setIcon(Cm2_IconDepot.getSpecifiedIcon("down"));
		this.btn4.setIcon(Cm2_IconDepot.getSpecifiedIcon("CV_left"));
		// setting the frames bounds regarding to the definded scale
		final Dimension screensize = this.frame.getToolkit().getScreenSize();
		this.frame
				.setSize(
						(int) (screensize.width
								* CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.HELP_FRAME_SCALE + screensize.width / 16),
						(int) (screensize.height * CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.HELP_FRAME_SCALE));
		this.frame.setLocation(
				screensize.width / 2 - this.frame.getWidth() / 2,
				screensize.height / 2 - this.frame.getHeight() / 2);

		this.frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.frame.setIconImage(Cm2_IconDepot.getSpecifiedIcon("Toolbar_Help")
				.getImage());

		// putting the components together
		this.splitpane.setLeftComponent(new JScrollPane(this.tree));
		this.splitpane.setRightComponent(new JScrollPane(this.editorpane));
		this.frame.getContentPane().add(this.splitpane, BorderLayout.CENTER);
		this.panel.setLayout(new BoxLayout(this.panel, BoxLayout.X_AXIS));
		this.panel.add(Box.createHorizontalGlue());
		this.panel.add(this.btn4);
		this.panel.add(Box.createHorizontalStrut(25));
		this.panel.add(this.btn1);
		this.panel.add(Box.createHorizontalStrut(5));
		this.panel.add(this.btn2);
		this.panel.add(Box.createHorizontalGlue());
		this.panel.add(this.btn3);
		this.panel.add(Box.createHorizontalStrut(5));
		this.frame.getContentPane().add(this.panel, BorderLayout.SOUTH);

		this.btn1.setPreferredSize(new Dimension(150, 30));
		this.btn2.setPreferredSize(new Dimension(150, 30));
		this.btn4.setPreferredSize(new Dimension(40, 30));
		// furnish the tree with nice icons...
		final DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();

		renderer
				.setOpenIcon(Cm2_IconDepot.getSpecifiedIcon("Help_Node_opened"));
		renderer.setClosedIcon(Cm2_IconDepot
				.getSpecifiedIcon("Help_Node_closed"));
		renderer.setLeafIcon(Cm2_IconDepot.getSpecifiedIcon("Help_Leaf"));
		// renderer.setBackgroundNonSelectionColor(tree.getBackground());

		this.tree.setCellRenderer(renderer);

		// comfigurating the components
		this.tree.setBorder(BorderFactory.createRaisedBevelBorder());
		this.tree.setRootVisible(false);
		this.tree.setEditable(false);
		this.tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		this.tree.setShowsRootHandles(false);

		this.editorpane.setEditable(false);
		this.editorpane.setEditorKit(new HTMLEditorKit());
		this.editorpane.setMargin(new Insets(10, 10, 10, 10));

		this.splitpane.setOneTouchExpandable(true);
		this.splitpane.setContinuousLayout(true);

		// adding a listener to the editorpane that reacts on hyperlinksclicks
		this.editorpane.addHyperlinkListener(new HyperlinkListener() {

			public void hyperlinkUpdate(final HyperlinkEvent arg0) {
				HelpDialog.this.hyperlinkClicked(arg0);

			}

		});

		// adding an Actionlistener to the trwo buttons
		this.btn1.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {

				/*
				 * moves one page backwards in the tree. If it arrives a closed
				 * node, the node will be expanded and the selection will jump
				 * to the last position in this node Or, if the position is
				 * already on top, it will jump to the end of the whole tree
				 */
				HelpDialog.this.position--;
				if (HelpDialog.this.position > 0) {
					final int oldcount = HelpDialog.this.tree.getRowCount();
					HelpDialog.this.tree.expandRow(HelpDialog.this.position);
					final int newcount = HelpDialog.this.tree.getRowCount();
					if (oldcount != newcount) {

						HelpDialog.this.position += newcount - oldcount;
						HelpDialog.this.position += 1;
					}

				} else
					HelpDialog.this.position = HelpDialog.this.tree
							.getRowCount();

				HelpDialog.this.tree.setSelectionRow(HelpDialog.this.position);
			}

		});
		this.btn2.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {

				/*
				 * same as above, only other direction; but if a closed node
				 * gets selected, position will remain there
				 */
				HelpDialog.this.position++;
				if (HelpDialog.this.position < HelpDialog.this.tree
						.getRowCount()) {
					HelpDialog.this.tree.expandRow(HelpDialog.this.position);

				} else
					HelpDialog.this.position = 1;

				HelpDialog.this.tree.setSelectionRow(HelpDialog.this.position);
			}

		});
		this.btn4.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {

				if (HelpDialog.this.history.size() > 1) {

					HelpDialog.this.jumpToPage(HelpDialog.this.history
							.lastElement());
					HelpDialog.this.history
							.removeElementAt(HelpDialog.this.history.size() - 1);
					if (HelpDialog.this.history.size() == 1)
						HelpDialog.this.btn4.setVisible(false);
				}

			}
		});
		this.btn3.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {

				HelpDialog.this.frame.dispose();
			}

		});

		this.frame.setVisible(true);

		// adding a listener for selections made in the tree
		this.tree.addTreeSelectionListener(new TreeSelectionListener() {

			public void valueChanged(final TreeSelectionEvent arg0) {
				HelpDialog.this.treeClicked(arg0);

				try {
					if (HelpDialog.this.tree.getSelectionRows()[0] == 0)
						HelpDialog.this.editorpane.setBackground(Color.black);
					else
						HelpDialog.this.editorpane
								.setBackground(DataPool.J3DSettings.bright_appearance ? null
										: Color.GRAY);
				} catch (final Exception e) {
					HelpDialog.this.editorpane.setBackground(null);
				}
			}

		});

		if (this.tree.getRowCount() > location)
			this.tree.setSelectionRow(location);

	}

	/**
	 * reads in the TOC file and builds up the tree by the data
	 * 
	 * @throws Exception
	 */
	private void buildIndexfromFile() throws Exception {

		this.references = new Hashtable<String, String>();

		this.quickhelp = new Hashtable<String, String>();
		this.tree = new JTree();
		final DOMParser parser = new DOMParser();

		// running on a Linux system, "file:///" is needed
		// running on a Windows system, "file://" is needed
		if (File.separator.matches("/")) {
			this.addLinuxSeparator = "/";
		}

		parser.parse("file:/" + this.addLinuxSeparator
				+ this.toc_file.getPath());

		this.rootNode = new DefaultMutableTreeNode();

		// filling the tree's root node with children
		this.fillTree(parser.getDocument().getDocumentElement(), this.rootNode);

		// adding the rootNode to the tree and expands it
		final DefaultTreeModel model = new DefaultTreeModel(this.rootNode);
		this.tree.setModel(model);

		for (int i = 0; i < this.tree.getRowCount(); i++) {
			this.tree.expandRow(i);
		}

	}

	/**
	 * tries to display the requested page
	 * 
	 * @param filename
	 *            - the file to display
	 */
	private void jumpToPage(final String filename) {

		// final URL url = ClassLoader.getSystemResource(filename);

		final URL url;
		try {

			url = new URL("file:/"
					+ this.addLinuxSeparator
					+ new File(this.home + "cm2help").getAbsoluteFile()
							.getParentFile().getAbsolutePath() + File.separator
					+ filename);

			// System.out.println(url);
		} catch (final MalformedURLException e1) {
			e1.printStackTrace();
			return;
		}

		((JComponent) this.splitpane.getRightComponent())
				.setBorder(BorderFactory.createTitledBorder(new File(filename)
						.getName()));

		SwingUtilities.invokeLater(new Thread() {
			@Override
			public void run() {
				try {
					if (url.toString().endsWith(".txt")) {
						HelpDialog.this.editorpane.setBackground(Color.white);
						HelpDialog.this.editorpane.setForeground(Color.black);
					} else {
						HelpDialog.this.editorpane.setBackground(null);
						HelpDialog.this.editorpane.setForeground(null);

					}

					HelpDialog.this.editorpane.setPage(url);
				} catch (final IOException e) {
					JOptionPane.showMessageDialog(HelpDialog.this.frame,
							"The requested Page\n"
									+ new File(url.getFile()).getName()
									+ "\ncannot be displayed!\n\nReason:\n"
									+ e.toString(), "Error",
							JOptionPane.WARNING_MESSAGE);
				}
			}
		});

	}

	private void jumpToPage(final URL url) {

		((JComponent) this.splitpane.getRightComponent())
				.setBorder(BorderFactory.createTitledBorder(new File(url
						.getFile()).getName()));

		SwingUtilities.invokeLater(new Thread() {
			@Override
			public void run() {
				try {
					if (url.toString().endsWith(".txt")) {
						HelpDialog.this.editorpane.setBackground(Color.white);
						HelpDialog.this.editorpane.setForeground(Color.black);
					} else {
						HelpDialog.this.editorpane.setBackground(null);
						HelpDialog.this.editorpane.setForeground(null);

					}
					HelpDialog.this.editorpane.setPage(url);
				} catch (final IOException e) {
					JOptionPane.showMessageDialog(HelpDialog.this.frame,
							"The requested Page\n"
									+ new File(url.getFile()).getName()
									+ "\ncannot be displayed!\n\nReason:\n"
									+ e.toString(), "Error",
							JOptionPane.WARNING_MESSAGE);
				}
			}
		});

	}

	/**
	 * Fills a node in the tree by its referring node in the DomTree. This
	 * function work recursive, so once started, most likely with the rootnode
	 * of both trees, this method calls itself all over again until the tree is
	 * finished.
	 * 
	 * @param node
	 *            - the recent Node from the DomTree
	 * @param currentNode
	 *            - the recent Node in the JTree
	 */
	private void fillTree(final Node node,
			final DefaultMutableTreeNode currentNode) {

		if (node.getNodeType() == Node.ELEMENT_NODE) {

			String title = null;
			String filepath = null;

			final NamedNodeMap attrs = node.getAttributes();
			if (attrs != null) {

				/*
				 * determines the title for the new tree node, and its
				 * additional filename (if one exists)
				 */
				if (attrs.getNamedItem("name") != null) {
					title = attrs.getNamedItem("name").getNodeValue();

					if (attrs.getNamedItem("file") != null) {
						filepath = attrs.getNamedItem("file").getNodeValue();

					}
				}

				if (attrs.getNamedItem("caller") != null
						&& attrs.getNamedItem("file") != null) {
					this.quickhelp.put(attrs.getNamedItem("caller")
							.getNodeValue(), attrs.getNamedItem("file")
							.getNodeValue());

				}
				final DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(
						title);

				// if at least a title is available, the node get added
				if (title != null)
					currentNode.add(newNode);

				/*
				 * and if also a filepath exists, it will put into the
				 * references by its path in the tree
				 */
				if (title != null && filepath != null) {
					String pathstring = "";

					for (int i = 0; i < newNode.getPath().length; i++) {

						pathstring += newNode.getPath()[i].toString();

						if (i < newNode.getPath().length - 1)
							pathstring += ", ";
					}

					this.references.put(pathstring, filepath);
					this.orderedlist.addElement(filepath);

				}

				// if the current node has children, thread will proceed to them
				// first
				final NodeList children = node.getChildNodes();

				if (children != null) {

					for (int i = 0; i < children.getLength(); i++) {

						this.fillTree(children.item(i), newNode);

					}
				}

			}

		}
	}

	/**
	 * Called if an Event gets fired from the JTree. Opens the selection then
	 * 
	 * @param arg0
	 */
	public void treeClicked(final TreeSelectionEvent arg0) {

		// Checking if event is valid for our purpose
		if (this.tree.getSelectionRows() != null
				&& this.splitpane.getLeftComponent() != null) {

			// enables the buttons, in order to be able to browse the tree with
			// them
			this.btn1.setEnabled(true);
			this.btn2.setEnabled(true);

			// setting the internal position to the selection
			this.position = this.tree.getSelectionRows()[0];

			// creating a key to access the corresponding filepath in references
			// Hashtable
			String key = arg0.getNewLeadSelectionPath().toString();

			key = key.substring(1, key.length() - 1);

			// retrieving the filepath
			final String filepath = this.references.get(key);

			// if a filepath exists, open the file; else do nothing
			if (filepath != null)
				this.jumpToPage(filepath);

			this.history.clear();
			this.btn4.setVisible(false);

			final URL url;
			try {
				url = new URL("file:/"
						+ this.addLinuxSeparator
						+ new File(this.home + "cm2help").getAbsoluteFile()
								.getParentFile().getAbsolutePath()
						+ File.separator + filepath);
				// System.out.println(url);
			} catch (final MalformedURLException e1) {
				e1.printStackTrace();
				return;
			}
			this.history.addElement(url);

		}

	}

	/**
	 * Invoked if a Hyperlink is clicked on the editorPane.
	 * 
	 * @param arg0
	 */
	public void hyperlinkClicked(final HyperlinkEvent arg0) {

		if (arg0.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {

			/*
			 * tries to display the url retrieved from the clicked link in the
			 * OS browser
			 */

			if (arg0.getDescription().startsWith("http")) {
				Cm2_Toolkit.openWebPageOnSystemBrowser(arg0.getDescription());
				return;
			}
			/*
			 * tries to display the url retrieved from the clicked link sets the
			 * bordername to the name of the file then
			 */
			try {

				this.history.addElement(this.editorpane.getPage());
				this.editorpane.setPage(arg0.getURL());

				((JComponent) this.splitpane.getRightComponent())
						.setBorder(BorderFactory.createTitledBorder(new File(
								arg0.getURL().getFile()).getName()));

				this.btn4.setVisible(true);
				// opens a message if page cannot be displayed
			} catch (final Exception e) {
				JOptionPane.showMessageDialog(this.frame,
						"The requested Page\n" + arg0.getURL()
								+ "\ncannot be displayed!\n\nReason:\n"
								+ e.toString(), "Error",
						JOptionPane.WARNING_MESSAGE);
			}
		}
	}

	/**
	 * Called at startup Opens the Help if the programm is started for the first
	 * time
	 * 
	 */
	public static void popUpHelpIfItsApplicationsFirstRun() {
		try {
			ConfigParser.getValue("Show_Help_At_First_Run");

		} catch (final Exception e) {
			new HelpDialog().popupHelp();
			ConfigParser.deleteAllAt("Show_Help_At_First_Run");
			ConfigParser.addValue("Show_Help_At_First_Run", "Done");
		}

	}

	public void popupHelp(final Object parent) {
		int pos = 0;

		if (this.quickhelp.get(parent.getClass().getSimpleName()) != null) {
			final String s = this.quickhelp.get(parent.getClass()
					.getSimpleName());

			for (int i = 0; i < this.orderedlist.size(); i++) {
				if (this.orderedlist.get(i).equalsIgnoreCase(s)) {
					pos = i;
					break;
				}

			}
			this.popupHelp(pos);

		}

	}

}
