/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.JTextComponent;
import javax.swing.text.PlainDocument;

/**
 * A document that limits the text length of a textfield to a fixed size. Also
 * upper cases all characters instantly
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class LimitedDocument extends PlainDocument {

	private static final long serialVersionUID = 1L;

	private final int limit;

	private final JTextComponent c;
	
	

	/**
	 * Constructs a new document with the specified limit of characters
	 * 
	 * @param c
	 * @param limit
	 */
	public LimitedDocument(final JTextComponent c, final int limit) {
		this.c = c;
		this.limit = limit;
	}

	@Override
	public void insertString(final int offset, String str, final AttributeSet a)
			throws BadLocationException {
		str = str.toUpperCase();
		if ((this.c.getText() + str).length() > this.limit)
			return;
		else
			super.insertString(offset, str, a);
	}
	
	

}
