/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.gui.HelpBtn;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * The dialog that lets the user enter his text for the title section of the
 * membrane
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
@SuppressWarnings("serial")
public class ManageTitleSectionDialog extends JDialog implements ActionListener {

	private JButton cancelBtn, okBtn;

	private JPanel buttonbar;

	private JPanel header1;

	private JTextField tf1, tf2, tf3;

	private final MembraneModel model;

	private JLabel h1;

	private JPanel ground1, ground2, ground3, ground4;

	private JPanel ground;

	private JButton dateBtn;

	private JTabbedPane tab;

	private ImageIcon warningIcon;

	private final int width = 710;

	private EditPane title;

	private EditPane source;

	private EditPane keywds;

	private EditPane expdta;

	private EditPane author;

	private EditPane jrnl;

	private EditPane remark;

	public ManageTitleSectionDialog() {
		this.model = DataPool.getMembraneModel();

	}

	/**
	 * Constructs the Gui and shows the Dialog. Blocks until user cancels or
	 * accepts
	 * 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public void popUp() {

		this.setTitle("Manage Pdb Title Section");

		this.warningIcon = Cm2_IconDepot.getSpecifiedIcon("Warning");

		this.tf1 = new JTextField();

		this.tf2 = new JTextField();

		this.tf3 = new JTextField();

		this.buttonbar = new JPanel();

		this.header1 = new JPanel();
		this.header1.setLayout(new BoxLayout(this.header1, BoxLayout.X_AXIS));

		this.h1 = new JLabel(" Manage the title section of the membrane",
				JLabel.LEADING);

		this.header1.add(this.h1);
		this.header1.add(Box.createHorizontalGlue());
		this.header1.add(Box.createVerticalStrut(40));

		// header1.setBackground(Color.white);

		this.dateBtn = new JButton("Today");

		this.cancelBtn = new JButton("Discard");

		this.okBtn = new JButton("Apply Changes");

		this.tab = new JTabbedPane();

		this.ground = new JPanel();
		this.ground1 = new JPanel();
		this.ground2 = new JPanel();
		this.ground3 = new JPanel();
		this.ground4 = new JPanel();

		this.ground1.setBorder(BorderFactory
				.createTitledBorder("Header  (max. 40 characters)"));
		this.ground2.setBorder(BorderFactory
				.createTitledBorder("Code   (max. 4 characters)"));

		String possiblemonth = "";
		for (int i = 0; i < Cm2_Toolkit.MONTHS.length; i++) {
			possiblemonth += Cm2_Toolkit.MONTHS[i];
			if (i < Cm2_Toolkit.MONTHS.length - 1)
				possiblemonth += ",";
		}
		this.ground3
				.setBorder(BorderFactory
						.createTitledBorder("Redefine Date   (DD-MMM-YY). Possible month values: {"
								+ possiblemonth + "}"));

		this.ground1.add(this.tf1);

		this.ground2.add(this.tf2);
		this.ground3.add(this.tf3);
		this.ground3.add(this.dateBtn);

		this.ground.add(this.ground1);
		this.ground.add(this.ground2);
		this.ground.add(this.ground3);
		this.ground.add(this.ground4);

		this.setModal(true);

		this.getContentPane().setLayout(
				new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));

		this.ground.setLayout(new BoxLayout(this.ground, BoxLayout.Y_AXIS));

		this.getContentPane().add(this.header1);

		this.getContentPane().add(this.tab);
		this.getContentPane().add(this.buttonbar);

		this.buttonbar
				.setLayout(new BoxLayout(this.buttonbar, BoxLayout.X_AXIS));
		this.buttonbar.add(Box.createHorizontalGlue());
		this.buttonbar.add(this.okBtn);
		this.buttonbar.add(Box.createHorizontalStrut(10));
		this.buttonbar.add(this.cancelBtn);
		this.buttonbar.add(Box.createHorizontalGlue());
		this.buttonbar.add(new HelpBtn(this));
		this.buttonbar.add(Box.createHorizontalStrut(5));

		this.dateBtn.addActionListener(this);
		this.cancelBtn.addActionListener(this);
		this.okBtn.addActionListener(this);

		this.h1.setFont(new Font("Serif", Font.PLAIN, 18));

		this.tf1
				.setFont(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.FONT_PDB_TEXTFIELDS);
		this.tf2
				.setFont(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.FONT_PDB_TEXTFIELDS);
		this.tf3
				.setFont(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.FONT_PDB_TEXTFIELDS);

		this.tf1.setPreferredSize(new Dimension(333, this.tf1
				.getPreferredSize().height));
		this.tf2.setPreferredSize(new Dimension(45,
				this.tf2.getPreferredSize().height));
		this.tf3.setPreferredSize(new Dimension(85,
				this.tf3.getPreferredSize().height));

		this.tf1.setDocument(new LimitedDocument(this.tf1, 40));
		this.tf2.setDocument(new LimitedDocument(this.tf2, 4));
		this.tf3.setDocument(new LimitedDocument(this.tf3, 9));

		// make sure this window is always on top(inside the application)
		this.addWindowFocusListener(new WindowFocusListener() {

			public void windowGainedFocus(final WindowEvent e) {

			}

			public void windowLostFocus(final WindowEvent e) {

				ManageTitleSectionDialog.this.requestFocus();

			}

		});

		this.tab.addTab("First Row", this.ground);
		if (this.model.getHeader() != null)
			this.tf1.setText(this.model.getHeader());
		if (this.model.getCode() != null)
			this.tf2.setText(this.model.getCode());
		if (this.model.getDate() != null)
			this.tf3.setText(this.model.getDate());

		this.title = new EditPane("TITLE", "TITLE ", 1);
		this.tab.addTab(this.title.tabname, this.title);
		if (this.model.getTitle() != null)
			this.title.setContents(this.model.getTitle());

		this.source = new EditPane("SOURCE", "SOURCE", 2);
		this.tab.addTab(this.source.tabname, this.source);
		if (this.model.getSOURCE() != null)
			this.source.setContents(this.model.getSOURCE());

		this.keywds = new EditPane("KEYWDS", "KEYWDS", 3);
		this.tab.addTab(this.keywds.tabname, this.keywds);
		if (this.model.getKEYWDS() != null)
			this.keywds.setContents(this.model.getKEYWDS());

		this.expdta = new EditPane("EXPDTA", "EXPDTA", 4);
		this.tab.addTab(this.expdta.tabname, this.expdta);
		if (this.model.getEXPDTA() != null)
			this.expdta.setContents(this.model.getEXPDTA());

		this.author = new EditPane("AUTHOR", "AUTHOR", 5);
		this.tab.addTab(this.author.tabname, this.author);
		if (this.model.getAuthor() != null)
			this.author.setContents(this.model.getAuthor());

		this.jrnl = new EditPane("JRNL", "JRNL  ", 6);
		this.tab.addTab(this.jrnl.tabname, this.jrnl);
		if (this.model.getJRNL() != null)
			this.jrnl.setContents(this.model.getJRNL());

		this.remark = new EditPane("REMARK", "REMARK", 7);
		this.tab.addTab(this.remark.tabname, this.remark);
		if (this.model.getRemark() != null)
			this.remark.setContents(this.model.getRemark());

		final JPanel p1 = new JPanel();
		this.ground1.add(p1);
		p1.setPreferredSize(new Dimension(this.width
				- this.tf1.getPreferredSize().width - 20, this.tf1
				.getPreferredSize().height));

		final JPanel p2 = new JPanel();
		this.ground2.add(p2);
		p2.setPreferredSize(new Dimension(this.width
				- this.tf2.getPreferredSize().width - 20, this.tf2
				.getPreferredSize().height));

		final JPanel p3 = new JPanel();
		this.ground3.add(p3);
		p3.setPreferredSize(new Dimension(this.width
				- this.tf3.getPreferredSize().width - 20
				- this.dateBtn.getPreferredSize().width, this.tf3
				.getPreferredSize().height));

		this.ground4.setPreferredSize(new Dimension(this.width, 300));

		this.pack();

		final Dimension dim = this.getToolkit().getScreenSize();
		this.setLocation(dim.width / 2 - this.getWidth() / 2, dim.height / 2
				- this.getHeight() / 2);

		this.setResizable(false);
		this.setVisible(true);

	}

	@SuppressWarnings("unchecked")
	public void actionPerformed(final ActionEvent arg0) {

		if (arg0.getSource() == this.okBtn) {

			final String date = this.tf3.getText().toUpperCase().trim();
			String datereport = "";
			try {
				final String[] strings = date.split("-");
				if (Integer.parseInt(strings[0]) > 31
						|| Integer.parseInt(strings[0]) < 01
						|| Integer.parseInt(strings[2]) > 31
						|| Integer.parseInt(strings[2]) < 01)
					datereport = "Invalid number values";
				if (date.length() != 9)
					datereport = "Invalid date format";
				if (strings[1].length() != 3)
					datereport = "Invalid month";

				boolean in = false;
				for (int i = 0; i < Cm2_Toolkit.MONTHS.length; i++) {
					if (strings[1].equals(Cm2_Toolkit.MONTHS[i])) {
						in = true;
						break;
					}
				}
				if (in == false)
					datereport = "Invalid month";
			} catch (final Exception e) {

				datereport = "Invalid date format";

			}

			if (datereport.length() > 0) {
				JOptionPane.showMessageDialog(this, datereport, "Invalid Date",
						JOptionPane.ERROR_MESSAGE);
				return;
			}

			this.model.setHeader(this.tf1.getText());
			this.model.setCode(this.tf2.getText());
			this.model.setDate(date);

			this.model.setTitle(this.title.getContents());
			this.model.setSOURCE(this.source.getContents());

			this.model.setKeywds(this.keywds.getContents());
			this.model.setExpdta(this.expdta.getContents());
			this.model.setAuthor(this.author.getContents());
			this.model.setJrnl(this.jrnl.getContents());
			this.model.setRemark(this.remark.getContents());

			this.dispose();
		}

		if (arg0.getSource() == this.cancelBtn) {
			this.dispose();
		}

		if (arg0.getSource() == this.dateBtn) {
			this.tf3.setText(Cm2_Toolkit.createCurrentDate());
		}

	}

	/**
	 * Contains a textarea, a fuler and a combobox for predefined tokens, if
	 * available Used for editing complex sections in the title area
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class EditPane extends JPanel implements CaretListener,
			ActionListener, AdjustmentListener {

		int[] colpos = new int[9];

		String tabname;

		String tag;

		private final String[] suggests;

		private final JTextArea tagpane = new JTextArea("      ");

		private final JPanel textpanel = new JPanel();

		private final JPanel top = new JPanel() {
			@Override
			public void paintComponent(final Graphics g) {
				super.paintComponent(g);
				g.setFont(g.getFont().deriveFont(10f));
				final int h = this.getHeight();

				if (EditPane.this.invalidnumberformat) {
					g.drawImage(ManageTitleSectionDialog.this.warningIcon
							.getImage(), 60 - EditPane.this.textpanel
							.getVisibleRect().x, h - 15, null);
					g.drawString("Invalid number format!",
							22 - EditPane.this.textpanel.getVisibleRect().x,
							h - 21);
				} else if (EditPane.this.invalidnumberposition) {
					g.drawImage(ManageTitleSectionDialog.this.warningIcon
							.getImage(), 60 - EditPane.this.textpanel
							.getVisibleRect().x, h - 15, null);
					g.drawString("Number position not well formed!",
							2 - EditPane.this.textpanel.getVisibleRect().x,
							h - 34);
					g.drawString("Right-justified alignment preferred!",
							2 - EditPane.this.textpanel.getVisibleRect().x,
							h - 21);
				}

				if (EditPane.this.eightyplus) {

					g
							.drawImage(
									ManageTitleSectionDialog.this.warningIcon
											.getImage(),
									30
											+ EditPane.this.colpos[EditPane.this.colpos.length - 1]
											- EditPane.this.textpanel
													.getVisibleRect().x,
									h - 15, null);
					g
							.drawString(
									"Text out of range!",
									-3
											+ EditPane.this.colpos[EditPane.this.colpos.length - 1]
											- EditPane.this.textpanel
													.getVisibleRect().x, h - 21);

				}
			}
		};

		private final JPanel ruler;

		private final JPanel bottom = new JPanel();

		private final JScrollPane scroll;

		private JComboBox suggestionbox;

		boolean eightyplus = false;

		boolean invalidnumberformat = false;

		boolean invalidnumberposition = false;

		private final int texth = 400;

		private final int rulerh = 10;

		int tabindex;

		String[] cols = { "11-20", "21-30", "31-40", "41-50", "51-60", "61-70",
				"71-80", "80+" };

		JTextArea tp = new JTextArea() {
			@Override
			public void paintComponent(final Graphics g) {
				super.paintComponent(g);
				final int h = this.getHeight();

				for (int i = 0; i < EditPane.this.colpos.length; i++) {
					// if (i == 0)
					// g.setColor(null);
					// else
					// g.setColor(Color.LIGHT_GRAY);
					this.paintLine(g, EditPane.this.colpos[i]
							- EditPane.this.colpos[0], h);
				}

			}

			private void paintLine(final Graphics g, final int x, final int h) {

				for (int i = 0; i < h; i += 5) {
					g.drawLine(x, i, x, i + 1);
				}
			}
		};

		private final MyDocument document = new MyDocument(this.tp);

		public EditPane(final String tabname, final String tag,
				final int tabindex) {

			this.ruler = new JPanel() {

				public void paintComponent(final Graphics g) {
					super.paintComponent(g);

					final int h = this.getHeight();

					for (int i = 0; i < EditPane.this.colpos.length; i++) {
						g.drawLine((EditPane.this.colpos[i] + 1)
								- EditPane.this.textpanel.getVisibleRect().x,
								h - 6, (EditPane.this.colpos[i] + 1)
										- EditPane.this.textpanel
												.getVisibleRect().x, h);

					}

					g.setFont(g.getFont().deriveFont(9f));

					g.drawString("1-6", 17 - EditPane.this.textpanel
							.getVisibleRect().x, h - 3);
					g.drawString("7-10", 57 - EditPane.this.textpanel
							.getVisibleRect().x, h - 3);

					for (int i = 0; i < EditPane.this.colpos.length; i++) {
						if (i < 1)
							continue;
						// if (i == colpos.length - 1)
						// g.setColor(Color.red.darker());
						// else
						// g.setColor(Color.DARK_GRAY);
						final int pos = EditPane.this.colpos[i]
								+ EditPane.this.tagpane.getWidth()
								- EditPane.this.textpanel.getVisibleRect().x;

						g
								.drawString(EditPane.this.cols[i - 1],
										pos - 18, h - 3);

					}

				}
			};

			this.tabname = tabname;
			this.tag = tag;
			this.tabindex = tabindex;
			this.suggests = ManageTitleSectionDialog.this
					.createSuggests(tabname);
			this.tp.setDocument(this.document);
			this.scroll = new JScrollPane(this.textpanel);
			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

			this.textpanel.setLayout(new BoxLayout(this.textpanel,
					BoxLayout.X_AXIS));
			this.bottom.setLayout(new BoxLayout(this.bottom, BoxLayout.X_AXIS));
			this.add(this.top);
			this.add(this.ruler);
			this.add(this.scroll);
			this.add(this.bottom);

			this.ruler.setBorder(BorderFactory.createEtchedBorder());

			this.top.setPreferredSize(new Dimension(
					ManageTitleSectionDialog.this.width, 45));
			this.ruler.setPreferredSize(new Dimension(
					ManageTitleSectionDialog.this.width, this.rulerh));
			this.scroll.setPreferredSize(new Dimension(
					ManageTitleSectionDialog.this.width, this.texth));
			this.bottom.setPreferredSize(new Dimension(
					ManageTitleSectionDialog.this.width, 30));

			this.tagpane.setMaximumSize(new Dimension(72, this.texth));
			this.tagpane.setMinimumSize(new Dimension(72, this.texth));

			this.textpanel.add(this.tagpane);
			this.textpanel.add(this.tp);

			this.tagpane
					.setFont(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.FONT_PDB_TEXTFIELDS);
			this.tp
					.setFont(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.FONT_PDB_TEXTFIELDS);

			this.tagpane.setEditable(false);
			// tagpane.setEnabled(false);
			// tagpane.setForeground(Color.DARK_GRAY);

			this.tp.addCaretListener(this);
			this.scroll.getHorizontalScrollBar().addAdjustmentListener(this);
			this.scroll.getVerticalScrollBar().addAdjustmentListener(this);

			final JLabel l = new JLabel();
			l
					.setFont(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.FONT_PDB_TEXTFIELDS);
			for (int i = 0; i < this.colpos.length; i++) {

				if (i == 0) {
					l.setText(l.getText() + "OOOOOO");
				} else if (i == 1) {
					l.setText(l.getText() + "OOOO");
				}

				else
					l.setText(l.getText() + "OOOOOOOOOO");

				this.colpos[i] = l.getPreferredSize().width;
			}

			if (this.suggests != null) {
				final DefaultComboBoxModel cm = new DefaultComboBoxModel();
				this.suggestionbox = new JComboBox(cm);
				cm.addElement("Insert token");
				this.suggestionbox.addActionListener(this);
				this.suggestionbox
						.setFont(CM2_CONSTANTS.GUI_SETTINGS_AND_CONTROLS.FONT_PDB_TEXTFIELDS);
				this.bottom.add(this.suggestionbox);

				for (int i = 0; i < this.suggests.length; i++) {
					cm.addElement(this.suggests[i]);
				}

				final JPanel p = new JPanel();

				p.setPreferredSize(new Dimension(
						ManageTitleSectionDialog.this.width
								- this.suggestionbox.getPreferredSize().width,
						this.bottom.getPreferredSize().height));
				this.suggestionbox.setMaximumSize(new Dimension(
						this.suggestionbox.getPreferredSize().width, 20));
				this.suggestionbox.setMinimumSize(new Dimension(
						this.suggestionbox.getPreferredSize().width, 20));
				this.bottom.add(p);

			}

		}

		public void setContents(final String s) {
			if (s.length() < 1)
				this.tp.setText("    ");
			else
				this.tp.setText(s);

		}

		public String getContents() {
			if (this.tp.getText().trim().length() < 1)
				return null;
			else
				return this.tp.getText();
		}

		private void updateWarnings() {
			if (this.eightyplus || this.invalidnumberformat
					|| this.invalidnumberposition) {
				ManageTitleSectionDialog.this.tab.setIconAt(this.tabindex,
						ManageTitleSectionDialog.this.warningIcon);

			} else {
				ManageTitleSectionDialog.this.tab
						.setIconAt(this.tabindex, null);

			}

			this.top.repaint();

		}

		public void caretUpdate(final CaretEvent arg0) {

			if (this.tp.getText() == null || this.tp.getText().length() < 1) {
				this.tagpane.setText("      ");
				return;
			}

			String tags = "";
			final int linecount = this.tp.getLineCount();
			for (int i = 0; i < linecount; i++) {
				tags += this.tag;
				if (i < linecount - 1)
					tags += "\n";
			}
			this.tagpane.setText(tags);

			this.eightyplus = false;
			this.invalidnumberformat = false;
			this.invalidnumberposition = false;
			final String[] all = this.tp.getText().split("\n");

			if (all.length < 1)
				return;

			for (int i = 0; i < all.length; i++) {
				if (all[i].length() > 74)
					this.eightyplus = true;

				final int m = 4;
				final int l = all[i].length();
				final String sub = all[i].substring(0, Math.min(m, l));
				final String s = sub.trim();
				if (s.length() > 0) {
					try {
						Integer.parseInt(s);

					} catch (final Exception e) {
						this.invalidnumberformat = true;

					}
					if (this.invalidnumberformat == false) {
						try {

							new Integer(sub.substring(3, 4));

						} catch (final Exception e) {
							this.invalidnumberposition = true;
							this.invalidnumberformat = false;

						}
					}
				}
			}

			this.updateWarnings();

		}

		public void actionPerformed(final ActionEvent arg0) {
			if (this.suggestionbox.getSelectedIndex() == 0)
				return;
			final String s = (String) ((DefaultComboBoxModel) this.suggestionbox
					.getModel()).getSelectedItem();
			this.suggestionbox.setSelectedIndex(0);

			if (this.tabname.equals("SOURCE"))
				this.addToken(s, true, true);
			else if (this.tabname.equals("JRNL"))
				this.addToken(Cm2_Toolkit.extendRight(s, 7), false, false);
			else if (this.tabname.equals("EXPDTA"))
				this.addToken(s, false, false);
		}

		private void addToken(String token, final boolean setnumber,
				final boolean appendquote) {
			final int cp = this.tp.getCaretPosition();

			int index = 1;
			if (this.tabname.equals("REMARK"))
				index = 0;

			final String oldtext = this.tp.getText();
			if (oldtext != null) {

				final String[] strings = oldtext.split("\n");

				for (int i = 0; i < strings.length; i++) {
					try {
						final String pre = strings[i].substring(0, 4).trim();
						index = Math.max(Integer.parseInt(pre), index);
					} catch (final Exception e) {

					}
				}

			}

			String toset = "";
			if (oldtext.trim().length() > 0 && setnumber) {

				if (index < 9999)
					toset = "\n" + Cm2_Toolkit.extendLeft("" + (index + 1), 4)
							+ " ";
				else
					toset = "\n" + Cm2_Toolkit.extendLeft("", 4) + " ";
			} else if (oldtext.trim().length() > 0) {
				toset = "\n" + "    ";
			} else
				toset = "" + "    ";

			if (appendquote)
				token += ": ";

			toset += token;

			int in = cp;
			while (in < oldtext.length()) {

				if (oldtext.charAt(in) == '\n')
					break;
				in++;
			}

			String fin = oldtext.substring(0, in) + toset
					+ oldtext.substring(in);

			if (setnumber) {
				final String[] strings = fin.split("\n");

				int newindex = 1;
				if (this.tabname.equals("REMARK"))
					newindex = 0;

				fin = "";
				int previous = -1;
				for (int i = 0; i < strings.length; i++) {
					try {
						final String pre = strings[i].substring(0, 4).trim();
						final int n = Integer.parseInt(pre);

						if (n != previous) {
							newindex++;
							previous = n;
						} else
							newindex = n;
						if (newindex < 9999)
							strings[i] = Cm2_Toolkit.extendLeft(
									"" + (newindex), 4)
									+ strings[i].substring(4);
						else
							strings[i] = Cm2_Toolkit.extendLeft("", 4)
									+ strings[i].substring(4);

					} catch (final Exception e) {

					}

					if (i < strings.length - 1)
						fin += strings[i] + "\n";
					else
						fin += strings[i];
				}
			}

			this.tp.setText(fin);

			this.caretUpdate(null);

			this.tp.scrollRectToVisible(new Rectangle(-72, this.tp
					.getVisibleRect().y, 1, 1));

			this.tp.requestFocus();

			this.tp.setCaretPosition(in + toset.length());

		}

		public void adjustmentValueChanged(final AdjustmentEvent e) {
			this.ruler.repaint();
			this.top.repaint();
			if (this.tp.getPreferredSize().height < this.texth)
				return;
			this.tagpane.setMaximumSize(new Dimension(72, this.tp
					.getPreferredSize().height));
			this.tagpane.setMinimumSize(new Dimension(72, this.tp
					.getPreferredSize().height));
		}

	}

	/**
	 * The document for the textareas in the edit panes.
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	class MyDocument extends PlainDocument {

		private final JTextArea TA;

		public MyDocument(final JTextArea tp) {
			this.TA = tp;
		}

		public void insertString(final int offset, String str,
				final AttributeSet a) throws BadLocationException {
			final int cp = this.TA.getCaretPosition();
			final String text = this.TA.getText();
			int pos = 0;
			for (int i = 0; i < text.length(); i++) {
				if (i >= cp)
					break;
				if (text.charAt(i) == '\n')
					pos++;

			}
			if (str.equalsIgnoreCase("\n")) {
				try {
					final String[] strings = text.split("\n");
					final String re = strings[pos].substring(0, 4).trim();
					str += Cm2_Toolkit.extendLeft("" + Integer.parseInt(re), 4)
							+ " ";
				} catch (final Exception e) {
					str += "    ";
				}
				this.TA.scrollRectToVisible(new Rectangle(-72, this.TA
						.getVisibleRect().y, 1, 1));
			}
			str = str.toUpperCase();
			str = str.replaceAll("\t", " ");
			super.insertString(offset, str, a);
		}

	}

	/**
	 * Creates a list of predefines tokens for the suggestionbox
	 * 
	 * @param tabname
	 * @return
	 */
	private String[] createSuggests(final String tabname) {

		String[] s = null;

		if (tabname.equals("SOURCE"))
			s = new String[] { "MOL_ID", "ORGANISM_SCIENTIFIC",
					"ORGANISM_COMMON", "STRAIN", "VARIANT", "SYNTHETIC",
					"FRAGMENT", "CELL_LINE", "ORGAN", "TISSUE", "CELL",
					"ORGANELLE", "SECRETION", "CELLULAR_LOCATION",
					"EXPRESSION_SYSTEM", "EXPRESSION_SYSTEM_STRAIN",
					"EXPRESSION_SYSTEM_VARIANT", "EXPRESSION_SYSTEM_CELL_LINE",
					"EXPRESSION_SYSTEM_ORGAN", "EXPRESSION_SYSTEM_TISSUE",
					"EXPRESSION_SYSTEM_CELL", "EXPRESSION_SYSTEM_ORGANELLE",
					"EXPRESSION_SYSTEM_CELLULAR_LOCATION",
					"EXPRESSION_SYSTEM_VECTOR_TYPE",
					"EXPRESSION_SYSTEM_VECTOR", "OTHER_DETAILS" };

		if (tabname.equals("EXPDTA"))
			s = new String[] { "X-RAY DIFFRACTION", "ELECTRON DIFFRACTION",
					"ELECTRON MICROSCOPY", "CRYO-ELECTRON MICROSCOPY",
					"FIBER DIFFRACTION", "FLUORESCENCE TRANSFER",
					"NEUTRON DIFFRACTION", "NMR", "THEORETICAL MODEL",
					"SOLUTION SCATTERING",
					"SOLUTION SCATTERING, THEORETICAL MODEL", };

		if (tabname.equals("JRNL"))
			s = new String[] { "  AUTH", "  TITL", "  EDIT", "  REF", "  PUBL",
					"  REFN" };

		return s;

	}

}
