/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.Engine.MembraneOperations;
import org.cellmicrocosmos.cm2.gui.CompatibilityPanel;
import org.cellmicrocosmos.cm2.gui.HelpBtn;
import org.cellmicrocosmos.cm2.io.Compatibility;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;

/**
 * This dialog is used to change the PDB restriction settings.
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class PdbSettingsWindow extends JFrame implements ActionListener {

	private static final long serialVersionUID = 1L;

	private final MembraneOperations operations;

	JButton ok = new JButton("Ok");

	JButton cancel = new JButton("Cancel");

	NumberingPanel npanel = new NumberingPanel();

	TransformationsPanel tpanel = new TransformationsPanel();

	MyCompatibilityPanel cpanel = new MyCompatibilityPanel();

	/**
	 * Builds up the frame
	 * 
	 * @param membraneOperations
	 */
	public PdbSettingsWindow(final MembraneOperations membraneOperations) {

		this.operations = membraneOperations;

		this.ok.addActionListener(this);
		this.cancel.addActionListener(this);

		final JPanel center = new JPanel();
		final JPanel bar = new JPanel();

		center.setLayout(new BoxLayout(center, BoxLayout.X_AXIS));

		final JPanel rightone = new JPanel();
		rightone.setLayout(new BoxLayout(rightone, BoxLayout.Y_AXIS));
		rightone.setBorder(BorderFactory.createTitledBorder(null,
				"Compatibility", TitledBorder.DEFAULT_JUSTIFICATION,
				TitledBorder.DEFAULT_POSITION, rightone.getFont().deriveFont(
						Font.BOLD, rightone.getFont().getSize2D() + 2f)));
		final JLabel clabel = new JLabel(
				"<html>A list of known pdb-viewers/tools<br>and their compatibilities with the<br>current settings.<br><br>Roll over an entry for details</html>");

		rightone.add(Box.createVerticalStrut(4));
		rightone.add(clabel);

		rightone.add(Box.createVerticalStrut(10));
		rightone.add(this.cpanel);
		rightone.add(Box.createVerticalGlue());

		center.add(this.npanel);
		center.add(this.tpanel);
		center.add(rightone);

		final JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));

		final JLabel hlabel = new JLabel(
				" Options concerning the contents of the output file",
				SwingConstants.LEADING);
		hlabel.setFont(new Font("Serif", Font.PLAIN, 18));
		header.add(hlabel);
		header.add(Box.createHorizontalGlue());
		header.add(Box.createVerticalStrut(40));

		JPanel main = new JPanel();
		main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));
		
		main.add(header, BorderLayout.NORTH);
		main.add(center, BorderLayout.CENTER);
		main.add(bar, BorderLayout.SOUTH);

		final JScrollPane scrollMain = new JScrollPane(main);
		scrollMain.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollMain.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		
		this.getContentPane().add(scrollMain);
		
		bar.setLayout(new BoxLayout(bar, BoxLayout.X_AXIS));
		bar.add(Box.createHorizontalGlue());
		bar.add(this.ok);
		bar.add(Box.createHorizontalStrut(10));
		bar.add(this.cancel);
		bar.add(Box.createHorizontalGlue());
		bar.add(new HelpBtn(this));
		bar.add(Box.createHorizontalStrut(5));

		this.setUpButtons();

		this.setTitle("PDB Output Settings");
		this.setIconImage(Cm2_IconDepot.getSpecifiedIcon("Toolbar_PDBSettings")
				.getImage());
		this.pack();
		this.setLocationByPlatform(true);
		this.setResizable(false);
		this.setVisible(true);
		this.ok.requestFocusInWindow();

		this.npanel.check.addActionListener(this.cpanel);
		this.npanel.radio1.addActionListener(this.cpanel);
		this.npanel.radio2.addActionListener(this.cpanel);
		this.npanel.radio3.addActionListener(this.cpanel);
		this.npanel.radio4.addActionListener(this.cpanel);
		this.npanel.radio5.addActionListener(this.cpanel);
		this.npanel.radio6.addActionListener(this.cpanel);
		this.npanel.radio7.addActionListener(this.cpanel);
		this.npanel.radio8.addActionListener(this.cpanel);
		this.npanel.radio9.addActionListener(this.cpanel);

		this.tpanel.check.addActionListener(this.cpanel);
		this.tpanel.check2.addActionListener(this.cpanel);
		this.tpanel.check3.addActionListener(this.cpanel);
		this.tpanel.check4.addActionListener(this.cpanel);
		this.tpanel.check5.addActionListener(this.cpanel);
		this.tpanel.check8.addActionListener(this.cpanel);
		this.tpanel.radio1.addActionListener(this.cpanel);
		this.tpanel.radio2.addActionListener(this.cpanel);
		this.tpanel.radio3.addActionListener(this.cpanel);
		this.tpanel.radio4.addActionListener(this.cpanel);
		this.tpanel.radio5.addActionListener(this.cpanel);
		this.tpanel.radio6.addActionListener(this.cpanel);

		this.cpanel.update();

	}

	public void actionPerformed(final ActionEvent arg0) {

		if (arg0.getSource() == this.ok) {
			
			// check if the DomainID and LayerID match and provide an option to change the setting again
			if (this.tpanel.comboDomainID2ResName.getSelectedIndex() == this.tpanel.comboLayerID2ResName.getSelectedIndex() &&
				this.tpanel.comboLayerID2ResName.getSelectedIndex() > 0) {
				int val = JOptionPane.showOptionDialog(
								null,
								"The position of the DomainID and the LayerID in the ResName record match!\n\n" +
								"Do you really think this makes sense?\n\nI" +
								"It might me better to select only one option and to deactivate the other one or to change one position.\n" +
								"Remember: in case you are using only a single layer and no domains you should deactivate both options!"
								, "DomainID and LayerID match",
								JOptionPane.YES_NO_OPTION,
								JOptionPane.WARNING_MESSAGE, null,
								new String[] { "Yes, I know what I am doing!", "Stop, let me fix this!" }, null);

				if (val != JOptionPane.YES_OPTION)
					return;

			}
			
			if (this.npanel.radio1.isSelected())
				DataPool.LimitationSettings.atomserials = 0;
			else if (this.npanel.radio2.isSelected())
				DataPool.LimitationSettings.atomserials = 1;
			else
				DataPool.LimitationSettings.atomserials = 2;

			if (this.npanel.radio4.isSelected())
				DataPool.LimitationSettings.chains = 0;
			else if (this.npanel.radio5.isSelected())
				DataPool.LimitationSettings.chains = 1;
			else
				DataPool.LimitationSettings.chains = 2;

			DataPool.LimitationSettings.caseSensitiveChainIds = this.npanel.check
					.isSelected();

			DataPool.LimitationSettings.assignResSeqNumbers = this.tpanel.check2
					.isSelected();

			DataPool.LimitationSettings.adoptTitleSections = this.tpanel.check3
					.isSelected();

			DataPool.CoordinateTransformationSettings.sortByType = this.tpanel.check4
					.isSelected();

			DataPool.LimitationSettings.reverseParsing = this.tpanel.check5
					.isSelected();
			
			DataPool.LimitationSettings.exportTopology = this.tpanel.check6
					.isSelected();
			
			DataPool.LimitationSettings.exportAPLVoro = this.tpanel.check7
					.isSelected();
			
			DataPool.PPESettings.saveOPMDummyLayersInPDB = this.tpanel.check8
					.isSelected(); 

			DataPool.LimitationSettings.all99999 = this.npanel.radio7
					.isSelected();

			DataPool.LimitationSettings.restartNumberingWith0 = this.npanel.radio9
					.isSelected();
			
			if (this.tpanel.comboDomainID2ResName.getSelectedIndex() == 1)
				DataPool.LimitationSettings.domainID2ResNameAtPosition = 1;
			else if (this.tpanel.comboDomainID2ResName.getSelectedIndex() == 2)
				DataPool.LimitationSettings.domainID2ResNameAtPosition = 2;
			else if (this.tpanel.comboDomainID2ResName.getSelectedIndex() == 3)
				DataPool.LimitationSettings.domainID2ResNameAtPosition = 3;
			else
				DataPool.LimitationSettings.domainID2ResNameAtPosition = -1;
			
			if (this.tpanel.comboLayerID2ResName.getSelectedIndex() == 1)
				DataPool.LimitationSettings.layerID2ResNameAtPosition = 1;
			else if (this.tpanel.comboLayerID2ResName.getSelectedIndex() == 2)
				DataPool.LimitationSettings.layerID2ResNameAtPosition = 2;
			else if (this.tpanel.comboLayerID2ResName.getSelectedIndex() == 3)
				DataPool.LimitationSettings.layerID2ResNameAtPosition = 3;
			else
				DataPool.LimitationSettings.layerID2ResNameAtPosition = -1;
			
			if (this.tpanel.radio1.isSelected())
				DataPool.CoordinateTransformationSettings.shiftXZ = 0;
			else if (this.tpanel.radio2.isSelected())
				DataPool.CoordinateTransformationSettings.shiftXZ = 1;
			else
				DataPool.CoordinateTransformationSettings.shiftXZ = 2;

			if (this.tpanel.radio4.isSelected())
				DataPool.CoordinateTransformationSettings.shiftY = 0;
			else if (this.tpanel.radio5.isSelected())
				DataPool.CoordinateTransformationSettings.shiftY = 1;
			else
				DataPool.CoordinateTransformationSettings.shiftY = 2;

			DataPool.CoordinateTransformationSettings.swapYZ = this.tpanel.check
					.isSelected();

			if (DataPool.getMembraneModel() != null)
				this.operations.refreshViews(false, true, false, false, false,
						false);

			this.dispose();
		}
		if (arg0.getSource() == this.cancel) {
			this.dispose();
		}

	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
	 * 
	 */
	private class TransformationsPanel extends JPanel {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		JRadioButton radio1 = new JRadioButton("None");
		JRadioButton radio2 = new JRadioButton("To physical center");
		JRadioButton radio3 = new JRadioButton("To center by defined bounds");
		JRadioButton radio4 = new JRadioButton("None");
		JRadioButton radio5 = new JRadioButton("To physical center");
		JRadioButton radio6 = new JRadioButton(
				"To the bottom (only positive y-values)");

		JCheckBox check = new JCheckBox("Swap Y/Z axis");
		JCheckBox check2 = new JCheckBox(
				"Assign sequential resSeq's for each molecule");
		JCheckBox check3 = new JCheckBox(
				"Adopt title sections of other molecule members");
		
		JCheckBox check4 = new JCheckBox(
				"Sort Molecules by Type, not by Membrane Side");
		
		JCheckBox check5 = new JCheckBox(
				"Write additional information for reverse parsing");
		
		JCheckBox check6 = new JCheckBox(
				"Write additional GROMACS topology files");
		
		JCheckBox check7 = new JCheckBox(
				"Write additional APL@Voro config file");
		
		JCheckBox check8 = new JCheckBox(
				"Write the OPM membrane layer dummy markers");

		JComboBox comboDomainID2ResName = new JComboBox(new String[]{	"     Do not write DomainID in ResName (Standard)", 
																		"     Write DomainID in ResName, start at 1st position", 
																		"     Write DomainID in ResName, start at 2nd position", 
																		"     Write DomainID in ResName, start at 3rd position"});
		
		JComboBox comboLayerID2ResName = new JComboBox(new String[]{	"     Do not write LayerID in ResName (Standard)", 
																		"     Write LayerID in ResName, start at 1st position", 
																		"     Write LayerID in ResName, start at 2nd position", 
																		"     Write LayerID in ResName, start at 3rd position"});
		
		JPanel cpPanel = new JPanel();

		JPanel otherPanel = new JPanel();
		
		JPanel ppePanel = new JPanel();

		public TransformationsPanel() {

			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			this.add(this.cpPanel);
			this.add(Box.createVerticalStrut(8));
			this.add(this.otherPanel);
			this.add(Box.createVerticalStrut(8));
			this.add(this.ppePanel);
			this.add(Box.createVerticalGlue());

			this.cpPanel
					.setLayout(new BoxLayout(this.cpPanel, BoxLayout.Y_AXIS));
			this.otherPanel.setLayout(new BoxLayout(this.otherPanel,
					BoxLayout.Y_AXIS));
			this.ppePanel.setLayout(new BoxLayout(this.ppePanel,
					BoxLayout.Y_AXIS));
			final ButtonGroup bg = new ButtonGroup();
			bg.add(this.radio1);
			bg.add(this.radio2);
			bg.add(this.radio3);
			final ButtonGroup bg2 = new ButtonGroup();
			bg2.add(this.radio4);
			bg2.add(this.radio5);
			bg2.add(this.radio6);

			this.cpPanel.setBorder(BorderFactory.createTitledBorder(null,
					"Origin of Coordinates",
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, this.getFont().deriveFont(
							Font.BOLD, this.getFont().getSize2D() + 2f)));
			this.otherPanel.setBorder(BorderFactory.createTitledBorder(null,
					"Other Options", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, this.getFont().deriveFont(
							Font.BOLD, this.getFont().getSize2D() + 2f)));
			this.ppePanel.setBorder(BorderFactory.createTitledBorder(null,
					"Protein Placement Expansion", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, this.getFont().deriveFont(
							Font.BOLD, this.getFont().getSize2D() + 2f)));

			final JPanel subj1 = new JPanel();
			final JPanel subj2 = new JPanel();

			subj1.setLayout(new BoxLayout(subj1, BoxLayout.Y_AXIS));
			subj2.setLayout(new BoxLayout(subj2, BoxLayout.Y_AXIS));

			this.cpPanel.add(subj1);
			this.cpPanel.add(subj2);

			subj1
					.setBorder(BorderFactory
							.createTitledBorder("Planar Shifting"));
			subj2
					.setBorder(BorderFactory
							.createTitledBorder("Height Shifting"));

			final JLabel sl1 = new JLabel(
					"<html>"
							+ "Define the shifting of the center on the XZ plane</html>");

			final JLabel sl2 = new JLabel("<html>"
					+ "Define the shifting of the center on the Y Axis</html>");

			final Color fg = this.getForeground().brighter();
			this.radio1.setForeground(fg);
			this.radio2.setForeground(fg);
			this.radio3.setForeground(fg);
			this.radio4.setForeground(fg);
			this.radio5.setForeground(fg);
			this.radio6.setForeground(fg);
			this.check.setForeground(fg);
			this.check2.setForeground(fg);
			this.check3.setForeground(fg);
			this.check4.setForeground(fg);
			this.check5.setForeground(fg);
			this.check6.setForeground(fg);
			this.check7.setForeground(fg);
			this.check8.setForeground(fg);

			Dimension comboDim = new Dimension(301,20);
			
			this.comboDomainID2ResName.setForeground(fg);
			this.comboDomainID2ResName.setLayout(new BoxLayout(comboDomainID2ResName, BoxLayout.Y_AXIS));
			this.comboDomainID2ResName.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			this.comboDomainID2ResName.setSize(comboDim);
			this.comboDomainID2ResName.setMinimumSize(comboDim);
			this.comboDomainID2ResName.setMaximumSize(comboDim);

			this.comboLayerID2ResName.setForeground(fg);
			this.comboLayerID2ResName.setLayout(new BoxLayout(comboLayerID2ResName, BoxLayout.Y_AXIS));
			this.comboLayerID2ResName.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			this.comboLayerID2ResName.setSize(comboDim);
			this.comboLayerID2ResName.setMinimumSize(comboDim);
			this.comboLayerID2ResName.setMaximumSize(comboDim);
			
			subj1.add(sl1);

			subj1.add(this.radio1);
			subj1.add(this.radio2);
			subj1.add(this.radio3);

			subj2.add(sl2);

			subj2.add(this.radio4);
			subj2.add(this.radio5);
			subj2.add(this.radio6);

			final JPanel subj3 = new JPanel();
			final JPanel subj4 = new JPanel();
			final JPanel subj5 = new JPanel();
			final JPanel subj6 = new JPanel();
			final JPanel subj7 = new JPanel();
			
			final JPanel subj8 = new JPanel();

			subj3.setLayout(new BoxLayout(subj3, BoxLayout.Y_AXIS));
			subj4.setLayout(new BoxLayout(subj4, BoxLayout.Y_AXIS));
			subj5.setLayout(new BoxLayout(subj5, BoxLayout.Y_AXIS));
			subj6.setLayout(new BoxLayout(subj6, BoxLayout.Y_AXIS));
			subj7.setLayout(new BoxLayout(subj7, BoxLayout.Y_AXIS));
			
			subj8.setLayout(new BoxLayout(subj8, BoxLayout.Y_AXIS));

			this.otherPanel.add(Box.createVerticalStrut(5));
			this.otherPanel.add(subj3);
			this.otherPanel.add(Box.createVerticalStrut(5));
			this.otherPanel.add(subj4);
			this.otherPanel.add(Box.createVerticalStrut(5));
			this.otherPanel.add(subj5);
			this.otherPanel.add(Box.createVerticalStrut(5));
			this.otherPanel.add(subj6);
			this.otherPanel.add(Box.createVerticalStrut(5));
			this.otherPanel.add(subj7);
			this.otherPanel.add(Box.createVerticalStrut(5));
			
			this.ppePanel.add(Box.createVerticalStrut(5));
			this.ppePanel.add(subj8);
			this.ppePanel.add(Box.createVerticalStrut(5));

			subj3.add(this.check);
			subj3.add(Box.createHorizontalGlue());

			subj4.add(this.check2);
			subj4.add(Box.createHorizontalGlue());

			subj5.add(this.check3);
			subj5.add(Box.createHorizontalGlue());

			subj6.add(this.check4);
			subj6.add(Box.createHorizontalGlue());

			subj7.add(this.check5);
			subj7.add(Box.createHorizontalGlue());

			subj7.add(this.check6);
			subj7.add(Box.createHorizontalGlue());
			
			subj7.add(this.check7);
			subj7.add(Box.createHorizontalGlue());

			subj7.add(this.comboDomainID2ResName);
			subj7.add(Box.createHorizontalGlue());
			
			subj7.add(this.comboLayerID2ResName);
			subj7.add(Box.createHorizontalGlue());
			
			subj8.add(this.check8);
			subj8.add(Box.createHorizontalGlue());
			
			// check4.addActionListener(new ActionListener() {
			//
			// public void actionPerformed(ActionEvent e) {
			// if(check4.isSelected()) {
			// check3.setSelected(false);
			// check3.setEnabled(false);
			// }else {
			// check3.setEnabled(true);
			// }
			//				
			//					
			// }
			//				
			// });

		}

	}

	/**
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class NumberingPanel extends JPanel {

		private static final long serialVersionUID = 1L;

		JRadioButton radio1 = new JRadioButton();

		JRadioButton radio2 = new JRadioButton();

		JRadioButton radio3 = new JRadioButton();

		JRadioButton radio4 = new JRadioButton();

		JRadioButton radio5 = new JRadioButton();

		JRadioButton radio6 = new JRadioButton();

		JCheckBox check = new JCheckBox();

		JRadioButton radio7 = new JRadioButton();

		JRadioButton radio8 = new JRadioButton();
		
		JRadioButton radio9 = new JRadioButton();

		JPanel restrictionsPanel = new JPanel();

		public NumberingPanel() {
			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			this.add(this.restrictionsPanel);

			this.restrictionsPanel.setLayout(new BoxLayout(
					this.restrictionsPanel, BoxLayout.Y_AXIS));

			final ButtonGroup bg = new ButtonGroup();
			bg.add(this.radio1);
			bg.add(this.radio2);
			bg.add(this.radio3);
			final ButtonGroup bg2 = new ButtonGroup();
			bg2.add(this.radio4);
			bg2.add(this.radio5);
			bg2.add(this.radio6);

			final ButtonGroup bg3 = new ButtonGroup();
			bg3.add(this.radio7);
			bg3.add(this.radio8);
			bg3.add(this.radio9);

			final ActionListener al = new ActionListener() {

				public void actionPerformed(final ActionEvent arg0) {
					if (NumberingPanel.this.radio3.isSelected()) {

						NumberingPanel.this.radio8.setEnabled(true);
						NumberingPanel.this.radio9.setEnabled(true);
					} else {
						NumberingPanel.this.radio7.setSelected(true);

						NumberingPanel.this.radio8.setEnabled(false);
						NumberingPanel.this.radio9.setEnabled(false);
					}

				}

			};
			this.radio1.addActionListener(al);
			this.radio2.addActionListener(al);
			this.radio3.addActionListener(al);
			this.radio4.addActionListener(al);
			this.radio5.addActionListener(al);
			this.radio6.addActionListener(al);

			this.restrictionsPanel.setBorder(BorderFactory.createTitledBorder(
					null, "Manage Restrictions",
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, this.getFont().deriveFont(
							Font.BOLD, this.getFont().getSize2D() + 2f)));

			final JPanel subj1 = new JPanel();
			final JPanel subj2 = new JPanel();

			subj1.setLayout(new BoxLayout(subj1, BoxLayout.Y_AXIS));
			subj2.setLayout(new BoxLayout(subj2, BoxLayout.Y_AXIS));

			this.restrictionsPanel.add(subj1);
			this.restrictionsPanel.add(Box.createVerticalStrut(2));
			this.restrictionsPanel.add(subj2);

			subj1.setBorder(BorderFactory
					.createTitledBorder("Atom serial numbers"));
			subj2.setBorder(BorderFactory.createTitledBorder("Chain ID's"));

			final JLabel sl1 = new JLabel(
					"<html>"
							+ "In a pdb file, every atom entry must have a unique serial number. Those are required<br>"
							+ "in order to associate the atom to other instances in the CONECT Section. Serials are<br>"
							+ "located between column 7 and 11, what reduces their maximum value to 99999. This<br>"
							+ "number will easily be exceeded in this application.<br>"
							+ "If you allow to write more than 99998 atom entries into the file, those exceeding<br>"
							+ "the limit will all have the serial number 99999, what makes it impossible to implement<br>"
							+ "any CONECTS in the file, plus some readers might be incompatible with this case.<br>"
							+ "Choose what atom entries should have consecutive serials and therefore what limits<br>"
							+ "you will have when building your membrane</html>");

			this.radio1
					.setText("Serials for both PROTEIN and LIPID atoms. Complete CONECTs possible");
			this.radio2
					.setText("Serials only for PROTEIN atoms. CONECTs only for proteins");
			this.radio3.setText("No serials at all. No CONECTs");

			final JLabel sl2 = new JLabel(
					"<html>"
							+ "Each residue chain must have its unique identifier. This has to be a single<br>"
							+ "alphabetical character that has its place in column 22. Therefore, the number<br>"
							+ "of chains is limited to 26. If you activate case sensitive Chain ID's, this<br>"
							+ "number is doubled to 52, but beware: Case sensitive chain ID's is an extended<br>"
							+ "Standard and might not be compatible with older readers!<br>"
							+ "Choose a restriction for this issue</html>");

			this.radio4
					.setText("Chains for both PROTEIN and LIPID atoms. Full secondary structure possible");
			this.radio5
					.setText("Chains only for PROTEIN atoms. Secondary structure only for proteins");
			this.radio6
					.setText("No Chains at all. No secondary structure possible");
			this.check.setText("Allow case sensitive Chain ID's");

			final Color fg = this.getForeground().brighter();
			this.radio1.setForeground(fg);
			this.radio2.setForeground(fg);
			this.radio3.setForeground(fg);
			this.radio4.setForeground(fg);
			this.radio5.setForeground(fg);
			this.radio6.setForeground(fg);
			this.check.setForeground(fg);

			subj1.add(sl1);

			subj1.add(this.radio1);
			subj1.add(this.radio2);
			subj1.add(this.radio3);

			subj2.add(sl2);

			subj2.add(this.radio4);
			subj2.add(this.radio5);
			subj2.add(this.radio6);

			subj2.add(this.check);

			final JPanel subj3 = new JPanel();

			subj3.setLayout(new BoxLayout(subj3, BoxLayout.Y_AXIS));

			final JLabel sl3 = new JLabel("<html>"
					+ "Serials of unnumbered ATOM entries:</html>");

			this.radio7.setText("Assign \"99999\" to all");
			this.radio8
					.setText("Number them from 1 to 99999, then start over again with 1");
			this.radio9
					.setText("Number them from 1 to 99999, then start over again with 0 (Gromacs only)");

			this.radio7.setForeground(fg);
			this.radio8.setForeground(fg);
			this.radio9.setForeground(fg);

			subj3.add(sl3);

			subj3.add(this.radio7);
			subj3.add(this.radio8);
			subj3.add(this.radio9);

			subj1.add(Box.createVerticalStrut(5));
			subj1.add(subj3);

		}
	}

	public void setUpButtons() {

		if (DataPool.LimitationSettings.chains == 0)
			this.npanel.radio4.setSelected(true);
		else if (DataPool.LimitationSettings.chains == 1)
			this.npanel.radio5.setSelected(true);
		else
			this.npanel.radio6.setSelected(true);

		if (DataPool.LimitationSettings.caseSensitiveChainIds)
			this.npanel.check.setSelected(true);

		if (DataPool.LimitationSettings.atomserials == 0)
			this.npanel.radio1.doClick();
		else if (DataPool.LimitationSettings.atomserials == 1)
			this.npanel.radio2.doClick();
		else
			this.npanel.radio3.doClick();

		if (DataPool.LimitationSettings.all99999)
			this.npanel.radio7.setSelected(true);
		else {
			if (DataPool.LimitationSettings.restartNumberingWith0) {
				this.npanel.radio8.setSelected(false);
				this.npanel.radio9.setSelected(true);
			} else {
				this.npanel.radio8.setSelected(true);
				this.npanel.radio9.setSelected(false);
			}
		}
		
		this.tpanel.check2
				.setSelected(DataPool.LimitationSettings.assignResSeqNumbers);
		this.tpanel.check3
				.setSelected(DataPool.LimitationSettings.adoptTitleSections);

		this.tpanel.check4
				.setSelected(DataPool.CoordinateTransformationSettings.sortByType);
		
		if (DataPool.LimitationSettings.reverseParsing) {
			this.tpanel.check5.setSelected(false);
			this.tpanel.check5.doClick();
		}

		this.tpanel.check6.setSelected(DataPool.LimitationSettings.exportTopology);
		
		this.tpanel.check7.setSelected(DataPool.LimitationSettings.exportAPLVoro);
		
		if (DataPool.CoordinateTransformationSettings.shiftXZ == 0)
			this.tpanel.radio1.setSelected(true);
		else if (DataPool.CoordinateTransformationSettings.shiftXZ == 1)
			this.tpanel.radio2.setSelected(true);
		else
			this.tpanel.radio3.setSelected(true);

		if (DataPool.CoordinateTransformationSettings.shiftY == 0)
			this.tpanel.radio4.setSelected(true);
		else if (DataPool.CoordinateTransformationSettings.shiftY == 1)
			this.tpanel.radio5.setSelected(true);
		else
			this.tpanel.radio6.setSelected(true);

		this.tpanel.check
				.setSelected(DataPool.CoordinateTransformationSettings.swapYZ);
		
		this.tpanel.check8.setSelected(DataPool.PPESettings.saveOPMDummyLayersInPDB);
		
		if (DataPool.LimitationSettings.domainID2ResNameAtPosition == 1)
			this.tpanel.comboDomainID2ResName.setSelectedIndex(1);
		else if (DataPool.LimitationSettings.domainID2ResNameAtPosition == 2)
		this.tpanel.comboDomainID2ResName.setSelectedIndex(2);
		else if  (DataPool.LimitationSettings.domainID2ResNameAtPosition == 3)
			this.tpanel.comboDomainID2ResName.setSelectedIndex(3);
		else
			this.tpanel.comboDomainID2ResName.setSelectedIndex(0);
		
		if (DataPool.LimitationSettings.layerID2ResNameAtPosition == 1)
			this.tpanel.comboLayerID2ResName.setSelectedIndex(1);
		else if (DataPool.LimitationSettings.layerID2ResNameAtPosition == 2)
		this.tpanel.comboLayerID2ResName.setSelectedIndex(2);
		else if  (DataPool.LimitationSettings.layerID2ResNameAtPosition == 3)
			this.tpanel.comboLayerID2ResName.setSelectedIndex(3);
		else
			this.tpanel.comboLayerID2ResName.setSelectedIndex(0);

	}

	private class MyCompatibilityPanel extends CompatibilityPanel implements
			ActionListener {

		private static final long serialVersionUID = 1L;

		public void actionPerformed(final ActionEvent e) {
			this.update();

		}

		public String checkCompatibility(final String name) {

			int as = Compatibility.ATOMSERIALS_BOTH;
			if (PdbSettingsWindow.this.npanel.radio2.isSelected())
				as = Compatibility.ATOMSERIALS_PROTSONLY;
			else if (PdbSettingsWindow.this.npanel.radio3.isSelected())
				as = Compatibility.ATOMSERIALS_NONE;

			int cid = Compatibility.ATOMSERIALS_BOTH;
			if (PdbSettingsWindow.this.npanel.radio5.isSelected())
				cid = Compatibility.ATOMSERIALS_PROTSONLY;
			else if (PdbSettingsWindow.this.npanel.radio6.isSelected())
				cid = Compatibility.ATOMSERIALS_NONE;

			int sXZ = Compatibility.TRANSFORM_NONE;

			if (PdbSettingsWindow.this.tpanel.radio2.isSelected())
				sXZ = Compatibility.TRANSFORM_PHYSICAL;
			else if (PdbSettingsWindow.this.tpanel.radio3.isSelected())
				sXZ = Compatibility.TRANSFORM_BOTTOM;

			int sY = Compatibility.TRANSFORM_NONE;

			if (PdbSettingsWindow.this.tpanel.radio5.isSelected())
				sY = Compatibility.TRANSFORM_PHYSICAL;
			else if (PdbSettingsWindow.this.tpanel.radio6.isSelected())
				sY = Compatibility.TRANSFORM_BOTTOM;

			return this.getToolText(name, as, cid,
					PdbSettingsWindow.this.npanel.check.isSelected(),
					PdbSettingsWindow.this.tpanel.check2.isSelected(),
					PdbSettingsWindow.this.npanel.radio7.isSelected(),
					PdbSettingsWindow.this.npanel.radio9.isSelected(),
					PdbSettingsWindow.this.tpanel.check3.isSelected(),
					PdbSettingsWindow.this.tpanel.check.isSelected(), 
					sXZ, 
					sY, 
					PdbSettingsWindow.this.tpanel.check4.isSelected(),
					PdbSettingsWindow.this.tpanel.check5.isSelected());
		}
	}

	public void dispose() {

		super.dispose();
		System.gc();
	}

}
