/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.elements;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.vecmath.Matrix3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.Engine.MembraneOperations;
import org.cellmicrocosmos.cm2.interfaces.DatabaseViewListener;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.io.PdbReader_3D.PdbAttributes;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


@SuppressWarnings("serial")
/**
 * 
 * The View that shows all the Properties of the current selection
 * 
 * Different Modes: MINFO - shows the Properties of the current Membrane Model
 * ENTRYP / ENTRYL - shows the properties of a DataBaseEntry SSELECT - shows the
 * properties of the selected protein instance MSELECT - shows the properties of
 * the selected Proteins, if multiple selected LIPID - shows the properties of
 * the selected Lipid template ERROR - shows a message that no data can be
 * displayed OFF - turns this view off
 * 
 * @author Tim Dingersen
 * 
 */
public class PropertiesView extends JPanel {

	private Molecule_3D[] currentMolecules;

	boolean isMembraneAttrsShowing = false;

	private final int MINFO = 0;

	private final int ENTRYP = 1;

	private final int ENTRYL = 7;

	private final int SSELECT = 2;

	private final int MSELECT = 3;

	private final int LIPID = 4;

	private final int ERR = 5;

	private final int OFF = 6;

	private final int NEUTRAL = 8;

	final Font font1 = new Font("Arial", Font.BOLD, 11);

	final Font font2 = new Font("Arial", Font.PLAIN, 10);

	final Font fontheader = new Font("Arial", Font.PLAIN, 15);

	HeaderPanel headerpanel = new HeaderPanel();

	Panel P = new Panel();

	JScrollPane scroll;

	private int mode = this.OFF;

	private StringWrapper[] wrappers;

	private float xcoord;

	private float zcoord;

	private float ytrans;

	private Matrix3f matrix;

	private long filesizesingle;

	private int lipidcountExtra;

	private int lipidcountIntra;

	private long filesizemulti;

	private final int borderL = 15;

	private final int rowspace = 12;

	private final BufferedImage image1 = new BufferedImage(16, 16,
			Transparency.BITMASK);
	
	private final BufferedImage image1w = new BufferedImage(16, 16,
			Transparency.BITMASK);

	private final BufferedImage image2 = new BufferedImage(16, 16,
			Transparency.BITMASK);
	
	private final BufferedImage image2w = new BufferedImage(16, 16,
			Transparency.BITMASK);

	private final BufferedImage image3 = new BufferedImage(16, 16,
			Transparency.BITMASK);
	
	private final BufferedImage image3w = new BufferedImage(16, 16,
			Transparency.BITMASK);

	private final BufferedImage image4 = new BufferedImage(24, 16,
			Transparency.BITMASK);

	private final BufferedImage image4w = new BufferedImage(24, 16,
			Transparency.BITMASK);
	
	private final BufferedImage image5 = new BufferedImage(24, 16,
			Transparency.BITMASK);
	
	private final BufferedImage image5w = new BufferedImage(24, 16,
			Transparency.BITMASK);
	
	private final BufferedImage image6 = new BufferedImage(24, 16,
			Transparency.BITMASK);
	
	private final BufferedImage image6w = new BufferedImage(24, 16,
			Transparency.BITMASK);
	
	private DatabaseViewListener dblistener;

	private MembraneOperations melistener;
	
	

	public PropertiesView() {

		this.scroll = new JScrollPane(this.P);

		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		this.add(this.headerpanel);
		this.add(this.scroll);

		this.image1.getGraphics().setColor(Color.white);
		// image1.getGraphics().fillRect(0, 0, 16, 16);
		this.image1w.getGraphics().setColor(Color.white);
		// image1w.getGraphics().fillRect(0, 0, 16, 16);
		this.image2.getGraphics().setColor(Color.white);
		// image2.getGraphics().fillRect(0, 0, 16, 16);
		this.image2w.getGraphics().setColor(Color.white);
		// image2w.getGraphics().fillRect(0, 0, 16, 16);
		this.image3.getGraphics().setColor(Color.white);
		// image3.getGraphics().fillRect(0, 0, 16, 16);
		this.image3w.getGraphics().setColor(Color.white);
		// image3w.getGraphics().fillRect(0, 0, 16, 16);
		this.image4.getGraphics().setColor(Color.white);
		// image4.getGraphics().fillRect(0, 0, 16, 16);
		this.image4w.getGraphics().setColor(Color.white);
		// image4w.getGraphics().fillRect(0, 0, 16, 16);
		this.image5.getGraphics().setColor(Color.white);
		// image5.getGraphics().fillRect(0, 0, 16, 16);
		this.image5w.getGraphics().setColor(Color.white);
		// image5w.getGraphics().fillRect(0, 0, 16, 16);
		this.image6.getGraphics().setColor(Color.white);
		// image6.getGraphics().fillRect(0, 0, 16, 16);
		this.image6w.getGraphics().setColor(Color.white);
		// image6.getGraphics().fillRect(0, 0, 16, 16);

		this.image1.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_Prot").getImage(), 0, 0,
				null);
		this.image1w.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_Prot_w").getImage(), 0, 0,
				null);
		this.image2.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_LipEC").getImage(), 0, 0,
				null);
		this.image2w.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_LipEC_w").getImage(), 0, 0,
				null);
		this.image3.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_LipIC").getImage(), 0, -4,
				null);
		this.image3w.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_LipIC_w").getImage(), 0, -4,
				null);
		this.image4.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_Area").getImage(), 0, 0,
				null);
		this.image4w.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_Area_w").getImage(), 0, 0,
				null);
		this.image5.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_LipECratio").getImage(), 0, 0,
				null);
		this.image5w.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_LipECratio_w").getImage(), 0, 0,
				null);
		this.image6.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_LipICratio").getImage(), 0, 0,
				null);
		this.image6w.getGraphics().drawImage(
				Cm2_IconDepot.getSpecifiedIcon("CV_LipICratio_w").getImage(), 0, 0,
				null);

	}

	/**
	 * Displays the specified properties
	 * 
	 * @param file
	 * @param type
	 */
	public void showProperties(final File file, final int type) {
		this.isMembraneAttrsShowing = false;
		this.currentMolecules = null;
		final PdbReader_3D reader = new PdbReader_3D();

		if (file == null) {
			this.clearView(false);
			return;
		}

		try {
			final PdbAttributes attrs = reader.readAttributes(file);
			final Molecule_3D molecule = new Protein_3D();

			int mode;
			if (type == 0)
				mode = this.ENTRYP;
			else if (type == 1)
				mode = this.ENTRYL;
			else
				mode = this.NEUTRAL;

			molecule.setPDBFile(file);
			molecule.setDate(attrs.getDate());
			molecule.setTitle(attrs.getTitles());
			molecule.setHeader(attrs.getHeader());
			molecule.setSOURCE(attrs.getSpecies());
			molecule.setAuthor(attrs.getAuthor());
			molecule.setPDBcode(attrs.getCode());

			try {
				molecule.setModelSet(DatabaseView.ModelByFile.get(
						molecule.getPDBFile().getAbsolutePath()).intValue());
			} catch (final Exception e) {

			}

			this.showIt(molecule, mode);

		} catch (final IOException e) {
			this.clearView(true);

			return;
		}

	}

	/**
	 * displays properties of the passed molecules
	 * 
	 * @param molecule
	 */
	public void showProperties(final Molecule_3D molecule[]) {
		this.isMembraneAttrsShowing = false;
		this.currentMolecules = molecule;

		if (molecule == null) {
			this.showMembraneInfo();
			return;
		}
		if (molecule.length == 1) {
			this.showIt(molecule[0], this.SSELECT);
		} else if (molecule.length > 1) {
			this.showIt(molecule[0], this.MSELECT);
		} else
			this.clearView(false);

	}

	public void showLipid(final Lipid_3D lip) {
		this.isMembraneAttrsShowing = false;
		this.currentMolecules = null;

		// MembraneModel model = DataPool.getMembraneModel();
		// Lipid_3D[] lip;
		// lip = model.getExampleLipidsByPDBCodeExtra(code);
		// if (lip.length == 0)
		// lip = model.getExampleLipidsByPDBCodeIntra(code);
		// if (lip.length == 0) {
		// this.clearView(false);
		// return;
		// }
		this.currentMolecules = new Lipid_3D[] { lip };
		this.showIt(lip, this.LIPID);

	}

	public void showMembraneInfo() {
		this.isMembraneAttrsShowing = true;
		this.currentMolecules = null;

		this.showIt(null, this.MINFO);
	}

	public void clearView(final boolean error) {
		this.isMembraneAttrsShowing = false;
		this.currentMolecules = null;

		if (error)
			this.showIt(null, this.ERR);
		else
			this.showIt(null, this.OFF);
	}

	private void showIt(final Molecule_3D mol, int mode) {

		if (mol == null
				&& (mode == this.SSELECT || mode == this.MSELECT
						|| mode == this.LIPID || mode == this.ENTRYP || mode == this.ENTRYL))
			mode = this.MINFO;
		this.mode = mode;
		this.wrappers = new StringWrapper[12];

		this.xcoord = 0;
		this.zcoord = 0;
		this.ytrans = 0;

		this.matrix = null;

		this.headerpanel.setMode(mode);

		if (mode == this.OFF) {
			this.P.setVisible(false);

			return;
		} else
			this.P.setVisible(true);

		final int minW = 190;
		final int maxW = 600;

		int W = this.scroll.getSize().width - 20;
		if (W < minW)
			W = minW;
		if (W > maxW)
			W = maxW;

		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.LIPID) {

			if (mol.getPDBFile() != null) {
				int m;
				if (mode == this.ENTRYP || mode == this.SSELECT)
					m = 0;
				else
					m = 1;
				final String usercomment = this.dblistener
						.retrieveUserCommentFromDB(mol.getPDBFile(), m);
				if (usercomment != null)
					this.wrappers[9] = new StringWrapper(W, "User Notes",
							usercomment);
			}

		}

		if (mode == this.SSELECT && mol instanceof Protein_3D) {

			if (((Protein_3D) mol).getID() != null) {
				if (((Protein_3D) mol).getID().length() > 0)
					this.wrappers[8] = new StringWrapper(W, "Instance ID",
							((Protein_3D) mol).getID());
			}

		}
		if (mode == this.MSELECT && mol instanceof Protein_3D) {

			this.wrappers[8] = new StringWrapper(W, "Instance ID", "Multiple");

		}
		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.MSELECT || mode == this.LIPID
				|| mode == this.NEUTRAL) {

			if (mol.getPDBcode() != null) {
				if (mol.getPDBcode().length() > 0)
					this.wrappers[0] = new StringWrapper(W, "Code", mol
							.getPDBcode());
			}

		}

		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.MSELECT || mode == this.LIPID
				|| mode == this.NEUTRAL) {

			if (mol.getHeader() != null) {
				if (mol.getHeader().length() > 0)
					this.wrappers[1] = new StringWrapper(W, "Header", mol
							.getHeader());
			}

		}
		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.MSELECT || mode == this.LIPID
				|| mode == this.NEUTRAL) {

			if (mol.getTitle() != null) {
				if (mol.getTitle().length() > 0)
					this.wrappers[2] = new StringWrapper(W, "Title", mol
							.getTitle());
			}

		}
		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.MSELECT || mode == this.LIPID
				|| mode == this.NEUTRAL) {

			if (mol.getSOURCE() != null) {
				if (mol.getSOURCE().length() > 0)
					this.wrappers[3] = new StringWrapper(W, "Source", mol
							.getSOURCE());
			}

		}
		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.MSELECT || mode == this.LIPID
				|| mode == this.NEUTRAL) {

			if (mol.getPDBFile() != null) {
				if (mol.getPDBFile().length() > 0)
					this.wrappers[4] = new StringWrapper(W, "Pdb File", mol
							.getPDBFile().getAbsolutePath());
			}

		}
		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.LIPID) {

			if (mol.getModelSet() != -1) {

				this.wrappers[5] = new StringWrapper(W, "Active Model", ""
						+ mol.getModelSet());
			}

		}
		if (mode == this.SSELECT||mode==this.MSELECT
				|| mode == this.LIPID) {

			if (!mol.getCurrentProfileName().equals("Default_Profile")) {

				this.wrappers[11] = new StringWrapper(W, "Current Profile", ""
						+ mol.getCurrentProfileName());
			}

		}
		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.MSELECT || mode == this.LIPID
				|| mode == this.NEUTRAL) {

			if (mol.getAuthor() != null) {
				if (mol.getAuthor().length() > 0)
					this.wrappers[6] = new StringWrapper(W, "Author", mol
							.getAuthor());
			}

		}
		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.MSELECT || mode == this.LIPID
				|| mode == this.NEUTRAL) {

			if (mol.getDate() != null) {
				if (mol.getDate().length() > 0)
					this.wrappers[7] = new StringWrapper(W, "Date", mol
							.getDate());
			}

		}
		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.MSELECT || mode == this.LIPID
				|| mode == this.NEUTRAL) {

			if (mol.getAtomicMass() > 0) {

				this.wrappers[10] = new StringWrapper(W, "Atomic Mass (u)", ""
						+ mol.getAtomicMass());
			}

		}

		final MembraneModel model = DataPool.getMembraneModel();

		if (mode == this.MINFO) {

			if (model.getCode() != null) {
				if (model.getCode().length() > 0)
					this.wrappers[0] = new StringWrapper(W, "Code", model
							.getCode());
			}

		}

		if (mode == this.MINFO) {

			if (model.getHeader() != null) {
				if (model.getHeader().length() > 0)
					this.wrappers[1] = new StringWrapper(W, "Header", model
							.getHeader());
			}

		}

		if (mode == this.MINFO) {

			if (DataPool.getCurrentMembraneFile() != null) {

				this.wrappers[4] = new StringWrapper(W, "Pdb File", DataPool
						.getCurrentMembraneFile().getAbsolutePath());
			} else
				this.wrappers[4] = new StringWrapper(W, "Pdb File",
						"Not yet determined");

		}

		if (mode == this.MINFO) {

			if (model.getDate() != null) {
				if (model.getDate().length() > 0)
					this.wrappers[7] = new StringWrapper(W, "Date", model
							.getDate());
			}

		}

		if (mode == this.SSELECT) {

			this.xcoord = mol.getX();
			this.zcoord = mol.getZ();
			this.ytrans = mol.getYTranslation();

			this.matrix = mol.getRotationMatrix();

		}

		if (mode == this.ENTRYP || mode == this.ENTRYL || mode == this.SSELECT
				|| mode == this.MSELECT || mode == this.LIPID
				|| mode == this.NEUTRAL) {

			if (mol.getPDBFile().canRead())
				this.filesizesingle = mol.getPDBFile().length();
			else {
				final String[] s = DataPool.getRawPdbData(mol.getPDBFile()
						.getAbsolutePath());
				this.filesizesingle = 0;
				for (int i = 0; i < s.length; i++) {
					this.filesizesingle += s[i].length();
				}
			}

		}

		if (mode == this.MSELECT) {
			this.filesizemulti = this.filesizesingle
					* DataPool.getMembraneModel().getProteinsByPDBCode(
							mol.getPDBcode()).length;

		}

		if (mode == this.LIPID) {
			this.lipidcountExtra = DataPool.getMembraneModel()
					.getLipidsByPDBCodeExtra(mol.getPDBcode()).length;
			this.lipidcountIntra = DataPool.getMembraneModel()
					.getLipidsByPDBCodeIntra(mol.getPDBcode()).length;
			this.filesizemulti = this.filesizesingle
					* (this.lipidcountExtra + this.lipidcountIntra);
		}

		int H = 20;

		if (mode == this.MINFO) {
			H += 80;
			H += this.rowspace * 3;

		}

		if (mode == this.SSELECT || mode == this.MSELECT || mode == this.LIPID
				|| mode == this.ENTRYP || mode == this.ENTRYL
				|| mode == this.NEUTRAL) {

			H += 18;
			H += this.rowspace;

		}

		if (mode == this.SSELECT) {

			H += 18;

			H += this.rowspace / 2;

			H += this.rowspace * 4;

		}

		for (int i = 0; i < this.wrappers.length; i++) {

			if (this.wrappers[i] == null)
				continue;

			H += 21;
			H += this.wrappers[i].rows.size() * this.rowspace;
		}

		// if(H<scroll.getSize().height)H=scroll.getSize().height;

		this.P.setPreferredSize(new Dimension(W, H));

		this.P.revalidate();

		this.P.repaint();

	}

	public void doLayout() {
		super.doLayout();
		this.headerpanel.setPreferredSize(new Dimension(this.getWidth(), 30));
		this.scroll.setPreferredSize(new Dimension(this.getWidth(), this
				.getHeight() - 35));

	}

	public boolean isMembraneAttrsShowing() {
		return this.isMembraneAttrsShowing;
	}

	public Molecule_3D[] getCurrentMolecules() {
		return this.currentMolecules;
	}

	/**
	 * Wraps a string into an array of lines
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class StringWrapper {

		public int borderR = 10;

		public String title;

		public Vector<String> rows = new Vector<String>();;

		public StringWrapper(final int W, final String title,
				final String contents) {

			this.title = title;

			final Vector<String> words = new Vector<String>();
			int lastone = 0;

			for (int i = 0; i < contents.length(); i++) {

				final char c = contents.charAt(i);

				if (c == ' ' || c == '\n' || c == '\t'
						|| i == contents.length() - 1) {
					String sub = contents.substring(lastone, i);

					sub = sub.replaceAll(" ", "");
					sub = sub.replaceAll("\n", "");
					sub.replace("\t", "");
					if (sub.length() > 0) {
						if (c == ' ')
							sub += " ";
						words.add(sub);
					}

					lastone = i;

				}

				if (c == '-' || c == '/' || c == '\\' || c == ','
						|| i == contents.length() - 1) {
					String sub = contents.substring(lastone, i + 1);

					sub = sub.replaceAll(" ", "");
					sub = sub.replaceAll("\n", "");
					sub.replace("\t", "");
					if (sub.length() > 0) {

						if (words.size() > 0) {
							if (words.get(words.size() - 1).endsWith(","))
								sub = " " + sub;
						}

						words.add(sub);
					}

					lastone = i + 1;

				}

			}

			int index = 0;

			while (index < words.size()) {

				if (this.rows.size() == 0) {
					this.rows.add(words.get(index));
					index++;
					continue;
				}

				final JLabel label = new JLabel(this.rows
						.get(this.rows.size() - 1)
						+ words.get(index));
				label.setFont(PropertiesView.this.font2);
				if (label.getPreferredSize().width < W - 10) {
					this.rows.set(this.rows.size() - 1, this.rows.get(this.rows
							.size() - 1)
							+ words.get(index));
					index++;
					continue;
				} else {
					this.rows.add(words.get(index));
					index++;
				}

			}

			for (int i = 0; i < this.rows.size(); i++) {
				if (this.rows.get(i).startsWith(" "))
					this.rows.set(i, this.rows.get(i).replaceFirst(" ", ""));
			}

		}

	}

	/**
	 * The Headercomponent of this view
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class HeaderPanel extends JPanel {

		private int mode = PropertiesView.this.OFF;

		public HeaderPanel() {
			super();

		}

		public void setMode(final int mode) {
			this.mode = mode;
			this.repaint();
		}

		public void paintComponent(final Graphics g) {

			String string;
			if (this.mode == PropertiesView.this.ENTRYP)
				string = "File Info (PROTEIN)";
			else if (this.mode == PropertiesView.this.ENTRYL)
				string = "File Info (LIPID)";
			else if (this.mode == PropertiesView.this.SSELECT)
				string = "Current Selection";
			else if (this.mode == PropertiesView.this.MSELECT)
				string = "Current Selections";
			else if (this.mode == PropertiesView.this.LIPID)
				string = "Lipid Template";
			else if (this.mode == PropertiesView.this.MINFO) {
				if (PropertiesView.this.melistener.getHighlightedMicroDomain() < 0)
					string = "All Areas";
				else if (PropertiesView.this.melistener
						.getHighlightedMicroDomain() == 0)
					string = "Default Area";
				else {
					string = "Microdomain "
							+ PropertiesView.this.melistener
									.getHighlightedMicroDomain();
				}
			} else if (this.mode == PropertiesView.this.NEUTRAL)
				string = "External File";
			else {
				g.setColor(new JPanel().getBackground());
				g.fillRect(0, 0, this.getWidth(), this.getHeight());
				return;
			}

			g.setColor(this.getBackground());
			g.fillRect(0, 0, this.getWidth(), this.getHeight());

			if (this.mode == PropertiesView.this.ENTRYP)
				g.setColor(CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_PROTEIN_COLOR);
			else if (this.mode == PropertiesView.this.SSELECT
					|| this.mode == PropertiesView.this.MSELECT)
				g.setColor(new Color(
						CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_PROTEIN_COLOR
								.getRed(),
						CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_PROTEIN_COLOR
								.getGreen() - 10,
						CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_PROTEIN_COLOR
								.getBlue() - 60));
			else if (this.mode == PropertiesView.this.ENTRYL)
				g.setColor(CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_LIPID_COLOR);
			else if (this.mode == PropertiesView.this.LIPID)
				g.setColor(new Color(
						CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_LIPID_COLOR
								.getRed() - 10,
						CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_LIPID_COLOR
								.getGreen() - 20,
						CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_LIPID_COLOR
								.getBlue() - 20));
			else if (this.mode == PropertiesView.this.NEUTRAL)
				g.setColor(Color.gray);
			else if (this.mode == PropertiesView.this.MINFO)
				g.setColor(new Color(230, 230, 245));
			g.fillRoundRect(12, 5, this.getWidth(), this.getHeight() - 10, 20,
					20);

			g.setColor(Color.black);
			g.setFont(PropertiesView.this.fontheader);

			final JLabel label = new JLabel(string);
			label.setFont(PropertiesView.this.fontheader);

			final Dimension size = label.getPreferredSize();

			g.drawString(string, (int) this.getWidth() / 2 - (int) size.width
					/ 2, 22);
		}

	}

	/**
	 * The Panel that displays the specified contents of the selection
	 * 
	 * @author Tim Dingersen dackelnose@gmx.net
	 * 
	 */
	private class Panel extends JPanel {

		public void paintComponent(final Graphics g) {
			super.paintComponent(g);

			if (PropertiesView.this.mode == PropertiesView.this.OFF) {
				g.setColor(this.getBackground());
				g.fillRect(0, 0, this.getWidth(), this.getHeight());
				return;
			}

			g.setColor(this.getBackground());
			g.fillRect(0, 0, this.getWidth(), this.getHeight());

			g.setColor(this.getForeground());

			int level = 15;

			int[] sequence;

			if (PropertiesView.this.mode == PropertiesView.this.ERR) {

				final Dimension dim = new JLabel("No Data to Display")
						.getPreferredSize();
				g.drawString("No Data to Display", (int) this.getWidth() / 2
						- (int) dim.width / 2, (int) this.getHeight() / 2 + 5);
				return;
			}

			if (PropertiesView.this.mode == PropertiesView.this.MINFO) {

				final MembraneModel model = DataPool.getMembraneModel();

				g.setFont(PropertiesView.this.font1);
				g.drawString("Molecule Counts", 5, level);
				level += 5;
				int fontWidth = 10;

				final int layerid = DataPool.getCurrentLayerID();
				
				int numberOfProteins = model.getAllProteinsInDomain(PropertiesView.this.melistener.getHighlightedMicroDomain(), layerid).length;
				int numberOfExtraLipids = model.getAllLipidsExtraInDomain(PropertiesView.this.melistener.getHighlightedMicroDomain(), layerid).length;
				int numberOfIntraLipids = model.getAllLipidsIntraInDomain(PropertiesView.this.melistener.getHighlightedMicroDomain(), layerid).length;
				
				int xPosOfExtraLipids = 25+(((int)Math.log10(numberOfProteins)+1)*fontWidth);
				int xPosOfIntraLipids = 20+xPosOfExtraLipids+((int)Math.log10(numberOfExtraLipids)+1)*fontWidth;
				
				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image1w:PropertiesView.this.image1, 5, level, null);
				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image2w:PropertiesView.this.image2, xPosOfExtraLipids, level, null);
				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image3w:PropertiesView.this.image3, xPosOfIntraLipids, level, null);
//				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image1w:PropertiesView.this.image1, 5, level, null);
//				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image2w:PropertiesView.this.image2, 60, level, null);
//				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image3w:PropertiesView.this.image3, 115, level, null);
				
				level += 13;

				g.setFont(PropertiesView.this.font2
						.deriveFont(PropertiesView.this.font2.getSize2D() + 1));

				g.drawString(numberOfProteins+"", 25, level);
				g.drawString(numberOfExtraLipids+"", xPosOfExtraLipids+20, level);
				g.drawString(numberOfIntraLipids+"", xPosOfIntraLipids+20, level);
//				g.drawString(numberOfProteins+"", 30, level);
//				g.drawString(numberOfExtraLipids+"", 80, level);
//				g.drawString(numberOfIntraLipids+"", 135, level);
				
				
				
				level += PropertiesView.this.rowspace;
				level += 10;

				g.setFont(PropertiesView.this.font1);
				g.drawString("Areas in Angstrom", 5, level);
				level += 5;
				
				double areaOfAllMicrodomains = 0;
				// this is only needed if there are microdomains and the default area is selected
				if (model.countMicroDomains()>0 && PropertiesView.this.melistener.getHighlightedMicroDomain() == 0) {
					for (int i=0;i<model.countMicroDomains();i++){
						areaOfAllMicrodomains =
							areaOfAllMicrodomains + model.getMicroDomain(i).getArea();
					}
				}
				double areaOfSelectedMicrodomain = 
								((PropertiesView.this.melistener.getHighlightedMicroDomain() < 1)?
										model.getArea()-
											((PropertiesView.this.melistener.getHighlightedMicroDomain() == 0)?
													areaOfAllMicrodomains:0):
										model.getMicroDomain(PropertiesView.this.melistener.getHighlightedMicroDomain() - 1).getArea() - areaOfAllMicrodomains );
				String areaOfAllMicrodomainsString = areaOfSelectedMicrodomain+"";
				
				int xPosOfExtraDensity = 35+(((int)Math.log10(areaOfSelectedMicrodomain)+3)*fontWidth);

				double lipidDensityExtra = (model.getAllLipidsExtraInDomain(
												PropertiesView.this.melistener.getHighlightedMicroDomain(), layerid).length!=0)?
														areaOfSelectedMicrodomain /
												new Double(model.getAllLipidsExtraInDomain(
														PropertiesView.this.melistener.getHighlightedMicroDomain(), layerid).length):0;                                              //200
				String lipidDensityExtraString = Double.toString(lipidDensityExtra);
				
				int xPosOfIntraDensity = 30+xPosOfExtraDensity+((int)Math.log10(lipidDensityExtra)+3)*fontWidth;
				
				double lipidDensityIntra = (model.getAllLipidsIntraInDomain(
												PropertiesView.this.melistener.getHighlightedMicroDomain(), layerid).length!=0)?
														areaOfSelectedMicrodomain /
												new Double(model.getAllLipidsIntraInDomain(
														PropertiesView.this.melistener.getHighlightedMicroDomain(), layerid).length):0;                                              //264
				String lipidDensityIntraString = Double.toString(lipidDensityIntra);
				
				// 170 and 235
				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image4w:PropertiesView.this.image4, 5, level, null);
				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image5w:PropertiesView.this.image5, xPosOfExtraDensity, level, null);
				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image6w:PropertiesView.this.image6, xPosOfIntraDensity, level, null);
//				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image4w:PropertiesView.this.image4, 5, level, null);
//				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image5w:PropertiesView.this.image5, 100, level, null);
//				g.drawImage(DataPool.J3DSettings.bright_appearance?PropertiesView.this.image6w:PropertiesView.this.image6, 165, level, null);
				level += 13;
				
				g.setFont(PropertiesView.this.font2
						.deriveFont(PropertiesView.this.font2.getSize2D() + 1));
				
				g.drawString(areaOfAllMicrodomainsString.substring(0, (areaOfAllMicrodomainsString.indexOf(".")+3<areaOfAllMicrodomainsString.length())?areaOfAllMicrodomainsString.indexOf(".")+3:areaOfAllMicrodomainsString.length()), 35, level);
				g.drawString(lipidDensityExtraString.substring(0, (lipidDensityExtraString.indexOf(".")+3<lipidDensityExtraString.length())?lipidDensityExtraString.indexOf(".")+3:lipidDensityExtraString.length()), xPosOfExtraDensity+30, level);
				g.drawString(lipidDensityIntraString.substring(0, (lipidDensityIntraString.indexOf(".")+3<lipidDensityIntraString.length())?lipidDensityIntraString.indexOf(".")+3:lipidDensityIntraString.length()), xPosOfIntraDensity+30, level);
//				g.drawString(areaOfAllMicrodomainsString.substring(0, (areaOfAllMicrodomainsString.indexOf(".")+3<areaOfAllMicrodomainsString.length())?areaOfAllMicrodomainsString.indexOf(".")+3:areaOfAllMicrodomainsString.length()), 35, level);
//				g.drawString(lipidDensityExtraString.substring(0, (lipidDensityExtraString.indexOf(".")+3<lipidDensityExtraString.length())?lipidDensityExtraString.indexOf(".")+3:lipidDensityExtraString.length()), 130, level);
//				g.drawString(lipidDensityIntraString.substring(0, (lipidDensityIntraString.indexOf(".")+3<lipidDensityIntraString.length())?lipidDensityIntraString.indexOf(".")+3:lipidDensityIntraString.length()), 194, level);
				
				
				
				

				
				
				
				level += PropertiesView.this.rowspace;
				level += 10;

				g.setFont(PropertiesView.this.font1);
				g.drawString("Bounds in Angstrom", 5, level);
				level += 16;
				g.setFont(PropertiesView.this.font2
						.deriveFont(PropertiesView.this.font2.getSize2D() + 1));

				if (PropertiesView.this.melistener.getHighlightedMicroDomain() < 1) {
					g.drawString(model.getSize().width + " X "
							+ model.getSize().height,
							PropertiesView.this.borderL, level);
				} else {
					final MicroDomain_3D md = model
							.getMicroDomain(PropertiesView.this.melistener
									.getHighlightedMicroDomain() - 1);
					g.drawString(md.getBoundingBox().width + " X "
							+ md.getBoundingBox().height,
							PropertiesView.this.borderL, level);
				}
				
				
				
				level += PropertiesView.this.rowspace;
				level += 6;

				g.setFont(PropertiesView.this.font1);
				g.drawString("Estimated Filesize", 5, level);
				level += 16;
				g.setFont(PropertiesView.this.font2
						.deriveFont(PropertiesView.this.font2.getSize2D() + 1));

				final long size = model.calculateTotalFileSize(
						PropertiesView.this.melistener
								.getHighlightedMicroDomain(), layerid);
				if (size > 0)
					g.drawString("< "
							+ Cm2_Toolkit.convertFileSizeToString(size),
							PropertiesView.this.borderL, level);
				else
					g.drawString("Not Determined", PropertiesView.this.borderL,
							level);

				level += PropertiesView.this.rowspace;
				level += 6;

			}

			if (PropertiesView.this.mode == PropertiesView.this.SSELECT
					|| PropertiesView.this.mode == PropertiesView.this.MSELECT
					|| PropertiesView.this.mode == PropertiesView.this.LIPID
					|| PropertiesView.this.mode == PropertiesView.this.ENTRYP
					|| PropertiesView.this.mode == PropertiesView.this.ENTRYL
					|| PropertiesView.this.mode == PropertiesView.this.NEUTRAL)
				sequence = new int[] { 8, 0, 9, 4,11, 5, 1, 2, 3, 6, 7, 10 };
			else
				sequence = new int[] { 4, 5, 0, 1, 2, 3, 6, 7, 9 };

			for (int i = 0; i < sequence.length; i++) {

				if (sequence[i] == 1
						&& (PropertiesView.this.mode == PropertiesView.this.SSELECT
								|| PropertiesView.this.mode == PropertiesView.this.MSELECT
								|| PropertiesView.this.mode == PropertiesView.this.LIPID
								|| PropertiesView.this.mode == PropertiesView.this.ENTRYP
								|| PropertiesView.this.mode == PropertiesView.this.ENTRYL || PropertiesView.this.mode == PropertiesView.this.NEUTRAL)) {

					g.setFont(PropertiesView.this.font1);
					g.drawString("Filesize", 5, level);
					level += 16;

					g.setFont(PropertiesView.this.font2);

					if (PropertiesView.this.mode == PropertiesView.this.SSELECT
							|| PropertiesView.this.mode == PropertiesView.this.ENTRYP
							|| PropertiesView.this.mode == PropertiesView.this.ENTRYL
							|| PropertiesView.this.mode == PropertiesView.this.NEUTRAL) {
						g
								.drawString(
										Cm2_Toolkit
												.convertFileSizeToString(PropertiesView.this.filesizesingle),
										PropertiesView.this.borderL, level);

					} else if (PropertiesView.this.mode == PropertiesView.this.MSELECT) {

						g
								.drawString(
										Cm2_Toolkit
												.convertFileSizeToString(PropertiesView.this.filesizesingle)
												+ "  ( Of All: "
												+ Cm2_Toolkit
														.convertFileSizeToString(PropertiesView.this.filesizemulti)
												+ " )",
										PropertiesView.this.borderL, level);
					} else
						g
								.drawString(
										Cm2_Toolkit
												.convertFileSizeToString(PropertiesView.this.filesizesingle)
												+ "  ( In Membrane: "
												+ Cm2_Toolkit
														.convertFileSizeToString(PropertiesView.this.filesizemulti)
												+ " )",
										PropertiesView.this.borderL, level);

					level += PropertiesView.this.rowspace;

					level += 6;
				}

				if (sequence[i] == 1
						&& PropertiesView.this.mode == PropertiesView.this.SSELECT) {

					g.setFont(PropertiesView.this.font1);
					g.drawString("Geometry", 5, level);
					level += 16;

					g.setFont(new Font("Arial", Font.PLAIN, 11));

					g
							.drawString("Position",
									PropertiesView.this.borderL, level);
					g.drawString("Rotation", PropertiesView.this.borderL + 70,
							level);
					level += PropertiesView.this.rowspace / 2;
					g.setFont(new Font("Arial", Font.PLAIN, 11));
					g.drawString("Pos X", PropertiesView.this.borderL, level
							+ PropertiesView.this.rowspace);
					g.drawString("Pos Z", PropertiesView.this.borderL, level
							+ PropertiesView.this.rowspace * 2);
					g.drawString("Height", PropertiesView.this.borderL, level
							+ PropertiesView.this.rowspace * 3);

					level += PropertiesView.this.rowspace;

					g.setFont(PropertiesView.this.font2);

					g.drawString((int) PropertiesView.this.xcoord + "",
							PropertiesView.this.borderL + 35, level);
					g.drawString((int) PropertiesView.this.zcoord + "",
							PropertiesView.this.borderL + 35, level
									+ PropertiesView.this.rowspace);

					String trans = "" + PropertiesView.this.ytrans;

					try {
						final int a = trans.indexOf(".");
						trans = trans.substring(0, a + 2);
					} catch (final Exception e) {

					}
					;
					g.drawString(trans, PropertiesView.this.borderL + 35, level
							+ PropertiesView.this.rowspace * 2);

					if (PropertiesView.this.matrix != null) {
						final String m00 = "" + PropertiesView.this.matrix.m00;
						final String m10 = "" + PropertiesView.this.matrix.m10;
						final String m20 = "" + PropertiesView.this.matrix.m20;
						final String m01 = "" + PropertiesView.this.matrix.m01;
						final String m11 = "" + PropertiesView.this.matrix.m11;
						final String m21 = "" + PropertiesView.this.matrix.m21;
						final String m02 = "" + PropertiesView.this.matrix.m02;
						final String m12 = "" + PropertiesView.this.matrix.m12;
						final String m22 = "" + PropertiesView.this.matrix.m22;
						final String[] matVal = { m00, m01, m02, m10, m11, m12,
								m20, m21, m22 };

						for (int j = 0; j < matVal.length; j++) {
							if (matVal[j].length() > 5) {
								matVal[j] = matVal[j].substring(0, 5);
							}
						}

						g.drawRoundRect(PropertiesView.this.borderL + 65,
								level - 10, 108,
								PropertiesView.this.rowspace * 3, 15, 15);

						g.setColor(this.getBackground().brighter());
						g.drawLine(PropertiesView.this.borderL + 70,
								level - 10, PropertiesView.this.borderL + 173,
								level - 10);
						g.drawLine(PropertiesView.this.borderL + 70, level - 10
								+ PropertiesView.this.rowspace * 3,
								PropertiesView.this.borderL + 173, level - 10
										+ PropertiesView.this.rowspace * 3);

						g.setColor(this.getForeground());

						g.drawString(matVal[0] + " ",
								PropertiesView.this.borderL + 70, level);
						g.drawString(matVal[1] + " ",
								PropertiesView.this.borderL + 105, level);
						g.drawString(matVal[2] + " ",
								PropertiesView.this.borderL + 140, level);
						level += PropertiesView.this.rowspace;
						g.drawString(matVal[3] + " ",
								PropertiesView.this.borderL + 70, level);
						g.drawString(matVal[4] + " ",
								PropertiesView.this.borderL + 105, level);
						g.drawString(matVal[5] + " ",
								PropertiesView.this.borderL + 140, level);
						level += PropertiesView.this.rowspace;
						g.drawString(matVal[6] + " ",
								PropertiesView.this.borderL + 70, level);
						g.drawString(matVal[7] + " ",
								PropertiesView.this.borderL + 105, level);
						g.drawString(matVal[8] + " ",
								PropertiesView.this.borderL + 140, level);

						level += PropertiesView.this.rowspace;

						level += 6;
					}
				}

				final StringWrapper wrapper = PropertiesView.this.wrappers[sequence[i]];
				if (wrapper == null)
					continue;

				g.setFont(PropertiesView.this.font1);
				g.drawString(wrapper.title, 5, level);
				level += 15;
				if (sequence[i] == 7)
					g
							.setFont(PropertiesView.this.font2
									.deriveFont(PropertiesView.this.font2
											.getSize2D() + 2));
				else
					g.setFont(PropertiesView.this.font2);
				for (int j = 0; j < wrapper.rows.size(); j++) {
					g.drawString(wrapper.rows.get(j),
							PropertiesView.this.borderL, level);
					level += PropertiesView.this.rowspace;
				}
				level += 6;

			}

		}

	}

	public void updateCurrentView() {

		if (this.currentMolecules != null)
			this.showIt(this.currentMolecules[0], this.mode);
		else
			this.showIt(null, this.mode);
	}

	public void addMembraneListener(final MembraneOperations membraneOperations) {
		this.melistener = membraneOperations;

	}

	public void addDatabaseViewListener(
			final DatabaseViewListener databaseOperations) {
		this.dblistener = databaseOperations;

	}
	
//	private Color getColor() {
//		if(DataPool.J3DSettings.bright_appearance)return new Color(15, 10, 20);
//		else return new Color(5,10,5);
//	}
	
}
