/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io;

import java.util.Vector;

/**
 * This special Exception will be thrown by the MembraneIO if at least one of
 * the membrane xml's Entries is invalid. If so, this Exception wont be thrown
 * instantly, it will first collect errors and will be thrown later at the end
 * of the membrane loading process. By this way, all invalid Entries can be
 * summarized at one time
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class InvalidEntriesException extends Exception {

	private static final long serialVersionUID = 1L;

	private final Vector<String> errorlist = new Vector<String>();

	/**
	 * makes an Entry in the Errorlist
	 * 
	 * @param string
	 */
	public void addEntry(final String string,Exception e) {

//		if(e!=null)
//		e.printStackTrace();
		
		if (this.errorlist.size() <= 12)
			this.errorlist.addElement(string);
		if (this.errorlist.size() == 12)
			this.errorlist.addElement("...");
	}

	/**
	 * returns error at specified index
	 * 
	 * @param index
	 * @return
	 */
	public String getEntryAt(final int index) {
		return this.errorlist.get(index);
	}

	/**
	 * returns the number of entries made
	 * 
	 * @return
	 */
	public int getEntryCount() {
		return this.errorlist.size();
	}

	/**
	 * concatenates all error Messages to one String, seperated by the new Line
	 * indicator "\n"
	 * 
	 * @return - the assembled String
	 */
	public String getMessageString() {

		String message = "";

		for (int i = 0; i < this.errorlist.size(); i++)
			message += this.errorlist.get(i) + "\n";

		return message;
	}

}
