/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io.ppe.toolbox;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;

import org.cellmicrocosmos.cm2.io.ppe.toolbox.PPE_CONSTANTS.CONF_PARSER;
import org.cellmicrocosmos.cm2.io.ppe.toolbox.PPE_CONSTANTS.IO;


/**
 * The ConfParser reads in the ppe.conf and generates different ArrayLists.
 * 
 * @author Christian Gamroth
 * 
 */
public class ConfParser {

	private Reader reader;
	private BufferedReader confreader;

	/**
	 * Constructs a new ConfParser.
	 */
	public ConfParser() {
		try {
			final File conffile = new File(IO.SETTINGS_DIR, IO.CONF_FILENAME);
			this.reader = new FileReader(conffile);
			this.confreader = new BufferedReader(this.reader);
		} catch (final FileNotFoundException e) {
			System.out.println(">> FileNotFoundException at ConfParser(): ");
			System.out.println(e);
		}
	}

	/**
	 * Generates an ArrayList with connection data.
	 * 
	 * @param key The key for indentifying the source.
	 * @return an ArrayList with connection data
	 */
	public ArrayList<String> parseForPortConfig(final String key) {
		String templine = null;
		final ArrayList<String> list = new ArrayList<String>();
		try {
			while ((templine = this.confreader.readLine()) != null) {
				while (!templine.equals(CONF_PARSER.START_TOKEN)) {
					templine = this.confreader.readLine();
				}
				if ((templine = this.confreader.readLine()).equals(key + ":")) {
					while (!templine.equals(CONF_PARSER.PORT_SUBKEY + ":")) {
						templine = this.confreader.readLine();
					}
					while (!(templine = this.confreader.readLine())
							.equals(CONF_PARSER.SUB_TOKEN)) {
						list.add(templine);
					}
					list.trimToSize();
					return list;
				}
			}
		} catch (final IOException e) {
			System.out
					.println(">> IOException at parseForPortConfig(String): ");
			System.out.println(e);
		}
		return null;
	}

	/**
	 * Generates an ArrayList with parsing-elements.
	 * 
	 * @param key The key for identifying the source.
	 * @return an ArrayList with parsing-elements
	 */
	public ArrayList<String> parseForElements(final String key) {
		String templine = null;
		final ArrayList<String> list = new ArrayList<String>();
		try {
			while ((templine = this.confreader.readLine()) != null) {
				while (!templine.equals(CONF_PARSER.START_TOKEN)) {
					templine = this.confreader.readLine();
				}
				if ((templine = this.confreader.readLine()).equals(key + ":")) {
					while (!templine.equals(CONF_PARSER.XML_SUBKEY + ":")) {
						templine = this.confreader.readLine();
					}
					while (!(templine = this.confreader.readLine())
							.equals(CONF_PARSER.END_TOKEN)) {
						list.add(templine);
					}
					list.trimToSize();
					return list;
				}
			}
		} catch (final IOException e) {
			System.out.println(">> IOException at parseForElements(String): ");
			System.out.println(e);
		}
		return null;
	}

	/**
	 * Closes the open streams and frees the space.
	 */
	public void freeSpace() {
		try {
			this.confreader.close();
			this.reader.close();
		} catch (final IOException e) {
			System.out.println(">> IOException at closeAllStreams(): ");
			System.out.println(e);
		}
	}

}
