/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io.ppe.toolbox;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.gui.dialogs.StatusDialog;
import org.cellmicrocosmos.cm2.io.ppe.network.http.OPMConnection;
import org.cellmicrocosmos.cm2.io.ppe.network.http.PdbTMConnection;
import org.cellmicrocosmos.cm2.io.ppe.toolbox.PPE_CONSTANTS.CONF_PARSER;
import org.cellmicrocosmos.cm2.io.ppe.toolbox.PPE_CONSTANTS.IO;
import org.cellmicrocosmos.cm2.io.ppe.toolbox.PPE_CONSTANTS.PPE_UPDATER;
import org.cellmicrocosmos.cm2.io.ppe.transform.xml.SAXCustomizer;
import org.cellmicrocosmos.cm2.io.ppe.transform.xml.XMLCustomizer;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * The class PPEUpdater updates the pdbtm.mif and looks for
 * new data bank updates of the different protein data banks.
 * 
 * @author Christian Gamroth, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class PPEUpdater {
	
	private static PrintWriter pwriter = null;
	
	private static XMLCustomizer customizer = null;
	
	private static OPMConnection opmCon = null;
	
	private static PdbTMConnection pdbtmCon = null;
	
	private static boolean updateOPM = true;
	
	/**
	 * Looks for new entries in the implemented protein data banks.
	 * 
	 * @return <code>true</code> if an update is necessary
	 */
	public static boolean lookUpForUpdates(int database) {
		try {
			if (database == PPE_CONSTANTS.DB.PDBTM) {
				if (!new File(IO.SETTINGS_DIR,
						IO.MIF_FILENAME_PDBTM).exists())
					return true;
				final File updatefile = new File(IO.SETTINGS_DIR,
						IO.UPDATE_FILENAME);
				if (!updatefile.exists()) {
					return true;
				}
				final BufferedReader breader = new BufferedReader(new FileReader(
						updatefile));
				final PdbTMConnection con = new PdbTMConnection();
				boolean lookForUpdates = true;
				String temp = null;
				while ((temp = breader.readLine()) != null) {
					final StringTokenizer tokenizer = new StringTokenizer(temp);
					if (tokenizer.nextToken().equals(PPE_UPDATER.PDBTM)) {
						final int counttemp = Integer.parseInt(tokenizer
								.nextToken());
						if (counttemp == Integer.parseInt(con.getCountOfPdbTM())) {
							lookForUpdates = false;
						}
					}
					/* An dieser Stelle weitere Quellen einfuegen:
					if (tokenizer.nextToken().equals(PPE_UPDATER.###)) { ... */
				}
				breader.close();
				return lookForUpdates;
				
			}else if (database == PPE_CONSTANTS.DB.OPM) {
				return updateOPM;
			}
		} catch (final FileNotFoundException e) {
			System.out
					.println(">> FileNotFoundException at lookUpForUpdates(): ");
			System.out.println(e);
		} catch (final IOException e) {
			System.out.println(">> IOException at lookUpForUpdates(): ");
			System.out.println(e);
		}
		return false;
	}

	/**
	 * Performs an update of the pdbtm.mif and other files. Creates or updates
	 * the update.txt at last.
	 * 
	 * @param dialog The StatusDialog for showing the progress to the user.
	 * @return <code>true</code> if the update was succesful
	 */
	public static void makeUpdate(final int database) {
		
		final JButton cancelButton = new JButton("Cancel");

		final StatusDialog dialog = new StatusDialog(cancelButton, true);

		final File updatefile = new File(IO.SETTINGS_DIR,
				IO.UPDATE_FILENAME);
		final File updatefileBackup = new File(IO.SETTINGS_DIR,
				IO.UPDATE_FILENAME+IO.BACKUP_FILENAME_ADDON);
		
		final Thread t = new Thread() {
			public void run() {

				try {
					
					if (database == PPE_CONSTANTS.DB.PDBTM) {
						dialog.setProgress("Connecting to PDBTM", 0, true);
						pdbtmCon = new PdbTMConnection();
						customizer = new SAXCustomizer(PPE_CONSTANTS.DB.PDBTM, pdbtmCon);
//						final File updatefile = new File(IO.SETTINGS_DIR,
//								IO.UPDATE_FILENAME);
//						final File updatefileBackup = new File(IO.SETTINGS_DIR,
//								IO.UPDATE_FILENAME+IO.BACKUP_FILENAME_ADDON);
						
						Vector<String> databaseNames = new Vector<String>();
						Vector<Integer> databaseSizes = new Vector<Integer>();
						
//						System.out.println("updatefile "+updatefile.getAbsolutePath());
						
						if (updatefile.exists()) {
							
							Cm2_Toolkit.copyFile(updatefile, updatefileBackup);
							
							// save the size of all other database entries
							String temp = null;
							final BufferedReader breader = new BufferedReader(new FileReader(
									updatefile));
							while ((temp = breader.readLine()) != null) {
								final StringTokenizer tokenizer = new StringTokenizer(temp);
								String actualDatabaseName = tokenizer.nextToken();
								if (!actualDatabaseName.equals(PPE_UPDATER.PDBTM)) {
									databaseNames.add(actualDatabaseName);
									databaseSizes.add(Integer.parseInt(tokenizer
											.nextToken()));
								}
							}
							breader.close();
							updatefile.delete();
							updatefile.createNewFile();
						}
						pwriter = new PrintWriter(new FileWriter(
								updatefile));
						for(int i=0;i<databaseNames.size();i++)
							pwriter.println(databaseNames.get(i) + " " + databaseSizes.get(i));
						pwriter.println(PPE_UPDATER.PDBTM + " " + pdbtmCon.getCountOfPdbTM());
						pwriter.flush();
						dialog.setProgress("Downloading from PDBTM", 20, true);
						customizer.customizeThisFile(pdbtmCon.getInputStreamFromXML(),
								CONF_PARSER.PDBTM_KEY);
						pwriter.close();
						
						if (updatefileBackup.exists())
							updatefileBackup.delete();
						
//							System.out.println(">> Update beendet ...");
						dialog.setProgress("Finished", 100, true);
						
						DataPool.initMIFHandler();
					} else if(database == PPE_CONSTANTS.DB.OPM) {

						dialog.setProgress("Connecting to OPM", 0, true);
						opmCon = new OPMConnection();
						customizer = new SAXCustomizer(PPE_CONSTANTS.DB.OPM, opmCon);
						final File updatefile = new File(IO.SETTINGS_DIR,
								IO.UPDATE_FILENAME);				
						
						Vector<String> databaseNames = new Vector<String>();
						Vector<Integer> databaseSizes = new Vector<Integer>();
						int databaseNamesIDofOPM = 0;
						if (updatefile.exists()) {
							
							Cm2_Toolkit.copyFile(updatefile, updatefileBackup);
							
							// save the size of all other database entries
							String temp = null;
							final BufferedReader breader = new BufferedReader(new FileReader(
									updatefile));
							while ((temp = breader.readLine()) != null) {
								final StringTokenizer tokenizer = new StringTokenizer(temp);
								String actualDatabaseName = tokenizer.nextToken();
								databaseNames.add(actualDatabaseName);
								databaseSizes.add(Integer.parseInt(tokenizer
										.nextToken()));
							}
							breader.close();
							updatefile.delete();
							updatefile.createNewFile();
						}
//							dialog.setProgress("Downloading from OPM", 20, true);
						customizer.customizeThisFile(opmCon.getInputStreamFromXML(),
								CONF_PARSER.OPM_KEY);
						pwriter = new PrintWriter(new FileWriter(
								updatefile));
						for(int i=0;i<databaseNames.size();i++)
							if (!databaseNames.get(i).equals(PPE_UPDATER.OPM))
								pwriter.println(databaseNames.get(i) + " " + databaseSizes.get(i));
							else
								databaseNamesIDofOPM = i;
						pwriter.println(PPE_UPDATER.OPM + " " + opmCon.getCountOfOPM());
						pwriter.flush();
						pwriter.close();
						
						if (updatefileBackup.exists())
							updatefileBackup.delete();
						
//							System.out.println(">> Update beendet ...");
						dialog.setProgress("Finished", 100, true);
						
//							updateOPM = !(con.getCountOfOPM().equals(databaseSizes.get(databaseNamesIDofOPM)));
						
						try {
							
//							System.out.println("|"+opmCon.getCountOfOPM()+"|"+databaseSizes.get(databaseNamesIDofOPM));
							if (databaseSizes.size() > databaseNamesIDofOPM &&
									Integer.parseInt(opmCon.getCountOfOPM()) != databaseSizes.get(databaseNamesIDofOPM)) {
								DataPool.initMIFHandler();
							} else {
								File mifileBackup = new File(IO.SETTINGS_DIR,IO.MIF_FILENAME_OPM+IO.BACKUP_FILENAME_ADDON);
								if (mifileBackup.exists()) {
//									mifileBackup.delete();
									mifileBackup.renameTo(new File(IO.SETTINGS_DIR, IO.MIF_FILENAME_OPM));
								}
								JOptionPane.showMessageDialog(null,
										"OPM Update is not necessary.", "OPM Update",
										JOptionPane.PLAIN_MESSAGE);
							}
						}catch(final ArrayIndexOutOfBoundsException e) {
							System.out.println("There happened an error when updating the opm.mif file. Please redo the OPM Update in the PDB Update Window.");
							e.printStackTrace();
						}

					}

					dialog.dispose();

				} catch (final Exception e) {
					e.printStackTrace();
				}

			}
		};
		t.setPriority(Thread.MIN_PRIORITY);
		t.start();
		cancelButton.addActionListener(new ActionListener() {

			@SuppressWarnings("deprecation")
			public void actionPerformed(final ActionEvent arg0) {

				t.stop();
		
				try {
					if (pwriter != null) {
						pwriter.flush();
						pwriter.close();
					}
				} catch (Exception e) {
					System.err.println("Error closing PrintWriter.");
//					e.printStackTrace();
				}
				if (updatefileBackup.exists()) {
					if (updatefile.exists())
						updatefile.delete();					
					updatefileBackup.renameTo(new File(IO.SETTINGS_DIR,	IO.UPDATE_FILENAME));
				}
				
				String fileName = "";
				
				if (database == PPE_CONSTANTS.DB.PDBTM) {
					fileName = IO.MIF_FILENAME_PDBTM;
					try {
						if (pdbtmCon.getInputStreamFromXML()!=null)
							pdbtmCon.getInputStreamFromXML().close();
					} catch (IOException e) {
						System.err.println("Error closing PDBTM InputStream.");
//						e.printStackTrace();
					}
				} else if (database == PPE_CONSTANTS.DB.OPM) {
					fileName = IO.MIF_FILENAME_OPM;
					try {
						if (opmCon.getInputStreamFromXML()!=null)
							opmCon.getInputStreamFromXML().close();
					} catch (IOException e) {
						System.err.println("Error closing OPM InputStream.");
//						e.printStackTrace();
					}
				}
				
				if (customizer!=null)
					((SAXCustomizer)customizer).closeStreams();
				else {
					System.err.println("Customizer is not initialized.");
				}
				
				File mifileBackup = new File(IO.SETTINGS_DIR, fileName+IO.BACKUP_FILENAME_ADDON);
				if (mifileBackup!=null && mifileBackup.exists()) {
					File mifile = new File(IO.SETTINGS_DIR, fileName);
					mifile.delete();
					if (!mifileBackup.renameTo(new File(IO.SETTINGS_DIR, fileName)))
						Cm2_Toolkit.complain("There was an error restoring the backup file for\n"+IO.SETTINGS_DIR+File.separator+fileName+
												"\nduring canceling the operation.\n\nThere might be problems with the Protein Placing Extension (PPE) when restarting CmME.\n\n"+
												"Please press the update button again or manually check the file and backup file.", 
												"Problem during Canceling");
				}
				
				dialog.dispose();

			}

		});
		
	}
	
	/**
	 * Get the lastModified number of the file defined by the database.
	 * @param database
	 * @return
	 */
	public static long lastModified(int database) {
		File mifile = new File(IO.SETTINGS_DIR, database==PPE_CONSTANTS.DB.PDBTM?IO.MIF_FILENAME_PDBTM:IO.MIF_FILENAME_OPM);
		if (mifile.exists())
			return mifile.lastModified();
		else
			return 0;
	}
	
	/**
	 * Set the ContenFile information of the file defined by the database.
	 * @param database
	 * @return
	 */
	public static void setContentsFile(int database, JLabel label) {
		File mifile = new File(IO.SETTINGS_DIR, database==PPE_CONSTANTS.DB.PDBTM?IO.MIF_FILENAME_PDBTM:IO.MIF_FILENAME_OPM);
		if (mifile.exists()) {
			label.setText(mifile.getName());
			label.setToolTipText(mifile.getAbsolutePath());
		} else {
			label.setText("-");
		}
	}

}
