/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io.ppe.toolbox;

import java.io.File;

import org.cellmicrocosmos.cm2.io.FileIOHandler;


/**
 * PPE_CONSTANTS contains some subclasses with defined constants.
 * 
 * @author Christian Gamroth, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class PPE_CONSTANTS {

	/**
	 * IO contains constants for the input and output of the
	 * Protein Placement Expansion.
	 * 
	 * @author Administrator
	 * 
	 */
	public static class DB {

		public static int PDBTM = 0;
		
		public static int OPM = 1;

	}

	/**
	 * IO contains constants for the input and output of the
	 * Protein Placement Expansion.
	 * 
	 * @author Christian Gamroth
	 * 
	 */
	public static class IO {

		public static final File SETTINGS_DIR = new File(FileIOHandler.home
				+ "cm2settings" + File.separator);

		public static final String CONF_FILENAME = "ppe.conf";

		public static final String MIF_FILENAME_PDBTM = "pdbtm.mif";

		public static final String MIF_FILENAME_OPM = "opm.mif";
		
		public static final String XML_FILENAME_PDBTM = "pdbtmall.xml";

		public static final String UPDATE_FILENAME = "update.txt";
		
		public static final String BACKUP_FILENAME_ADDON = "_backup";

	}

	/**
	 * CONF_PARSER contains constants belonging to the ppe.conf.
	 * 
	 * @author Christian Gamroth
	 * 
	 */
	public static class CONF_PARSER {

		public static final String PDBTM_KEY = "&_PDBTM";
		
		public static final String OPM_KEY = "&_OPM";

		public static final String PORT_SUBKEY = "&_PORT_CONFIG";

		public static final String XML_SUBKEY = "&_XML_CONFIG";

		public static final String START_TOKEN = "---------->";

		public static final String SUB_TOKEN = "---->";

		public static final String END_TOKEN = "/--------->";

	}

	/**
	 * MIFILE contains constants which are necessary for the
	 * generation of a MIFile by the MIFHandler.
	 * 
	 * @author Christian Gamroth
	 * 
	 */
	public static class MIFILE {

		public static final String START_ELEMENT = "PPE";

		public static final String NEW_ENTRY = "ENTRY";

		public static final String PDBTM_ENTRY = "pdbtm";
		
		public static final String OPM_ENTRY = "opm";

		public static final String NORMAL_ENTRY = "NORMAL";

		public static final String ROWX_ENTRY = "ROWX";

		public static final String ROWY_ENTRY = "ROWY";

		public static final String ROWZ_ENTRY = "ROWZ";

	}

	/**
	 * CONNECTION contains constants which are necessary for the connections.
	 * 
	 * @author Christian Gamroth
	 * 
	 */
	public static class CONNECTION {

		public static final String SUBDIR_PDB_ALL = "count_all.txt";

		public static final String SUBDIR_PDBTM_ALL = "count_tm.txt";

		public static final String SUBDIR_PDBTM_ALPHA = "count_tma.txt";

		public static final String SUBDIR_PDBTM_BETA = "count_tmb.txt";

		public static final String SUBDIR_PDBTM_XMLFILE = "pdbtmall";

		public static final String REQUEST_KEY_AUTHORIZATION = "Authorization";

	}

	/**
	 * PPE_UPDATER contains constants which are necessary for
	 * generating an update.txt.
	 * 
	 * @author Christian Gamroth
	 * 
	 */
	public static class PPE_UPDATER {

		public static final String PDBTM = "pdbtm";
		
		public static final String OPM = "opm";

	}

}
