/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithm;

import java.awt.Color;
import java.awt.Dimension;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.vecmath.Point2d;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * 
 * This panel displays multiple graphs that can be updated frequently
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
@SuppressWarnings("serial")
public class GraphPanel extends JPanel {

	public final XYSeriesCollection dataset = new XYSeriesCollection();

	public final ChartPanel chartPanel;
	public JFreeChart chart;
	public final XYPlot plot;

	/**
	 * Construcs a new Graphpanel
	 * 
	 * @param title
	 *            the title that is displayed on top
	 * @param Xlabel
	 *            the displayed name of the X-Axis
	 * @param Ylabel
	 *            the displayed name if the Y-axis
	 * @param graphs
	 *            the name for each graph that will be displayed
	 * @param colors
	 *            the color for each graph to use
	 */
	public GraphPanel(String title, String Xlabel, String Ylabel,
			String[] graphs, Color[] colors) {

		JFreeChart chart = ChartFactory.createXYLineChart(title, Xlabel,
				Ylabel, dataset, PlotOrientation.VERTICAL, true, true, false);

		chartPanel = new ChartPanel(chart);

		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		this.add(chartPanel);

		plot = chart.getXYPlot();
		plot.setBackgroundPaint(this.getBackground().brighter());

		plot.setRangeGridlinesVisible(false);
		plot.setDomainGridlinesVisible(false);
		plot.setRangeZeroBaselineVisible(true);
		plot.setRangeZeroBaselinePaint(Color.GREEN);
		plot.getDomainAxis().setStandardTickUnits(
				NumberAxis.createIntegerTickUnits());

		final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();

		plot.setRenderer(renderer);
		renderer.setBaseSeriesVisibleInLegend(false);

		for (int i = 0; i < graphs.length; i++) {
			XYSeries series = new XYSeries(graphs[i]);
			dataset.addSeries(series);
			renderer.setSeriesLinesVisible(i, true);
			renderer.setSeriesShapesVisible(i, false);
			renderer.setSeriesPaint(i, colors[i]);

			dataset.setAutoWidth(true);
		}

	}

	/**
	 * Sets the coordinates of the graphs. the first dimension of the passed
	 * array seperates the graphs. The second dimension should contain the
	 * points for each graph.
	 * 
	 * @param point2ds
	 *            the current graph coordinates, for each graph separetely
	 */
	public void setData(Point2d[][] point2ds) {

		List<?> allSeries = dataset.getSeries();

		try {

			for (int i = 0; i < allSeries.size(); i++) {
				XYSeries s = (XYSeries) allSeries.get(i);
				s.clear();

				for (int j = 0; j < point2ds[i].length; j++) {

					if (point2ds[i][j].y == Double.NaN)
						s.add(new Double(point2ds[i][j].x), null);
					else {

						s.add(point2ds[i][j].x, point2ds[i][j].y);
					}
				}

			}

		} catch (Throwable e) {
			// System.out.println(e);
		}

	}

	/**
	 *@inheritDoc
	 */
	public void setPreferredSize(Dimension dim) {
		super.setPreferredSize(dim);
		chartPanel.setPreferredSize(dim);
	}

}
