/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Rectangle;
import java.util.Hashtable;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point2f;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.membranepackingalgorithm.IllegalOperationException;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithmGUI;


/**
 * This algorithm places lipids at randomly picked positions. 
 * Offers various settings for the user helping to make the resulting
 * membrane look nice.
 * 
 * @author Tim Dingersen dackelnose@gmx.net, comments by Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class AdvancedRandomPlacing extends MembranePackingAlgorithm {
	JRadioButton radio1 = new JRadioButton("{0,1,...,359}     Degr. (All)");

	JRadioButton radio2 = new JRadioButton(
			"{0,90,180,270} Degr. (Quarter Turns)");

	JRadioButton radio3 = new JRadioButton(
			"{0}                     Degr. (None)");

	JSlider sl1 = new JSlider();

	JSlider sl2 = new JSlider();

	JSlider sl3 = new JSlider();

	JPanel p1 = new JPanel();

	JPanel p2 = new JPanel();

	JPanel p3 = new JPanel();

	JPanel p4 = new JPanel();

	final int[] rotations90 = { 0, 90, 180, 270 };

	private boolean stop = false;
	
	private int userSelectedDomain = -1;

	public boolean buildAndShowGui() {
		
		// check if the user selected a membrane
		if (this.getSelectedArea() != -1) {

			int val = JOptionPane
					.showConfirmDialog(
							null,
							"You have currently selected a particular area.\nDo you want to consider this area only?\n",
							"Selected Area", JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE);

			if (val == JOptionPane.YES_OPTION) {
				userSelectedDomain = this.getSelectedArea();
			}

		}

		final JPanel sec = new JPanel();
		
		final JPanel main = new JPanel() {

			private static final long serialVersionUID = -2257568683661885677L;

			public void doLayout() {
				if (MembranePackingAlgorithmGUI.getDialogFrame().isVisible())
					return;
				final int w = sec.getPreferredSize().width;
				sec.setPreferredSize(new Dimension(w,
						this.getPreferredSize().height));
				AdvancedRandomPlacing.this.p1.setPreferredSize(new Dimension(w,
						15));
				AdvancedRandomPlacing.this.p2.setPreferredSize(new Dimension(w,
						15));
				AdvancedRandomPlacing.this.p3.setPreferredSize(new Dimension(w,
						15));
				AdvancedRandomPlacing.this.p4.setPreferredSize(new Dimension(w,
						15));
				super.doLayout();

			}
		};

		sec.setLayout(new BoxLayout(sec, BoxLayout.Y_AXIS));

		this.p1.setLayout(new BoxLayout(this.p1, BoxLayout.X_AXIS));
		this.p2.setLayout(new BoxLayout(this.p2, BoxLayout.X_AXIS));
		this.p3.setLayout(new BoxLayout(this.p3, BoxLayout.X_AXIS));
		this.p4.setLayout(new BoxLayout(this.p4, BoxLayout.X_AXIS));

		sec.add(Box.createVerticalStrut(30));

		final JLabel l1 = new JLabel("  Overall Density of Lipids",
				SwingConstants.LEADING);
		this.p1.add(l1);
		this.p1.add(Box.createHorizontalGlue());
		l1.setFont(l1.getFont().deriveFont(Font.BOLD, 11f));
		sec.add(this.p1);
		sec.add(Box.createVerticalStrut(20));
		sec.add(this.sl1);
		sec.add(Box.createVerticalGlue());
		final JLabel l2 = new JLabel("  Possible Turns along Y-Axis",
				SwingConstants.LEADING);
		this.p2.add(l2);
		this.p2.add(Box.createHorizontalGlue());
		l2.setFont(l2.getFont().deriveFont(Font.BOLD, 11f));
		sec.add(Box.createVerticalStrut(20));
		sec.add(this.p2);
		sec.add(Box.createVerticalStrut(20));

		final JPanel r1 = new JPanel();
		final JPanel r2 = new JPanel();
		final JPanel r3 = new JPanel();
		r1.setLayout(new BoxLayout(r1, BoxLayout.X_AXIS));
		r2.setLayout(new BoxLayout(r2, BoxLayout.X_AXIS));
		r3.setLayout(new BoxLayout(r3, BoxLayout.X_AXIS));

		r1.add(this.radio1);
		r2.add(this.radio2);
		r3.add(this.radio3);

		r1.add(Box.createHorizontalGlue());
		r2.add(Box.createHorizontalGlue());
		r3.add(Box.createHorizontalGlue());
		sec.add(r1);
		sec.add(r2);
		sec.add(r3);

		sec.add(Box.createVerticalGlue());
		final JLabel l3 = new JLabel("  Tolerance for Random Tilts",
				SwingConstants.LEADING);
		this.p3.add(l3);
		this.p3.add(Box.createHorizontalGlue());
		l3.setFont(l3.getFont().deriveFont(Font.BOLD, 11f));
		sec.add(Box.createVerticalStrut(20));
		sec.add(this.p3);
		sec.add(Box.createVerticalStrut(20));
		sec.add(this.sl2);
		sec.add(Box.createVerticalGlue());
		final JLabel l4 = new JLabel("  Tolerance for Random Vertical Shifts",
				SwingConstants.LEADING);
		this.p4.add(l4);
		this.p4.add(Box.createHorizontalGlue());
		l4.setFont(l4.getFont().deriveFont(Font.BOLD, 11f));
		sec.add(Box.createVerticalStrut(20));
		sec.add(this.p4);
		sec.add(Box.createVerticalStrut(20));
		sec.add(this.sl3);

		sec.add(Box.createVerticalStrut(20));

		main.add(sec, BorderLayout.WEST);

		final ButtonGroup bg = new ButtonGroup();
		bg.add(this.radio1);
		bg.add(this.radio2);
		bg.add(this.radio3);

		this.sl1.setMinimum(1);
		this.sl1.setMaximum(63);

		this.sl2.setMinimum(0);
		this.sl2.setMaximum(45);

		this.sl3.setMinimum(0);
		this.sl3.setMaximum(100);

		this.sl1.setMajorTickSpacing(1);
		this.sl2.setMajorTickSpacing(1);
		this.sl3.setMajorTickSpacing(1);

		this.sl1.setSnapToTicks(true);
		this.sl2.setSnapToTicks(true);
		this.sl3.setSnapToTicks(true);

		final Hashtable<Integer,JLabel> hash1 = new Hashtable<Integer,JLabel>();

		hash1.put(new Integer(1), new JLabel("Low"));
		hash1.put(new Integer(25), new JLabel("Medium"));
		hash1.put(new Integer(50), new JLabel("High"));
		hash1.put(new Integer(63), new JLabel("        Superb       "));
		((JLabel) hash1.get(new Integer(63))).setForeground(Color.red);
		this.sl1.setLabelTable(hash1);
		this.sl1.setPaintTicks(true);
		this.sl1.setPaintLabels(true);

		final Hashtable<Integer,JLabel> hash2 = new Hashtable<Integer,JLabel>();
		hash2.put(new Integer(0), new JLabel("0"));
		hash2.put(new Integer(10), new JLabel("10"));
		hash2.put(new Integer(20), new JLabel("20"));
		hash2.put(new Integer(30), new JLabel("30"));
		hash2.put(new Integer(40), new JLabel("40"));
		hash2
				.put(new Integer(45), new JLabel(
						"                 45    Degr.  "));

		this.sl2.setLabelTable(hash2);
		this.sl2.setPaintTicks(true);
		this.sl2.setPaintLabels(true);

		final Hashtable<Integer,JLabel> hash3 = new Hashtable<Integer,JLabel>();
		hash3.put(new Integer(0), new JLabel("0"));
		hash3.put(new Integer(10), new JLabel("1"));
		hash3.put(new Integer(20), new JLabel("2"));
		hash3.put(new Integer(30), new JLabel("3"));
		hash3.put(new Integer(40), new JLabel("4"));
		hash3.put(new Integer(50), new JLabel("5"));
		hash3.put(new Integer(60), new JLabel("6"));
		hash3.put(new Integer(70), new JLabel("7"));
		hash3.put(new Integer(80), new JLabel("8"));
		hash3.put(new Integer(90), new JLabel("9"));
		hash3
				.put(new Integer(100), new JLabel(
						"               10     Angs.  "));

		this.sl3.setLabelTable(hash3);
		this.sl3.setPaintTicks(true);
		this.sl3.setPaintLabels(true);

		this.radio1.setSelected(true);

		this.sl1.setValue(25);
		this.sl2.setValue(5);
		this.sl3.setValue(10);

		return MembranePackingAlgorithmGUI.popUpGUI(main,
				"Starting \"Advanced Random Placing Algorithm\"",
				"Set up Parameters");

	}

	/**
	 * The runtime of this algorithm is:
	 * 
	 * g*(s*(A+b*B)) = g*s*n, where
	 * 
	 *  g = sectors of the membrane grid
	 *  s = salary, generated by the user-defined and the membrane size
	 *  A = side A (newlipid) placing attempt
	 *  b = number of side B retries
	 *  B = side B (otherlipid) placing attempt
	 *  n = (T+o*O) = each lipid
	 */
	public void fillWithLipids() throws IllegalOperationException {

		final Rectangle bounds = new Rectangle(0, 0,
				this.getMembraneSize().width, this.getMembraneSize().height);

		// the grid devisor constant
		final int steps = 20;

		// the sector boundaries of each grid segment
		final float sectX = (float) bounds.width / steps;
		final float sectZ = (float) bounds.height / steps;

		// the salary depends on the membrane dimensions
		float salary = (bounds.width * bounds.height) / 2;
		
		// define the overall density of membrane
		final float density = (((float) this.sl1.getValue()) * 2) / 100000;

		float remove = 1;

		remove += ((((float) this.sl1.getValue()) * 2 - 75) / 75) * (-1);

		salary *= density;

		if (this.sl1.getValue() * 2 > 100)
			salary += ((this.sl1.getValue() * 2 - 100)) * salary;

		// define the actual grid segment X/Z-values
		float LX = bounds.x; 			//left 
		float RX = bounds.x + sectX;	//right
		float UZ = bounds.y;			//up
		float DZ = bounds.y + sectZ;	//down

		boolean dirtoggle = true;

		final int todo = (steps * steps) + steps;
		int progress = 1;

		final int oppretries = this.sl1.getValue();//opposite retries

		boolean toggleSide = (this.lipidsEC.length!=0)?true:false;
		
		//  RUNTIMEVAR: g
		while (true) {
			if (this.stop)
				return;

			//  RUNTIMEVAR: s
			float budget = salary;
			
			// original position from Tim, did not work if there was only 
			// one lipid type on both membrane layer; the filling of the
			// intra side was wrong:
			// boolean toggleSide = (this.lipidsEC.length!=0)?true:false;
			
			// define the actual grid segment rectangle
			final Rectangle currentbounds = new Rectangle((int) LX, (int) UZ,
					(int) (RX - LX), (int) (DZ - UZ));

			while (budget > 0) {
				if (this.stop)
					return;
				// change membrane side
				toggleSide = !toggleSide;

				// newlipid placing
				//  RUNTIMEVAR: A
				Lipid newlipid;

				// generate a random position inside the current actual grid segment
				final Point2f newpos = this
						.createRandomXZPosition(currentbounds);

				// handle the actual layer ID
				final int layerid = this.getCurrentLayerID();
				final int domainID = this
						.getMicroDomainAtPoint(newpos, layerid);
				if (userSelectedDomain!=-1 && userSelectedDomain!=domainID) 
					break;

				// create a new lipid at
				//  intra default area
				if (toggleSide && domainID < 1) {
					newlipid = this.createNextIntracellularLipid(layerid);
				//  intra microdomain area
				} else if (toggleSide && domainID > 0) {
					newlipid = this.getMicroDomainAt(domainID - 1)
							.createNextIntracellularLipid();
				//  extra default area
				} else if (!toggleSide && domainID < 1) {
					newlipid = this.createNextExtracellularLipid(layerid);
				//  extra microdomain area
				} else {
					newlipid = this.getMicroDomainAt(domainID - 1)
							.createNextExtracellularLipid();
				}
				
				// if there were problems creating the newlipid, skip this run
				if (newlipid == null) {
					budget -= remove;
					continue;
				}

				float shiftex;

				// generate and add the random shift along the Y-axis to the 
				//  actual lipid, this may be deactivated by the user by 
				//  setting the according slider to 0
				shiftex = this.generateRandomShift();
				final Point3f firstpos = new Point3f(newpos.x, newlipid
						.getYTranslation()
						+ shiftex, newpos.y);
				newlipid.setPosition(firstpos);

				// generate and add the random rotation along the Y-axis
				//  to the actual lipid, this may be deactivated by the
				//  user by using the according radio button
				newlipid.rotY(this.generateRandomRotation());
				this.tilt(newlipid);

				// skip this placement if there are intersections
				if (this.intersectsBorders(newlipid)) {
					budget -= remove;
					continue;
				}
				if (this.intersects(newlipid)) {
					budget -= remove;
					continue;
				}

				// add newlipid to the membrane
				if (toggleSide)
					this.addIntracellularLipid(newlipid);
				else
					this.addExtracellularLipid(newlipid);

				
				// otherlipid placing
				//  RUNTIMVAR: B
				Lipid otherlipid;
				
				// create the otherlipid for the opposite membrane side at the correct domain
				if (toggleSide && domainID < 1)
					otherlipid = this.createNextExtracellularLipid(layerid);
				else if (toggleSide && domainID > 0)
					otherlipid = this.getMicroDomainAt(domainID - 1)
							.createNextExtracellularLipid();
				else if (!toggleSide && domainID < 1)
					otherlipid = this.createNextIntracellularLipid(layerid);
				else
					otherlipid = this.getMicroDomainAt(domainID - 1)
							.createNextIntracellularLipid();

				// if there were problems creating the otherlipid, skip this run
				if (otherlipid == null) {
					budget -= remove;
					continue;
				}
				// create the otherlipid with the same configuration as the added newlipid
				//  only the Y-translation differs (because it is at the other side)
				final Point3f secondpos = new Point3f(newpos.x, otherlipid
						.getYTranslation()
						+ shiftex, newpos.y);
				otherlipid.setPosition(secondpos);

				// backup the matrix
				final Matrix3f old = this.copyMatrix(otherlipid
						.getRotationMatrix());

				// retries trying to set the opposite lipid
				//  RUNTIMEVAR: b
				for (int t = 0; t < oppretries; t++) {
					otherlipid.setRotationMatrix(this.copyMatrix(old));
					// generate a random rotation
					otherlipid.rotY(this.generateRandomRotation());
					// generate a random tilt
					this.tilt(otherlipid);
					
					// if there is an intersection, continue with this loop
					if (this.intersectsBorders(otherlipid)) {
						budget -= remove;
						continue;
					}
					if (this.intersects(otherlipid)) {
						budget -= remove;
						continue;
					}

					// add the lipid successfully and escape the loop
					if (toggleSide)
						this.addExtracellularLipid(otherlipid);
					else
						this.addIntracellularLipid(otherlipid);

					break;
				}

			}
			
			// show the actual progress
			progress++;
			this.setDisplayedProgress((int) (progress * (100f / todo)));

			// update the actual grid segment X/Z-values
			if (dirtoggle) {
				LX += sectX;
				RX += sectX;
			} else {
				LX -= sectX;
				RX -= sectX;
			}

			if ((RX > (bounds.x + bounds.width) || LX < bounds.x)) {
				dirtoggle = !dirtoggle;
				UZ += sectZ;
				DZ += sectZ;
			}

			// finish the algorithm if the grid is exceeded
			if (DZ > (bounds.y + bounds.height))
				break;

		}

		// /////////////////////////////

		// ////////////////////////////////

		// for (int i = 0; i < this.getMicroDomainCount(); i++) {
		//
		// MicroDomain micro = this.getMicroDomainAt(i);
		// if (!micro.hasAnyExtracellularLipids()
		// && !micro.hasAnyIntracellularLipids())
		// return;
		//
		// bounds = micro.getBounds();
		//
		// steps = 20;
		//
		// sectX = (float) bounds.width / steps;
		// sectZ = (float) bounds.height / steps;
		//
		// salary = (bounds.width * bounds.height) / 2;
		// density = (((float) sl1.getValue()) * 2) / 100000;
		//
		// remove = 1;
		//
		// remove += ((((float) sl1.getValue()) * 2 - 75) / 75) * (-1);
		//
		// salary *= density;
		//
		// if (sl1.getValue() * 2 > 100)
		// salary += ((float) (sl1.getValue() * 2 - 100)) * salary;
		//
		// LX = bounds.x;
		// RX = bounds.x + sectX;
		// UZ = bounds.y;
		// DZ = bounds.y + sectZ;
		//
		// toggle = true;
		//
		// todo = (steps * steps) + steps;
		// progress = 1;
		//
		// bothsides = micro.hasAnyExtracellularLipids()
		// && micro.hasAnyIntracellularLipids();
		//
		// while (true) {
		// if (stop)
		// return;
		// float budget = salary;
		//
		// boolean ECfirstthenIC = false;
		//
		// while (budget > 0) {
		// if (stop)
		// return;
		// ECfirstthenIC = !ECfirstthenIC;
		// Lipid ex;
		// if (bothsides) {
		// if (ECfirstthenIC)
		// ex = micro.createNextExtracellularLipid();
		// else
		// ex = micro.createNextIntracellularLipid();
		// } else if (micro.hasAnyExtracellularLipids())
		// ex = micro.createNextExtracellularLipid();
		// else
		// ex = micro.createNextIntracellularLipid();
		//
		// float ranX = random.nextInt((int) sectX);
		// float ranZ = random.nextInt((int) sectZ);
		//
		// ranX += LX;
		// ranZ += UZ;
		//
		// ranX += random.nextFloat();
		// ranZ += random.nextFloat();
		//
		// float shiftex;
		// if (bothsides) {
		// if (ECfirstthenIC)
		// shiftex = this.generateRandomShift();
		// else
		// shiftex = -this.generateRandomShift();
		// } else if (micro.hasAnyExtracellularLipids())
		// shiftex = this.generateRandomShift();
		// else
		// shiftex = -this.generateRandomShift();
		//
		// Point3f firstpos = new Point3f(ranX, ex.getYTranslation()
		// + shiftex, ranZ);
		//
		// ex.setPosition(firstpos);
		//
		// ex.rotY(this.generateRandomRotation());
		// this.tilt(ex);
		//
		// if (this.intersectsBorders(ex)) {
		// budget -= remove;
		// continue;
		// }
		// if (!micro.isInsideArea(ex) || this.intersects(ex)) {
		// budget -= remove;
		// continue;
		// }
		//
		// try {
		// if (bothsides) {
		// if (ECfirstthenIC)
		// this.addExtracellularLipid(ex);
		// else
		// this.addIntracellularLipid(ex);
		// } else if (micro.hasAnyExtracellularLipids())
		// this.addExtracellularLipid(ex);
		// else
		// this.addIntracellularLipid(ex);
		//
		// } catch (IllegalOperationException e) {
		//
		// }
		//
		// if (bothsides) {
		// Lipid in;
		// boolean added = false;
		// for (int j = 0; j < 3; j++) {
		// if (stop)
		// return;
		// if (ECfirstthenIC)
		// in = micro.createNextIntracellularLipid();
		// else
		// in = micro.createNextExtracellularLipid();
		// float addX = random.nextFloat();
		//
		// float addZ = random.nextFloat();
		//
		// if (random.nextBoolean())
		// addX *= (-1);
		// if (random.nextBoolean())
		// addZ *= (-1);
		//
		// Point3f secondpos = new Point3f(ranX + addX, in
		// .getYTranslation()
		// + shiftex, ranZ + addZ);
		//
		// in.setPosition(secondpos);
		// in.rotY(this.generateRandomRotation());
		// this.tilt(in);
		// if (this.intersectsBorders(in)) {
		// budget -= remove;
		// continue;
		// }
		// if (!micro.isInsideArea(in) || this.intersects(in)) {
		// budget -= remove;
		// continue;
		// }
		//
		// try {
		// if (ECfirstthenIC)
		// this.addIntracellularLipid(in);
		// else
		// this.addExtracellularLipid(in);
		//
		// added = true;
		// break;
		// } catch (IllegalOperationException e) {
		//
		// }
		//
		// }
		// if (added)
		// continue;
		//
		// while (budget > 0) {
		// if (stop)
		// return;
		// ranX = random.nextInt((int) sectX);
		// ranZ = random.nextInt((int) sectZ);
		//
		// ranX += LX;
		// ranZ += UZ;
		//
		// ranX += random.nextFloat();
		// ranZ += random.nextFloat();
		//
		// if (ECfirstthenIC)
		// in = micro.createNextIntracellularLipid();
		// else
		// in = micro.createNextExtracellularLipid();
		//
		// float shiftin;
		// if (ECfirstthenIC)
		// shiftin = -this.generateRandomShift();
		// else
		// shiftin = this.generateRandomShift();
		//
		// firstpos = new Point3f(ranX, in.getYTranslation()
		// + shiftin, ranZ);
		//
		// in.setPosition(firstpos);
		// in.rotY(this.generateRandomRotation());
		// this.tilt(in);
		// if (this.intersectsBorders(in)) {
		// budget -= remove;
		//
		// continue;
		// }
		// if (!micro.isInsideArea(in) || this.intersects(in)) {
		// budget -= remove;
		//
		// continue;
		// }
		//
		// try {
		// if (ECfirstthenIC)
		// this.addIntracellularLipid(in);
		// else
		// this.addExtracellularLipid(in);
		//
		// break;
		// } catch (IllegalOperationException e) {
		//
		// }
		//
		// }
		//
		// }
		// }
		// progress++;
		//
		// this
		// .setDisplayedProgress((int) ((float) progress * (100f / (float)
		// todo)));
		//
		// if (toggle) {
		// LX += sectX;
		// RX += sectX;
		// } else {
		// LX -= sectX;
		// RX -= sectX;
		// }
		//
		// if ((RX > (bounds.x + bounds.width) || LX < bounds.x)) {
		// toggle = !toggle;
		// UZ += sectZ;
		// DZ += sectZ;
		// }
		//
		// if (DZ > (bounds.y + bounds.height))
		// break;
		//
		// }
		//
		// }

	}

	/**
	 * Generate a random shift along the Y-axis.
	 * @return float random shift
	 */
	private float generateRandomShift() {
		final int slshiftval = this.sl3.getValue();
		if (slshiftval == 0)
			return 0;

		float ret = ((float) this.random.nextInt(slshiftval + 1)) / 10;
		if (this.random.nextBoolean())
			ret = -ret;
		return ret;

	}

	/**
	 * Tilt the given lipid alng the X- and Z-axis. 
	 * 
	 * @param lip the lipid 
	 */
	private void tilt(final Lipid lip) {
		if (this.sl2.getValue() == 0)
			return;

		int tiltX = this.random.nextInt(this.sl2.getValue() + 1);
		int tiltZ = this.random.nextInt(this.sl2.getValue() + 1);
		;

		if (this.random.nextBoolean())
			tiltX = 360 - tiltX;
		if (this.random.nextBoolean())
			tiltZ = 360 - tiltZ;

		lip.rotX(tiltX);
		lip.rotZ(tiltZ);

	}

	/**
	 * Generate a random rotation along the Y-axis.
	 * 
	 * @return int random rotation
	 */
	private int generateRandomRotation() {
		if (this.radio1.isSelected())
			return this.random.nextInt(360);
		else if (this.radio2.isSelected())
			return this.rotations90[this.random.nextInt(4)];
		else
			return 0;
	}

	public String getFullInformation() {

		final String s = "This algorithm, like \'Random Algorithm\' places\n"
				+ "lipids at randomly picked positions. Offers various\n"
				+ "settings for the user helping to make the resulting\n"
				+ "membrane look nice"

				+ "\n\n\n" + "Functioning: Advanced Trial and Error placing\n"
				+ "Orientation: Geometric\n"
				+ "Speed: Average (depends on setup values)\n"
				+ "Biological correctness: None\n\n"
				
				+ "Author: Tim Dingersen\n"
				+ "Version: 1.01";
		return "<html>" + s.replaceAll("\n", "<br>") + "</html>";
	}

	public String getName() {

		return "Advanced Random Placing";
	}

	public String getShortInformation() {

		return "Advanced random based lipid filling algorithm (Version 1.01)";
	}

	public String getStringRepresentationOfSetupParameters() {
		String s = "Density: ";

		if (this.sl1.getValue() < 18)
			s += "Low";
		else if (this.sl1.getValue() < 36)
			s += "Medium";
		else if (this.sl1.getValue() < 55)
			s += "High";
		else
			s += "Superb";

		s += "\n";
		s += "Tolerance for Tilts :" + this.sl2.getValue();
		s += "\n";
		s += "Tolerance for Shifts :" + this.sl3.getValue() / 10;
		s += "\n";
		s += "Selected Layer/Domain: "+userSelectedDomain+"/"+this.getCurrentLayerID();
		return s;
	}

	public void stopAlgorithm() {
		this.stop = true;

	}

	public boolean supportsAbsoluteValues() {
		return false;
	}

	public boolean supportsMicrodomains() {
		return true;
	}

	public boolean overrideExistingLipids() {
		return false;
	}

	public Component getRuntimeControlPanel() {
		return null;
	}
	
}
