/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.Vector;

import javax.media.j3d.BoundingBox;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.vecmath.Point2f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector2f;
import javax.vecmath.Vector4d;

import org.cellmicrocosmos.cm2.membranepackingalgorithm.IllegalOperationException;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithmGUI;


/**
 * 
 * Distributes lipids onto a membrane trying to reach the pre-defined absolute
 * lipid numbers.
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class Distributor extends MembranePackingAlgorithm {

	private boolean stop = false;

	private boolean pause = false;
	
	JScrollPane guipanel;

	JSpinner spinnerExtra = new JSpinner();
	JSpinner spinnerExtraRatio = new JSpinner();
	JSpinner spinnerIntra = new JSpinner();
	JSpinner spinnerIntraRatio = new JSpinner();
	JButton applyBtn = new JButton("Apply Counts");
	JButton pauseBtn = new JButton("Pause");
	JCheckBox stopIfRatioReached = new JCheckBox("AutoStop");
	
	public JCheckBox cbAlgoAddLipids = new JCheckBox("Add new Lipids");
	public JCheckBox cbAlgoApproach = new JCheckBox("Approach Proteins or Membrane Center");
	public JCheckBox cbAlgoShake = new JCheckBox("Shake");
	public JCheckBox cbAlgoTwist = new JCheckBox("Twist");

	private Rectangle2D.Double bounds;

	private boolean haveToUpdateCounts = false;
//	private boolean pressedUpdateCounts = false;

	private Lipid[] lipEC;

	private Lipid[] lipIC;

	// contains the absolute distances of 
	//  lip.z-low.z(0 or w), lip.x-top.x(1 or x), lip.z-top.z(2 or y), lip.x-low.x(3 or z),
	//  where lip is the lipid position, top is the upper end of the boundary box and
	//  low is the lower end of the boundary box:
	//  1----|    2
	//            |      
	//   -   X    -
	//   |
	//   0   |----3
	private Vector4d[] diststancesEC;
	private Vector4d[] diststancesIC;

	// contains the absolute path (representing the lipid type) 
	//  and radius on the x/z plane for each lipid
	private Hashtable<String, Float> radsEC;
	private Hashtable<String, Float> radsIC;

	private RandomPlacing RP = null;
	
	private int userSelectedDomain = -1;
	
	public boolean buildAndShowGui() {
		
		this.setStartInBackgroundAllowed(false);

		// check if the user selected a membrane
		if (this.getSelectedArea() != -1) {

			int val = JOptionPane
					.showConfirmDialog(
							null,
							"You have currently selected a particular area.\nDo you want to consider this area only?\n",
							"Selected Area", JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE);

			if (val == JOptionPane.YES_OPTION) {
				userSelectedDomain = this.getSelectedArea();
				
				if ((this.getAllSampleLipids_ExtraCellular(this.getCurrentLayerID(),userSelectedDomain).length == 0) &&
					(this.getAllSampleLipids_IntraCellular(this.getCurrentLayerID(),userSelectedDomain).length == 0)) {
					JOptionPane
						.showMessageDialog(
								null,
								"Your selected area does not contain any Sample Lipids.\nPlease add at least one Sample Lipid from the Local Database.");
					return false;
				}
			}

		}
		
		this.bounds = new Rectangle2D.Double(0, 0,
				this.getMembraneSize().width, this.getMembraneSize().height);

		double areaOfAllMicrodomains = 0;
		// this is only needed if there are microdomains and the default area is selected
		if (this.getMicroDomainCount()>0 && this.userSelectedDomain == 0) {
			for (int i=0;i<this.getMicroDomainCount();i++){
				areaOfAllMicrodomains =
					areaOfAllMicrodomains + this.getMicroDomainAt(i).getArea();
			}
		}

		final double areaOfSelectedMicrodomain = 
						((this.userSelectedDomain < 1)?
								this.getMembraneArea()-
									((this.userSelectedDomain == 0)?areaOfAllMicrodomains:0):
								this.getMicroDomain(this.userSelectedDomain).getArea() - areaOfAllMicrodomains );
		
		JPanel guicontent = new JPanel();
		
		guicontent.add(Box.createHorizontalGlue());
		guicontent.add(new JLabel("Lipids EC", SwingConstants.TRAILING));
		guicontent.add(Box.createHorizontalStrut(5));
		guicontent.add(this.spinnerExtra);
		guicontent.add(Box.createHorizontalStrut(5));
		guicontent.add(this.spinnerExtraRatio);
		guicontent.add(Box.createHorizontalStrut(5));
		guicontent.add(new JLabel("<HTML>Lipids/&Aring;&sup2;</HTML>", SwingConstants.LEADING));
		guicontent.add(Box.createHorizontalStrut(25));
		guicontent.add(new JLabel("Lipids IC", SwingConstants.TRAILING));
		guicontent.add(Box.createHorizontalStrut(5));
		guicontent.add(this.spinnerIntra);
		guicontent.add(Box.createHorizontalStrut(5));
		guicontent.add(this.spinnerIntraRatio);
		guicontent.add(Box.createHorizontalStrut(5));
		guicontent.add(new JLabel("<HTML>Lipids/&Aring;&sup2;</HTML>", SwingConstants.LEADING));
		guicontent.add(Box.createHorizontalStrut(20));
		guicontent.add(this.applyBtn);
		guicontent.add(Box.createHorizontalStrut(10));
		guicontent.add(this.stopIfRatioReached);
		guicontent.add(Box.createHorizontalStrut(10));
		guicontent.add(this.pauseBtn);
		guicontent.add(Box.createHorizontalGlue());
		
		this.guipanel = new JScrollPane(guicontent);
		this.guipanel.setPreferredSize(new Dimension(100,guicontent.getPreferredSize().height+15));
		this.guipanel.setMinimumSize(new Dimension(100,guicontent.getMinimumSize().height+15));
		this.guipanel.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		this.guipanel.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		
		this.stopIfRatioReached.setSelected(true);
		this.stopIfRatioReached.setToolTipText("Select this if the algorithm should automatically stop if the defined ratios are reached.");
		
		int startLipidsEC = 0;
		int startLipidsIC = 0;
		int maxLipidsEC = 0;
		int maxLipidsIC = 0;

		// compute the maximum number of lipids if algorithm is used in absolute values mode
		if (this.isComputeAbsoluteValuesSelected()) {

			Integer[] percentagesEC = this.getPercentages_ExtraCellular(this.getCurrentLayerID(), userSelectedDomain);
			for (int i=0;i<percentagesEC.length; i++) {
				startLipidsEC += percentagesEC[i];
			}
			maxLipidsEC = startLipidsEC;
			
			Integer[] percentagesIC = this.getPercentages_IntraCellular(this.getCurrentLayerID(), userSelectedDomain);
			for (int i=0;i<percentagesIC.length; i++) {
				startLipidsIC += percentagesIC[i];
			}
			maxLipidsIC = startLipidsIC;
					
		// compute the maximum number of lipids if algorithm is used in default percentages mode
		} else {
			
			startLipidsEC = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA).size();
			startLipidsIC = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA).size();
			
			maxLipidsEC = (int) ((this.bounds.width * this.bounds.height) / 3);
			maxLipidsIC = (int) ((this.bounds.width * this.bounds.height) / 3);
			
		}

		// generate the spinners for the extracellular side and add the listeners
		this.spinnerExtra.setModel(new SpinnerNumberModel(startLipidsEC, 0, maxLipidsEC, 1));
		this.spinnerExtraRatio.setModel(new SpinnerNumberModel(0.0d, 0.0d, 1000.0d, 1.0d));
		this.spinnerExtraRatio.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				JSpinner actualSpinner = (JSpinner)e.getSource();
				spinnerExtraRatio.setValue(actualSpinner.getValue());
				float newValue = new Float(new Double(	areaOfSelectedMicrodomain / 
														(Double) spinnerExtraRatio.getValue()	) );
				spinnerExtra.setValue(Math.round(newValue));
			}
		});
		this.spinnerExtra.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				JSpinner actualSpinner = (JSpinner)e.getSource();
				spinnerExtra.setValue(actualSpinner.getValue());
				if ((Integer) spinnerExtra.getValue() != 0)
					spinnerExtraRatio.setValue(new Double(	areaOfSelectedMicrodomain /
															new Double((Integer)spinnerExtra.getValue())) );
				else
					spinnerExtraRatio.setValue(0.0d);
			}
		});
		if (this.getAllSampleLipids_ExtraCellular(this.getCurrentLayerID(),userSelectedDomain).length == 0) {
			spinnerExtra.setEnabled(false);
			spinnerExtraRatio.setEnabled(false);
		}
		this.spinnerExtra.setValue(startLipidsEC);
		if ((Integer) spinnerExtra.getValue() != 0)
			spinnerExtraRatio.setValue(new Double(	areaOfSelectedMicrodomain /
													new Double((Integer)spinnerExtra.getValue())) );
		else
			spinnerExtraRatio.setValue(0.0d);
		
		// generate the spinners for the intracellular side and add the listeners
		this.spinnerIntra.setModel(new SpinnerNumberModel(startLipidsIC, 0, maxLipidsIC, 1));
		this.spinnerIntraRatio.setModel(new SpinnerNumberModel(0.0d, 0.0d, 1000.0d, 1.0d));
		this.spinnerIntraRatio.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				JSpinner actualSpinner = (JSpinner)e.getSource();
				spinnerIntraRatio.setValue(actualSpinner.getValue());
				float newValue = new Float(new Double(	areaOfSelectedMicrodomain / 
														(Double) spinnerIntraRatio.getValue()	));
				spinnerIntra.setValue(Math.round(newValue));
			}
		});
		this.spinnerIntra.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				JSpinner actualSpinner = (JSpinner)e.getSource();
				spinnerIntra.setValue(actualSpinner.getValue());
				if ((Integer) spinnerIntra.getValue() != 0)
					spinnerIntraRatio.setValue(new Double(	areaOfSelectedMicrodomain /
															new Double((Integer)spinnerIntra.getValue())) );
				else
					spinnerIntraRatio.setValue(0.0d);
			}
		});
		if (this.getAllSampleLipids_IntraCellular(this.getCurrentLayerID(),userSelectedDomain).length == 0) {
			spinnerIntra.setEnabled(false);
			spinnerIntraRatio.setEnabled(false);
		}
		this.spinnerIntra.setValue(startLipidsIC);
		if ((Integer) spinnerIntra.getValue() != 0)
			spinnerIntraRatio.setValue(new Double(	areaOfSelectedMicrodomain /
													new Double((Integer)spinnerIntra.getValue())) );
		else
			spinnerIntraRatio.setValue(0.0d);
		
		this.applyBtn.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {
				Distributor.this.setStartingTime(System.currentTimeMillis());
				Distributor.this.haveToUpdateCounts = true;
//				Distributor.this.pressedUpdateCounts = true;
				if (Distributor.this.pause) {
					Distributor.this.pause = false;
					Distributor.this.pauseBtn.setText("Pause");
				}
				if (Distributor.this.isStartInBackgroundSelected())
					Distributor.this.sendToSystemTray();
			}

		});
		
		this.pauseBtn.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {
				Distributor.this.pause = !Distributor.this.pause;
				if (Distributor.this.pause)
					Distributor.this.pauseBtn.setText("Distribute");
				else
					Distributor.this.pauseBtn.setText("Pause");
			}

		});

		
		
		// building up the options panel for the Random Placing GUI
		JPanel controlPanel = new JPanel();
		
		cbAlgoAddLipids.setSelected(true);
		cbAlgoApproach.setSelected(true);
		cbAlgoShake.setSelected(true);
		cbAlgoTwist.setSelected(true);
		
		controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.Y_AXIS));
		controlPanel.add(cbAlgoAddLipids);
		controlPanel.add(cbAlgoApproach);
		controlPanel.add(cbAlgoShake);
		controlPanel.add(cbAlgoTwist);

		/*
		 * showing the GUI to the user. The returnvalue indicates if the user
		 * confirmed or cancelled the dialog so far
		 */
		boolean returnValue = MembranePackingAlgorithmGUI.popUpGUI(controlPanel,
				"Distributor v. 1.11", "Choose applied sub-algorithms");
		
		// if nothing was selected exit the algorithm
		if (!cbAlgoAddLipids.isSelected() && !cbAlgoApproach.isSelected() && !cbAlgoShake.isSelected() && !cbAlgoTwist.isSelected()) {
			return false;
		} else {		
			return returnValue;
		}
		
//		return true;
	}

	public String getFullInformation() {
		final String s = "This algorithm enables you to specify a total count\n"
				+ "of lipids or the area per lipid in A² on each membrane\n"
				+ "side. The differencing number will then either be\n"
				+ "added to or removed from the membrane.\n"
				+ "Furthermore, all lipids will undergo an even\n"
				+ "distribution as good as possible.\n\n"
				+ "You can choose if the runtime should be infinite.\n"
				+ "This means that you can quit it manually as soon as \n"
				+ "you are satisfied with the results. By default, the\n"
				+ "algorithm automatically quits if the defined \n"
				+ "distribution is achieved.\n"
				+ "Microdomains are fully supported: If you have selected\n" 
				+ "a special microdomain before running the algorithm,\n"
				+ "only the area of the selection is taken into account.\n"
				+ "\n"
				+ "This algorithm is best to use if you want to create an\n"
				+ "initial state for MD-simulations because the average\n"
				+ "space between the lipids is rather high, depending on\n"
				+ "the chosen density.\n"
				+ "\n"
				+ "Note: The Starting Time is reset to the actual time\n"
				+ "each time the Apply Button was pressed."
				+ "\n\n\n"
				+ "Functioning: Repulsion between lipids\n"
				+ "Orientation: Geometric\n"
				+ "Speed: Fast\n"
				+ "Biological correctness: None\n\n" 
				
				+ "Authors: Tim Dingersen, Bj&ouml;rn Sommer\n"
				+ "Version: 1.2";
		
		return "<html>" + s.replaceAll("\n", "<br>") + "</html>";
	}

	public String getShortInformation() {

		return "Places a defined number of lipids and spreads them evenly (Version 1.2)";
	}

	public String getName() {

		return "Distributor";
	}

	public Component getRuntimeControlPanel() {
//		this.updatePanel();
		return this.guipanel;
	}

	public String getStringRepresentationOfSetupParameters() {
		
		String s="Distributor Settings:\n";
		s += " Number/Ratio of extra. lipids: "+spinnerExtra.getValue()+"/"+spinnerExtraRatio.getValue()+"\n";
		s += " Number \" finally reached     : "+this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA).size()+"\n";
		s += " Number/Ratio of intra. lipids: "+spinnerIntra.getValue()+"/"+spinnerIntraRatio.getValue()+"\n";
		s += " Number \"      finally reached: "+this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA).size()+"\n";
		s += "\n";
		
		s += "Applied Random Placing Sub-Algorithms:\n";
		if (cbAlgoAddLipids.isSelected())
			s += " Add Lipids\n";
		if (cbAlgoApproach.isSelected())
			s += " Approach\n";
		if (cbAlgoShake.isSelected())
			s += " Shake\n";
		if (cbAlgoTwist.isSelected())
			s += " Twist\n";
		s += "\n";
		
		s += "Selected Layer/Domain: "+userSelectedDomain+"/"+this.getCurrentLayerID();
		
		s += "Selected Layer/Domain        : "+userSelectedDomain+"/"+this.getCurrentLayerID();
		
		return s;
	}

	public boolean overrideExistingLipids() {

		return false;
	}

	public void stopAlgorithm() {
		if (this.RP != null) {
			this.RP.stopAlgorithm();

		}
		this.stop = true;

	}
	

	public boolean supportsAbsoluteValues() {
		return true;
	}

	public boolean supportsMicrodomains() {
		return true;
	}

	/**
	 * The runtime of this algorithm is:
	 * 
	 * r*((n*M || RP) + n*D), where
	 * 
	 *  r = repetitions until the user stopped the algorithm or the defined values are reached
	 *  n = number of actual lipids in the membrane
	 *  M = mix up attempt (repositioning of already placed lipid)
	 *  D = distribute procedure
	 *   
	 * The runtime of RandomPlacing (RP) is:
	 * 
	 * RP = (l*A+n*P+l*S), where
	 * 
	 *  l = limit = membrane width + membrane depth
	 *  A = lipid random placing attempt
	 *  n = number of actual lipids in the membrane
	 *  P = protein or membrane center approaching
	 *  S = shaking and/or twisting
	 * 
	 */
	public void fillWithLipids() throws Exception {

		this.updateStuff();
		this.applyBtn.setEnabled(false);
		
		// pause the distribution if there are already lipids added, this should preserve
		// the constancy if a filled area should be extended by additional lipids
		if ((this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA).size() != 0) || 
			(this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA).size() != 0)) {
			Distributor.this.pause = true;
			Distributor.this.pauseBtn.setText("Distribute");
		} 
//		else {
//			// RUNTIMEVAR: [n*M]
//			System.out.println("mix up 1");
//			this.mixUp();
//		}
		
		// limits the number of lipids to compare during the Distribution process
		int cutoff = 4;
		boolean toggle = false;
		this.haveToUpdateCounts = true;

		while (true) {

			if (this.haveToUpdateCounts) {
				this.pauseBtn.setEnabled(false);
				// this method contains the RandomPlacing run
				//  RUNTIMEVAR: [n*M] + RP
				this.performUpdateCounts();
				this.updateStuff();
				this.pauseBtn.setEnabled(true);
				this.haveToUpdateCounts = false;
			}

			final int tohaveEC = (Integer) this.spinnerExtra.getValue();
			final int tohaveIC = (Integer) this.spinnerIntra.getValue();

			final int haveEC = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA)
					.size();
			final int haveIC = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA)
					.size();

			if (tohaveEC != haveEC || tohaveIC != haveIC) {
				this.applyBtn.setEnabled(true);
			} else {
				this.applyBtn.setEnabled(false);
				// use this to stop immediately without one further distribution run
//				if (this.stopIfRatioReached.isSelected() && tohaveEC!=0 && tohaveIC!=0)
//					this.stop = true;
			}

			// the Distribution part of the algorithm starts here
			//  RUNTIMEVAR: n*D
			
			if (this.stop)
				break;
			if (this.pause)
				continue;
			this.displayInformation("Distributing... Press [STOP] to finish!");
			toggle = !toggle;

			final Lipid[] lipids = toggle ? this.lipEC : this.lipIC;
			final int size = lipids.length;

			// if(anneal==false&&this.checkDistribution())
			// this.anneal=true;

			if (size < 2)
				continue;

			final Vector4d[] boxes = toggle ? this.diststancesEC
					: this.diststancesIC;

			final Hashtable<String, Float> radiuses = toggle ? this.radsEC
					: this.radsIC;

			// the following loop will check every 3rd lipid,
			//  if there are only two lipids, every second lipid will be checked
			//  this is only important to give the user more often the possibility to
			//  pause or stop the algorithm; anyway, normally all lipids are checked here
			final int times = size == 2 ? 2 : 3;

			for (int e = 0; e < times; e++) {
				
				if (this.stop)
					break;
				if (this.pause) {
					e--;
					continue;
				}

				for (int i = (size - 1) - e; i >= 0; i -= times) {
					if (this.stop)
						break;
					if (this.pause) {
						i += times;
						continue;
					}

					final Lipid current = lipids[i];
					final Vector4d box = boxes[i];
					final Point3f p = current.getPosition();
					final Vector2f current2DPosOnPlane = new Vector2f(p.x, p.z);
					// get the list with lipids sorted according their distance to current
					final ComparableLipid[] closest = this.getClosest(current,
							lipids, radiuses);

					// this is only relevant if the membrane contains 
					//  less lipids than the cutoff needs
					if (closest.length < cutoff)
						cutoff = closest.length;

					final ComparableLipid[] others = new ComparableLipid[cutoff];

					// get the nearest lipids, their number is restricted by the cutoff
					for (int a = 0; a < cutoff; a++) {
						others[a] = closest[a];
					}

					// get the suggested new position of the actual lipid, by pushing
					//  it towards its nearest neighbours
					Vector2f suggestion = this.pushIt(current2DPosOnPlane, others);

					// the suggestion has now to be checked according the box values,
					// if it intersects the borders
					//  x----|    y
					//            |      
					//   -   X    -
					//   |
					//   w   |----z
					if (suggestion.y - box.w <= 0) {
						suggestion = new Vector2f(suggestion.x, (float) box.w
								+ (this.random.nextFloat() * 5));
					} else if (suggestion.y + box.y >= this.bounds.height) {
						suggestion = new Vector2f(
								suggestion.x,
								(float) (this.bounds.height - box.y - (this.random
										.nextFloat() * 5)));
					}
					if (suggestion.x + box.x >= this.bounds.width) {
						suggestion = new Vector2f((float) (this.bounds.width
								- box.x - (this.random.nextFloat() * 5)),
								suggestion.y);
					} else if (suggestion.x - box.z <= 0) {
						suggestion = new Vector2f((float) box.z
								+ (this.random.nextFloat() * 5), suggestion.y);
					}

					// the new position has to be inside the correct microdomain,
					//  if it is outside, it tried for one time to shift the lipid randomly
					//  inside the correct microdomain agrain
					if (this.getMicroDomainAtPoint(new Point2f(suggestion),
							this.getCurrentLayerID()) != current
							.getDomainIdentifier()) {
						suggestion = new Vector2f(this.random
								.nextInt((int) this.bounds.width - 1),
								this.random
										.nextInt((int) this.bounds.height - 1));
						if (this.getMicroDomainAtPoint(new Point2f(suggestion),
								this.getCurrentLayerID()) != current
								.getDomainIdentifier())
							continue;
					}

					// backup the position 
					final Point3f oldpos = new Point3f(current.getPosition());
					
					// set finally the new position
					current.setPosition(new Point3f(suggestion.x, current
							.getPosition().y, suggestion.y));

					// if there is an intersection with the borders, restore the old position
					if (this.intersectsBorders(current)
							|| this.intersects(current))
						current.setPosition(oldpos);

				}

			}
			
			if (tohaveEC == haveEC && tohaveIC == haveIC && this.stopIfRatioReached.isSelected() /**&& this.pressedUpdateCounts*/)
				this.stop = true;

		}
		
	}

//	private boolean checkDistribution() {
//
//		for (int a = 0; a < 2; a++) {
//			final Lipid[] lipids = a == 0 ? this.lipEC : this.lipIC;
//			final int[] totaldistances = new int[] { 0, 0, 0, 0 };
//			final int length = lipids.length;
//
//			for (int i = 0; i < length; i++) {
//				totaldistances[0] += (int) lipids[i].getPosition().x;
//			}
//			for (int i = 0; i < length; i++) {
//				totaldistances[1] += this.bounds.width
//						- (int) lipids[i].getPosition().x;
//			}
//			for (int i = 0; i < length; i++) {
//				totaldistances[2] += (int) lipids[i].getPosition().z;
//			}
//			for (int i = 0; i < length; i++) {
//				totaldistances[3] += this.bounds.height
//						- (int) lipids[i].getPosition().z;
//			}
//
//			System.out.println(Math.abs(totaldistances[0] - totaldistances[1]));
//			System.out.println(Math.abs(totaldistances[2] - totaldistances[3]));
//			System.out.println("");
//		}
//
//		return false;
//	}

	/**
	 * Change the positions of all lipids (for both sides) randomly.
	 * If the new position is not valid, restore the old position.
	 * 
	 *  RUNTIMEVAR: n*M
	 */
	private void mixUp() {

		for (int a = 0; a < 2; a++) {
			final Vector<Lipid> lips = this
					.getAllLipidsInDomain(userSelectedDomain, a == 0 ? MembranePackingAlgorithm.EXTRA
							: MembranePackingAlgorithm.INTRA);
			for (int i = 0; i < lips.size(); i++) {
				final Lipid l = lips.get(i);
				final Point3f oldpos = new Point3f(l.getPosition());
				l.setPosition(new Point3f(this.random
						.nextInt((int) this.bounds.width), oldpos.y,
						this.random.nextInt((int) this.bounds.height)));

				if (this.getMicroDomainAtPoint(
						new Point2f(l.getPosition().x, l.getPosition().z),l.getLayerID()) != l.getDomainIdentifier()
						|| this.intersectsBorders(l) || this.intersects(l))
					l.setPosition(oldpos);
			}
		}

	}

	/**
	 * This method is needed to update the values for the different 
	 * real lipid position/bounding box position distances and
	 * for the storage of the lipid type radii
	 *
	 */
	private void updateStuff() {

		this.lipEC = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA).toArray(
				new Lipid[0]);
		this.lipIC = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA).toArray(
				new Lipid[0]);

		this.diststancesEC = new Vector4d[this.lipEC.length];
		this.diststancesIC = new Vector4d[this.lipIC.length];

		this.radsEC = new Hashtable<String, Float>();
		this.radsIC = new Hashtable<String, Float>();

		for (int i = 0; i < this.diststancesEC.length; i++) {
			final BoundingBox b = this.lipEC[i].getBounds();
			final Point3d lower = new Point3d();
			final Point3d upper = new Point3d();
			b.getLower(lower);
			b.getUpper(upper);

			this.diststancesEC[i] = new Vector4d(
					Math.abs(this.lipEC[i].getPosition().z - lower.z), 
					Math.abs(this.lipEC[i].getPosition().x - upper.x), 
					Math.abs(this.lipEC[i].getPosition().z - upper.z), 
					Math.abs(this.lipEC[i].getPosition().x - lower.x));

			if (!this.radsEC.containsKey(this.lipEC[i].getOriginalMolecule()
					.getPDBFile().getAbsolutePath()))
				this.radsEC.put(this.lipEC[i].getOriginalMolecule()
						.getPDBFile().getAbsolutePath(), this.lipEC[i]
						.calculateRadius());
		}

		for (int i = 0; i < this.diststancesIC.length; i++) {
			final BoundingBox b = this.lipIC[i].getBounds();
			final Point3d lower = new Point3d();
			final Point3d upper = new Point3d();
			b.getLower(lower);
			b.getUpper(upper);

			this.diststancesIC[i] = new Vector4d(
					Math.abs(this.lipIC[i].getPosition().z - lower.z), 
					Math.abs(this.lipIC[i].getPosition().x - upper.x), 
					Math.abs(this.lipIC[i].getPosition().z - upper.z), 
					Math.abs(this.lipIC[i].getPosition().x - lower.x));

			if (!this.radsIC.containsKey(this.lipIC[i].getOriginalMolecule()
					.getPDBFile().getAbsolutePath()))
				this.radsIC.put(this.lipIC[i].getOriginalMolecule()
						.getPDBFile().getAbsolutePath(), this.lipIC[i]
						.calculateRadius());
		}

	}

	/**
	 * This method removes or adds lipids to the membrane according to the
	 * user defined values. It uses the RandomPlacing algorithm for the
	 * adding of lipids.
	 * 
	 * RUNTIMEVAR: RP
	 * 
	 * @throws IllegalOperationException
	 */
	private void performUpdateCounts() throws IllegalOperationException {

		final int tohaveEC = (Integer) this.spinnerExtra.getValue();
		final int tohaveIC = (Integer) this.spinnerIntra.getValue();

		final int haveEC = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA).size();
		final int haveIC = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA).size();

		final int diffEC = tohaveEC - haveEC;
		final int diffIC = tohaveIC - haveIC;

		final Vector<Lipid> toremove = new Vector<Lipid>();
		if (diffEC <= 0) {

			final int size = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA).size();
			for (int i = size - 1; i >= size - Math.abs(diffEC); i--) {

				toremove.add(this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA).get(i));

			}
		}
		if (diffIC <= 0) {
			final int size = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA).size();
			for (int i = size - 1; i >= size - Math.abs(diffIC); i--) {

				toremove.add(this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA).get(i));

			}
		}

		this.removeLipidsFromMembrane(toremove.toArray(new Lipid[0]));

		if (toremove.isEmpty() == false) {
			this.mixUp();
		}

		if ((diffEC > 0 || diffIC > 0) == false)
			return;

		this.RP = new RandomPlacing() {

			public void addLipids(int userSelectedDomain, boolean isIC)
					throws IllegalOperationException {

				if (this.getAllLipidsInDomain(Distributor.this.userSelectedDomain, MembranePackingAlgorithm.EXTRA).size() == tohaveEC
						&& this.getAllLipidsInDomain(Distributor.this.userSelectedDomain, MembranePackingAlgorithm.INTRA).size() == tohaveIC) {
					Distributor.this.RP.stopAlgorithm();
					return;
				}
				if (this.getAllLipidsInDomain(Distributor.this.userSelectedDomain, MembranePackingAlgorithm.EXTRA).size() == tohaveEC)
					isIC = true;
				else if (this.getAllLipidsInDomain(Distributor.this.userSelectedDomain, MembranePackingAlgorithm.INTRA).size() == tohaveIC)
					isIC = false;

				super.addLipids(Distributor.this.userSelectedDomain, isIC);

			}
		};

		this.RP.lipidsEC = this.lipidsEC;
		this.RP.lipidsIC = this.lipidsIC;
		this.RP.microDomains = this.microDomains;
		this.RP.setListenerAndPrepare(this.membraneViewListener, this.basicOperations, this.databaseOperations, this.calculationManager);

		this.RP.cbAddLipids.setSelected(true);
		this.RP.cbApproach.setSelected(false);
		this.RP.cbShake.setSelected(false);
		this.RP.cbTwist.setSelected(false);
		this.RP.rb2.setSelected(true);
		
		this.RP.cbAlgoAddLipids.setSelected(this.cbAlgoAddLipids.isSelected());
		this.RP.cbAlgoApproach.setSelected(this.cbAlgoApproach.isSelected());
		this.RP.cbAlgoShake.setSelected(this.cbAlgoShake.isSelected());
		this.RP.cbAlgoTwist.setSelected(this.cbAlgoTwist.isSelected());
		
		// this.RP.started=true;
		this.RP.fillWithLipids();
		
//		if (this.stopIfRatioReached.isSelected() &&
//				this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA).size() == tohaveEC
//				&& this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA).size() == tohaveIC) {
//			this.stop = true;
//		} else {
			this.stop = false;
//		}

		this.manuallyUpdateStatistics();
	}

	/**
	 * 
	 * Computes a new position for the actual lipid by pushing it towards the 
	 * positions of the nearest lipids. The positions of the lipids to compare
	 * are weighted by the difference of the farest distance from this group
	 * and the actual distance of the lipid to compare. This means, the position
	 * of the farest lipid is only used as a reference point, it is not directly
	 * used for the computation of the new position for the actual lipid.
	 * 
	 * @param ref2DPos the reference point of the actual lipid
	 * @param closest the list of lipids to compare
	 * @return the new position the lipid should be pushed to
	 */
	private Vector2f pushIt(final Vector2f ref2DPos, final ComparableLipid[] closest) {

		final Vector2f newone = new Vector2f(ref2DPos);
		final int numberOfClosest = closest.length;
		final float largestDistance = closest[numberOfClosest - 1].length;
		for (int i = 0; i < numberOfClosest; i++) {
			final Point3f p = closest[i].lipid.getPosition();

			final Vector2f v = new Vector2f(p.x, p.z);
			v.sub(ref2DPos);
			v.normalize();

			// this weights the value of the distances
			v.scale(largestDistance - closest[i].length);

			newone.sub(v);

		}

		return newone;
	}

	/**
	 * This method returns an array of all CompareableLipids which were 
	 * sorted according the distance to the given lipid to evaluate.  
	 * 
	 * The radii is substracted from each distance, if the actual radius
	 * is smaller than the actual distance.
	 * 
	 * @param lipid the lipid to examinine 
	 * @param lipids the list of all lipids which should be compared to lipid
	 * @param radiuses the radii of all lipid types
	 * @return
	 */
	private ComparableLipid[] getClosest(final Lipid lipid,
			final Lipid[] lipids, final Hashtable<String, Float> radiuses) {

		final int length = lipids.length;
		final Point3f pos = lipid.getPosition();
		final Vector2f orig = new Vector2f(pos.x, pos.z);

		final ComparableLipid[] toSort = new ComparableLipid[length - 1];
		int c = 0;
		for (int i = 0; i < length; i++) {

			if (lipids[i] == lipid)
				continue;
			final Point3f otherpos = lipids[i].getPosition();
			final Vector2f v = new Vector2f(otherpos.x, otherpos.z);
			v.sub(orig);
			toSort[c] = new ComparableLipid(lipids[i], v.length());

			float amplifier = 0f;

			// get the radius of the actual lipid to compare
			if (radiuses.containsKey(lipids[i].getOriginalMolecule()
					.getPDBFile().getAbsolutePath()))
				amplifier = radiuses.get(
						lipids[i].getOriginalMolecule().getPDBFile()
								.getAbsolutePath()).floatValue();

			// if the radius of the actual lipid to compare is smaller than
			//  the distance between this lipid and the other one,
			//  subtract the radius (amplifier) from the distance (length)
			if (toSort[c].length >= (amplifier / 10f))
				toSort[c].length -= (amplifier / 10f);

			c++;

		}

		Arrays.sort(toSort);

		return toSort;
	}

//	private void updatePanel() {
//		System.out.println("this.spinnerExtra 6 "+this.spinnerExtra.getValue());
//		this.spinnerExtra.setValue(this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA)
//				.size());
//		this.spinnerIntra.setValue(this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA)
//				.size());
//	}

	/**
	 * This class is used to find the closest lipid from all lipids in the 
	 * method getClosest(). 
	 * It contains a lipid and the distance from this lipid to the actual
	 * evaluated lipid.
	 */
	class ComparableLipid implements Comparable<Object> {

		public Lipid lipid;
		private float length;

		public ComparableLipid(final Lipid lipid, final float length) {
			this.lipid = lipid;
			this.length = length;
		}

		public int compareTo(final Object arg0) {
			final ComparableLipid other = (ComparableLipid) arg0;
			if (this.length < other.length)
				return -1;
			else
				return 1;
		}

	}
	
}
