/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.vecmath.Point2f;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.membranepackingalgorithm.IllegalOperationException;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithmGUI;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MicroDomain;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Molecule;


/**
 * 
 * Places the lipids in rows for each Domain\n\n"
 * 
 * --> Works best with only one single type of\n"
 *     lipid used on each membrane side and\n"
 *     without any random rotation"
 * 			
 * @author Tim Dingersen dackelnose@gmx.net, comments by Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class LinearPlacing extends MembranePackingAlgorithm {

	final int[] rotations90 = { 0, 90, 180, 270 };

	JTabbedPane tab;

	private boolean stopAlgorithm = false;

	boolean toggleSide = false;

	Vector<Lipid> EC;

	Vector<Lipid> IC;

	Vector<Molecule> domlipsEC = new Vector<Molecule>();

	Vector<Molecule> domlipsIC = new Vector<Molecule>();

	Rectangle mainBounds;

	private boolean hasalreadylipids;
	
	int userSelectedDomain = -1;

	public boolean buildAndShowGui() {

		// check if the user selected a membrane
		if (this.getSelectedArea() != -1) {

			int val = JOptionPane
					.showConfirmDialog(
							null,
							"You have currently selected a particular area.\nDo you want to consider this area only?\n",
							"Selected Area", JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE);

			if (val == JOptionPane.YES_OPTION) {
				userSelectedDomain = this.getSelectedArea();
			}

		}
		
		this.tab = new JTabbedPane();

		if (userSelectedDomain==-1) {

			this.tab.addTab("Default", new MyPanel());
			for (int i = 0; i < this.getMicroDomainCount(); i++) {
				if (this.getMicroDomainAt(i).getLayerID() != this
						.getCurrentLayerID())
					continue;
				this.tab.addTab("Domain " + this.getMicroDomainAt(i).getDomainID(),
						new MyPanel());
			}

		} else {
			
			if (userSelectedDomain==0) {
				this.tab.addTab("Default", new MyPanel());
			} else {
				this.tab.addTab("Default", new JPanel());
			}
			for (int i = 0; i < this.getMicroDomainCount(); i++) {
				if (this.getMicroDomainAt(i).getLayerID() != this
						.getCurrentLayerID())
					continue;
				if (userSelectedDomain==this.getMicroDomainAt(i).getDomainID()) {
					this.tab.addTab("Domain " + this.getMicroDomainAt(i).getDomainID(),
							new MyPanel());
				} else {
					this.tab.addTab("Domain " + this.getMicroDomainAt(i).getDomainID(),
							new JPanel());
				}
			}
			
		}

		return MembranePackingAlgorithmGUI.popUpGUI(this.tab,
				"Starting \"Linear Placing Algorithm\"", "Set up Parameters");
	}

	@SuppressWarnings("serial")
	class MyPanel extends JPanel {

		JRadioButton radio1 = new JRadioButton("{0,1,...,359}     Degr. (All)");

		JRadioButton radio2 = new JRadioButton(
				"{0,90,180,270} Degr. (Quarter Turns)");

		JRadioButton radio3 = new JRadioButton(
				"{0}                     Degr. (None)");

		public MyPanel() {
			super();

			final JLabel label = new JLabel("Random Y-Rotations");

			this.setLayout(new GridLayout(4, 0));

			this.add(label);
			this.add(this.radio3);
			this.add(this.radio2);
			this.add(this.radio1);

			final ButtonGroup bg = new ButtonGroup();
			bg.add(this.radio3);
			bg.add(this.radio2);
			bg.add(this.radio1);

			this.radio3.setSelected(true);
		}

	}

	public void fillWithLipids() throws IllegalOperationException {

		this.hasalreadylipids = this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.EXTRA)
				.size() > 0
				|| this.getAllLipidsInDomain(userSelectedDomain, MembranePackingAlgorithm.INTRA).size() > 0;

		final int layerid = this.getCurrentLayerID();
		
		// no domain was selected by the user
		if (userSelectedDomain==-1) {

			int c = 1;
			for (int i = 0; i < this.getMicroDomainCount(); i++) {
				if (this.getMicroDomainAt(i).getLayerID() == layerid)
					this.fillDomain(this.getMicroDomainAt(i), c, this
							.getMicroDomainAt(i).getLayerID());
				else
					continue;
				c++;
			}
	
			// // for(int i=0;i<this.getLayerCount();i++) {
			// this.fillDomain(null,0,this.getLayerAtIndex(i).getLayerId());
			// }
			this.fillDomain(null, 0, layerid);
			// this.fillDomain(null, 0,0);
		
			// a special domain has been selected by the user
		} else {
			
			if (userSelectedDomain != 0) {
//				int c = 1;
				for (int i = 0; i < this.getMicroDomainCount(); i++) {
					if (this.getMicroDomainAt(i).getLayerID() == layerid && i+1 == userSelectedDomain)
						this.fillDomain(this.getMicroDomainAt(i), i+1, this
								.getMicroDomainAt(i).getLayerID());
					else
						continue;
//					c++;
				}
			} else {
				this.fillDomain(null, 0, layerid);
			}
			
		}

	}

	/**
	 * Fills the actual selected domain with linear rows beginning in
	 * north west and finishing in south east.
	 * 
	 * The runtime of this algorithm is:
	 * 
	 * n, where
	 * 
	 *  n = newlipids 
	 *  
	 * @param MicroDomain domain
	 * @param int index
	 * @param int layerID
	 * @throws IllegalOperationException
	 */
	private void fillDomain(final MicroDomain domain, final int index,
			final int layerID) throws IllegalOperationException {

		// check, if the actual domain to compute is the user selected domain
		//  otherwise exit the procedure
		if (layerID != this.getCurrentLayerID())
			return;
		
		// get the bounds of the selected domain or the whole membrane
		this.mainBounds = domain != null ? domain.getBounds() : new Rectangle(
				0, 0, this.getMembraneSize().width,
				this.getMembraneSize().height);

		this.EC = new Vector<Lipid>();
		this.IC = new Vector<Lipid>();

		double actualXPosEC = this.mainBounds.x;
		double actualXPosIC = this.mainBounds.x;

		boolean placingSuccess = false;

		double actualZPosEC = this.mainBounds.y;
		double actualZPosIC = this.mainBounds.y;
		
		// the main loop to place all lipids
		//  RUNTIMEVAR: n
		while (true) {
			
			// if the user pressed stop, exit here
			if (this.stopAlgorithm)
				return;

			// change the actual membrane side
			this.toggleSide = !this.toggleSide;
			
			Lipid newlipid;

			// if the last placing did not succeeded and both actual maximum lipid bounds
			//  lie outside the membrane bounds, the placing process is finished
			if (!placingSuccess && actualZPosEC > this.mainBounds.getMaxY()
					&& actualZPosIC > this.mainBounds.getMaxY())
				break;

			// create the next intracellular lipid, if there are no sample lipids
			//  at this membrane side, finish the computation of this side
			if (this.toggleSide) {

				newlipid = domain != null ? domain
						.createNextIntracellularLipid() : this
						.createNextIntracellularLipid(layerID);

				final int countSampleLipids = domain != null ? domain
						.countSampleLipids_IntraCellular() : this
						.countSampleLipids_Intracellular(layerID);

				if (countSampleLipids < 1) {
					actualZPosIC = this.mainBounds.getMaxY() + 1;
					continue;
				}

			// create the next extracellular lipid, if there are no sample lipids
			//  at this membrane side, finish the computation of this side
			} else {

				newlipid = domain != null ? domain
						.createNextExtracellularLipid() : this
						.createNextExtracellularLipid(layerID);
						
				final int countSampleLipids = domain != null ? domain
						.countSampleLipids_ExtraCellular() : this
						.countSampleLipids_Extracellular(layerID);

				if (countSampleLipids < 1) {
					actualZPosEC = this.mainBounds.getMaxY() + 1;
					continue;
				}

			}

			// add the DesiredAtomDistance to the lipids YTranslation 
			if (newlipid.getYTranslation() < 0f)
				newlipid.setYTranslation(newlipid.getYTranslation()
						- this.getDesiredAtomDistance() / 2f);
			else if (newlipid.getYTranslation() > 0f)
				newlipid.setYTranslation(newlipid.getYTranslation()
						+ this.getDesiredAtomDistance() / 2f);

			// do the random rotation
			newlipid.rotY(this.generateRandomRotation(index));

			// add the DesiredAtomDistance to the Rectangle
			Rectangle2D.Double newlipidbounds = this.extendRectangle(newlipid
					.getXZBounds());

			// compute the next X position in line
			double nextXPos = newlipidbounds.width / 2;
			if (this.toggleSide)
				nextXPos += actualXPosIC;
			else
				nextXPos += actualXPosEC;

			// generate and set the actual temporary position by combining the computed
			//  X position in line with the actual Z position of the line
			final Point2f newpos = new Point2f((float) nextXPos,
					this.toggleSide ? (float) actualZPosIC : (float) actualZPosEC);
			newlipid.setPosition(new Point3f(newpos.x,
					newlipid.getPosition().y, newpos.y));

			// add the DesiredAtomDistance to the Rectangle
			newlipidbounds = this.extendRectangle(newlipid.getXZBounds());

			// check if the end of the line is reached,
			//  if so, start over with the next line
			//  - the X position is defined by the most left position of the actual domain or membrane area
			//  - the Z position is defined by the actual Z position + the lipid width along the Z axis
			//  if so, change the position of the lipid
			//   and add the half of the lipid bounds width along the X axis to the X position
			if (newlipidbounds.getMaxX() > this.mainBounds.getMaxX()) {

				placingSuccess = false;
				if (this.toggleSide) {
					actualXPosIC = this.mainBounds.x;
					actualZPosIC += newlipidbounds.height;
				} else {
					actualXPosEC = this.mainBounds.x;
					actualZPosEC += newlipidbounds.height;
				}

				newlipid.setPosition(new Point3f(this.mainBounds.x
						+ (float) newlipidbounds.width / 2f, newlipid
						.getPosition().y, this.toggleSide ? (float) actualZPosIC
						: (float) actualZPosEC));

			}

			// this.crawlDown(newlipid);

			// finalize the position by adding 1 to the X and Z position
			//  and add the half of the lipid bounds width along the Z axis to the Z position
			newlipid.setPosition(new Point3f(newlipid.getPosition().x + 1,
					newlipid.getPosition().y, newlipid.getPosition().z
							+ (float) newlipidbounds.height / 2f + 1));

			// add the DesiredAtomDistance to the Rectangle
			newlipidbounds = this.extendRectangle(newlipid.getXZBounds());

			// check if the end of the domain or membrane is reached,
			//  if so, skip the rest of this run and start over with the next new lipid
			//  - the X position is defined by the sum of the actual position and the actual lipid width along the X axis
			if (newlipidbounds.getMaxY() > this.mainBounds.getMaxY()) {
				if (this.toggleSide)
					actualXPosIC += newlipidbounds.width;
				else
					actualXPosEC += newlipidbounds.width;
				continue;
			}

			// check the actual position of the lipid for intersections
			//  if there are intersections, skip the rest of this run and 
			//  start over with the next new lipid
			if (domain != null) {

				if (!domain.isInsideArea(newlipid)

						|| this.intersectsWithProteins(newlipid)
						|| this.intersectsBorders(newlipid)
						|| (this.hasalreadylipids ? this.intersects(newlipid)
								: this.intersectsMoleculeWithMolecules(
										newlipid, this.toggleSide ? this.domlipsIC
												: this.domlipsEC))) {
					if (this.toggleSide)
						actualXPosIC += newlipidbounds.width;
					else
						actualXPosEC += newlipidbounds.width;
					continue;
				}
			} else {

				if (this.intersectsBorders(newlipid)
						|| this.intersectsAnyMicroDomain(newlipid)
						|| this.intersects(newlipid)) {
					if (this.toggleSide)
						actualXPosIC += newlipidbounds.width;
					else
						actualXPosEC += newlipidbounds.width;
					continue;
				}
				
			}

			// finally add the lipid to the actual domain or membrane
			if (this.toggleSide) {
				this.addIntracellularLipid(newlipid);
				this.IC.addElement(newlipid);
				if (domain != null)
					this.domlipsIC.addElement(newlipid);
			} else {
				this.addExtracellularLipid(newlipid);
				this.EC.addElement(newlipid);
				if (domain != null)
					this.domlipsEC.addElement(newlipid);
			}

			// add the bounds of the actual lipid to the actual X position in line
			if (this.toggleSide)
				actualXPosIC += newlipidbounds.width;
			else
				actualXPosEC += newlipidbounds.width;
			
			// inform the next run that the placing process succeeded
			placingSuccess = true;

		}

		this.EC = null;
		this.IC = null;

	}

	/**
	 * Extend the rectangle by the DesiredAtomDistance at all sides.
	 * @param bounds
	 * @return
	 */
	private Rectangle2D.Double extendRectangle(final Rectangle2D.Double bounds) {

		return new Rectangle2D.Double(bounds.x - this.getDesiredAtomDistance()
				/ 2f, bounds.y - this.getDesiredAtomDistance() / 2f,
				bounds.width + this.getDesiredAtomDistance(), bounds.height
						+ this.getDesiredAtomDistance());
	}

	// private void crawlDown(Lipid newlipid) {
	//
	// Rectangle2D.Double bounds = newlipid.getXZBounds();
	//
	// Vector<Lipid> lips = toggle ? IC : EC;
	//
	// int length = lips.size();
	//
	// double np = 0;
	// if (!lips.isEmpty())
	// np = mainBounds.height - lips.lastElement().getXZBounds().y;
	//
	// Rectangle2D.Double longbounds = new Rectangle2D.Double(bounds.x + 0.1,
	// mainBounds.y, bounds.x + bounds.width - 0.1, mainBounds.height
	// - 0.1 - np);
	// double max = mainBounds.y;
	// for (int i = 0; i < length; i++) {
	// Lipid l = lips.get(i);
	//
	// Rectangle2D.Double other = l.getXZBounds();
	//
	// if (longbounds.intersects(other)) {
	//
	// max = Math.max(max, other.getMaxY());
	// }
	//
	// }
	//
	// Point3f newone = new Point3f(newlipid.getPosition().x, newlipid
	// .getYTranslation(), (float) max + ((float) bounds.height / 2));
	//
	// newlipid.setPosition(newone);
	//
	// }

	public String getFullInformation() {
		final String s = "Places the lipids in rows for each Domain\n\n"
				+ "-->Works best with only one single type of\n"
				+ "lipid used on each membrane side and\n"
				+ "without any random rotation"

				+ "\n\n\n" + "Functioning: Linear positioning\n"
				+ "Orientation: Geometric\n" + "Speed: Fast\n"
				+ "Biological correctness: None\n\n"
				
				+ "Author: Tim Dingersen\n"
				+ "Version: 1.02";
		return "<html>" + s.replaceAll("\n", "<br>") + "</html>";
	}

	public String getName() {

		return "Linear Placing";
	}

	public String getShortInformation() {

		return "Simple Linear Placing, Version 1.02";
	}

	public String getStringRepresentationOfSetupParameters() {
		String s = "";
		if (userSelectedDomain==-1) {
			for (int p=0; p<this.tab.getComponentCount(); p++) {
				if (((MyPanel) this.tab.getComponentAt(p)).radio1.isSelected())
					s += "Random Y-Rotations for Domain "+p+" {0,1,...,359} Degr. (All)\n";
				else if (((MyPanel) this.tab.getComponentAt(p)).radio2.isSelected())
					s += "Random Y-Rotations for Domain "+p+" {0,90,180,270} Degr. (Quarter Turns)\n";
				else if (((MyPanel) this.tab.getComponentAt(p)).radio3.isSelected())
					s += "Random Y-Rotations for Domain "+p+" {0} Degr. (None)\n";
			}
		} else {
			if (((MyPanel) this.tab.getComponentAt(userSelectedDomain)).radio1.isSelected())
				s += "Random Y-Rotations for Domain "+userSelectedDomain+" {0,1,...,359} Degr. (All)\n";
			else if (((MyPanel) this.tab.getComponentAt(userSelectedDomain)).radio2.isSelected())
				s += "Random Y-Rotations for Domain "+userSelectedDomain+" {0,90,180,270} Degr. (Quarter Turns)\n";
			else if (((MyPanel) this.tab.getComponentAt(userSelectedDomain)).radio3.isSelected())
				s += "Random Y-Rotations for Domain "+userSelectedDomain+" {0} Degr. (None)\n";
			s += "\n" + "Selected Layer/Domain: "+userSelectedDomain+"/"+this.getCurrentLayerID();
		}
		
		return s;
	}

	public void stopAlgorithm() {
		this.stopAlgorithm = true;

	}
	

	public boolean supportsAbsoluteValues() {
		return false;
	}

	public boolean supportsMicrodomains() {
		return true;
	}

	/**
	 * Generates a random rotation for the given microdomain.
	 * @param index
	 * @return
	 */
	private int generateRandomRotation(final int index) {

		final MyPanel panel = (MyPanel) this.tab.getComponentAt(index);

		if (panel.radio1.isSelected())
			return this.random.nextInt(360);
		else if (panel.radio2.isSelected())
			return this.rotations90[this.random.nextInt(4)];
		else
			return 0;
	}

	public boolean overrideExistingLipids() {

		return false;
	}

	public Component getRuntimeControlPanel() {
		return null;
	}

}
