/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point2f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.membranepackingalgorithm.IllegalOperationException;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithmGUI;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MicroDomain;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Protein;


/**
 * Places the lipids randomly within membrane bounds and may 
 * try to approach proteins or the membrane center and
 * twisting and shaking the lipids.
 * 
 * @author Tim Dingersen dackelnose@gmx.net, comments by Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class RandomPlacing extends MembranePackingAlgorithm {

	private boolean stopAlgorithm = false;
	Point3f middle;

	// private JToolBar controlpanel = new JToolBar();
	private final JPanel panel = new JPanel();
	// private JPanel radiopanel1 = new JPanel();
	// private JPanel radiopanel2 = new JPanel();

	public JCheckBox cbAddLipids = new JCheckBox("Add new Lipids");
	public JCheckBox cbApproach = new JCheckBox("Approach");
	public JCheckBox cbShake = new JCheckBox("Shake");
	public JCheckBox cbTwist = new JCheckBox("Twist");
	
	public JCheckBox cbAlgoAddLipids = new JCheckBox("Add new Lipids");
	public JCheckBox cbAlgoApproach = new JCheckBox("Approach Proteins or Membrane Center");
	public JCheckBox cbAlgoShake = new JCheckBox("Shake");
	public JCheckBox cbAlgoTwist = new JCheckBox("Twist");

	public JRadioButton rb1 = new JRadioButton("Auto");
	public JRadioButton rb2 = new JRadioButton("Manual: ");
	boolean manualButtonSelected = false;

	// public JButton start = new JButton("Start");
	Scheduler scheduler = new Scheduler();
	private int limitOneSide;
	
	int userSelectedDomain = -1;

	// public boolean started=false;

//	Vector<Protein>prots=new Vector<Protein>();
//	Vector<Lipid>lipsEc=new Vector<Lipid>();
//	Vector<Lipid>lipsIc=new Vector<Lipid>();
//	
	public RandomPlacing() {

		this.cbAddLipids.setToolTipText("Add new lipids regularly");
		this.cbApproach
				.setToolTipText("<html>Makes lipids move towards their closest protein (or<br>"
						+ "towards the middle if no proteins exist)</html>");
		this.cbShake.setToolTipText("Moves existing lipids randomly around");
		this.cbTwist.setToolTipText("Rotates existing lipids randomly");

		// controlpanel.setLayout(new BoxLayout(controlpanel,
		// BoxLayout.X_AXIS));
		// radiopanel1.setLayout(new BoxLayout(radiopanel1, BoxLayout.X_AXIS));
		// radiopanel2.setLayout(new BoxLayout(radiopanel2, BoxLayout.X_AXIS));

		this.panel.setLayout(new BoxLayout(this.panel, BoxLayout.X_AXIS));

		// panel.setFloatable(false);
		final JLabel l = new JLabel("Schedule actions:", SwingConstants.LEADING);
		l.setFont(l.getFont().deriveFont(l.getFont().getSize2D() + 1));
		this.panel.add(l);
		this.panel.add(Box.createHorizontalStrut(15));
		this.panel.add(this.rb1);
		this.panel.add(Box.createHorizontalStrut(20));
		this.panel.add(new JLabel("|"));
		this.panel.add(Box.createHorizontalStrut(15));
		this.panel.add(this.rb2);
		this.panel.add(Box.createHorizontalStrut(10));
		this.panel.add(this.cbAddLipids);
		this.panel.add(this.cbApproach);
		this.panel.add(this.cbShake);
		this.panel.add(this.cbTwist);
		this.panel.add(Box.createHorizontalStrut(50));
		this.panel.add(Box.createHorizontalGlue());
		// this.panel.add(start);
		this.panel.add(Box.createHorizontalStrut(5));
		this.cbAddLipids.setSelected(true);
		this.cbApproach.setSelected(true);
		this.cbShake.setSelected(true);
		this.cbTwist.setSelected(true);

		final ButtonGroup bg = new ButtonGroup();
		bg.add(this.rb1);
		bg.add(this.rb2);

		this.rb1.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {

				RandomPlacing.this.cbAddLipids.setEnabled(false);
				RandomPlacing.this.cbApproach.setEnabled(false);
				RandomPlacing.this.cbShake.setEnabled(false);
				RandomPlacing.this.cbTwist.setEnabled(false);
				RandomPlacing.this.scheduler.setEnabled(true);

			}

		});
		this.rb2.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {

				if (RandomPlacing.this.cbAddLipids.isVisible())
					RandomPlacing.this.cbAddLipids.setEnabled(true);
				if (RandomPlacing.this.cbApproach.isVisible())
					RandomPlacing.this.cbApproach.setEnabled(true);
				if (RandomPlacing.this.cbShake.isVisible())
					RandomPlacing.this.cbShake.setEnabled(true);
				if (RandomPlacing.this.cbTwist.isVisible())
					RandomPlacing.this.cbTwist.setEnabled(true);
				RandomPlacing.this.scheduler.setEnabled(false);
				RandomPlacing.this.manualButtonSelected = true;

			}

		});

		this.rb1.doClick();

		final ActionListener ac = new ActionListener() {

			public void actionPerformed(final ActionEvent e) {

				final boolean b = RandomPlacing.this.cbAddLipids.isSelected()
						|| RandomPlacing.this.cbApproach.isSelected()
						|| RandomPlacing.this.cbShake.isSelected()
						|| RandomPlacing.this.cbTwist.isSelected();
				if (!b)
					((JCheckBox) e.getSource()).setSelected(true);
			}
		};

		this.cbAddLipids.addActionListener(ac);
		this.cbApproach.addActionListener(ac);
		this.cbShake.addActionListener(ac);
		this.cbTwist.addActionListener(ac);

		// start.addActionListener(this);

	}

	public boolean buildAndShowGui() {

		// check if the user selected a membrane
		if (this.getSelectedArea() != -1) {

			int val = JOptionPane
					.showConfirmDialog(
							null,
							"You have currently selected a particular area.\nDo you want to consider this area only?\n",
							"Selected Area", JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE);

			if (val == JOptionPane.YES_OPTION) {
				userSelectedDomain = this.getSelectedArea();
			}

		}
		
		
		// building up the options panel for the Random Placing GUI
		JPanel controlPanel = new JPanel();
		
		cbAlgoAddLipids.setSelected(true);
		cbAlgoApproach.setSelected(true);
		cbAlgoShake.setSelected(true);
		cbAlgoTwist.setSelected(true);
		
		controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.Y_AXIS));
		controlPanel.add(cbAlgoAddLipids);
		controlPanel.add(cbAlgoApproach);
		controlPanel.add(cbAlgoShake);
		controlPanel.add(cbAlgoTwist);

		/*
		 * showing the GUI to the user. The returnvalue indicates if the user
		 * confirmed or cancelled the dialog so far
		 */
		boolean returnValue = MembranePackingAlgorithmGUI.popUpGUI(controlPanel,
				"Random Placing v. 1.1", "Choose applied sub-algorithms");
		
		// if nothing was selected exit the algorithm
		if (!cbAlgoAddLipids.isSelected() && !cbAlgoApproach.isSelected() && !cbAlgoShake.isSelected() && !cbAlgoTwist.isSelected()) {
			return false;
		} else {		
			return returnValue;
		}
	}

	/**
	 * The runtime of this algorithm is:
	 * 
	 * r*(l*A+n*P+l*S), where
	 * 
	 *  r = repetitions until the user stopped the algorithm
	 *  l = limit = membrane width + membrane depth
	 *  A = lipid random placing attempt
	 *  n = number of actual lipids in the membrane
	 *  P = protein or membrane center approaching
	 *  S = shaking and/or twisting
	 * 
	 */
	public void fillWithLipids() throws IllegalOperationException {

		// check the selected sub-algorithms
		if (!cbAlgoAddLipids.isSelected()) {
			cbAddLipids.setVisible(false);
			cbAddLipids.setSelected(false);
		}
		if (!cbAlgoApproach.isSelected()) {
			cbApproach.setVisible(false);
			cbApproach.setSelected(false);
		}
		if (!cbAlgoShake.isSelected()) {
			cbShake.setVisible(false);
			cbShake.setSelected(false);
		}
		if (!cbAlgoTwist.isSelected()) {
			cbTwist.setVisible(false);
			cbTwist.setSelected(false);
		}
		
		this.middle = new Point3f(this.getMembraneSize().width / 2, 0, this
				.getMembraneSize().height / 2);

		this.limitOneSide = (this.getMembraneSize().width + this.getMembraneSize().height) / 2;
		this.scheduler.setLimit(this.limitOneSide - (this.limitOneSide / 8));
		final int limitTwoSides = this.limitOneSide * 2;

		boolean app1 = true;
		boolean app2 = true;
		while (true) {

			// try to add new lipids
			this.displayInformation("Adding new Lipids");
			for (int i = 0; i < limitTwoSides; i++) {
				if (this.stopAlgorithm)
					return;
				if (this.cbAddLipids.isSelected()) {
					this.addLipids(userSelectedDomain, true);
					this.addLipids(userSelectedDomain, false);
				}
			}

			// approach proteins or the membrane center if selected
			if (this.cbApproach.isSelected()) {
				this.displayInformation("Approaching (EC)");
				if (!this.performApproach(MembranePackingAlgorithm.EXTRA))
					app1 = false;
				this.displayInformation("Approaching (IC)");
				if (!this.performApproach(MembranePackingAlgorithm.INTRA))
					app2 = false;
				if (app1 == false || app2 == false)
					this.scheduler.setApproach(false);
			}

			// shake and twist the lipids if selected
			if (this.cbShake.isSelected() || this.cbTwist.isSelected()) {
				int d;
				if (this.cbShake.isSelected() && this.cbTwist.isSelected())
					d = 2;
				else if (this.cbShake.isSelected())
					d = 1;
				else
					d = 0;
				this.displayInformation("Shake / Twist");
				this.shakeLipid(MembranePackingAlgorithm.EXTRA, d);
				this.shakeLipid(MembranePackingAlgorithm.INTRA, d);
			}

			this.scheduler.next();

		}

	}

	/**
	 * Tries to add new lipids to the membrane.
	 * @param userSelectedDomain
	 * @param toggleSide
	 * @throws IllegalOperationException
	 */
	public void addLipids(final int userSelectedDomain, final boolean toggleSide)
			throws IllegalOperationException {
		Lipid newlipid;

		final Point2f newpos = this.createRandomXZPosition(null);

		final int domainID = this.getMicroDomainAtPoint(newpos, this
				.getCurrentLayerID());
		
		if (userSelectedDomain != -1 && domainID != userSelectedDomain)
			return;

		final MicroDomain dom = this.getMicroDomain(domainID);

		if (dom != null) {
			if (toggleSide)
				newlipid = dom.createNextIntracellularLipid();
			else
				newlipid = dom.createNextExtracellularLipid();

		} else {

			if (toggleSide)
				newlipid = this.createNextIntracellularLipid();
			else
				newlipid = this.createNextExtracellularLipid();
		}

		if (newlipid == null)
			return;

		newlipid.rotY(this.random.nextInt(359));

		newlipid.rotX(-2 + this.random.nextInt(4) + this.random.nextFloat());
		newlipid.rotZ(-2 + this.random.nextInt(4) + this.random.nextFloat());

		newlipid.setPosition(new Point3f(newpos.x, newlipid.getPosition().y,
				newpos.y));

		if (this.intersects(newlipid) || this.intersectsBorders(newlipid)) {

			boolean goon = false;

			for (int r = 1; r < 10; r++) {
				newlipid.rotY(this.random.nextInt(359));
				if (!this.intersects(newlipid)
						&& !this.intersectsBorders(newlipid)) {
					goon = true;
					break;
				}

			}

			if (!goon) {
				this.scheduler.miss(toggleSide);
				return;
			}

		}

		if (toggleSide)
			this.addIntracellularLipid(newlipid);
		else
			this.addExtracellularLipid(newlipid);

	}

	/**
	 * Lipids approach proteins or the membrane center at the given side.
	 * @param side
	 * @return int side
	 */
	private boolean performApproach(final int side) {

		final Vector<Lipid> lips = this.getAllLipidsInDomain(userSelectedDomain, side);
		
		if (lips.isEmpty())
			return true;
		
		int succ = 0;
		final int size = lips.size();

		for (int i = 0; i < size; i++) {
			if (this.stopAlgorithm || !this.cbApproach.isSelected())
				return true;

			final Lipid current = lips.get(i);

			Point3f target;
			final Protein prot = this.getNearestProtein(current);
			if (prot != null)
				target = prot.getPosition();
			else
				target = this.middle;

			final Point3f oldpos = current.getPosition();
			final Matrix3f oldm = this.copyMatrix(current.getRotationMatrix());

			final Vector3f direction = new Vector3f(target.x, 0, target.z);

			direction.sub(new Vector3f(oldpos.x, 0, oldpos.z));
			direction.normalize();

			final Point3f newpos = new Point3f(oldpos);
			final Vector3f toadd = new Vector3f(direction.x
					+ ((-1 + this.random.nextFloat()) / 2f), 0, direction.z
					+ ((-1 + this.random.nextFloat()) / 2f));

			toadd.scale(0.5f);

			newpos.add(toadd);
			current.setPosition(newpos);
			if (this.intersects(current) || this.intersectsBorders(current)) {

				boolean fit = false;
				for (int a = 0; a < 3; a++) {

					current.rotY(-3 + this.random.nextInt(6)
							+ this.random.nextFloat());
					if (this.intersects(current)
							|| this.intersectsBorders(current)) {
						continue;

					} else {
						fit = true;
						succ++;
						break;
					}
				}
				if (!fit) {
					current.setPosition(oldpos);
					current.setRotationMatrix(oldm);

				}

			} else if (this.getMicroDomainAtPoint(new Point2f(newpos.x,
					newpos.z), current.getLayerID()) != current
					.getDomainIdentifier()) {
				current.setPosition(oldpos);
				current.setRotationMatrix(oldm);

			} else {
				succ++;

			}

		}

		if (succ < (size / 25))
			return false;
		else
			return true;

	}


	
	
	int lastShakenLipidExtra = 0;
	int lastShakenLipidIntra = 0;

	/**
	 * Lipids shake or twist at the given side.
	 * Mode defines, if shake and/or twist are activated.
	 * @param int side
	 * @param int mode
	 */
	private void shakeLipid(final int side, final int mode) {

		final Vector<Lipid> lips = this.getAllLipidsInDomain(userSelectedDomain, side);
		
		if (lips.isEmpty())
			return;

		// sets the limit, each time this method is run, only this number of lipids is
		//  shaked and/or twisted, the lastShakenLipidExtra/Intra variables store,
		//  where to continue in the next run
		int counter = this.limitOneSide;
		for (int i = side == 0 ? this.lastShakenLipidExtra : this.lastShakenLipidIntra; i < lips.size(); i++) {
			if (this.stopAlgorithm)
				return;
			counter--;
			if (i >= lips.size() - 1) {
				i = 0;
			}
			if (counter < 0) {
				if (side == MembranePackingAlgorithm.EXTRA)
					this.lastShakenLipidExtra = i;
				else
					this.lastShakenLipidIntra = i;
				break;
			}
			final Lipid current = lips.get(i);
			// twist only
			if (mode == 0) {
				final Matrix3f oldm = this.copyMatrix(current
						.getRotationMatrix());
				boolean fit = false;
				for (int j = 0; j < 3; j++) {

					current.rotY(this.random.nextInt(359));
					if (this.intersects(current)
							|| this.intersectsBorders(current)) {
						continue;
					} else {
						fit = true;
						break;
					}
				}
				if (!fit)
					current.setRotationMatrix(oldm);
			// shake only
			} else if (mode == 1) {
				final Point3f oldpoint = new Point3f(current.getPosition());
				float rx = this.random.nextFloat();
				float rz = this.random.nextFloat();
				if (this.random.nextBoolean())
					rx *= -1;
				if (this.random.nextBoolean())
					rz *= -1;
				current.setPosition(new Point3f(oldpoint.x + rx, oldpoint.y,
						oldpoint.z + rz));
				if (this.intersects(current)
						|| this.intersectsBorders(current)
						|| this.getMicroDomainAtPoint(new Point2f(current
								.getPosition().x, current.getPosition().z),
								current.getLayerID()) != current
								.getDomainIdentifier()) {
					current.setPosition(oldpoint);
				}
			// shake and twist
			} else {
				final Point3f oldpoint = new Point3f(current.getPosition());
				final Matrix3f oldm = this.copyMatrix(current
						.getRotationMatrix());

				float rx = this.random.nextFloat();
				float rz = this.random.nextFloat();
				if (this.random.nextBoolean())
					rx *= -1;
				if (this.random.nextBoolean())
					rz *= -1;
				current.setPosition(new Point3f(oldpoint.x + rx, oldpoint.y,
						oldpoint.z + rz));
				boolean fit = false;
				for (int j = 0; j < 3; j++) {
					current.rotY(this.random.nextInt(360));
					if (this.intersects(current)
							|| this.intersectsBorders(current)) {
						continue;

					} else {
						fit = true;
						break;
					}
				}
				if (!fit
						|| this.getMicroDomainAtPoint(new Point2f(current
								.getPosition().x, current.getPosition().z),
								current.getLayerID()) != current
								.getDomainIdentifier()) {
					current.setRotationMatrix(oldm);
					current.setPosition(oldpoint);
				}
			}
		}
	}

	public String getFullInformation() {
		final String s = "Simply places the lipids randomly within the bounds"

		+ "\n\n\n" + "Functioning: Trial and Error placing\n"
				+ "Orientation: Geometric\n" + "Speed: Fast\n"
				+ "Biological correctness: None\n\n" 
				
				+ "Author: Tim Dingersen\n" 
				+ "Version: 1.1";
		return "<html>" + s.replaceAll("\n", "<br>") + "</html>";
	}

	public String getName() {

		return "Random Placing";
	}

	public String getShortInformation() {

		return "Places lipids at random position (Version 1.1)";
	}

	public String getStringRepresentationOfSetupParameters() {

		String s = "Applied Sub-Algorithms:\n";

		if (cbAddLipids.isVisible())
			s += " Add Lipids\n";
		if (cbApproach.isVisible())
			s += " Approach\n";
		if (cbShake.isVisible())
			s += " Shake\n";
		if (cbTwist.isVisible())
			s += " Twist\n";
		s += "\n";
		
		s += "Mode:\n";
		if (!RandomPlacing.this.manualButtonSelected)
			s += " Auto (Reproducible)\n";
		else
			s += " Manual (Non-Reproducible)\n";
		s += "\n";
		
		s += "Selected Layer/Domain: "+userSelectedDomain+"/"+this.getCurrentLayerID();
		
		return s;
	}

	public void stopAlgorithm() {
		this.stopAlgorithm = true;
	}


	public boolean supportsAbsoluteValues() {
		return true;
	}
	
	public boolean supportsMicrodomains() {
		return true;
	}

	public boolean overrideExistingLipids() {
		return false;
	}

	public Component getRuntimeControlPanel() {
		return this.panel;
	}

	class Scheduler {

		private int limit;
		public final int limitapp = 50;
		private int miss1 = 0;
		private int miss2 = 0;

		private boolean approach = true;
		private boolean enabled;

		public void setEnabled(final boolean b) {
			this.enabled = b;
			if (b) {
				if (RandomPlacing.this.cbAddLipids.isVisible())
					RandomPlacing.this.cbAddLipids.setSelected(true);
				RandomPlacing.this.cbApproach.setSelected(false);
				RandomPlacing.this.cbShake.setSelected(false);
				RandomPlacing.this.cbTwist.setSelected(false);
			} else {
				if (RandomPlacing.this.cbAddLipids.isVisible())
					RandomPlacing.this.cbAddLipids.setSelected(true);
				if (RandomPlacing.this.cbApproach.isVisible())
					RandomPlacing.this.cbApproach.setSelected(true);
				if (RandomPlacing.this.cbShake.isVisible())
					RandomPlacing.this.cbShake.setSelected(true);
				if (RandomPlacing.this.cbTwist.isVisible())
					RandomPlacing.this.cbTwist.setSelected(true);
			}

		}

		public void setApproach(final boolean b) {
			this.approach = b;

		}

		public void miss(final boolean toggle) {
			if (!this.enabled)
				return;
			if (toggle)
				this.miss1++;
			else
				this.miss2++;

		}

		public void setLimit(final int limit) {
			this.limit = limit;

		}

		// NEW>=Version 1.1: added by Bjoern:
		//  the Scheduler had to be changed because he used to hang in a infinite loop 
		//  especially when using absolute values and the target values were nearly reached; 
		//  this setting seems to achieve appropriate results in a much faster way
		public void next() {
			if (this.enabled) {
				if (this.miss1 >= this.limit || this.miss2 >= this.limit) {
					if (RandomPlacing.this.cbApproach.isVisible())
						RandomPlacing.this.cbApproach.setSelected(this.approach);
					if (RandomPlacing.this.cbShake.isVisible())
						RandomPlacing.this.cbShake.setSelected(true);
					if (RandomPlacing.this.cbTwist.isVisible())
						RandomPlacing.this.cbTwist.setSelected(true);
				} else {
					RandomPlacing.this.cbApproach.setSelected(false);
					RandomPlacing.this.cbShake.setSelected(false);
					RandomPlacing.this.cbTwist.setSelected(false);
					this.miss1 = 0;
					this.miss2 = 0;
				}
			} else {
				this.miss1 = 0;
				this.miss2 = 0;
			}
		}
		
		//  OLD<=Version 1.02
//		public void next() {
//			if (this.enabled) {
//
//				if (this.miss1 >= this.limit || this.miss2 >= this.limit) {
//
//					RandomPlacing.this.cb2.setSelected(this.approach);
//					RandomPlacing.this.cb3.setSelected(true);
//					RandomPlacing.this.cb4.setSelected(true);
//
//				} else {
//					RandomPlacing.this.cb2.setSelected(false);
//					RandomPlacing.this.cb3.setSelected(false);
//					RandomPlacing.this.cb4.setSelected(false);
//				}
//			}
//
//			this.miss1 = 0;
//			this.miss2 = 0;
//
//		}

	}

	// public void actionPerformed(ActionEvent e) {
	//	
	// start.setVisible(false);
	// started=true;
	//		
	// }

}
