/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.HashSet;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point2f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithmGUI;

/**
 * 
 * The Wanderer is an algorithm looking for an opportunity to rest. He is always
 * looking for free space where to keep standing. In addition, he is also able
 * to look for fellows and to move closer to them. If the Wanderer is able to
 * dance, he will turn himself until he has the closest position possible to his
 * fellow.
 * 
 * @author Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class TheWandererRookie extends MembranePackingAlgorithm implements
		ItemListener, ChangeListener {

	// 2D Placement variables (X,Z)
	public boolean wantsToStop = false;
	public boolean wanderForFellow = false;
	public boolean wanderDancing = false;

	private JSpinner stepSizeSpinner = new JSpinner();
	private float stepSize = 1.0f;
	private SpinnerNumberModel stepSizeSpinnerModel = new SpinnerNumberModel(
			1.0f, 0.0001f, 100.0f, 0.1f);
	private JLabel stepSizeLabel = new JLabel(
			"<html>&nbsp;StepSize&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</html>");

	private JSpinner defineMaxNullRunsSpinner = new JSpinner();
	private int maxNullRuns = 5;
	private SpinnerNumberModel defineMaxNullRunsSpinnerModel = new SpinnerNumberModel(
			5, 0, 50, 1);
	private JLabel defineMaxNullRunsLabel = new JLabel(
			"<html>&nbsp;Possible Missteps&nbsp;&nbsp;</html>");

	private int modus = 0;

	private JCheckBox chk1, chk2, chk3, chk4, chk5, chk6, chk7, chk8, chk9;

	private JLabel chk9Label = new JLabel("and dance");

	HashMap<String, Float> originalLipidTypeHeights = new HashMap<String, Float>();

	/**
	 * handles users GUI settings
	 */
	public void itemStateChanged(ItemEvent arg0) {

		if (arg0.getSource().equals(chk1)) {
			if (chk1.isSelected()) {
				if (chk2 != null && chk3 != null) {
					chk2.setSelected(false);
					chk3.setSelected(false);
					chk4.setSelected(false);
				}
				modus = 0;
			}

		} else if (arg0.getSource().equals(chk2)) {
			if (chk2.isSelected()) {
				chk1.setSelected(false);
				chk3.setSelected(false);
				chk4.setSelected(false);
				modus = 1;
			}

		} else if (arg0.getSource().equals(chk3)) {
			if (chk3.isSelected()) {
				chk1.setSelected(false);
				chk2.setSelected(false);
				chk4.setSelected(false);
				modus = 2;
			}
		} else if (arg0.getSource().equals(chk4)) {
			if (chk4.isSelected()) {
				chk1.setSelected(false);
				chk2.setSelected(false);
				chk3.setSelected(false);
				modus = 3;
			}
		} else if (arg0.getSource().equals(chk5)) {
			if (chk5.isSelected()) {
			} else {
			}
		} else if (arg0.getSource().equals(chk8)) {
			if (chk8.isSelected()) {
				wanderForFellow = true;
				chk9.setEnabled(true);
				chk9Label.setEnabled(true);
			} else {
				wanderForFellow = false;
				wanderDancing = false;
				chk9.setSelected(false);
				chk9.setEnabled(false);
				chk9Label.setEnabled(false);

			}
		} else if (arg0.getSource().equals(chk9)) {
			if (chk9.isSelected()) {
				wanderDancing = true;
			} else {
				wanderDancing = false;
			}
		}

		if (chk1.isSelected() == false && chk2.isSelected() == false
				&& chk3.isSelected() == false && chk4.isSelected() == false) {
			chk1.setSelected(true);
			modus = 0;
		}

	}

	/**
	 * Handles User's GUI settings
	 */
	public void stateChanged(ChangeEvent arg0) {
		// StepSize
		if (arg0.getSource().equals(stepSizeSpinner)) {
			stepSize = new Float(stepSizeSpinner.getValue().toString());
			// MaxNullRuns
		} else if (arg0.getSource().equals(defineMaxNullRunsSpinner)) {
			maxNullRuns = new Integer(defineMaxNullRunsSpinner.getValue()
					.toString());
		}

	}

	/**
	 * Generates the GUI with different options.
	 */
	public boolean buildAndShowGui() {

		JPanel mainpanel = new JPanel();
		mainpanel.setLayout(new BoxLayout(mainpanel, BoxLayout.Y_AXIS));

		JPanel panel10 = new JPanel();
		panel10.setLayout(new BoxLayout(panel10, BoxLayout.X_AXIS));
		panel10.setBackground(Color.GRAY);
		panel10.add(new JLabel(
				"<html>&nbsp;&nbsp;&nbsp;Wanderer Settings</html>"));

		stepSizeSpinner = new JSpinner();
		stepSizeSpinner.setMaximumSize(new Dimension(60, 50));
		stepSizeSpinner.setMinimumSize(new Dimension(60, 50));
		stepSizeSpinner.setModel(stepSizeSpinnerModel);
		panel10.add(stepSizeSpinner);
		panel10.add(stepSizeLabel);
		stepSizeSpinner.addChangeListener(this);

		defineMaxNullRunsSpinner = new JSpinner();
		defineMaxNullRunsSpinner.setMaximumSize(new Dimension(60, 50));
		defineMaxNullRunsSpinner.setMinimumSize(new Dimension(60, 50));
		defineMaxNullRunsSpinner.setModel(defineMaxNullRunsSpinnerModel);
		panel10.add(defineMaxNullRunsSpinner);
		panel10.add(defineMaxNullRunsLabel);
		defineMaxNullRunsSpinner.addChangeListener(this);

		chk8 = new JCheckBox();
		chk8.addItemListener(this);
		chk8.setSelected(false);
		chk8.setBackground(Color.GRAY);
		panel10.add(chk8);
		panel10.add(new JLabel("<html>Wander for Fellow</html>"));

		chk9 = new JCheckBox();
		chk9.addItemListener(this);
		chk9.setSelected(false);
		chk9.setBackground(Color.GRAY);
		chk9.setEnabled(false);
		panel10.add(chk9);
		chk9Label.setEnabled(false);
		panel10.add(chk9Label);

		JPanel panel11 = new JPanel();
		panel11.setLayout(new BoxLayout(panel11, BoxLayout.X_AXIS));
		panel11.add(new JLabel("<html>&nbsp;</html>"));

		JPanel panel2 = new JPanel();
		panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));
		panel2.setBorder(BorderFactory.createEtchedBorder());

		JPanel panel3 = new JPanel();
		panel3.setLayout(new BoxLayout(panel3, BoxLayout.X_AXIS));
		chk1 = new JCheckBox();
		chk1.addItemListener(this);
		chk1.setSelected(true);
		panel3.add(chk1);
		panel3.add(new JLabel(
				"<html>Activate for rotating lipids along the Y-Axis in 1 degree steps (1,2,3....360)</html>"));

		JPanel panel4 = new JPanel();
		panel4.setLayout(new BoxLayout(panel4, BoxLayout.X_AXIS));
		chk2 = new JCheckBox();
		chk2.addItemListener(this);
		chk2.setSelected(false);
		panel4.add(chk2);
		panel4.add(new JLabel(
				"<html>Activate for rotating lipids along the Y-Axis in 45 degree steps (0,45,90...315,360)</html>"));

		JPanel panel5 = new JPanel();
		panel5.setLayout(new BoxLayout(panel5, BoxLayout.X_AXIS));
		chk3 = new JCheckBox();
		chk3.addItemListener(this);
		chk3.setSelected(false);
		panel5.add(chk3);
		panel5.add(new JLabel(
				"<html>Activate for rotating lipids along the Y-Axis in 90 degree steps (90,180,270,360)</html>"));

		JPanel panel6 = new JPanel();
		panel6.setLayout(new BoxLayout(panel6, BoxLayout.X_AXIS));
		chk4 = new JCheckBox();
		chk4.addItemListener(this);
		chk4.setSelected(false);
		panel6.add(chk4);
		panel6.add(new JLabel(
				"<html>Activate for no rotation along the Y-Axis</html>"));

		chk5 = new JCheckBox();
		chk5.addItemListener(this);
		chk5.setSelected(true);

		JPanel panel9 = new JPanel();
		panel9.setLayout(new BoxLayout(panel9, BoxLayout.X_AXIS));
		panel9.setBackground(Color.GRAY);

		panel9.add(new JLabel(
				"<html>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Activate for</html>"));

		chk6 = new JCheckBox();
		chk6.addItemListener(this);
		chk6.setSelected(true);
		chk6.setBackground(Color.GRAY);
		panel9.add(chk6);
		panel9.add(new JLabel("<html>Extracellular</html>"));

		chk7 = new JCheckBox();
		chk7.addItemListener(this);
		chk7.setSelected(true);
		chk7.setBackground(Color.GRAY);
		panel9.add(chk7);
		panel9.add(new JLabel("<html>Intracellular Lipids</html>"));

		panel2.add(panel10);
		panel2.add(panel11);

		panel2.add(panel3);
		panel2.add(panel4);
		panel2.add(panel5);
		panel2.add(panel6);

		panel2.add(panel9);

		mainpanel.add(panel2);

		return MembranePackingAlgorithmGUI.popUpGUI(mainpanel,
				"The Wanderer Rookie",
				"Choose your routing and start wandering ...");
	}

	/**
	 * The runtime of this algorithm is:
	 * 
	 * n*([F+]D[+C]), where
	 * 
	 * n = number of attempts to place new lipids F = place first lipid in row,
	 * with: FreeEast, FreeSouth, [FellowWest, FellowNorth, [RotateNorthWest]] D
	 * = place a follow-up lipid directly, with: FreeEast, FreeSouth,
	 * [FellowWest, FellowNorth, [RotateNorthWest]] C = place a correcting
	 * follow-up lipid if the first attempt did not succed, with:
	 * ([FellowNorth,] FreedomSouth, FreedomEast, [FellowNorth, FellowWest]) ||
	 * (FreedomEast, FreedomSouth, [FellowWest, FellowNorth, [RotateNorthWest]])
	 */
	public void fillWithLipids() throws Exception {

		// initialize variables
		boolean firstLipidInRow = true;
		Point3f oldLipidPosition = new Point3f(0.0f, 0.0f, 0.0f);
		float oldAddedLipidRadius = 0.0f;
		Lipid oldAddedLipid = null;
		Lipid newLipidToAdd = null;
		HashSet<Lipid> lipidsOfActualRuns = new HashSet<Lipid>();
		Point3f newLipidPosition = null;
		int newLipidRotation = 0;
		float old1stInRowZPosition = 0.0f;
		boolean southIntersected = true;
		int newLipidDomainID = 0;
		int nullRuns = 0;
		int progressTotal = (this.getMembraneSize().height) * 2;
		int progress = 0;
		int progressNew = 0;

		// initialize z variables
		// get the number of runs for the actual side of membrane
		int numberOfRuns = 0;
		float plusForExtraMinusForIntra = 1.0f;
		// compute values for the progress bar and the number of runs
		// only intracellular
		if ((this.hasMembraneAnyIntracellularLipids() && chk7.isSelected())
				&& !(this.hasMembraneAnyExtracellularLipids() && chk6
						.isSelected())) {
			numberOfRuns++;
			plusForExtraMinusForIntra = -1.0f;
			progressTotal = progressTotal / 2;
			this.displayInformation("Wander on the intracellular side.");
			// only extracellular
		} else if (!(this.hasMembraneAnyIntracellularLipids() && chk7
				.isSelected())
				&& (this.hasMembraneAnyExtracellularLipids() && chk6
						.isSelected())) {
			progressTotal = progressTotal / 2;
			this.displayInformation("Wander on the extracellular side.");
			// no membrane side selected, prepare to exit
		} else if ((!this.hasMembraneAnyIntracellularLipids() || !chk7
				.isSelected())
				&& (!this.hasMembraneAnyExtracellularLipids() || !chk6
						.isSelected()))
			numberOfRuns = 2;
		// extra- and intracellular
		else
			this.displayInformation("Wander on the extracellular side.");

		// while both membrane sided are not complete
		while (numberOfRuns < 2) {

			Point3f validPosition = new Point3f();

			// creating the first lipid in a row
			if (firstLipidInRow) {

				// calculate the position of the new lipid, add it to the
				// membrane, set the position

				newLipidDomainID = this.getMicroDomainAtXZ(oldAddedLipidRadius,
						old1stInRowZPosition + oldAddedLipidRadius);

				newLipidToAdd = this.createNewLipid(newLipidToAdd,
						newLipidDomainID, numberOfRuns);

				this.storeOriginalLipidTypeHeight(newLipidToAdd);

				// add the actual lipid to the hashcode to identify if it is
				// part of the actual runs
				lipidsOfActualRuns.add(newLipidToAdd);
				// saving the radius for the calculation of the new lipid and
				// the one in the next run
				oldAddedLipidRadius = newLipidToAdd.calculateRadius();
				// set the random rotation
				newLipidRotation = this.getRandomRotationInMode(modus);
				newLipidToAdd.rotY(newLipidRotation);
				// set the position
				newLipidPosition = new Point3f(oldAddedLipidRadius,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(newLipidToAdd)),
						old1stInRowZPosition + oldAddedLipidRadius);
				newLipidToAdd.setPosition(newLipidPosition);
				// add the lipid to the membrane
				this.addNewLipid(newLipidToAdd, numberOfRuns);

				validPosition = wanderForFreedomEast(newLipidToAdd,
						newLipidDomainID, plusForExtraMinusForIntra);
				validPosition = wanderForFreedomSouth(newLipidToAdd,
						newLipidDomainID, plusForExtraMinusForIntra);
				if (wanderForFellow) {
					validPosition = wanderForFellowWest(newLipidToAdd,
							newLipidDomainID, plusForExtraMinusForIntra);
					validPosition = wanderForFellowNorth(newLipidToAdd,
							newLipidDomainID, plusForExtraMinusForIntra);
					if ((validPosition != null) && (wanderDancing)) {
						Point3f tempPos = wanderForOptimalRotationNorthWest(
								newLipidToAdd, newLipidDomainID,
								plusForExtraMinusForIntra, modus);
						if (tempPos != null)
							validPosition = tempPos;
					}
				}

				if (validPosition != null) {
					oldAddedLipid = newLipidToAdd;
					old1stInRowZPosition = newLipidToAdd.getPosition().z;
				} else {
					// remove the lost wanderer
					Lipid[] lipidArray = { newLipidToAdd };
					this.removeLipidsFromMembrane(lipidArray);
				}
				southIntersected = false;
				firstLipidInRow = false;

				// remove a lost wanderer
				if (validPosition == null) {
					Lipid[] lipidArray = { newLipidToAdd };
					this.removeLipidsFromMembrane(lipidArray);
				}
			}

			// place a follow-up lipid (first attempt)

			// saving the position for the calculation of the next lipid in the
			// next run
			if (oldAddedLipid != null)
				oldLipidPosition = oldAddedLipid.getPosition();
			if (oldLipidPosition == null)
				oldLipidPosition = new Point3f(0.0f, 0.0f, 0.0f);
			// calculate the position of the new lipid, add it to the membrane,
			// set the position
			newLipidDomainID = this.getMicroDomainAtXZ(oldLipidPosition.x
					+ oldAddedLipidRadius, oldLipidPosition.z);

			newLipidToAdd = this.createNewLipid(newLipidToAdd,
					newLipidDomainID, numberOfRuns);

			this.storeOriginalLipidTypeHeight(newLipidToAdd);

			// add the actual lipid to the hashcode to identify if it is part of
			// the actual runs
			lipidsOfActualRuns.add(newLipidToAdd);
			// saving the radius for the calculation of the next lipid in the
			// next run
			oldAddedLipidRadius = newLipidToAdd.calculateRadius();
			// set the random rotation
			newLipidRotation = this.getRandomRotationInMode(modus);
			newLipidToAdd.rotY(newLipidRotation);
			// set the position
			newLipidPosition = new Point3f(oldLipidPosition.x
					+ oldAddedLipidRadius,
					(plusForExtraMinusForIntra * this
							.getLipidInMembraneHeight(newLipidToAdd)),
					oldLipidPosition.z);
			newLipidToAdd.setPosition(newLipidPosition);
			// add the lipid to the membrane
			this.addNewLipid(newLipidToAdd, numberOfRuns);

			// start the wandering
			if ((this.intersectsWithLipids(newLipidToAdd))
					|| (intersectsWithProteins(newLipidToAdd))
					|| (intersectsBorders(newLipidToAdd)))
				validPosition = wanderForFreedomEast(newLipidToAdd,
						newLipidDomainID, plusForExtraMinusForIntra);
			if ((this.intersectsWithLipids(newLipidToAdd))
					|| (intersectsWithProteins(newLipidToAdd))
					|| (intersectsBorders(newLipidToAdd)))
				validPosition = wanderForFreedomSouth(newLipidToAdd,
						newLipidDomainID, plusForExtraMinusForIntra);
			if (wanderForFellow) {
				validPosition = wanderForFellowWest(newLipidToAdd,
						newLipidDomainID, plusForExtraMinusForIntra);
				validPosition = wanderForFellowNorth(newLipidToAdd,
						newLipidDomainID, plusForExtraMinusForIntra);
				if ((validPosition != null) && (wanderDancing)) {
					Point3f tempPos = wanderForOptimalRotationNorthWest(
							newLipidToAdd, newLipidDomainID,
							plusForExtraMinusForIntra, modus);
					if (tempPos != null)
						validPosition = tempPos;
				}
			}

			// place a follow-up lipid (second attempt)

			// if the placing did not work so far, create a new line
			if (validPosition == null) {
				newLipidPosition = new Point3f(oldAddedLipidRadius,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(newLipidToAdd)),
						old1stInRowZPosition + oldAddedLipidRadius);
				newLipidToAdd.setPosition(newLipidPosition);
				// if the lipid is out in the southern desert
				if (intersectsBorderSouth(newLipidToAdd)) {
					if (nullRuns >= maxNullRuns)
						southIntersected = true;
					newLipidPosition = new Point3f(oldAddedLipidRadius,
							(plusForExtraMinusForIntra * this
									.getLipidInMembraneHeight(newLipidToAdd)),
							this.getMembraneSize().height - oldAddedLipidRadius);
					newLipidToAdd.setPosition(newLipidPosition);
					if (wanderForFellow)
						validPosition = wanderForFellowNorth(newLipidToAdd,
								newLipidDomainID, plusForExtraMinusForIntra);
					if (validPosition == null)
						validPosition = wanderForFreedomSouth(newLipidToAdd,
								newLipidDomainID, plusForExtraMinusForIntra);
					if (validPosition == null)
						validPosition = wanderForFreedomEast(newLipidToAdd,
								newLipidDomainID, plusForExtraMinusForIntra);
					if (wanderForFellow) {
						validPosition = wanderForFellowNorth(newLipidToAdd,
								newLipidDomainID, plusForExtraMinusForIntra);
						validPosition = wanderForFellowWest(newLipidToAdd,
								newLipidDomainID, plusForExtraMinusForIntra);
					}
					// if the lipid is at the beginning of a row
				} else {
					validPosition = wanderForFreedomEast(newLipidToAdd,
							newLipidDomainID, plusForExtraMinusForIntra);
					validPosition = wanderForFreedomSouth(newLipidToAdd,
							newLipidDomainID, plusForExtraMinusForIntra);
					if (wanderForFellow) {
						validPosition = wanderForFellowWest(newLipidToAdd,
								newLipidDomainID, plusForExtraMinusForIntra);
						validPosition = wanderForFellowNorth(newLipidToAdd,
								newLipidDomainID, plusForExtraMinusForIntra);
						if ((validPosition != null) && (wanderDancing)) {
							Point3f tempPos = wanderForOptimalRotationNorthWest(
									newLipidToAdd, newLipidDomainID,
									plusForExtraMinusForIntra, modus);
							if (tempPos != null)
								validPosition = tempPos;
						}
					}
				}
				if (validPosition != null) {
					old1stInRowZPosition = newLipidToAdd.getPosition().z;
				}
			}
			if (validPosition != null) {
				// show progress bar
				if (numberOfRuns == 1)
					progressNew = ((int) ((float) (this.getMembraneSize().height + validPosition.z) * (100f / (float) progressTotal)));
				else
					progressNew = ((int) ((float) validPosition.z * (100f / (float) progressTotal)));
				if (progressNew > progress)
					progress = progressNew;
				this.setDisplayedProgress(progress);
				// save the actual lipid for the next run
				oldAddedLipid = newLipidToAdd;
			}
			if (validPosition == null)
				nullRuns++;
			else
				nullRuns = 0;
			if (southIntersected || (validPosition == null)
					|| (nullRuns >= maxNullRuns)) {
				// remove the lost wanderer
				Lipid[] lipidArray = { newLipidToAdd };
				this.removeLipidsFromMembrane(lipidArray);
				if (validPosition == null)
					firstLipidInRow = true;
				if ((southIntersected && validPosition == null)
						|| (nullRuns >= maxNullRuns)) {
					numberOfRuns++;
					nullRuns = 0;
					plusForExtraMinusForIntra = -1.0f;
					firstLipidInRow = true;
					oldLipidPosition = null;
					old1stInRowZPosition = 0.0f;
					this.displayInformation("Wander on the intracellular side.");
					if (!(this.hasMembraneAnyIntracellularLipids() && chk7
							.isSelected()))
						numberOfRuns++;
				}
			}
			southIntersected = false;
			if (wantsToStop) {
				removeIntersectingEarthOrSkyLipids(lipidsOfActualRuns);
				return;
			}
		}
		removeIntersectingEarthOrSkyLipids(lipidsOfActualRuns);
	}

	/**
	 * Stores the original lipid height. This should be used prior any changes
	 * on the lipid rotation.
	 * 
	 * @param Lipid
	 *            lipid
	 */
	private void storeOriginalLipidTypeHeight(Lipid lipid) {
		if (!originalLipidTypeHeights.containsKey(lipid.getType())) {
			Point3d lower = new Point3d();
			Point3d upper = new Point3d();
			lipid.getBounds().getLower(lower);
			lipid.getBounds().getUpper(upper);
			originalLipidTypeHeights.put(lipid.getType(), new Float(lower.y
					+ upper.y));
		}
	}

	/**
	 * Get the original height of the given lipid. This value was stored in the
	 * method: storeOriginalLipidTypeHeight(Lipid lipid)
	 * 
	 * @param lipid
	 * @return
	 */
	Float getOriginalLipidTypeHeight(Lipid lipid) {
		return originalLipidTypeHeights.get(lipid.getType());
	}

	/**
	 * Create a new lipid.
	 * 
	 * @param newLipidToAdd
	 * @param newLipidDomainID
	 * @param numberOfRuns
	 * @return
	 */
	public Lipid createNewLipid(Lipid newLipidToAdd, int newLipidDomainID,
			int numberOfRuns) {
		// calculate the position of the new lipid, add it to the membrane, set
		// the position
		if (newLipidDomainID < 1) {
			if (numberOfRuns == 0)
				newLipidToAdd = this.createNextExtracellularLipid();
			else
				newLipidToAdd = this.createNextIntracellularLipid();
		} else {
			if (numberOfRuns == 0)
				newLipidToAdd = this.getMicroDomainAt(newLipidDomainID - 1)
						.createNextExtracellularLipid();
			else
				newLipidToAdd = this.getMicroDomainAt(newLipidDomainID - 1)
						.createNextIntracellularLipid();
		}
		return newLipidToAdd;

	}

	/**
	 * Add the given lipid to the membrane.
	 * 
	 * @param newLipidToAdd
	 * @param numberOfRuns
	 * @throws Exception
	 */
	public void addNewLipid(Lipid newLipidToAdd, int numberOfRuns)
			throws Exception {
		// add the lipid to the membrane
		if (numberOfRuns == 0)
			this.addExtracellularLipid(newLipidToAdd);
		else
			this.addIntracellularLipid(newLipidToAdd);
	}

	/**
	 * Move the placed and trapped lipid to the east until it is free. Note: The
	 * given Lipid has to be added to the membrane first.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForFreedomEast(Lipid givenLipid, int microDomainID,
			float plusForExtraMinusForIntra) {
		if (!intersectsBorderEast(givenLipid)) {
			Point3f newLipidPosition = givenLipid.getPosition();
			int newLipidDomainID = this.getMicroDomainAtXZ(
					givenLipid.getPosition().x, givenLipid.getPosition().z);
			while ((this.intersectsWithLipids(givenLipid)
					|| (intersectsWithProteins(givenLipid))
					|| intersectsBorderWest(givenLipid) || (newLipidDomainID != microDomainID))
					&& !intersectsBorderEast(givenLipid)) {
				newLipidPosition = new Point3f(givenLipid.getPosition().x
						+ stepSize,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(givenLipid)),
						givenLipid.getPosition().z);
				newLipidDomainID = this.getMicroDomainAtXZ(newLipidPosition.x,
						newLipidPosition.z);
				givenLipid.setPosition(newLipidPosition);
				// rollback repositioning because of intersection
				if (intersectsBorderEast(givenLipid)) {
					newLipidPosition = new Point3f(givenLipid.getPosition().x
							- stepSize,
							(plusForExtraMinusForIntra * this
									.getLipidInMembraneHeight(givenLipid)),
							givenLipid.getPosition().z);
					givenLipid.setPosition(newLipidPosition);
					return null;
				}

				// remove
				if (wantsToStop) {
					return null;
				}

			}
			if (intersectsBorders(givenLipid))
				return null;
			else
				return newLipidPosition;
		}
		return null;
	}

	/**
	 * Move the placed and free lipid to the east until it meeting a fellow or
	 * the border. Note: The given Lipid has to be added to the membrane first.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForFellowEast(Lipid givenLipid, int microDomainID,
			float plusForExtraMinusForIntra) {
		if (!intersectsBorderEast(givenLipid)) {
			Point3f newLipidPosition = null;
			int newLipidDomainID = this.getMicroDomainAtXZ(
					givenLipid.getPosition().x, givenLipid.getPosition().z);
			while (!this.intersectsWithLipids(givenLipid)
					&& !intersectsWithProteins(givenLipid)
					&& !intersectsBorderEast(givenLipid)
					&& (newLipidDomainID == microDomainID)) {
				newLipidPosition = new Point3f(givenLipid.getPosition().x
						+ stepSize,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(givenLipid)),
						givenLipid.getPosition().z);
				newLipidDomainID = this.getMicroDomainAtXZ(newLipidPosition.x,
						newLipidPosition.z);
				givenLipid.setPosition(newLipidPosition);
				// rollback repositioning because of intersection
				if (this.intersectsWithLipids(givenLipid)
						|| intersectsWithProteins(givenLipid)
						|| intersectsBorderEast(givenLipid)
						|| (newLipidDomainID != microDomainID)) {
					newLipidPosition = new Point3f(givenLipid.getPosition().x
							- stepSize,
							(plusForExtraMinusForIntra * this
									.getLipidInMembraneHeight(givenLipid)),
							givenLipid.getPosition().z);
					givenLipid.setPosition(newLipidPosition);
					if (intersectsBorders(givenLipid))
						return null;
					else
						return newLipidPosition;
				}
			}
			return null;
		}
		return null;
	}

	/**
	 * Move the placed and trapped lipid to the west until it is free. Note: The
	 * given Lipid has to be added to the membrane first.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForFreedomWest(Lipid givenLipid, int microDomainID,
			float plusForExtraMinusForIntra) {
		if (!intersectsBorderWest(givenLipid)) {
			Point3f newLipidPosition = givenLipid.getPosition();
			int newLipidDomainID = this.getMicroDomainAtXZ(
					givenLipid.getPosition().x, givenLipid.getPosition().z);
			while ((this.intersectsWithLipids(givenLipid)
					|| intersectsWithProteins(givenLipid)
					|| intersectsBorderEast(givenLipid) || (newLipidDomainID != microDomainID))
					&& !intersectsBorderWest(givenLipid)) {
				newLipidPosition = new Point3f(givenLipid.getPosition().x
						- stepSize,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(givenLipid)),
						givenLipid.getPosition().z);
				newLipidDomainID = this.getMicroDomainAtXZ(newLipidPosition.x,
						newLipidPosition.z);
				givenLipid.setPosition(newLipidPosition);
				// rollback repositioning because of intersection
				if (intersectsBorderWest(givenLipid)) {
					newLipidPosition = new Point3f(givenLipid.getPosition().x
							+ stepSize,
							(plusForExtraMinusForIntra * this
									.getLipidInMembraneHeight(givenLipid)),
							givenLipid.getPosition().z);
					givenLipid.setPosition(newLipidPosition);
					return null;
				}
			}
			if (intersectsBorders(givenLipid))
				return null;
			else
				return newLipidPosition;
		}
		return null;
	}

	/**
	 * Move the placed and free lipid to the west until it meeting a fellow or
	 * the border. Note: The given Lipid has to be added to the membrane first.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForFellowWest(Lipid givenLipid, int microDomainID,
			float plusForExtraMinusForIntra) {
		if (!intersectsBorderWest(givenLipid)) {
			Point3f newLipidPosition = null;
			int newLipidDomainID = this.getMicroDomainAtXZ(
					givenLipid.getPosition().x, givenLipid.getPosition().z);
			while (!this.intersectsWithLipids(givenLipid)
					&& !intersectsWithProteins(givenLipid)
					&& !intersectsBorderWest(givenLipid)
					&& (newLipidDomainID == microDomainID)) {
				newLipidPosition = new Point3f(givenLipid.getPosition().x
						- stepSize,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(givenLipid)),
						givenLipid.getPosition().z);
				newLipidDomainID = this.getMicroDomainAtXZ(newLipidPosition.x,
						newLipidPosition.z);
				givenLipid.setPosition(newLipidPosition);
				// rollback repositioning because of intersection
				if (this.intersectsWithLipids(givenLipid)
						|| intersectsWithProteins(givenLipid)
						|| intersectsBorderWest(givenLipid)
						|| (newLipidDomainID != microDomainID)) {
					newLipidPosition = new Point3f(givenLipid.getPosition().x
							+ stepSize,
							(plusForExtraMinusForIntra * this
									.getLipidInMembraneHeight(givenLipid)),
							givenLipid.getPosition().z);
					givenLipid.setPosition(newLipidPosition);
					if (intersectsBorders(givenLipid))
						return null;
					else
						return newLipidPosition;
				}
			}
			return null;
		}
		return null;
	}

	/**
	 * Move the placed and trapped lipid to the north until it is free. Note:
	 * The given Lipid has to be added to the membrane first.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForFreedomNorth(Lipid givenLipid, int microDomainID,
			float plusForExtraMinusForIntra) {
		if (!intersectsBorderNorth(givenLipid)) {
			Point3f newLipidPosition = givenLipid.getPosition();
			int newLipidDomainID = this.getMicroDomainAtXZ(
					givenLipid.getPosition().x, givenLipid.getPosition().z);
			while ((this.intersectsWithLipids(givenLipid)
					|| intersectsWithProteins(givenLipid)
					|| intersectsBorderSouth(givenLipid) || (newLipidDomainID != microDomainID))
					&& !intersectsBorderNorth(givenLipid)) {
				newLipidPosition = new Point3f(givenLipid.getPosition().x,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(givenLipid)),
						givenLipid.getPosition().z - stepSize);
				newLipidDomainID = this.getMicroDomainAtXZ(newLipidPosition.x,
						newLipidPosition.z);
				givenLipid.setPosition(newLipidPosition);
				// rollback repositioning because of intersection
				if (intersectsBorders(givenLipid)) {
					newLipidPosition = new Point3f(givenLipid.getPosition().x,
							(plusForExtraMinusForIntra * this
									.getLipidInMembraneHeight(givenLipid)),
							givenLipid.getPosition().z + stepSize);
					givenLipid.setPosition(newLipidPosition);
					return null;
				}
			}
			if (intersectsBorders(givenLipid))
				return null;
			else
				return newLipidPosition;
		}
		return null;
	}

	/**
	 * Move the placed and free lipid to the north until it meeting a fellow or
	 * the border. Note: The given Lipid has to be added to the membrane first.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForFellowNorth(Lipid givenLipid, int microDomainID,
			float plusForExtraMinusForIntra) {
		if (!intersectsBorderNorth(givenLipid)) {
			Point3f newLipidPosition = null;
			int newLipidDomainID = this.getMicroDomainAtXZ(
					givenLipid.getPosition().x, givenLipid.getPosition().z);
			while (!this.intersectsWithLipids(givenLipid)
					&& !intersectsWithProteins(givenLipid)
					&& !intersectsBorderNorth(givenLipid)
					&& (newLipidDomainID == microDomainID)) {
				newLipidPosition = new Point3f(givenLipid.getPosition().x,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(givenLipid)),
						givenLipid.getPosition().z - stepSize);
				newLipidDomainID = this.getMicroDomainAtXZ(newLipidPosition.x,
						newLipidPosition.z);
				givenLipid.setPosition(newLipidPosition);
				// rollback repositioning because of intersection
				if (this.intersectsWithLipids(givenLipid)
						|| intersectsWithProteins(givenLipid)
						|| intersectsBorderNorth(givenLipid)
						|| (newLipidDomainID != microDomainID)) {
					newLipidPosition = new Point3f(givenLipid.getPosition().x,
							(plusForExtraMinusForIntra * this
									.getLipidInMembraneHeight(givenLipid)),
							givenLipid.getPosition().z + stepSize);
					givenLipid.setPosition(newLipidPosition);
					if (intersectsBorders(givenLipid))
						return null;
					else
						return newLipidPosition;
				}
			}
			return null;
		}
		return null;
	}

	/**
	 * Move the placed and trapped lipid to the south until it is free. Note:
	 * The given Lipid has to be added to the membrane first.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForFreedomSouth(Lipid givenLipid, int microDomainID,
			float plusForExtraMinusForIntra) {
		if (!intersectsBorderSouth(givenLipid)) {
			Point3f newLipidPosition = givenLipid.getPosition();
			int newLipidDomainID = this.getMicroDomainAtXZ(
					givenLipid.getPosition().x, givenLipid.getPosition().z);
			while ((this.intersectsWithLipids(givenLipid)
					|| intersectsWithProteins(givenLipid)
					|| intersectsBorderNorth(givenLipid) || (newLipidDomainID != microDomainID))
					&& !intersectsBorderSouth(givenLipid)) {
				newLipidPosition = new Point3f(givenLipid.getPosition().x,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(givenLipid)),
						givenLipid.getPosition().z + stepSize);
				newLipidDomainID = this.getMicroDomainAtXZ(newLipidPosition.x,
						newLipidPosition.z);
				givenLipid.setPosition(newLipidPosition);
				// rollback repositioning because of intersection
				if (intersectsBorders(givenLipid)) {
					newLipidPosition = new Point3f(givenLipid.getPosition().x,
							(plusForExtraMinusForIntra * this
									.getLipidInMembraneHeight(givenLipid)),
							givenLipid.getPosition().z - stepSize);
					givenLipid.setPosition(newLipidPosition);
					return null;
				}
			}
			if (intersectsBorders(givenLipid))
				return null;
			else
				return newLipidPosition;
		}
		return null;
	}

	/**
	 * Move the placed and free lipid to the south until it meeting a fellow or
	 * the border. Note: The given Lipid has to be added to the membrane first.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForFellowSouth(Lipid givenLipid, int microDomainID,
			float plusForExtraMinusForIntra) {
		if (!intersectsBorderSouth(givenLipid)) {
			Point3f newLipidPosition = null;
			int newLipidDomainID = this.getMicroDomainAtXZ(
					givenLipid.getPosition().x, givenLipid.getPosition().z);
			while (!this.intersectsWithLipids(givenLipid)
					&& !intersectsWithProteins(givenLipid)
					&& !intersectsBorderSouth(givenLipid)
					&& (newLipidDomainID == microDomainID)) {
				newLipidPosition = new Point3f(givenLipid.getPosition().x,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(givenLipid)),
						givenLipid.getPosition().z + stepSize);
				newLipidDomainID = this.getMicroDomainAtXZ(newLipidPosition.x,
						newLipidPosition.z);
				givenLipid.setPosition(newLipidPosition);
				// rollback repositioning because of intersection
				if (this.intersectsWithLipids(givenLipid)
						|| (intersectsWithProteins(givenLipid))
						|| intersectsBorderSouth(givenLipid)
						|| (newLipidDomainID != microDomainID)) {
					newLipidPosition = new Point3f(givenLipid.getPosition().x,
							(plusForExtraMinusForIntra * this
									.getLipidInMembraneHeight(givenLipid)),
							givenLipid.getPosition().z - stepSize);
					givenLipid.setPosition(newLipidPosition);
					if (intersectsBorders(givenLipid))
						return null;
					else
						return newLipidPosition;
				}
			}
			return null;
		}
		return null;
	}

	/**
	 * Move the placed and trapped lipid to the south-east until it is free.
	 * Note: The given Lipid has to be added to the membrane first.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForFreedomSouthEast(Lipid givenLipid,
			int microDomainID, float plusForExtraMinusForIntra) {
		if (!intersectsBorderSouth(givenLipid)
				&& !intersectsBorderEast(givenLipid)) {
			Point3f newLipidPosition = givenLipid.getPosition();
			int newLipidDomainID = this.getMicroDomainAtXZ(
					givenLipid.getPosition().x, givenLipid.getPosition().z);
			while ((this.intersectsWithLipids(givenLipid)
					|| (intersectsWithProteins(givenLipid))
					|| intersectsBorderNorth(givenLipid)
					|| intersectsBorderWest(givenLipid) || (newLipidDomainID != microDomainID))
					&& !intersectsBorderSouth(givenLipid)
					&& !intersectsBorderEast(givenLipid)) {
				newLipidPosition = new Point3f(givenLipid.getPosition().x
						+ stepSize,
						(plusForExtraMinusForIntra * this
								.getLipidInMembraneHeight(givenLipid)),
						givenLipid.getPosition().z + stepSize);
				newLipidDomainID = this.getMicroDomainAtXZ(newLipidPosition.x,
						newLipidPosition.z);
				givenLipid.setPosition(newLipidPosition);
				// rollback repositioning because of intersection
				if (intersectsBorders(givenLipid)) {
					newLipidPosition = new Point3f(givenLipid.getPosition().x
							- stepSize,
							(plusForExtraMinusForIntra * this
									.getLipidInMembraneHeight(givenLipid)),
							givenLipid.getPosition().z - stepSize);
					givenLipid.setPosition(newLipidPosition);
					return null;
				}
			}
			if (intersectsBorders(givenLipid))
				return null;
			else
				return newLipidPosition;
		}
		return null;
	}

	/**
	 * Wanders to the North and to the West for a Fellow and tries to find the
	 * optimal y-Rotation, following the mode.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @param yRotation
	 * @param mode
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForOptimalRotationNorthWest(Lipid givenLipid,
			int microDomainID, float plusForExtraMinusForIntra, int mode) {

		Point3f oldValidPosition = givenLipid.getPosition();
		Point3f validPosition = null;
		int tempYRotation = 0;
		Matrix3f oldLipidMatrix = givenLipid.getRotationMatrix();

		while (tempYRotation < 360) {
			if (mode == 0) {
				givenLipid.rotY(1);
				tempYRotation = tempYRotation + 1;
			} else if (mode == 1) {
				givenLipid.rotY(45);
				tempYRotation = tempYRotation + 45;
			} else if (mode == 2) {
				givenLipid.rotY(90);
				tempYRotation = tempYRotation + 90;
			} else if (mode == 3) {
				return oldValidPosition;
			}
			validPosition = wanderForFellowWest(givenLipid, microDomainID,
					plusForExtraMinusForIntra);
			validPosition = wanderForFellowNorth(givenLipid, microDomainID,
					plusForExtraMinusForIntra);
			if (validPosition != null) {
				oldValidPosition = validPosition;
				oldLipidMatrix = givenLipid.getRotationMatrix();
			}
		}
		if (validPosition == null) {
			validPosition = oldValidPosition;
			givenLipid.setRotationMatrix(oldLipidMatrix);
			givenLipid.setPosition(validPosition);
		}
		return validPosition;
	}

	/**
	 * Move the placed and free lipid to the east until it meeting a fellow or
	 * the border. Note: The given Lipid has to be added to the membrane first.
	 * 
	 * @param givenLipid
	 * @param microDomainID
	 * @param plusForExtraMinusForIntra
	 * @return Point3f the new lipid position, it is null if the position is not
	 *         valid
	 */
	public Point3f wanderForFellowVertical(Lipid givenLipid, int microDomainID,
			float plusForExtraMinusForIntra, float[] directXZ) {

		float xCorrectionMovementTarget = givenLipid.getPosition().x - 20;
		float zCorrectionMovementTarget = givenLipid.getPosition().z;

		if (!intersectsBorderEast(givenLipid)
				&& (xCorrectionMovementTarget != 0 || zCorrectionMovementTarget != 0)) {
			Point3f newLipidPosition = null;
			int newLipidDomainID = this.getMicroDomainAtXZ(
					givenLipid.getPosition().x, givenLipid.getPosition().z);
			while ((givenLipid.getPosition().x < xCorrectionMovementTarget)
					|| !this.intersectsWithLipids(givenLipid)
					&& !intersectsWithProteins(givenLipid)
					&& !intersectsBorders(givenLipid)
					&& (newLipidDomainID == microDomainID)) {
				newLipidPosition = new Point3f(givenLipid.getPosition().x
						+ ((-1 * plusForExtraMinusForIntra) * stepSize),
						givenLipid.getPosition().y, givenLipid.getPosition().z);

				newLipidDomainID = this.getMicroDomainAtXZ(newLipidPosition.x,
						newLipidPosition.z);
				givenLipid.setPosition(newLipidPosition);
				// rollback repositioning because of intersection
				if (this.intersectsWithLipids(givenLipid)
						|| intersectsWithProteins(givenLipid)
						|| intersectsBorders(givenLipid)
						|| (newLipidDomainID != microDomainID)) {
					newLipidPosition = new Point3f(givenLipid.getPosition().x
							- ((-1 * plusForExtraMinusForIntra) * stepSize),
							givenLipid.getPosition().y,
							givenLipid.getPosition().z);
					givenLipid.setPosition(newLipidPosition);
					if (intersectsBorders(givenLipid))
						return null;
					else
						return newLipidPosition;
				}

				// remove
				if (wantsToStop) {
					return null;
				}

			}
			return null;
		}
		return null;
	}

	/**
	 * 
	 * Returns a random integer for the roation in the given mode. mode = 0 :
	 * 0..360 mode = 1 : 0, 45, 90, 135, 180, 225, 270, 315 mode = 2 : 0, 90,
	 * 180, 270 mode = 3 : 0
	 * 
	 * @param mode
	 * @return a random integer
	 */
	public int getRandomRotationInMode(int mode) {
		int r = 0;
		if (mode == 0) {
			r = this.random.nextInt(360);
		} else if (mode == 1) {
			int r2 = this.random.nextInt(8);
			if (r2 == 1) {
				r = 0;
			} else if (r2 == 2) {
				r = 45;
			} else if (r2 == 3) {
				r = 90;
			} else if (r2 == 4) {
				r = 135;
			} else if (r2 == 5) {
				r = 180;
			} else if (r2 == 6) {
				r = 225;
			} else if (r2 == 7) {
				r = 270;
			} else if (r2 == 8) {
				r = 315;
			}
		} else if (mode == 2) {
			int r3 = this.random.nextInt(4);
			if (r3 == 1) {
				r = 0;
			} else if (r3 == 2) {
				r = 90;
			} else if (r3 == 3) {
				r = 180;
			} else if (r3 == 4) {
				r = 270;
			}
		} else if (mode == 3) {
			r = 0;
		}

		return r;
	}

	/**
	 * Returns the microdomain at the given x/z coordinates.
	 * 
	 * @param xPosition
	 * @param zPosition
	 * @return the number of the micordomain
	 */
	public int getMicroDomainAtXZ(float xPosition, float zPosition) {
		Point2f positionXZ = new Point2f(xPosition, zPosition);
		int microDomain = this.getMicroDomainAtPoint(positionXZ,
				this.getCurrentLayerID());
		return microDomain;
	}

	/**
	 * Remove all lost wanderers from the intra- and extracellular side.
	 * 
	 * @param lipidVectorEC
	 */
	public void removeIntersectingEarthOrSkyLipids(
			HashSet<Lipid> lipidsOfActualRuns) {
		// delete all Earth or Sky intersecting lipids
		for (Lipid l : lipidsOfActualRuns) {
			if (intersectsEarthOrSky(l)) {
				Lipid[] lipidArray = { l };
				this.removeLipidsFromMembrane(lipidArray);
			}
		}
	}

	/**
	 * Returns true is the given Lipid intersects with a Lipid on both sides of
	 * the membrane. Note: In comparison to normal algorithms this algorithm
	 * also shifts the lipids on their Y-Axis. In addition the rotation on the
	 * X- and Z-Axis changes the orientation. So it may happen, that a the given
	 * lipid may intersect with a lipid on the other side of the membrane.
	 * 
	 * @param givenLipid
	 * @return
	 */
	public boolean intersectsWithLipids(Lipid givenLipid) {
		return (intersectsWithLipid(givenLipid, 0) || intersectsWithLipid(
				givenLipid, 1));
	}

	/**
	 * 
	 * returns full information about this algorithm
	 */
	public String getFullInformation() {
		String s = "<html>"
				+ "This algorithm handels every single lipid as a kind of <br>"
				+ "wanderer. The Wanderer is able to move into different <br>"
				+ "directions. He first tries to find a free space. <br><br>"
				+ "If he succeeds, he can search in addition for a fellow.<br>"
				+ "In this case, he will try to move as close as possible<br>"
				+ "to a fellow lipid. If the Wanderer is in a good mood,<br>"
				+ "he can also start dancing. In this case, he will rotate<br>"
				+ "along his vertical axis, trying to move as close as<br>"
				+ "possible to his neighbor.<br><br>"
				+ "The size of steps and rotations can be adjusted:<br>"
				+ "Large steps will be quiet fast, but of course the  <br>"
				+ "gaps between the lipids will be larger.<br><br>"
				
				+ "<i>This algorithm can run multiple times without <br> "
				+ "editing the former placed lipids!</i><br><br>" +
				""
				+ "Author: Bj&ouml;rn Sommer<br>"
				+ "Version: 1.0<br></html>";
		return s;
	}

	public String getName() {
		return "The Wanderer Rookie";
	}

	public Component getRuntimeControlPanel() {
		return null;
	}

	/**
	 * returns short information about this algorithm
	 */
	public String getShortInformation() {
		return "Handles lipids as wanderers distributing lipids in a linear way";
	}

	/**
	 * 
	 * returns a string representation of users GUI settings
	 */
	public String getStringRepresentationOfSetupParameters() {

		String s = "";

		if (wanderForFellow)
			s += "Wander for Fellow"
					+ ((wanderDancing) ? (" (dancing)") : (""));
		else
			s += "Wander straight forward.";

		s += "\n";

		s += "Version: 1.0";
		
		s += "\n";

		s += "StepSize: " + stepSize;

		s += "\n";

		s += "Possible Missteps: " + maxNullRuns;

		s += "\n";

		if (chk1.isSelected()) {
			s += "Rotating lipids along the Y-Axis in 1 degree steps (1,2,3....360).";
		} else if (chk2.isSelected()) {
			s += "Rotating lipids along the Y-Axis in 45 degree steps (0,45,90...315,360).";
		} else if (chk3.isSelected()) {
			s += "Rotating lipids along the Y-Axis in 90 degree steps (90,180,270,360).";
		} else if (chk4.isSelected()) {
			s += "No rotation along the Y-Axis.";
		}

		s += "\n";

		if (chk5.isSelected())
			s += "Adjust XZ-Rotation.";
		else
			s += "Do not adjust XZ-Rotation.";

		s += "\n";

		if (chk6.isSelected())
			s += "Extracellular Lipids, ";
		else
			s += "No Extracellular Lipids, ";
		if (chk7.isSelected())
			s += "Intracellular Lipids.";
		else
			s += "No Intracellular Lipids.";

		return s;

	}

	public boolean overrideExistingLipids() {
		return false;
	}

	public void stopAlgorithm() {
		wantsToStop = true;

	}

	public boolean supportsAbsoluteValues() {
		return false;
	}

	public boolean supportsMicrodomains() {
		return true;
	}

}