/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms.atomlevelminimizer;

import java.awt.Dimension;
import java.util.Hashtable;

import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.membranepackingalgorithm.Atom;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.EnergyFunction;


/**
 * This class stores and handles values regarding the force field. Also contains the energy function or 
 * handles user-defined ones
 * 
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
public class Energies {

	private Atom_Level_Minimizer algo;

	public Hashtable<String, Hashtable<String, Float>> hashEpsilon = new Hashtable<String, Hashtable<String, Float>>();
	public Hashtable<String, Hashtable<String, Float>> hashSigma = new Hashtable<String, Hashtable<String, Float>>();

	
	public float masterEpsilon = 2f;
	public float masterSigma = 2f;

	public EnergyFunction currentFunction = null;

	public Double lowestValue = null;

	public Energies(Atom_Level_Minimizer algo) {
		this.algo = algo;

	}

	@SuppressWarnings("unchecked")
	private float calculateTotalEnergy(Atom a, EnergyFunction ef) {

		boolean continuousBox = algo.myData.settings.isBoxContinuous;

		Dimension box = algo.getMembraneSize();

		float totalEnergy = 0f;

		Point3f second = null;
		Point3f third = null;

		if (continuousBox && !algo.myData.settings.AutoCutoff) {

			float cutoff = algo.myData.settings.cutoff;
			Vector3f add = new Vector3f();
			boolean overBorder = false;
			if ((a.position.x + cutoff) > box.width) {
				add.add(new Vector3f(-box.width, 0, 0));
				overBorder = true;
			}
			if ((a.position.z + cutoff) > box.height) {
				add.add(new Vector3f(0, 0, -box.height));
				overBorder = true;
			}

			if (overBorder) {

				second = new Point3f(a.position);
				second.add(add);

			}

			add = new Vector3f();
			overBorder = false;
			if ((a.position.x - cutoff) < 0) {
				add.add(new Vector3f(box.width, 0, 0));
				overBorder = true;
			}
			if ((a.position.z - cutoff) < 0) {
				add.add(new Vector3f(0, 0, box.height));
				overBorder = true;
			}

			if (overBorder) {

				third = new Point3f(a.position);
				third.add(add);

			}

		}

		for (int i = 0; i < algo.myData.contents.atomsToCompare.length; i++) {

			Atom b = algo.myData.contents.atomsToCompare[i];

			

			float cutoff;

			Float r0 = null;
			if (algo.myData.settings.AutoCutoff) {

				if (ef == null) {
					r0 = this.masterSigma;
					try {
						

						Hashtable<String, Float> hash = ((Hashtable<String, Float>[]) a
								.getUserData())[1];
						r0 = hash.get(b.getParameterKey()).floatValue();
					} catch (Exception e) {

					}

					cutoff = r0 * 2.5f;
				} else {
					cutoff = ef.getCutoff(a, b, algo.myData.settings.cutoff);
				}

			} else
				cutoff = algo.myData.settings.cutoff;

			if (r0 == null) {
				r0 = this.masterSigma;
				try {

					Hashtable<String, Float> hash = ((Hashtable<String, Float>[]) a
							.getUserData())[1];
					r0 = hash.get(b.getParameterKey()).floatValue();
				} catch (Exception e) {

				}
			}

			if (continuousBox && algo.myData.settings.AutoCutoff) {
				second = null;
				third = null;
				Vector3f add = new Vector3f();
				boolean overBorder = false;
				if ((a.position.x + cutoff) > box.width) {
					add.add(new Vector3f(-box.width, 0, 0));
					overBorder = true;
				}
				if ((a.position.z + cutoff) > box.height) {
					add.add(new Vector3f(0, 0, -box.height));
					overBorder = true;
				}

				if (overBorder) {

					second = new Point3f(a.position);
					second.add(add);

				}

				add = new Vector3f();
				overBorder = false;
				if ((a.position.x - cutoff) < 0) {
					add.add(new Vector3f(box.width, 0, 0));
					overBorder = true;
				}
				if ((a.position.z - cutoff) < 0) {
					add.add(new Vector3f(0, 0, box.height));
					overBorder = true;
				}

				if (overBorder) {

					third = new Point3f(a.position);
					third.add(add);

				}

			}

			if (a != b) {

				if (ef == null) {
					boolean isNeighbor = this.isNeighbor(a, b);

					// System.out.println(isNeighbor);
					if (isNeighbor)
						continue;
				}

				Float epsilon = null;

				if ((b.position.x < a.position.x - cutoff)
						|| (b.position.x > a.position.x + cutoff)
						|| (b.position.z < a.position.z - cutoff)
						|| (b.position.z > a.position.z + cutoff)
						|| (b.position.y < a.position.y - cutoff)
						|| (b.position.y > a.position.y + cutoff)) {

				} else {

					float distance = a.position.distance(b.position);

					if (distance > cutoff) {

						totalEnergy += 0f;
					} else {

						if (ef == null) {

							epsilon = this.masterEpsilon;
							try {
								Hashtable<String, Float> hash = ((Hashtable<String, Float>[]) a
										.getUserData())[0];
								epsilon = hash.get(b.getParameterKey())
										.floatValue();
							} catch (Exception e) {

							}

							// System.out.println(epsilon*1000*Math.pow(10,-18)*1000);

							if (a.getAtomicMass() > 0 && b.getAtomicMass() > 0) {
								epsilon *= ((a.getAtomicMass() + b
										.getAtomicMass()) / 2f);
							}
							epsilon *= 4;

							totalEnergy += this.calculateLJPotential(
									a.position, b.position, r0, distance,
									epsilon);
						} else
							totalEnergy += ef.calculateEnergy(a, b,
									algo.myData.settings.constraintFactor,
									distance,
									algo.myData.settings.currentTemparature);
						// System.out.println("1 "+totalEnergy);

					}
				}
				if (second != null) {

					if ((b.position.x < second.x - cutoff)
							|| (b.position.x > second.x + cutoff)
							|| (b.position.z < second.z - cutoff)
							|| (b.position.z > second.z + cutoff)
							|| (b.position.y < second.y - cutoff)
							|| (b.position.y > second.y + cutoff)) {

					} else {

						float distance = second.distance(b.position);

						if (distance > cutoff) {

							totalEnergy += 0f;
						} else {

							Point3f p = a.getPosition();
							a.position = second;
							if (ef == null) {

								if (epsilon == null) {

									epsilon = this.masterEpsilon;
									try {
										Hashtable<String, Float> hash = ((Hashtable<String, Float>[]) a
												.getUserData())[0];
										epsilon = hash.get(b.getParameterKey())
												.floatValue();
									} catch (Exception e) {

									}

									if (a.getAtomicMass() > 0
											&& b.getAtomicMass() > 0) {
										epsilon *= ((a.getAtomicMass() + b
												.getAtomicMass()) / 2f);
									}
									epsilon *= 4;

								}
								totalEnergy += this.calculateLJPotential(
										second, b.position, r0, distance,
										epsilon);
							} else {

								totalEnergy += ef
										.calculateEnergy(
												a,
												b,
												algo.myData.settings.constraintFactor,
												distance,
												algo.myData.settings.currentTemparature);

							}
							a.position = p;

						}
					}

				}
				if (third != null) {

					if ((b.position.x < third.x - cutoff)
							|| (b.position.x > third.x + cutoff)
							|| (b.position.z < third.z - cutoff)
							|| (b.position.z > third.z + cutoff)
							|| (b.position.y < third.y - cutoff)
							|| (b.position.y > third.y + cutoff)) {

					} else {

						float distance = third.distance(b.position);

						if (distance > cutoff) {

							totalEnergy += 0f;
						} else {
							Point3f p = a.getPosition();
							a.position = third;
							if (ef == null) {

								if (epsilon == null) {

									epsilon = this.masterEpsilon;
									try {
										Hashtable<String, Float> hash = ((Hashtable<String, Float>[]) a
												.getUserData())[0];
										epsilon = hash.get(b.getParameterKey())
												.floatValue();
									} catch (Exception e) {

									}
									if (a.getAtomicMass() > 0
											&& b.getAtomicMass() > 0) {
										epsilon *= ((a.getAtomicMass() + b
												.getAtomicMass()) / 2f);
									}
									epsilon *= 4;

								}

								totalEnergy += this.calculateLJPotential(third,
										b.position, r0, distance, epsilon);
							} else {

								totalEnergy += ef
										.calculateEnergy(
												a,
												b,
												algo.myData.settings.constraintFactor,
												distance,
												algo.myData.settings.currentTemparature);

							}
							a.position = p;

						}
					}

				}
			}
		}

		return totalEnergy;

	}

	private boolean isNeighbor(Atom a, Atom b) {

		if (a.getParentalMolecule() != b.getParentalMolecule())
			return false;

		// return checkIt(a,b,0);

		for (int j = 0; j < a.bonds.length; j++) {
			if (a.bonds[j].neigbor == b) {
				return true;

			}
		}
		return false;
	}

	@SuppressWarnings("unused")
	private boolean checkIt(Atom ref, Atom other, int level) {

		level++;
		if (level > 4)
			return false;

		for (int j = 0; j < ref.bonds.length; j++) {
			if (ref.bonds[j].getBondedNeighbor() == other)
				return true;
			else if (checkIt(ref.bonds[j].getBondedNeighbor(), other, level))
				return true;
		}
		return false;

	}

	public float calculateLJPotential(Point3f a, Point3f b, Float r0,
			float distance, float epsilonTimes4) {

		float term = r0 / distance;

		float second = term;

		second = (float) Math.pow(second, 6);

		float first = second;

		first *= term;
		first *= term;
		first *= term;
		first *= term;
		first *= term;
		first *= term;

		float result = epsilonTimes4 * (first - second);

		return result;

	}

	

	public double totalEnergy(Atom atom) {

		if (algo.energies.currentFunction != null)
			return this.calculateTotalEnergy(atom,
					this.algo.energies.currentFunction);

		return this.calculateTotalEnergy(atom, null);

	}

	public double totalInternalEnergy(Atom atom) {

		if (algo.energies.currentFunction != null)
			return 0;

		double en = 0;

		if (algo.myData.settings.constraintFactor > 0) {

			for (int i = 0; i < atom.bonds.length; i++) {

				

				float length = atom.bonds[i].getCurrentLength();

				float origLength = atom.bonds[i].getOriginalLength();

				

				float d = length - origLength;

				d *= d;

				d *= (algo.myData.settings.constraintFactor * 2);

				if (d != Float.NaN)
					en += (1000 * d * (25f / (algo.myData.settings.currentTemparature > 0.1f ? algo.myData.settings.currentTemparature
							: 0.1f)));

			}

		}

		if (algo.myData.settings.atomMovement) {

			Atom[] others = atom.getParentalMolecule().getOriginalAtoms();

			for (int j = 0; j < others.length; j++) {
				if (others[j] == atom)
					continue;

				boolean isBonded = false;
				for (int k = 0; k < atom.bonds.length; k++) {
					if (atom.bonds[k].neigbor == others[j]) {
						isBonded = true;
						break;
					}
				}
				if (isBonded)
					continue;

				float distance = others[j].position.distance(atom.position);
				float reqDist;
				if (atom.isCoarseGrainParticle()
						|| others[j].isCoarseGrainParticle()) {
					reqDist = 2;

				} else {
					reqDist = (atom.getCovalentRadiusInAngstrom() + others[j]
							.getCovalentRadiusInAngstrom()) * 1.3f;

				}

				if (distance < reqDist) {
					float diff = reqDist - distance;

					diff *= diff;
					diff *= 2;

					if (diff != Float.NaN)
						en += 1000 * diff;

					break;
				}
			}

		}

		return en;

	}

	public double getLowestValuePossibleMultiplied() {

		double d;
		if (this.currentFunction != null) {
			d = this.currentFunction.getLowestPossibleReturnValue();
		} else {

			if (this.lowestValue != null)
				return lowestValue;

			float max = Float.MIN_VALUE;
			String[] types = algo.myData.contents
					.getAllDifferentParticleTypesInMembrane(algo);
			for (int i = 0; i < types.length; i++) {
				for (int j = 0; j < types.length; j++) {
					Float f = this.hashEpsilon.get(types[i]).get(types[j]);

					if (f != null) {

						max = Math.max(f, max);
					}
				}
			}

			max = Math.max(this.masterEpsilon, max);

			d = -max;

		}

		this.lowestValue = new Double(d
				* (double) algo.myData.contents.atomsToCompare.length);

		return this.lowestValue;
	}

}
