/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms.atomlevelminimizer;

import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.cellmicrocosmos.cm2.membranepackingalgorithm.Atom;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Lipid;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Molecule;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Protein;


/**
 * This class stores most of the current values, members and settings of the Algorithm
 * 
 * @author Tim Dingersen
 * dackelnose@gmx.net
 */
public class MyData {

	public CurrentSettings settings = new CurrentSettings();
	public Contents contents = new Contents();

	public MyData() {

	}

	class Contents {
		public Molecule[] moleculesToTreat;
		public Molecule[] moleculesToCompare;
		public Atom[] atomsToTreat;
		public Atom[] atomsToCompare;
		public Vector<Double> energyHistory = new Vector<Double>();

		public Double currentFreeEnergy = new Double(0);
		public int step = 0;
		public int stepsSinceLastCooling = 0;

		public void setRelevantMolecules(Molecule[] mols, int layer,
				int domain, Atom_Level_Minimizer algo) {
			Vector<Molecule> molsToTreat = new Vector<Molecule>();
			Vector<Molecule> molsToCompare = new Vector<Molecule>();

			boolean hydros = false;
			for (int i = 0; i < mols.length; i++) {

				molsToCompare.addElement(mols[i]);

				if (mols[i] instanceof Protein) {
					if (settings.alsoProteins) {

						if (settings.AllLayers) {
							molsToTreat.addElement(mols[i]);
						} else if (mols[i].getLayerID() == layer) {

							if (domain == -1
									|| (mols[i].getDomainIdentifier() == domain))
								molsToTreat.addElement(mols[i]);
						}

					}

				} else if (mols[i] instanceof Lipid) {

					if (settings.AllLayers) {
						molsToTreat.addElement(mols[i]);
					} else if (mols[i].getLayerID() == layer) {
						if (domain == -1
								|| (mols[i].getDomainIdentifier() == domain))
							molsToTreat.addElement(mols[i]);
					}

				}

			}

			this.moleculesToTreat = molsToTreat.toArray(new Molecule[0]);
			this.moleculesToCompare = molsToCompare.toArray(new Molecule[0]);

			Vector<Atom> toTreat = new Vector<Atom>();
			Vector<Atom> toCompare = new Vector<Atom>();
			for (int i = 0; i < this.moleculesToTreat.length; i++) {
				Atom[] atoms = this.moleculesToTreat[i].getAlignedAtomCopies();
				for (int j = 0; j < atoms.length; j++) {
					if (settings.alsoUnbonded) {

						toTreat.addElement(atoms[j]);
					} else if (atoms[j].bonds.length > 0) {
						toTreat.addElement(atoms[j]);
					}

				}

			}
			for (int i = 0; i < this.moleculesToCompare.length; i++) {
				Atom[] atoms = this.moleculesToCompare[i]
						.getAlignedAtomCopies();
				for (int j = 0; j < atoms.length; j++) {
					toCompare.addElement(atoms[j]);
				}

			}

			// Collections.shuffle(toCompare,algo.random);
			this.atomsToTreat = toTreat.toArray(new Atom[0]);
			this.atomsToCompare = toCompare.toArray(new Atom[0]);

			if (algo.energies != null && algo.energies.currentFunction != null) {
				algo.energies.currentFunction
						.MembersUpdated(this.atomsToCompare);
			}

			algo.energies.lowestValue = null;

			for (int i = 0; i < this.atomsToTreat.length; i++) {
				if (this.atomsToTreat[i].getElement().equals("H"))
					hydros = true;

				Hashtable<String, Float> h1 = algo.energies.hashEpsilon
						.get(this.atomsToTreat[i].getParameterKey());
				Hashtable<String, Float> h2 = algo.energies.hashSigma
						.get(this.atomsToTreat[i].getParameterKey());

				this.atomsToTreat[i].setUserData(new Hashtable[] { h1, h2 });
			}
			algo.hasHydrogens = hydros;
		}

		public Double[] getEnergyHistory() {

			return this.energyHistory.toArray(new Double[0]);
		}

		public String[] getAllDifferentParticleTypesInMembrane(
				Atom_Level_Minimizer algo) {

			Lipid[] lips = algo.getAllSampleLipidsInModel();
			Molecule[] mols = algo.getAllMoleculesInCurrentModel();
			Hashtable<String, String> hash = new Hashtable<String, String>();

			for (int a = 0; a < lips.length; a++) {

				for (int i = 0; i < lips[a].getOriginalAtoms().length; i++) {
					if (lips[a].getOriginalAtoms()[i].isCoarseGrainParticle()) {
						hash.put(lips[a].getOriginalAtoms()[i].getParticleID(),
								"");
					} else {
						hash
								.put(
										lips[a].getOriginalAtoms()[i]
												.getElement(), "");
					}
				}
			}
			for (int a = 0; a < mols.length; a++) {

				for (int i = 0; i < mols[a].getOriginalAtoms().length; i++) {
					if (mols[a].getOriginalAtoms()[i].isCoarseGrainParticle()) {
						hash.put(mols[a].getOriginalAtoms()[i].getParticleID(),
								"");
					} else {
						hash
								.put(
										mols[a].getOriginalAtoms()[i]
												.getElement(), "");
					}
				}
			}
			Enumeration<String> en = hash.keys();
			Vector<String> elements = new Vector<String>();
			while (en.hasMoreElements()) {
				elements.addElement(en.nextElement());
			}
			Collections.sort(elements);

			return elements.toArray(new String[0]);
		}

		public void addCurrentEnergyToHistory(Double currentFreeEnergy) {
			step++;
			this.currentFreeEnergy = currentFreeEnergy;
			this.energyHistory.addElement(currentFreeEnergy);
			if (this.energyHistory.size() > 75) {
				this.energyHistory.remove(0);
				this.energyHistory.trimToSize();
			}
		}

	}

	class CurrentSettings {

		public boolean AutoCutoff = true;
		public boolean AllLayers = false;
		public boolean alsoProteins = false;
		public boolean alsoUnbonded = false;
		float currentTemparature = 0f;
		public int coolingIntervalLength = 0;
		public float coolingRate = 0;

		float currentMovingFactor = 0f;

		public float cutoff = 0f;
		public boolean isBoxContinuous = true;

		public float pressureY = 0;
		public float pressureXZ = 0;
		public boolean solidBorders = true;
		public float constraintFactor = 0.2f;
		public boolean moleculeMovement = true;
		public boolean atomMovement = false;
		public int addNewLipidsFactor = 0;

	}

}
