/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithms.atomlevelminimizer;

import java.util.Hashtable;

/**
 * This class is used to create default values for the Lennard-Jones-parameter.
 * Creates some fixed values for epsilon and calculates sigma by adding known Van-der-Waals radii
 * 
 * @author Tim Dingersen
 * dackelnose@gmx.net
 *
 */
public class MyDefaults {

	public static final double MASTER_SIGMA = 3.0;
	public static final double MASTER_EPSILON = 1.6;

	public static Hashtable<String, Hashtable<String, Float>> createDefaultEpsilons() {
		Hashtable<String, Hashtable<String, Float>> hash = new Hashtable<String, Hashtable<String, Float>>();

		Hashtable<String, Float> C = new Hashtable<String, Float>();
		C.put("C", 1.02f);
		C.put("N", 1.58f);
		C.put("O", 1.41f);
		C.put("H", 0.8f);
		C.put("S", 2.18f);
		hash.put("C", C);

		Hashtable<String, Float> N = new Hashtable<String, Float>();
		N.put("C", 1.58f);
		N.put("N", 2.5f);
		N.put("O", 2.22f);
		N.put("H", 1.22f);
		N.put("S", 3.4f);
		hash.put("N", N);

		Hashtable<String, Float> O = new Hashtable<String, Float>();
		O.put("C", 1.41f);
		O.put("N", 2.22f);
		O.put("O", 2.0f);
		O.put("H", 1.17f);
		O.put("S", 3.05f);
		hash.put("O", O);

		Hashtable<String, Float> H = new Hashtable<String, Float>();
		H.put("C", 0.8f);
		H.put("N", 1.22f);
		H.put("O", 1.17f);
		H.put("H", 0.78f);
		H.put("S", 1.72f);
		hash.put("H", H);

		Hashtable<String, Float> S = new Hashtable<String, Float>();
		S.put("C", 2.18f);
		S.put("N", 3.4f);
		S.put("O", 3.05f);
		S.put("H", 1.72f);
		S.put("S", 2.10f);
		hash.put("S", S);

		return hash;
	}

	public static Hashtable<String, Hashtable<String, Float>> createDefaultSigmas(
			Atom_Level_Minimizer algo) {

		Hashtable<String, Hashtable<String, Float>> hash = new Hashtable<String, Hashtable<String, Float>>();
		String[] types = algo.gui.paraPanel.allTypes;

		for (int i = 0; i < types.length; i++) {

			Hashtable<String, Float> hash2 = new Hashtable<String, Float>();

			float radiusA = algo.getSystemVanDerWaalsRadius(types[i]);
			if (radiusA < 0)
				continue;

			for (int j = 0; j < types.length; j++) {

				float radiusB = algo.getSystemVanDerWaalsRadius(types[j]);
				if (radiusB < 0)
					continue;

				float ret = radiusA + radiusB;

				// ret= Math.round(ret * 100f) / 100.0f;
				hash2.put(types[j], ret);

			}

			hash.put(types[i], hash2);

		}



		return hash;
	}
}
