/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.misc;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.Toolkit;
import java.io.File;
import java.util.Hashtable;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JLabel;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;


/**
 * 
 * Stores all icons for the application. Those are read in at startup
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class Cm2_IconDepot {

	// containing all icons that are found in the icon directory
	private static Hashtable<String, ImageIcon> icons = new Hashtable<String, ImageIcon>();

	/*
	 * The cursors that are used in this app besides the default cursor
	 */
	public static Cursor DNDCursor;

	public static Cursor PAINT1Cursor;

	public static Cursor PAINT2Cursor;

	public static Cursor PAINT3Cursor;

	/**
	 * Initializes all icons
	 * 
	 */
	public static void readIcons() {

		File[] files = CM2_CONSTANTS.IMAGES_AND_ICONS.ICONS_PARENT
				.listFiles();
		
		// this is for backward compatibility with Cm2_1 only
		if (files == null)
			files = CM2_CONSTANTS.IMAGES_AND_ICONS.ICONS_PARENT_CM2_1.listFiles();
		
		for (int i = 0; i < files.length; i++) {

			try {
				// using imageIO to validate the image. That doesnt work with
				// the
				// toolkit
				ImageIO.read(files[i]);
				Cm2_IconDepot.icons.put(Cm2_Toolkit.getSimpleName(files[i]),
						new ImageIcon(Toolkit.getDefaultToolkit().getImage(
								files[i].getAbsolutePath())));

			} catch (final Exception e) {

			}

		}
		try {
			Cm2_IconDepot.DNDCursor = new JLabel().getToolkit().createCustomCursor(
					Cm2_IconDepot.getSpecifiedIcon("DnD_Cursor").getImage(),
					new Point(0, 0), "");
			Cm2_IconDepot.PAINT1Cursor = new JLabel().getToolkit()
					.createCustomCursor(
							Cm2_IconDepot.getSpecifiedIcon("Paint_Rect_Cursor")
									.getImage(), new Point(0, 0), "");
			Cm2_IconDepot.PAINT2Cursor = new JLabel().getToolkit()
					.createCustomCursor(
							Cm2_IconDepot.getSpecifiedIcon("Paint_Ellipse_Cursor")
									.getImage(), new Point(0, 0), "");
			Cm2_IconDepot.PAINT3Cursor = new JLabel().getToolkit()
					.createCustomCursor(
							Cm2_IconDepot.getSpecifiedIcon("Paint_Polygon_Cursor")
									.getImage(), new Point(0, 0), "");
		} catch (final Exception e) {
			
			System.err.println("There was an error initializing the icons.");
			e.printStackTrace();
			
		}

	}

	/**
	 * Returns the Icon that has the passed name. If the file cannot be read, an
	 * default "NoImage"-icon will be returned. Dir-Path is taken from
	 * Cm2_CONSTANTS
	 * 
	 */
	public static ImageIcon getSpecifiedIcon(final String whichone) {

		try {
			if (Cm2_IconDepot.icons.get(whichone) == null)
				throw new Exception();

			return Cm2_IconDepot.icons.get(whichone);
		} catch (final Exception e) {

			// try to load the default image if requested one not available
			try {
				return Cm2_IconDepot.icons.get("Missing_Icon");

			} catch (final Exception e1) {
				return new ImageIcon();
			}

		}

	}

}
