/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithm;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

/**
 * This class can be used in order to show your algorithm-GUI to the user
 * 
 * @author Sebastian Schneider, Tim Dingersen, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class MembranePackingAlgorithmGUI {

	private static boolean b;

	private static JButton b1;

	private static JButton b2;

	private static JDialog dia;

	/**
	 * opens a JDialog in which the JComponent that u have designed will be
	 * displayed. It also provides two buttons. One that will force the dialog
	 * to dispose and return true ("Ok" button) and the other will force the
	 * dialog to return false("cancel" button).
	 * 
	 * The title of the JDialog will be set by the String "name".
	 * 
	 * A description can be displayed above the JComponent. Please pay attention
	 * that the description String is well formated and sized to prevent a bad
	 * Look and Feel.
	 * 
	 * 
	 * @param userPanel a
	 *            Component that contains your entire GUI
	 * @param name the
	 *            name of the dialog
	 * @param description a
	 *            short descrition that is displayed above your Component
	 * @return true if user proceeds, false if he cancels
	 */
	public static boolean popUpGUI(final Component userPanel,
			final String name, final String description) {
		MembranePackingAlgorithmGUI.dia = new JDialog();
		MembranePackingAlgorithmGUI.dia.setTitle(name);
		final JPanel mainPanel = new JPanel();
		final JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));

		final JLabel area = new JLabel("", SwingConstants.LEADING);

		area.setText(description);
		// header.setBackground(Color.WHITE);
		header.add(Box.createHorizontalStrut(3));
		header.add(area);
		header.add(Box.createHorizontalGlue());

		area.setFont(area.getFont().deriveFont(Font.BOLD,
				header.getFont().getSize2D() + 4));
		mainPanel.setLayout(new BorderLayout());
		mainPanel.add(header, BorderLayout.PAGE_START);

		final JScrollPane scrolli = new JScrollPane(userPanel);
		scrolli.setBorder(BorderFactory.createEtchedBorder());

		mainPanel.add(scrolli, BorderLayout.CENTER);
		// userPanel.setBorder(BorderFactory.createEtchedBorder());
		// mainPanel.add(userPanel);
		final JPanel lowerPanel = new JPanel();
		lowerPanel.setLayout(new BoxLayout(lowerPanel, BoxLayout.X_AXIS));

		final JPanel left = new JPanel();
		final JPanel right = new JPanel();

		left.add(new JLabel(" "));
		right.add(new JLabel(" "));

		final JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		MembranePackingAlgorithmGUI.b1 = new JButton("ok");
		MembranePackingAlgorithmGUI.b1.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent arg0) {
				if (arg0.getSource().equals(MembranePackingAlgorithmGUI.b1)) {
					MembranePackingAlgorithmGUI.b = true;
					MembranePackingAlgorithmGUI.dia.dispose();
				}
			}
		});

		buttonPanel.add(MembranePackingAlgorithmGUI.b1);
		buttonPanel.add(new JLabel("        "));
		buttonPanel.add(new JLabel(" "));

		MembranePackingAlgorithmGUI.b2 = new JButton("cancel");
		MembranePackingAlgorithmGUI.b2.addActionListener(new ActionListener() {

			public void actionPerformed(final ActionEvent e) {
				if (e.getSource().equals(MembranePackingAlgorithmGUI.b2)) {
					MembranePackingAlgorithmGUI.b = false;
					MembranePackingAlgorithmGUI.dia.dispose();
				}
			}
		});

		buttonPanel.add(MembranePackingAlgorithmGUI.b2);

		lowerPanel.add(left);
		lowerPanel.add(buttonPanel);
		lowerPanel.add(right);
		mainPanel.add(lowerPanel, BorderLayout.PAGE_END);

		MembranePackingAlgorithmGUI.dia.add(mainPanel);
		MembranePackingAlgorithmGUI.dia.setAlwaysOnTop(true);
		final Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();

		MembranePackingAlgorithmGUI.dia.setModal(true);

		MembranePackingAlgorithmGUI.dia.pack();
		MembranePackingAlgorithmGUI.dia.setLocation(dim.width / 2
				- MembranePackingAlgorithmGUI.dia.getWidth() / 2, dim.height / 2
				- MembranePackingAlgorithmGUI.dia.getHeight() / 2);
		MembranePackingAlgorithmGUI.dia.setVisible(true);

		return MembranePackingAlgorithmGUI.b;
	}

	/**
	 * Returns the JDialog, if it is required somehow
	 */
	public static JDialog getDialogFrame() {
		return MembranePackingAlgorithmGUI.dia;
	}
}
