/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_1)
 *
 * Copyright (c) 2005, 2014 The CELLmicrocosmos 2 Development Team
 *                          (Bjrn Sommer et al., Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithm;

import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * This toolkit offers some useful static methods concerning calculations with
 * matrices
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class RotationToolkit {

	/**
	 * Simply clones a matrix. Works better than the vecmath clone method that
	 * appears to be a little buggy
	 * 
	 * @param m the
	 *            matrix to copy
	 * @return a copy of the passed matrix
	 */
	public static Matrix3f copyMatrix(final Matrix3f m) {
		return Cm2_Toolkit.copyMatrix(m);
	}

	/**
	 * Creates a new unit matrix
	 * 
	 * @return a new unit matrix
	 */
	public static Matrix3f createAUnitMatrix() {
		return Cm2_Toolkit.createAUnitMatrix();
	}

	/**
	 * Rotates a point according to the passed matrix. The passed point, which
	 * represents the starting location won't be changed; a new point will be
	 * returned instead. So its up to the caller to replace the old point with
	 * the return value or to use it for something else.
	 * 
	 * Besides, this method proves to work better than the transform of Matrix3f
	 * 
	 * @param matrix
	 *            the matrix to transform the point with
	 * @param point3f
	 *            the coordinate to be transformed
	 * @return a new point having the transformed location
	 */
	public static Point3f transformToNewPoint(final Matrix3f matrix,
			final Point3f point3f) {

		return Cm2_Toolkit.transformToNewPoint(matrix, point3f);

	}

	/**
	 * Rotates a matrix around the X Axis of the MembraneEditor's coordinate
	 * space.
	 * 
	 * @param current
	 *            the matrix to be rotated
	 * @param angle
	 *            the rotation angle in degrees
	 */
	public static void rotate_X_Axis_Of_CoordinateSystem(
			final Matrix3f current, final float angle) {

		Cm2_Toolkit.rotate_X_Axis_Of_CoordinateSystem(current, angle);

	}

	/**
	 * Rotates a matrix around the Y Axis of the MembraneEditor's coordinate
	 * space.
	 * 
	 * @param current
	 *            the matrix to be rotated
	 * @param angle
	 *            the rotation angle in degrees
	 */
	public static void rotate_Y_Axis_Of_CoordinateSystem(
			final Matrix3f current, final float angle) {

		Cm2_Toolkit.rotate_Y_Axis_Of_CoordinateSystem(current, angle);

	}

	/**
	 * Rotates a matrix around the Z Axis of the MembraneEditor's coordinate
	 * space.
	 * 
	 * @param current
	 *            the matrix to be rotated
	 * @param angle
	 *            the rotation angle in degrees
	 */
	public static void rotate_Z_Axis_Of_CoordinateSystem(
			final Matrix3f current, final float angle) {

		Cm2_Toolkit.rotate_Z_Axis_Of_CoordinateSystem(current, angle);

	}

	/**
	 * Simply rotates a matrix around its X Axis. space.
	 * 
	 * @param current
	 *            the matrix to be rotated
	 * @param angle
	 *            the rotation angle in degrees
	 */
	public static void rotate_X_Axis_Matrix(final Matrix3f current,
			final float angle) {
		Cm2_Toolkit.rotate_X_Axis_Matrix(current, angle);
	}

	/**
	 * Simply rotates a matrix around its Y Axis. space.
	 * 
	 * @param current
	 *            the matrix to be rotated
	 * @param angle
	 *            the rotation angle in degrees
	 */
	public static void rotate_Y_Axis_Matrix(final Matrix3f current,
			final float angle) {
		Cm2_Toolkit.rotate_Y_Axis_Matrix(current, angle);
	}

	/**
	 * Simply rotates a matrix around its Z Axis. space.
	 * 
	 * @param current
	 *            the matrix to be rotated
	 * @param angle
	 *            the rotation angle in degrees
	 */
	public static void rotate_Z_Axis_Matrix(final Matrix3f current,
			final float angle) {
		Cm2_Toolkit.rotate_Z_Axis_Matrix(current, angle);
	}

}
